// Package e -----------------------------
// @file      : artistinfoMsg.go
// @author    : JJXu
// @contact   : wavingbear@163.com
// @time      : 2023/10/24 14:58
// -------------------------------------------
package e

import "fmt"

type ErrorCode int

// String  使用 fmt.Sprintf(Err13ArtistUidIsNull) 返回 "画家UID不能为空"
func (e ErrorCode) String() string {
	if v, ok := MsgFlags[e]; ok {
		return v
	} else {
		return fmt.Sprintf("未知错误:[%d]", e)
	}
}

const (
	Err13IDIsNull        = 130001
	Err13DuplicateData   = 130002
	Err13ScriptIsRunning = 130003
	//用户相关错误
	Err13ArtistUidIsNull        = 130101
	Err13ArtistTnumIsnull       = 130102
	Err13UserIsUnLock           = 130103
	Err13UserNotFound           = 130104
	Err13IDNumISNULL            = 130105
	Err13TelNumISNull           = 130106
	Err13NoOperationAfterUnlock = 130107
	Err13RealNameIsNull         = 130108
	Err13OriginIsNull           = 130109
	ErrorTelNumInBlackList      = 130110
	ErrorIdCardInBlackList      = 130111
	//画作相关错误
	Err13ArtworkUidIsNull   = 130201
	Err13ArtworkTfnumIsNull = 130202
	Err13UnknownAuditNode   = 130203
	Err13ArtworkNotFound    = 130204
	Err13BaseAuditNotFinish = 130205
	//画家指数
	Err13DuplicateTile = 130301
	//经纪人补充信息
	Err13ArtPackageUidIsNull = 130701
	Err13AgentNotExist       = 130702
	Err13AgentNotSpecified   = 130703
	Err13AgentHasSpecified   = 130704
	Err13DataHaventGenerate  = 130705
	//合同
	Err13ContractDuplicate = 130801
	Err13IllegalContract   = 130802
)

var err13Msg = map[ErrorCode]string{
	Error13ArtistInfoCodeStart:  "画家宝模块错误",
	Error13ArtistInfoCodeEnd:    "画家宝模块错误",
	Err13IDIsNull:               "ID不能为空",
	Err13DuplicateData:          "数据已存在",
	Err13ScriptIsRunning:        "脚本正在运行，不可重复操作",
	Err13ArtistUidIsNull:        "画家UID不能为空",
	Err13ArtistTnumIsnull:       "画家编号不能为空",
	Err13UserIsUnLock:           "未锁定该用户，操作取消",
	Err13UserNotFound:           "找不到此用户",
	Err13IDNumISNULL:            "身份证号不能为空",
	Err13TelNumISNull:           "手机号不能为空",
	Err13RealNameIsNull:         "真实姓名不能为空",
	Err13OriginIsNull:           "数据来源不能为空",
	Err13NoOperationAfterUnlock: "未做任何操作，无法解锁",
	Err13AgentNotExist:          "经纪人不存在",
	Err13ArtworkUidIsNull:       "画作UID不能为空",
	Err13ArtworkTfnumIsNull:     "画作编号不能为空",
	Err13ArtworkNotFound:        "画作不存在",
	Err13BaseAuditNotFinish:     "画作基本信息未审批完成，操作取消",
	Err13UnknownAuditNode:       "未知的审批节点",
	Err13DuplicateTile:          "标题重复",
	Err13ArtPackageUidIsNull:    "画展包Uid为必填",
	Err13AgentNotSpecified:      "用户尚未指定经纪人",
	Err13AgentHasSpecified:      "您已指定经纪人，不可重复操作",
	Err13DataHaventGenerate:     "数据尚未生成",
	Err13ContractDuplicate:      "合同已存在，请勿重复操作",
	Err13IllegalContract:        "存在非法的合同",
}

func init() {
	for k, v := range err13Msg {
		if _, ok := MsgFlags[k]; ok {
			panic(fmt.Sprintf("错误代码%d冲突,请检查pkg/e目录下的代码", k))
		} else {
			MsgFlags[k] = v
		}
	}
}
