// Package router -----------------------------
// @file      : artistinfoRouter.go
// @author    : JJXu
// @contact   : wavingbear@163.com
// @time      : 2023/3/6 16:45
// -------------------------------------------
package router

import (
	"github.com/fonchain_enterprise/fonchain-main/pkg/middleware"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/artistInfo/area"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/artistInfo/asArtist"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/artistInfo/asArtshow"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/artistInfo/asArtwork"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/artistInfo/asContract"
	//"github.com/fonchain_enterprise/fonchain-main/pkg/service/artistInfo/asCopyright"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/artistInfo/asPush"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/artistInfo/asStatement"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/artistInfo/asUser"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/artistInfo/authPayment"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/artistInfo/blackList"
	inCircleArticle "github.com/fonchain_enterprise/fonchain-main/pkg/service/artistInfo/inCricleArticle"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/artistInfo/mailArtwork"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/backend"
	"github.com/gin-gonic/gin"
)

func NewAuditRouter(r *gin.Engine) {
	// 后端
	v1 := r.Group("api")
	v1.Use(middleware.CheckLogin(service.AccountProvider), middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
	v2 := r.Group("api/v2")
	v2.Use(middleware.CheckLogin(service.AccountProvider), middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
	noAuth := r.Group("api")
	//
	//========================================================================================
	//                                    画家
	authGroup := v1.Group("")
	{
		noCheckLoginRouter := r.Group("api")
		//cacheUserCheckLoginRouter := v1.Use(new(middleware.ApiCache).ApiCacheMiddleware(30 * time.Second))
		//cacheUserCheckLoginRouter.POST("user/view/list/get", asArtist.GetArtistProfileList)
		v1.POST("user/view/list/get", asArtist.GetArtistProfileList)
		v1.POST("artist/supplement/getSupplementDetail", asArtshow.ArtistSupplement.GetSupplementDetail)
		v1.POST("user/lock", asUser.LockUser)
		v1.POST("user/lockRollback", asUser.LockRollBack)                                    //取消解锁
		v1.POST("user/getInvitedList", asUser.GetInvitedListByBackendSide2)                  // 获取接受邀请的画家列表(邀请了哪些人)
		noCheckLoginRouter.POST("user/getInvitedList2", asUser.GetInvitedListByBackendSide2) // 获取接受邀请的画家列表(邀请了哪些人),无身份校验
		v1.POST("user/getInviterUserList", asUser.GetInviterUserList)                        // 获取邀请人列表（被哪些人邀请过）
		v1.POST("user/inviteStatic", asUser.InviteStatic)                                    // 邀请人数统计列表(只统计邀请人数大于1的画家列表)
		v1.POST("user/batchCreateQrCode", asUser.BatchCreateQrCode)                          // 批量生成二维码
		v1.POST("artist/import", asArtist.ImportUserToArtistService)                         // 导入画家宝画家到画家服务
		v1.POST("artist/importArtistToArtistinfo", asArtist.ImportArtistToArtistinfo)        //画家导入画家宝
		v1.POST("realNameAudit/query", asUser.GetRealNameAuditList)                          //获取实名审批列表
		v1.POST("realNameAudit/edit", asUser.EditRealNameAudit)                              //编辑实名审批信息
		//------------------------------User
		v1.POST("user/User/create", asUser.Handler.CreateUser)
		v1.POST("user/User/delete", asUser.Handler.DeleteUser)
		v1.POST("user/User/update", asUser.Handler.UpdateUser)
		v1.POST("user/User/detail", asUser.Handler.GetUserDetail)
		v1.POST("user/User/query", asUser.Handler.GetUserList)

		v1.POST("artistInfoUser/export/view", asUser.GetBackendUserViewList)
		v1.POST("artistInfoUser/export/excel", asUser.ExportBackendUserViewList)
		v1.POST("artistInfoUser/export/static", asUser.ExportBackendUserStatic)
	}
	//========================================================================================
	//                                    画作
	{
		//v1.POST("artwork/profile/create", asArtwork.CreateWorkProfile)                                         //创建画作首选项
		//v1.POST("artwork/profile/update", asArtwork.UpdateArtwork)                                             //更新画作首选项
		//v1.POST("artwork/profile/update2", asArtwork.UpdateArtwork2)                                           //更新画作首选项(根据页面进行了优化)
		v1.POST("artwork/profile/get", asArtwork.GetArtworkProfile) //查询画作详情
		//v1.POST("artwork/profile/preview/get", asArtwork.GetArtworkPreviewListAtUserSide)                      //查询画作预览列表(画作管理中的列表)
		v1.POST("artwork/profile/preview/history", asArtwork.GetArtworkHistoryList) //查询画作历史记录
		//v1.POST("artwork/list/query", asArtwork.GetArtworkList)                                                //查询画作列表 todo（暂时弃用）
		v1.POST("artwork/delete", asArtwork.DelArtwork) //删除画作
		//v1.POST("artwork/supplement/UnsupplementedList", asArtwork.Supplement.GetSupplementDataListAtUserSide) //获取当前的画作信息补充列表(用户端) //GetUnsupplementedDataList
		//v1.POST("artwork/supplement/update", asArtwork.Supplement.UpdateSupplement)                            //画作信息补充
		v1.POST("artwork/supplement/detail", asArtwork.Supplement.Detail)  //获取画作补充信息详情
		v1.POST("artwork/getArtistArtworkList", asArtwork.GetArtworkList2) //获取指定画家的画作列表
		v1.POST("artwork/import", asArtwork.ImportArtowrkToArtworkService) //导入画作到画作服务
	}
	//========================================================================================
	//                                    画展
	{
		v1.POST("artshow/UnfinishedAuditStatistics", asArtshow.GetAuditUnfinishedStatistics)     //获取画展模块未完成审批统计
		v1.POST("artshow/video/getvideos", asArtshow.ArtshowVideo.GetArtshowVideoListAtUserSide) //获取画展视频列表
		v1.POST("artshow/video/updateVideo", asArtshow.ArtshowVideo.UpdateArtshowVideo)          //更新视频信息
		v1.POST("artshow/video/getVideoDetail", asArtshow.ArtshowVideo.GetVideoDetail)           //获取视频详情

		v1.POST("artshow/artistIndex/getArtistIndexList", asArtshow.ArtshowIndex.GetArtistIndexListAtUserSide) //用户端获取画家指数列表
		v1.POST("artshow/artistIndex/updateArtistIndex", asArtshow.ArtshowIndex.UpdateArtistIndex)             //更新画家指数

		v1.POST("artshow/artistSupplement/getSupplementList", asArtshow.ArtistSupplement.GetArtistSupplementListAtUserSide) //用户端获取画家补充信息列表
		v1.POST("artshow/artistSupplement/updateSupplement", asArtshow.ArtistSupplement.UpdateArtistSupplement)             //更新画家补充信息

		v1.POST("artshow/artshowHistoryGroup", asArtshow.ArtshowHistoryGroup) //获取画家的画展历史记录
		v1.POST("artshow/list", asArtshow.ArtshowList)                        //获取画家的画展历史记录
		v1.POST("artshow/update-recommend", asArtshow.UpdateRecomend)         //获取画家的画展历史记录

	}
	{

		v1.POST("backend/artwork/auditFlow/audit", asArtwork.Supplement.AuditSupplement)           //画作审批(基本信息和补充信息共用一个接口)
		v1.POST("backend/artwork/auditFlow/reAudit", asArtwork.Supplement.ReAuditBaseInfo)         //重新审批(目前只有基本信息审批有这功能) Deprecated 此逻辑由前端控制，后台不需要处理
		v1.POST("backend/artwork/preview/list", asArtwork.GetArtworkPreviewListAtBackendSide2)     //画作基本信息列表(后端)
		v1.POST("backend/artwork/preview/detail", asArtwork.GetArtworkPreviewDetailAtBackendSide2) //画作基本信息详情
		//v1.POST("backend/artwork/supplement/list", asArtwork.Supplement.GetSupplementDataListAtBackendSide3) //画作信息补充列表(后端)
		v1.POST("backend/artwork/supplement/list", asArtwork.Supplement.GetViewArtworkSupplementDetailList)

		v1.POST("backend/artwork/viewReviewArtistStatic/query", asArtwork.Supplement.GetViewReviewArtistStaticList)
		v1.POST("backend/artwork/auditFlow/baseReviewAudit", asArtwork.Supplement.ReviewBaseArtworkV2)  //基本画作复审
		v1.POST("backend/artwork/auditFlow/baseReviewList", asArtwork.Supplement.ReviewBaseArtworkList) //复审画作列表
		v1.POST("backend/artwork/review/export/excel", asArtwork.Supplement.ExportArtworkReviewList)    //导出基本画作复审数据
		v1.POST("backend/artwork/review/export/static", asArtwork.Supplement.ExportArtworkReviewStatic) //导出复审静态资源
		v1.POST("backend/artwork/auditFlow/baseReviewAudit/save", asArtwork.Supplement.TempSaveReviewStatus)
		v1.POST("backend/artwork/auditFlow/baseReviewAudit/list", asArtwork.Supplement.TempSaveReviewStatusList)
		v1.POST("artistInfoArtwork/artworkBaseAuditHistory/query", asArtwork.Supplement.GetArtworkBaseAuditHistoryList)
		v1.POST("artistInfoArtwork/reviewAuditMessage/query", asArtwork.Supplement.GetReviewAuditMessageList)    //复审消息查询
		v1.POST("artistInfoArtwork/reviewAuditMessage/static", asArtwork.Supplement.GetReviewAuditMessageStatic) //复审消息阅读状态统计
		v1.POST("artistInfoArtwork/reviewAuditMessage/update", asArtwork.Supplement.UpdateReviewAuditMessage)    //更新复审消息，一键审批
		v1.POST("artistInfoArtwork/reviewAuditMessage/delete", asArtwork.Supplement.DeleteReviewAuditMessage)    //删除复审消息，一键删除已读消息

		v1.POST("backend/artwork/supplement/generate", asArtwork.Supplement.GenerateSupplementInfo)               //生成画作补充信息
		v1.POST("backend/menus/get", backend.MenusGet)                                                            //画作信息补充列表(后端)
		v1.POST("backend/artist/UnfinishedArtworkBase", backend.GetArtistListUnfinishedArtworkBase)               //获取未完成画作基本信息审批的艺术家列表
		v1.POST("backend/artist/UnfinishedArtworkSupplement", backend.GetArtistListOfUnfinishedArtworkSupplement) //获取未完成画作补充信息审批的艺术家列表
		v1.POST("backend/artist/ArtistsArtworkStaticList", asArtist.ArtistsArtworkStaticList)                     //画家的画作数据统计
		v1.POST("backend/export-field-list", backend.ExportFieldList)
		authGroup.Use(middleware.CheckLogin(service.AccountProvider))
		//获取导出信息
		//authGroup.POST("backend/artwork/preview/list-export", asArtwork.GetArtworkPreviewListAtBackendSideExport2) //画作基本信息列表(后端)
		v1.POST("backend/artwork/preview/list-export", asArtwork.GetArtworkPreviewListAtBackendSideExport2) //画作基本信息列表(后端)

		v1.POST("backend/artshow/video/generateVideoRecord", asArtshow.ArtshowVideo.GenerateArtshowVideoRecord)   //生成画展视频记录
		v1.POST("backend/artshow/video/getVideoRecords", asArtshow.ArtshowVideo.GetArtshowVideoListAtBackendSide) //获取待审批画展视频列表
		v1.POST("backend/artshow/video/auditVideo", asArtshow.ArtshowVideo.AuditArtshowVideo)                     //审批画展视频资料
		v1.POST("backend/artshow/video/getLockedArtistList", asArtshow.ArtshowVideo.GetLockedArtistList)          //获取锁定的画家列表（视频）

		v1.POST("backend/artshow/artistIndex/createArtistIndex", asArtshow.ArtshowIndex.GenerateArtistIndexRecords)          //生成画家指数记录
		v1.POST("backend/artshow/artistIndex/getArtistIndexRecords", asArtshow.ArtshowIndex.GetArtistIndexListAtBackendSide) //获取画家指数审批列表
		v1.POST("backend/artshow/artistIndex/auditArtistIndex", asArtshow.ArtshowIndex.AuditArtistIndex)                     //审批画家指数数据
		v1.POST("backend/artshow/artistIndex/getLockedArtistList", asArtshow.ArtshowIndex.GetLockedArtistList)               //获取锁定的画家列表（画家指数）
		v1.POST("backend/artshow/artistIndex/getArtistIndexDetail", asArtshow.ArtshowIndex.GetArtistIndexDetail)             //获取画家指数详情
		v1.POST("backend/artshow/artistIndex/staticDetail", asArtshow.ArtshowIndex.GetViewArtistIndexStaticDetail)           //锁定画家画家指数统计详情
		v1.POST("backend/artshow/artistIndex/staticList", asArtshow.ArtshowIndex.GetViewArtistIndexStaticList)               //锁定画家画家指数统计列表

		v1.POST("backend/artshow/artistSupplement/createSupplement", asArtshow.ArtistSupplement.GenerateArtistSupplementRecords)          //生成画家补充信息记录
		v1.POST("backend/artshow/artistSupplement/getSupplementRecords", asArtshow.ArtistSupplement.GetArtistSupplementListAtBackendSide) //获取成画家补充信息审批列表
		v1.POST("backend/artshow/artistSupplement/auditSupplement", asArtshow.ArtistSupplement.AuditArtistSupplement)                     //审批成画家补充信息数据
		//v1.POST("backend/artshow/artistSupplement/getSupplementList", asArtshow.ArtistSupplement.GetLockedArtistList)                   //获取锁定的画家列表（画家信息补充）

		v1.POST("artshow/broker/getItemList", asArtshow.BrokerSupplement.GetItemList)                                           //获取经纪人下拉菜单列表
		v1.POST("artshow/broker/updateRecord", asArtshow.BrokerSupplement.UpdateBrokerSupplementRecord)                         //更新经纪人补充信息
		v1.POST("artshow/broker/getSupplementDetail", asArtshow.BrokerSupplement.GetSupplementDetail)                           //获取经纪人补充信息详情
		v1.POST("artshow/broker/getSupplementList", asArtshow.BrokerSupplement.GetSupplementList)                               //获取经纪人补充信息列表s
		v1.POST("backend/artshow/broker/createBrokerSupplementRecord", asArtshow.BrokerSupplement.CreateBrokerSupplementRecord) //创建经纪人补充信息
		v1.POST("backend/artshow/broker/bindAndGetBroker", asArtshow.BrokerSupplement.BindAndGetArtPackageAndBrokerSupplement)  //绑定并获取画展包经纪人

		v1.POST("backend/artshow/generateAllSupplement", asArtshow.ArtshowIndex.GenerateAllSupplement) //生成所有画家补充信息（画展视频资料、画家指数、画家补充信息、经纪人补充信息）
	}
	//                                  经纪人画展
	{
		v1.POST("artshow/broker/updateArtistSupplement", asArtshow.ArtistSupplement.AuditArtistSupplement) //审批成画家补充信息数据
	}
	//========================================================================================
	//                                 经纪人代补充画展信息
	{
		v1.POST("artshow/broker/getArtistSupplementStatus", asArtshow.BrokerOperationControl.GetArtistSupplementStatus)           //获取画家信息的补充状态
		v1.POST("artshow/broker/getArtistSupplementDetail", asArtshow.BrokerOperationControl.GetArtistSupplementDetail)           //获取画家简介补充信息
		v1.POST("artshow/broker/getArtworkSupplementStatusList", asArtshow.BrokerOperationControl.GetArtworkSupplementStatusList) //获取画作补充状态列表
		v1.POST("artshow/broker/getArtistIndexSupplementList", asArtshow.BrokerOperationControl.GetArtistIndexSupplementList)     //获取画家指数列表
		v1.POST("artshow/broker/getArtistVideo", asArtshow.BrokerOperationControl.GetArtistVideo)                                 //获取画家视频
		v1.POST("artshow/broker/getArtworkSupplementDetail", asArtshow.BrokerOperationControl.GetArtworkSupplementDetail)         //获取画作补充信息
		v1.POST("artshow/broker/updateArtistInfo", asArtshow.BrokerOperationControl.UpdateArtistInfo)                             //更新画家信息
		v1.POST("artshow/broker/updateArtistVideoSupplement", asArtshow.BrokerOperationControl.UpdateArtistVideoSupplement)       //更新画家视频信息
		v1.POST("artshow/broker/createArtistIndexSupplement", asArtshow.BrokerOperationControl.CreateArtistIndexSupplement)       //更新画家指数
		v1.POST("artshow/broker/updateArtworkSupplement", asArtshow.BrokerOperationControl.UpdateArtworkSupplement)               //更新画作补充信息
		v2.POST("artshow/broker/getArtistSupplementDetail", asArtshow.BrokerOperationControl.GetArtistSupplementDetailV2)         //获取画家简介补充信息V2
		v2.POST("artshow/broker/updateArtistInfo", asArtshow.BrokerOperationControl.UpdateArtistInfoV2)                           //更新画家信息V2
		v1.POST("artshow/broker/getArtshowFlowPrice", asArtshow.BrokerOperationControl.GetArtshowFlowPrice)                       //获取平尺数底价详情
		v1.POST("artshow/broker/createArtshowFlowPrice", asArtshow.BrokerOperationControl.CreateArtshowFlowPrice)                 //获取平尺数底价详情
		v1.POST("artshow/broker/getSupplementFromArtistSystem", asArtshow.BrokerOperationControl.GetArtistSupplementFromSystem)   //从画家系统获取补充信息
		v1.POST("artshow/bankInfo/update", asArtshow.BrokerOperationControl.UpdateBankInfo)                                       //从画家系统获取补充信息
		v1.POST("artshow/bankInfo/get", asArtshow.BrokerOperationControl.GetBankInfo)                                             //从画家系统获取补充信息
		v1.POST("artshow/copyrightComment/list", asArtshow.BrokerOperationControl.GetCopyrightCommentList)
		v1.POST("artshow/copyrightComment/update", asArtshow.BrokerOperationControl.UpdateCopyrightComment)
		v1.POST("artshow/copyrightComment/detail", asArtshow.BrokerOperationControl.GetCopyrightCommentDetail)
	}
	//========================================================================================
	//									合同
	{
		v1.POST("contract/createContractMgmt", asContract.CreateContractMgmt)            //创建合同
		v1.POST("contract/contractListMgmt", asContract.ContractListMgmt)                //画家合同列表
		v1.POST("contract/contractListMgmt2", asContract.ContractListMgmtV2)             //画家合同列表v2
		v1.POST("contract/contractArtistStatList", asContract.GetContractArtistStatList) //画家合同统计列表
		v1.POST("contract/batchCreate", asContract.BatchCreateContract)                  //批量创建合同
		v1.POST("contract/reBatchCreate", asContract.ReBatchCreateContract)              //重新一键发起合同
		v1.POST("contract/artworkList", asContract.ContractArtworkList)                  //合同画作列表
	}
	//========================================================================================
	//									对账单
	{
		v1.POST("statement/uploadExcelOneTx", asStatement.UploadExcelOneTx)                         //导入物权附件一
		v1.POST("statement/uploadExcelTwoTx", asStatement.UploadExcelTwoTx)                         //导入物权附件二
		v1.POST("statement/uploadExcelOneCopy", asStatement.UploadExcelOneCopy)                     //导入版权附件一
		v1.POST("statement/uploadExcelTwoCopy", asStatement.UploadExcelTwoCopy)                     //导入版权附件二
		v1.POST("statement/getAllBatchTimeTx", asStatement.GetAllBatchTimeTx)                       //获取物权所有批次时间
		v1.POST("statement/getAllBatchTimeCopy", asStatement.GetAllBatchTimeCopy)                   //获取版权所有批次时间
		v1.POST("statement/getArtworkSoldTxList", asStatement.GetArtworkSoldTxList)                 //按要求获取画家列表（物权）
		v1.POST("statement/getArtworkSoldTxDetailList", asStatement.GetArtworkSoldTxDetailList)     //根据批次uid获取画家已售的画作物权列表
		v1.POST("statement/getArtworkSoldCopyList", asStatement.GetArtworkSoldCopyList)             //按要求获取画家列表（版权）
		v1.POST("statement/getArtworkSoldCopyDetailList", asStatement.GetArtworkSoldCopyDetailList) //根据批次uid获取画家已售的画作版权列表
	}
	//========================================================================================
	//									黑名单
	{
		v1.POST("blackList/create", blackList.Service.Create)
		v1.POST("blackList/delete", blackList.Service.Delete)
		v1.POST("blackList/search", blackList.Service.Search)
		v1.POST("blackList/update", blackList.Service.Update)
	}
	//========================================================================================
	//									展讯
	{
		//--------------------------------展讯文章内容
		v1.POST("inCircleArticle/InCircleArticle/create", inCircleArticle.Handler.CreateInCircleArticle)
		v1.POST("inCircleArticle/InCircleArticle/delete", inCircleArticle.Handler.DeleteInCircleArticle)
		v1.POST("inCircleArticle/InCircleArticle/update", inCircleArticle.Handler.UpdateInCircleArticle)
		v1.POST("inCircleArticle/InCircleArticle/detail", inCircleArticle.Handler.GetInCircleArticleDetail)
		v1.POST("inCircleArticle/InCircleArticle/query", inCircleArticle.Handler.GetInCircleArticleList)
		v1.POST("inCircleArticle/InCircleArticle/like", inCircleArticle.Handler.SetLike)             //点赞
		v1.POST("inCircleArticle/InCircleArticle/sync", inCircleArticle.Handler.SyncInCircleArticle) //点赞
		//--------------------------------展讯用户信息
		v1.POST("inCircleArticle/InCircleUser/create", inCircleArticle.Handler.CreateInCircleUser)
		v1.POST("inCircleArticle/InCircleUser/delete", inCircleArticle.Handler.DeleteInCircleUser)
		v1.POST("inCircleArticle/InCircleUser/update", inCircleArticle.Handler.UpdateInCircleUser)
		v1.POST("inCircleArticle/InCircleUser/detail", inCircleArticle.Handler.GetInCircleUserDetail)
		v1.POST("inCircleArticle/InCircleUser/query", inCircleArticle.Handler.GetInCircleUserList)
		//--------------------------------展讯文章评论
		v1.POST("inCircleArticle/Comment/create", inCircleArticle.Handler.CreateComment)
		v1.POST("inCircleArticle/Comment/delete", inCircleArticle.Handler.DeleteComment)
		v1.POST("inCircleArticle/Comment/update", inCircleArticle.Handler.UpdateComment)
		v1.POST("inCircleArticle/Comment/detail", inCircleArticle.Handler.GetCommentDetail)
		v1.POST("inCircleArticle/Comment/query", inCircleArticle.Handler.GetCommentList)
		//------------------------------Swiper轮播图
		v1.POST("inCircleArticle/Swiper/create", inCircleArticle.Handler.CreateSwiper)
		v1.POST("inCircleArticle/Swiper/delete", inCircleArticle.Handler.DeleteSwiper)
		v1.POST("inCircleArticle/Swiper/update", inCircleArticle.Handler.UpdateSwiper)
		v1.POST("inCircleArticle/Swiper/detail", inCircleArticle.Handler.GetSwiperDetail)
		v1.POST("inCircleArticle/Swiper/query", inCircleArticle.Handler.GetSwiperList)
	}
	{
		//------------------------------ExportField
		v1.POST("artistInfoUser/exportField/create", asUser.Handler.CreateExportField)
		v1.POST("artistInfoUser/exportField/delete", asUser.Handler.DeleteExportField)
		v1.POST("artistInfoUser/exportField/update", asUser.Handler.UpdateExportField)
		v1.POST("artistInfoUser/exportField/detail", asUser.Handler.GetExportFieldDetail)
		v1.POST("artistInfoUser/exportField/query", asUser.Handler.GetExportFieldList)
	}
	{
		//------------------------------省市区，银行网点查询
		v1.POST("area/queryBankList", area.GetBankListHandler)         //获取银行列表
		v1.POST("area/queryMainBankList", area.GetMainBankListHandler) //获取总行列表
		v1.POST("area/getTree", area.GetAreaTreeHandler)               //获取省市区树
	}
	//----------------------------------其它
	{
		noAuth.POST("tmp/uploadpic", service.UploadImg)
		v1.POST("test/microErr", service.MicroErr)
		v1.POST("script/syncLowerArtist", asUser.SyncLowArtistHandler) //同步低优先级画家
		noAuth.POST("fake/login", asUser.FakeLogin)                    //模拟登录
		noAuth.POST("unipush2v0/test", asPush.Test)
	}
	//========================================================================================
	////									asCopyright
	//{
	//	//------------------------------CopyrightReg
	//	v1.POST("copyright/artwork/update", asCopyright.Handler.UpdateCopyrightReg)
	//	v1.POST("copyright/artwork/updateStatus", asCopyright.Handler.UpdateCopyrightRegStatus)
	//	v1.POST("copyright/artwork/detail", asCopyright.Handler.GetCopyrightRegDetail)
	//	v1.POST("copyright/artwork/query", asCopyright.Handler.GetCopyrightRegList)
	//	v1.POST("copyright/artwork/record/query", asCopyright.Handler.GetCopyrightRegRecordList)
	//	v1.POST("copyright/artwork/getDciPayUrl", asCopyright.Handler.GetDciPayUrl) //获取支付链接
	//	//v1.GET("copyright/artwork/dciPayWs", asCopyright.DigitPayService.ConnDciPayWebsocket)
	//	v1.POST("copyright/status/static", asCopyright.Handler.GetCopyrightStatusStatic)
	//	v1.POST("copyright/sync/dciData", asCopyright.Handler.SyncDciData)                          //同步初审数据
	//	v1.POST("copyright/startSecondReview", asCopyright.Handler.StartSecondReview)               //开始复审
	//	v1.POST("copyright/syncPaymentStatus", asCopyright.Handler.SyncPaymentStatusFromThirdParty) //同步支付状态
	//}
	//========================================================================================
	//									authPayment
	{
		paymentRouter := v1.Group("")
		paymentRouter.Use(middleware.InitBody(), middleware.RecordOperation())
		//------------------------------AuthPayment
		//v1.POST("authPayment/detail", authPayment.Handler.GetAuthPaymentDetail)
		//v1.POST("authPayment/query", authPayment.Handler.GetAuthPaymentList)
		paymentRouter.POST("authArtshow/view/query", authPayment.Handler.GetAuthPaymentArtShowList) //可发起支付画展包视图列表（一级）
		paymentRouter.POST("authPayment/view/query", authPayment.Handler.GetViewAuthPaymentList)    //发起支付记录视图列表（二级）
		paymentRouter.POST("authPayment/update/per", authPayment.Handler.UpdateAuthPaymentPercent)  //更新比例
		paymentRouter.POST("authPayment/create", authPayment.Handler.CreateAuthPaymentSend)         //发起支付
		paymentRouter.POST("authPayment/update", authPayment.Handler.UpdateAuthPayment)             //更新支付
		paymentRouter.POST("authPayment/delete", authPayment.Handler.DeleteAuthPayment)             //删除支付
		paymentRouter.POST("authPayment/stat", authPayment.Handler.GetAuthPaymentStat)              //鉴证支付统计
	}
	//========================================================================================
	//									mailArtwork 寄画退画
	{
		//寄画
		v1.POST("mailArtwork/viewArtistMailStat/query", mailArtwork.Handler.GetViewArtistMailStatList)   //一级列表，画家寄画统计列表
		v1.POST("mailArtwork/viewMailArtworkData/query", mailArtwork.Handler.GetViewMailArtworkDataList) //二级列表 ， 寄画的画作列表，包含画作信息
		v1.POST("mailArtwork/cansend/query", mailArtwork.Handler.GetCanSendMailArtworkList)              //查询可寄出的画作列表
		v1.POST("mailArtwork/confirm", mailArtwork.Handler.ConfirmMailArtwork)                           //发起寄画/退画
		v1.POST("mailArtwork/courierEdit", mailArtwork.Handler.CourierEdit)                              //快递信息填写
		v1.POST("mailArtwork/courierDetail", mailArtwork.Handler.CourierDetail)                          //快递信息查询
		v1.POST("mailArtwork/putInStorage", mailArtwork.Handler.PutInStorage)                            //入库
		v1.POST("mailArtwork/artworkCourierGroup", mailArtwork.Handler.ArtworkCourierGroup)              //通过uid查询画作和快递信息，并按照寄出状态分组
		//退画
		v1.POST("returnArtwork/artistReturnStat/query", mailArtwork.Handler.GetArtistReturnStatDataList)   //一级列表,画家退画统计列表
		v1.POST("returnArtwork/viewReturnArtwork/query", mailArtwork.Handler.GetViewReturnArtworkDataList) //二级列表，查询已发起退回的画作列表
		v1.POST("returnArtwork/canreturn/query", mailArtwork.Handler.GetCanReturnArtworkList)              //查询可退回的画作列表

		v1.POST("mailArtwork/mailArtwork/delete", mailArtwork.Handler.DeleteMailArtwork) //删除寄画信息
		v1.POST("mailArtwork/mailArtwork/update", mailArtwork.Handler.UpdateMailArtwork) //更新寄画信息
		v1.POST("mailArtwork/mailArtwork/shortUrlTest", mailArtwork.Handler.ShortUrlTest)
	}
}
