package router

import (
	"github.com/fonchain_enterprise/fonchain-main/pkg/middleware"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/account"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/auth/department"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/auth/position"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/auth/rule"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/deliver"
	"github.com/gin-gonic/gin"
	"net/http"
)

func AuthRoute(router *gin.RouterGroup) {
	r := router.Group("")
	r.StaticFS("/pdfst/print_model", http.Dir("./data/print_model"))
	positionV2Route := r.Group("/position/v2")
	positionV2Route.Use(middleware.CheckLogin(service.AccountProvider), middleware.CheckAuth(service.AccountProvider, service.RuleProvider))
	{
		positionV2Route.Use(middleware.WarnLog(service.AccountProvider, service.GrpcLogImpl))
		positionV2Route.POST("create", position.CreateV2)
		positionV2Route.POST("update", position.UpdateV2)
		positionV2Route.POST("detail", position.DetailV2)
		positionV2Route.POST("list", position.ListV2)
		positionV2Route.POST("batch/remove", position.BatchRemove)
		positionV2Route.POST("batch/bind/auth", position.BatchBindAuth)
		positionV2Route.POST("model/create", position.CreateModel) //创建模版
		positionV2Route.POST("model/remove", position.DeleteModel) //删除模版
		positionV2Route.POST("model/list", position.ListModel)     //模版列表
		positionV2Route.POST("model/detail", position.DetailModel) //模版详情
		positionV2Route.POST("log/list", position.LogList)         //日志列表
	}

	deliverV2Route := r.Group("/api/deliver")
	deliverV2Route.Use(middleware.CheckLogin(service.AccountProvider), middleware.CheckAuth(service.AccountProvider, service.RuleProvider))
	{
		deliverV2Route.POST("create", deliver.Create)
		deliverV2Route.POST("update", deliver.Update)
		deliverV2Route.POST("detail", deliver.Detail)
		deliverV2Route.POST("list", deliver.List)
		deliverV2Route.POST("remove", deliver.Remove)
	}

	departmentRoute := r.Group("/department/v2")
	departmentRoute.Use(middleware.CheckLogin(service.AccountProvider))
	departmentRoute.POST("tree/filter", department.FilterTree)
	departmentRoute.Use(middleware.CheckAuth(service.AccountProvider, service.RuleProvider))
	{
		departmentRoute.Use(middleware.WarnLog(service.AccountProvider, service.GrpcLogImpl))
		departmentRoute.POST("boss/level", department.LevelBoss)   //绑定部门
		departmentRoute.POST("boss/direct", department.DirectBoss) //绑定部门
		departmentRoute.POST("create", department.CreateV2)
		departmentRoute.POST("update", department.UpdateV2)
		departmentRoute.POST("detail", department.DetailV2)
		departmentRoute.POST("tree/my", department.MyTree)
		departmentRoute.POST("tree/all", department.AllTree)
		departmentRoute.POST("tree/all/position", department.AllTreeWithPosition)
		departmentRoute.POST("remove", department.Remove)            //删除部门
		departmentRoute.POST("bind", department.Bind)                //绑定部门
		departmentRoute.POST("unbind", department.UnBind)            //绑定部门
		departmentRoute.POST("list/key", department.ListByAuth)      //绑定部门
		departmentRoute.POST("position", department.DetailsV2)       //绑定部门
		departmentRoute.POST("base/all/list", department.BaseListV2) //绑定部门
	}

	ruleV2Route := r.Group("/rule/v2")
	ruleV2Route.Use(middleware.CheckLogin(service.AccountProvider), middleware.CheckAuth(service.AccountProvider, service.RuleProvider))
	{
		ruleV2Route.Use(middleware.WarnLog(service.AccountProvider, service.GrpcLogImpl))
		ruleV2Route.POST("update", rule.UpdateV2)
		ruleV2Route.POST("detail", rule.DetailV2)
		ruleV2Route.POST("menu/list", rule.ListV2)
	}

	userV2Route := r.Group("/user/v2")
	userV2Route.POST("sample/account", account.SampleAccount) //登陆-校验这是否有其他人有登陆信息
	userV2Route.POST("login", account.UserLoginSqueezeOther)  //新登陆,挤掉其他人
	userV2Route.Use(middleware.CheckLogin(service.AccountProvider), middleware.CheckAuth(service.AccountProvider, service.RuleProvider))
	{
		userV2Route.Use(middleware.WarnLog(service.AccountProvider, service.GrpcLogImpl))
		userV2Route.POST("create", account.UserRegisterV2)                   //注册
		userV2Route.POST("update", account.UpdateV2)                         //更新
		userV2Route.POST("reset/pwd", account.ResetPwd)                      //更新
		userV2Route.POST("detail", account.DetailV2)                         //获取详情
		userV2Route.POST("boss/staff/detail", account.DetailNowDepartmentV2) // 部门领导获取该员工信息 (部门岗位仅仅限制领导可见的岗位)
		userV2Route.POST("boss/staff/update", account.BossDetailUpdate)      // 部门领导获取该员工信息 (部门岗位仅仅限制领导可见的岗位)
		userV2Route.POST("my/info", account.MyInfo)                          //获取详情
		userV2Route.POST("list", account.ListV2)                             //获取详情
		userV2Route.POST("batch/remove", account.BatchRemoveV2)              //删除
		userV2Route.POST("remove", account.RemoveV2)                         //删除
		userV2Route.POST("check/register", account.CheckBeforeRegister)      //删除
		userV2Route.POST("/face", account.Face)
	}
}
