package router

import (
	"github.com/fonchain_enterprise/fonchain-main/pkg/middleware"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/blockchain"
	service_chainbrowser "github.com/fonchain_enterprise/fonchain-main/pkg/service/chainbrowser"
	service_invoice "github.com/fonchain_enterprise/fonchain-main/pkg/service/invoice"
	"github.com/gin-gonic/gin"
)

func ChainBrowserRoute(r *gin.RouterGroup) {
	//noAuth := r.Group("")
	auth := r.Group("")
	eCardErpAuth := r.Group("eCard")
	eCardAuth := r.Group("e_card")
	auth.Use(middleware.CheckLogin(service.AccountProvider), middleware.CheckAuth(service.AccountProvider, service.RuleProvider), middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
	browser := auth.Group("browser")
	{
		browser.POST("home-data", service_chainbrowser.HomeData)
		browser.POST("block-list", service_chainbrowser.BlockList)
		browser.POST("tx-list", service_chainbrowser.TxList)
		browser.POST("blockhash-detail", service_chainbrowser.BlockHashDetail)
		browser.POST("txhash-detail", service_chainbrowser.TxHashDetail)
		browser.POST("filehash-list", service_chainbrowser.FileHashList)
		browser.POST("file-add-chain", service_chainbrowser.FileAddChain)
		browser.POST("filehash-detail", service_chainbrowser.FileHashDetail)
		browser.POST("refresh-file-chain", service_chainbrowser.RefreshFileChain)

		browser.POST("hash-detail", service_chainbrowser.HashDetail)
	}

	equity := auth.Group("equity")
	{
		equity.POST("update/free/resources", service_chainbrowser.UpdateFreeResources)                 // 修改系统赠送权益值
		equity.POST("get/all/free/resources", service_chainbrowser.GetAllFreeResources)                // 获取所有的系统赠送权益值
		equity.POST("update/consume/resources", service_chainbrowser.UpdateConsumeResources)           // 修改消费权益值的存证消费值
		equity.POST("get/all/consume/resources", service_chainbrowser.GetAllConsumeResources)          // 获取消费权益值的存证的消费值
		equity.POST("update/conversion/ratio", service_chainbrowser.UpdateConversionRatio)             // 更新权益兑换比例表
		equity.POST("get/all/refill/card/status", service_chainbrowser.GetAllRefillCardStatus)         // 获取未删除的充值卡金额的状态
		equity.POST("create/refill/card/amount/info", service_chainbrowser.CreateRefillCardAmountInfo) // 新增充值金额信息
		equity.POST("get/refill/card/amount/info", service_chainbrowser.GetRefillCardAmountInfo)       // 获取所有的充值卡金额信息
		equity.POST("stop/refill/card/amount/info", service_chainbrowser.StopRefillCardAmountInfo)     // 停用充值金额
		equity.POST("open/refill/card/amount/info", service_chainbrowser.OpenRefillCardAmountInfo)     // 启用充值金额信息
		equity.POST("delete/refill/card/amount/info", service_chainbrowser.DeleteRefillCardAmountInfo) // 删除充值金额信息

		equity.POST("get/all/pay/status", service_chainbrowser.GetAllPayStatus)                                                        // 获取所有的支付状态
		equity.POST("recharge/record/manage", service_chainbrowser.RechargeRecordManage)                                               // 充值卡订单管理
		equity.POST("recharge/record/manage/export/form", service_chainbrowser.RechargeRecordManageExportForm)                         // 充值卡订单管理表导出
		equity.POST("get/all/refund/status", service_chainbrowser.GetAllRefundStatus)                                                  // 获取所有的退款状态
		equity.POST("refund/apply/manage", service_chainbrowser.RefundApplyManage)                                                     // 充值卡退款申请管理
		equity.POST("refund/apply/manage/export/form", service_chainbrowser.RefundApplyManageExportForm)                               // 充值卡退款申请管理表导出
		equity.POST("wx/refund", service_chainbrowser.WxRefund)                                                                        // 退款允许
		equity.POST("ali/refund", service_chainbrowser.AliRefund)                                                                      // 退款允许
		equity.POST("refund/refuse", service_chainbrowser.RefundRefuse)                                                                // 退款驳回
		equity.POST("get/all/add/type", service_chainbrowser.GetAllAddType)                                                            // 获取所有的增加权益值的类型
		equity.POST("resource/usage/detail/add/manage", service_chainbrowser.ResourceUsageDetailAddManage)                             // 资源权益使用（充值）
		equity.POST("resource/usage/detail/add/manage/export/form", service_chainbrowser.ResourceUsageDetailAddManageExportForm)       // 资源权益使用表（充值）导出
		equity.POST("get/all/reduce/type", service_chainbrowser.GetAllReduceType)                                                      // 获取所有的消费权益值的类型
		equity.POST("resource/usage/detail/reduce/manage", service_chainbrowser.ResourceUsageDetailReduceManage)                       // 资源权益使用（消费）
		equity.POST("resource/usage/detail/reduce/manage/export/form", service_chainbrowser.ResourceUsageDetailReduceManageExportForm) // 资源权益使用（消费）导出

		equity.POST("get/resources/by/user/ids", service_chainbrowser.GetResourcesByUserIds) // 根据用户id列表查询资源

		equity.Use(middleware.CheckLogin(service.AccountProvider))
		equity.POST("refund/allow", service_chainbrowser.RefundAllow)                 // 退款允许
		equity.POST("admin/top/up/resource", service_chainbrowser.AdminTopUpResource) // 管理员赠送丰值

		invoice := auth.Group("invoice")
		invoice.Use(middleware.CheckLogin(service.AccountProvider))
		{
			invoice.POST("list", service_invoice.FindInvoice)     //发票列表
			invoice.POST("handle", service_invoice.HandelInvoice) //发票处理
		}
		BlockUser := auth.Group("block-user")
		BlockUser.Use(middleware.CheckLogin(service.AccountProvider))
		{
			BlockUser.POST("list", blockchain.BlockUserList)
			BlockUser.POST("setting", blockchain.BlockUserSetting)
		}

		{
			eCardErpAuth.POST("eCard-user-manage", blockchain.ECardUserManage)
			eCardErpAuth.POST("chain-record", blockchain.ChainRecord)
			eCardAuth.POST("save-info", blockchain.SaveInfo)
			eCardAuth.POST("exist", blockchain.UserExist)
			eCardAuth.POST("send", blockchain.Send)
			eCardAuth.POST("login", blockchain.Login)
			eCardAuth.POST("check", blockchain.Check)
		}
	}
}
