package router

import (
	"github.com/fonchain_enterprise/fonchain-main/pkg/middleware"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/dci"
	"github.com/gin-gonic/gin"
)

func DciRoute(r *gin.RouterGroup) {
	basic := r.Group("dci")
	noAuth := r.Group("dci")
	basic.Use(middleware.CheckLogin(service.AccountProvider), middleware.CheckAuth(service.AccountProvider, service.RuleProvider), middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
	file := basic.Group("file")
	{
		file.POST("upload", dci.GetUploadFile)
	}

	{
		noAuth.GET("area", dci.GetAreaCode)
		noAuth.POST("callback", dci.Callback)
	}

	user := basic.Group("user")
	{
		user.POST("add", dci.AddDciUser)
		user.POST("update", dci.UpdateDciUser)
		user.POST("info", dci.QueryDciUser)
	}

	work := basic.Group("work")
	{
		work.POST("create", dci.CreateDciRegistration)
		work.POST("info", dci.QueryDciRegistration)
		work.POST("pay-url", dci.GetDciPayUrl)
		work.POST("pay-info", dci.QueryDciPay)
		work.POST("cert-info", dci.GetDciRegistrationcert)
		work.POST("retry", dci.RetryDciRegistration)
		work.POST("close", dci.CloseDciRegistration)
	}

	reg := basic.Group("reg")
	{
		reg.POST("create", dci.CreateDciPreregistration)
		reg.POST("info", dci.QueryDciPreregistration)
	}

}
