package router

import (
	"github.com/fonchain_enterprise/fonchain-main/pkg/middleware"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/employee"
	"github.com/gin-gonic/gin"
)

func EmployeeFileRoute(r *gin.RouterGroup) {
	employeeFileRoute := r.Group("/employee-file")
	employeeFileRoute.Use(middleware.CheckLogin(service.AccountProvider))
	{
		//查询字典表
		employeeFileRoute.POST("dictionary/search", employee.SearchDictionaryList)
		//员工档案curd
		employeeFileRoute.POST("create", employee.CreateEmployeeFile)
		employeeFileRoute.POST("update", employee.UpdateEmployeeFile)
		employeeFileRoute.POST("new/changes", employee.MoreNewChanges)
		employeeFileRoute.POST("set/attendance/group", employee.TheAttendanceGroupIsSet)
		employeeFileRoute.POST("list", employee.FindEmployeeFileList)
		//展示列
		employeeFileRoute.POST("create/display", employee.CreateDisplayColumns)
		employeeFileRoute.POST("details/display", employee.DetailsDisplayColumns)
		employeeFileRoute.POST("all/display", employee.DisplayColumnAll)
		//导出
		employeeFileRoute.POST("download/excel", employee.DownloadExcel)
		employeeFileRoute.POST("download/docx", employee.DownloadDocx)
		employeeFileRoute.POST("create/excel", employee.CreateExcel)
		employeeFileRoute.POST("find/total", employee.QueryStatistics)
		//奖惩记录
		employeeFileRoute.POST("create/rewards/punishments", employee.CreateRewardsAndPunishments)
		employeeFileRoute.POST("find/rewards/punishments", employee.FindRewardsAndPunishments)
		employeeFileRoute.POST("download/rewards/punishments", employee.DownloadRewardsAndPunishments)
		employeeFileRoute.POST("create/rewards/punishments/excel", employee.CreateRewardsAndPunishmentsExcel)
		//离职记录
		employeeFileRoute.POST("create/turnover/record", employee.CreateTurnoverRecord)
		employeeFileRoute.POST("find/turnover/record", employee.FindTurnoverRecord)
		employeeFileRoute.POST("download/turnover/record", employee.DownloadTurnoverRecord)
		//薪资记录
		employeeFileRoute.POST("find/payroll/record", employee.FindPayrollRecordList)
		employeeFileRoute.POST("download/payroll/record", employee.DownloadPayrollRecordList)
		//合同记录
		employeeFileRoute.POST("create/contract/record", employee.CreateContractRecord)
		employeeFileRoute.POST("find/contract/record", employee.FindContractRecord)
		employeeFileRoute.POST("create/contract/excel", employee.CreateContractExcel)
		employeeFileRoute.POST("download/contract/record", employee.DownloadContractRecord)
		employeeFileRoute.POST("update/contract/attachment", employee.UpdateContractAttachment)
		//异动记录
		employeeFileRoute.POST("find/changes/record", employee.FindChangesRecord)
		employeeFileRoute.POST("download/changes/record", employee.DownloadChangesRecord)
		employeeFileRoute.POST("create/changes/excel", employee.CreateChangesExcel)
		employeeFileRoute.POST("update/changes/attachment", employee.UpdateChangesAttachment)
		//提醒记录
		employeeFileRoute.POST("find/reminder/record", employee.FindReminderList)
		employeeFileRoute.POST("processing/reminder", employee.ProcessingReminder)
		employeeFileRoute.POST("find/out/standing", employee.QueryOutstanding)
		//redis初始化
		employeeFileRoute.POST("init/redis/attendance-section", employee.InitRedisAttendance)
		employeeFileRoute.POST("init/redis/recent-img", employee.InitRedisRecentImg)
	}
}
