package router

import (
	"github.com/fonchain_enterprise/fonchain-main/pkg/middleware"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/exam"
	"github.com/gin-gonic/gin"
)

func ExamRoute(r *gin.RouterGroup) {
	noAuth := r.Group("")
	auth := r.Group("")
	//auth := r.Group("")
	auth.Use(middleware.CheckLogin(service.AccountProvider))

	examNoAuth := noAuth.Group("exam")
	{
		examNoAuth.POST("user/send", exam.SendExamMsg)
		examNoAuth.POST("user/reg", exam.UserExamRegister)
		examNoAuth.POST("user/login", exam.UserExamLogin)
	}

	examAuth := auth.Group("exam")
	{
		examAuth.POST("user/all", exam.GetAllUserInfo)

		examAuth.POST("user/save-answer", exam.SaveUserAnswerHis)
		examAuth.POST("user/save-train", exam.SaveUserTrainHis)
		examAuth.POST("user/save-exam", exam.SaveUserHis)
		examAuth.POST("user/answer-his", exam.UserHisInfo)

		examAuth.POST("save-exam", exam.SaveExam)
		examAuth.POST("exam-info", exam.GetExamInfo)
		examAuth.POST("exam-infos", exam.GetExamInfos)

		examAuth.POST("save-train", exam.SaveTrain)
		examAuth.POST("train-info", exam.GetTrainInfo)
		examAuth.POST("train-infos", exam.GetTrainInfos)
		examAuth.POST("update-train", exam.UpdateTrain)
		examAuth.POST("delete-train", exam.DeleteTrain)

		examAuth.POST("save-gift", exam.SaveGift)
		examAuth.POST("gift-info", exam.GetGiftInfo)

		examAuth.POST("save-question", exam.SaveQuestion)
		examAuth.POST("question-info", exam.GetQuestionInfo)
		examAuth.POST("check-question", exam.CheckQuestion)
		examAuth.POST("update-question", exam.UpdateQuestion)
		examAuth.POST("delete-question", exam.DeleteQuestion)
		examAuth.POST("detail-question", exam.DetailQuestion)
		examAuth.POST("import-questions", exam.ImportQuestions)

		examAuth.POST("exam-user", exam.GetExamUserInfo)
		examAuth.POST("fuzzy-query-user", exam.FuzzyQueryUser)
		examAuth.POST("export-excel", exam.ExportExcel)

		examAuth.POST("question-classify-info", exam.GetQuestionClassifies)

		examAuth.POST("save-exam-paper", exam.SaveExamPaper)
		examAuth.POST("update-exam-paper", exam.UpdateExamPaper)
		examAuth.POST("delete-exam-paper", exam.DeleteExamPaper)
		examAuth.POST("get-exam-papers", exam.GetExamPapers)
		examAuth.POST("detail-exam-paper", exam.DetailExamPaper)

		examAuth.POST("save-exam-task", exam.SaveExamTask)
		examAuth.POST("update-exam-task", exam.UpdateExamTask)
		examAuth.POST("delete-exam-task", exam.DeleteExamTask)
		examAuth.POST("get-exam-tasks", exam.GetExamTasks)
		examAuth.POST("detail-exam-task", exam.DetailExamTask)
		examAuth.POST("check-exam-task", exam.CheckExamTask)
		examAuth.POST("publish-exam-task", exam.PublishExamTask)
		examAuth.POST("update-exam-task-range", exam.UpdateExamTaskRange)

		examAuth.POST("save-user-exam-task", exam.SaveUserExam)
		examAuth.POST("get-person-exam-tasks", exam.GetPersonUserExamTaskRecords)
		examAuth.POST("get-person-exam—sub-tasks", exam.GetPersonUserExamSubTaskRecords)
		examAuth.POST("get-person-score", exam.GetPersonScore)
		examAuth.POST("grading-paper", exam.GradingPaper)
		examAuth.POST("data-user-task-details", exam.DataUserTaskDetails)
		examAuth.POST("data-person-monitor", exam.DataPersonMonitor)
		examAuth.POST("data-header-monitor", exam.DataHearMonitor)
		examAuth.POST("export-excel-data-person-monitor", exam.ExportExcelDataPersonMonitor)
	}

}
