package router

import (
	"github.com/fonchain_enterprise/fonchain-main/pkg/middleware"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/inventory"
	"github.com/gin-gonic/gin"
)

// InventoryRoute 库管接口
func InventoryRoute(Router *gin.RouterGroup) {
	inventoryRoute := Router.Group("api/inventory")
	inventoryRoute.Use(middleware.CheckLogin(service.AccountProvider), middleware.CheckAuth(service.AccountProvider, service.RuleProvider))
	{

		inventoryRoute.Use(middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
		inventoryRoute.POST("remove", inventory.Remove)
		inventoryRoute.POST("list", inventory.List)
		inventoryRoute.POST("detail", inventory.Detail)
		inventoryRoute.POST("detail/barcode", inventory.DetailBarCode)
		inventoryRoute.POST("create", inventory.Create)
		inventoryRoute.POST("update", inventory.Update)
		inventoryRoute.POST("log/detail", inventory.DetailProductLog)
		inventoryRoute.POST("log/list", inventory.ListProductLog)
		inventoryRoute.POST("stock/update", inventory.ChangeStock)
	}

	wareHouseRoute := Router.Group("api/warehouse")
	wareHouseRoute.GET("down", inventory.DownWareHouse)
	wareHouseRoute.Use(middleware.CheckLogin(service.AccountProvider), middleware.CheckAuth(service.AccountProvider, service.RuleProvider))
	{
		//wareHouseRoute.Use(middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
		wareHouseRoute.POST("remove", inventory.RemoveWareHouse)
		wareHouseRoute.POST("list", inventory.ListWareHouse)
		wareHouseRoute.POST("pick/count", inventory.CountPickUpWareHouse)
		wareHouseRoute.POST("export", inventory.ExportWareHouse)
		wareHouseRoute.POST("pay/list", inventory.WareHousePayBill) //付款单列表
		wareHouseRoute.POST("detail", inventory.DetailWareHouse)
		wareHouseRoute.POST("preinfo", inventory.GetProductPreInfo)
		wareHouseRoute.POST("pickup", inventory.PickUp)
		wareHouseRoute.POST("edit", inventory.EditWareHouse)
		wareHouseRoute.POST("end/at", inventory.GetEndAtYmd)
	}

}
