package router

import (
	"github.com/fonchain_enterprise/fonchain-main/pkg/middleware"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/oa"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/oa_new"
	"github.com/gin-gonic/gin"
)

func OaRoute(r *gin.RouterGroup) {

	oaRoute := r.Group("oa")
	oaRoute.POST("check-wifi", oa.CheckWifi)
	{
		oaRoute.Use(middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl), middleware.CheckLogin(service.AccountProvider))
		{
			oaRoute.Use(middleware.CheckAuth(service.AccountProvider, service.RuleProvider), middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
			oaRoute.POST("clock-in", oa.ClockIn)
			oaRoute.GET("action-type", oa.ConfirmActionType)
			oaRoute.POST("record", oa.OARecord)
			oaRoute.POST("record/miss", oa.MissRecordForMakeUpV2)
			oaRoute.POST("record/single", oa.OARecordSingle)
			oaRoute.POST("staff/collection-all", oa.AttendanceCollectionV2)
			oaRoute.POST("staff/collection-single", oa.StaffAttendanceCollectionV1)
			oaRoute.POST("staff/collection-detail", oa.StaffAttendanceCollectionDetailV2)
			oaRoute.POST("make-up/useful-num", oa.MakeUpNum) // 补卡 使用 以及 剩余 次数
			oaRoute.POST("abnormal", oa.QueryAbnormal)
			oaRoute.POST("record/other", oa.QueryOther)
			// 员工管理
			oaRoute.POST("users", oa.OaUserList)
			// 外勤确认
			oaRoute.POST("out-work-check", oa.OutWorkCheck)
			oaRoute.POST("out-work-check/add", oa.CreateOutWorkCheck)

			// 系统补卡
			oaRoute.POST("system/make-up", oa_new.SystemMakeUp)
		}

		// 考勤设置
		workingTimeRoute := oaRoute.Group("/working-time")
		{
			workingTimeRoute.Use(middleware.CheckAuth(service.AccountProvider, service.RuleProvider), middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
			workingTimeRoute.POST("options", oa.WorkingTimeList)
			workingTimeRoute.POST("create", oa.CreateWorkingTime)
			workingTimeRoute.POST("update", oa.SaveTimeOption)
			workingTimeRoute.POST("duration", oa.TimeDuration)
			//workingTimeRoute.POST("options", oa.WorkingTimeList)
			workingTimeRoute.GET("num", oa.QueryNoSettingWorkingTimeNum)
			workingTimeRoute.POST("best", oa.QueryWorkingTimeBest)
			workingTimeRoute.POST("in-use", oa.TimeOptionInUse)
			workingTimeRoute.POST("cancel", oa.CancelTimeOption)
			workingTimeRoute.POST("positions", oa_new.WorkingTimeWithPositions)
		}

		applyRoute := oaRoute.Group("/apply")
		{
			applyRoute.Use(middleware.CheckAuth(service.AccountProvider, service.RuleProvider), middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
			applyRoute.POST("leave-apply/info", oa.QueryLeaveApplyInfo)
			applyRoute.POST("list", oa.QueryOaApply)
			applyRoute.POST("over-time/duration", oa.OverTimeDuration)
			applyRoute.POST("info", oa_new.QueryOaApplyInfo)
		}

		settingRoute := oaRoute.Group("/setting")
		{
			settingRoute.Use(middleware.CheckAuth(service.AccountProvider, service.RuleProvider), middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
			settingRoute.POST("save", oa.SaveOaSetting)
			settingRoute.POST("detail", oa.QueryOaSetting)
			settingRoute.POST("remove", oa.DeleteOaSetting)
		}

		// 请假时长
		{
			oaRoute.POST("leave/duration", oa.LeaveDuration)
			oaRoute.POST("annual-leave/duration", oa.AnnualLeaveDuration)
			oaRoute.POST("leave/check", oa.CheckLeave)
		}

		// 考勤组
		workTimeTemplateRoute := oaRoute.Group("/work-time-template")
		{
			workTimeTemplateRoute.Use(middleware.CheckAuth(service.AccountProvider, service.RuleProvider), middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
			workTimeTemplateRoute.POST("template-save", oa_new.SaveWorkTimeTemplate)
			workTimeTemplateRoute.POST("template-list", oa_new.GetWorkTimeTemplateList)
			workTimeTemplateRoute.POST("template-remove", oa_new.DeleteWorkTimeTemplate)
			workTimeTemplateRoute.POST("template-effective", oa_new.WorkingTimeTemplateEffective)
			workTimeTemplateRoute.POST("template-not-effective", oa_new.WorkingTimeTemplateNotEffective)
			workTimeTemplateRoute.POST("template-relation-save", oa_new.SaveWorkTimeTemplateRelation)
			workTimeTemplateRoute.POST("template-relation-cancel", oa_new.BatchCancelWorkingTimeTemplateRelation)
			workTimeTemplateRoute.POST("map-disable", oa_new.ChickInMapDisable)
			workTimeTemplateRoute.POST("template-relation-save-now", oa_new.SaveWorkTimeTemplateRelationEffectiveNow)
			workTimeTemplateRoute.POST("template-date", oa_new.QueryWorkTimeTemplateDate)
		}

		// 打卡地址
		chickInMapRoute := oaRoute.Group("/chick-in-map")
		{
			chickInMapRoute.Use(middleware.CheckAuth(service.AccountProvider, service.RuleProvider), middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
			chickInMapRoute.POST("map-save", oa_new.SaveChickInMap)
			chickInMapRoute.POST("map-list", oa_new.GetChickInMapList)
			chickInMapRoute.POST("map-remove", oa_new.DeleteChickInMap)
		}

		// 考勤规则
		workTimeRuleRoute := oaRoute.Group("/work-time-rule")
		{
			workTimeRuleRoute.Use(middleware.CheckAuth(service.AccountProvider, service.RuleProvider), middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
			workTimeRuleRoute.POST("time-rule-save", oa_new.SaveWorkTimeRule)
			workTimeRuleRoute.POST("time-rule-list", oa_new.GetWorkTimeRuleList)
			workTimeRuleRoute.POST("time-rule-remove", oa_new.DeleteWorkTimeRule)
		}

		// 假勤规则
		oaRuleRoute := oaRoute.Group("/oa-rule")
		{
			oaRuleRoute.Use(middleware.CheckAuth(service.AccountProvider, service.RuleProvider), middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
			oaRuleRoute.POST("rule-save", oa_new.SaveOaRule)
			oaRuleRoute.POST("rule-list", oa_new.GetOaRuleList)
			oaRuleRoute.POST("rule-remove", oa_new.DeleteOaRule)
		}

		// 假期余额
		oaLeaveBalanceRoute := oaRoute.Group("/leave-balance")
		{
			oaLeaveBalanceRoute.Use(middleware.CheckAuth(service.AccountProvider, service.RuleProvider), middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
			oaLeaveBalanceRoute.POST("balance-list", oa_new.GetLeaveBalanceList)
			oaLeaveBalanceRoute.POST("balance-info", oa_new.GetLeaveBalanceInfo)
			oaLeaveBalanceRoute.POST("balance-create-excel", oa_new.CreateLeaveBalanceExcel)
			oaLeaveBalanceRoute.POST("balance-download-excel", oa_new.DownloadLeaveBalanceExcel)
			oaLeaveBalanceRoute.POST("balance-update-annual", oa_new.BalanceUpdateAnnual)
		}

		{
			oaRoute.POST("check-oa-month", oa.CheckOaMonth)
			oaRoute.POST("query-oa-month", oa.AttendanceCollectionTest)
		}

		collectionRoute := oaRoute.Group("/collection")
		{
			r.POST("oa/collection/date/inner", oa_new.StaffCollectionDetailDate)
			collectionRoute.Use(middleware.CheckAuth(service.AccountProvider, service.RuleProvider), middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
			collectionRoute.POST("detail", oa_new.StaffCollectionDetail)
			collectionRoute.POST("dates", oa_new.StaffCollectionDetailDates)
			collectionRoute.POST("date", oa_new.StaffCollectionDetailDate)
			collectionRoute.POST("all", oa_new.StaffCollectionDetailMonth)
			collectionRoute.POST("download", oa_new.StaffCollectionMonthDownload)
			collectionRoute.POST("download/list", oa_new.StaffCollectionMonthDownloadList)
			collectionRoute.POST("download/del", oa_new.StaffCollectionMonthDownloadDel)
			//collectionRoute.POST("download/old", oa_new.StaffCollectionMonthDownloadOld)
			collectionRoute.POST("download/action-time", oa_new.ExportActionRecord)

		}

		v2 := oaRoute.Group("/v2")
		{
			v2.Use(middleware.CheckAuth(service.AccountProvider, service.RuleProvider), middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
			v2.POST("clock-in", oa_new.ClockIn)
			v2.POST("action-type", oa_new.ActionType)
			v2.GET("chick-in-map", oa_new.ChickInMapInfo)
			v2.GET("workingTime-info", oa_new.WorkingTimeInfo)
			v2.POST("make-up/useful-num", oa_new.MakeUpNum)
			v2.POST("record/miss", oa_new.MissRecordForMakeUp)
			v2.POST("system/common", oa_new.SystemCommon)
			v2.POST("record/face", oa_new.QueryOtherRecord)
		}

		oaMonthFieldRoute := v2.Group("/oa-month-field")
		{
			oaMonthFieldRoute.POST("field-save", oa_new.SaveOaMonthField)
			oaMonthFieldRoute.POST("field-list", oa_new.FindOaMonthField)
		}

		durationRoute := v2.Group("/duration")
		{
			durationRoute.POST("over-time", oa_new.OverTimeDuration)
			durationRoute.POST("leave", oa_new.LeaveDuration)
			durationRoute.POST("annual-leave", oa_new.AnnualLeaveDuration)
			durationRoute.POST("businessTrip", oa_new.BusinessTripDuration)
			durationRoute.POST("out-work", oa_new.OutWorkDuration)
		}

		// 考勤组分类
		tempGroupRoute := oaRoute.Group("/temp-group")
		{
			tempGroupRoute.Use(middleware.CheckAuth(service.AccountProvider, service.RuleProvider), middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
			tempGroupRoute.POST("group-save", oa_new.SaveTemplateGroup)
			tempGroupRoute.POST("group-list", oa_new.QueryTemplateGroup)
			tempGroupRoute.POST("group-remove", oa_new.DeleteTemplateGroup)
		}

	}
}
