package router

import (
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/home_page"
	"net/http"

	"github.com/fonchain_enterprise/fonchain-main/pkg/service/children_competition"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/customer_service"

	"github.com/fonchain_enterprise/fonchain-main/pkg/service/goods_management"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/recruit"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/train"

	"github.com/fonchain_enterprise/fonchain-main/pkg/service/weekmeal"

	"github.com/fonchain_enterprise/fonchain-main/pkg/service/oa"

	"github.com/fonchain_enterprise/fonchain-main/pkg/service/order"

	"github.com/fonchain_enterprise/fonchain-main/pkg/middleware"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/account"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/approval"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/auth/department"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/auth/position"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/auth/rule"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/auth/rule_log"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/auth/supplyinfo"
	es_approval "github.com/fonchain_enterprise/fonchain-main/pkg/service/es_service/approval"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/push"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/task"
	"github.com/gin-contrib/gzip"
	"github.com/gin-contrib/pprof"
	"github.com/gin-gonic/gin"
)

//Router 管理""组
/*func Router(g *gin.RouterGroup) {
	// g.GET("/", func(c *gin.Context) {
	//   c.HTML(http.StatusOK, "login.html", nil)
	// })
	// g.GET("/login", func(c *gin.Context) {
	//   c.HTML(http.StatusOK, "login.html", nil)``
	// })
	g.GET("/", func(c *gin.Context) {
		c.HTML(http.StatusOK, "index.html", nil)
	})

}*/

// 路由配置
func NewRouter() *gin.Engine {
	//使用默认gin路由
	r := gin.Default()

	//Router(gin.Default().Group(""))
	originRoute := r.Group("")
	r.Use(gzip.Gzip(gzip.DefaultCompression))
	//加入日志中间件，跨域中间件
	privateGroup := r.Group("")
	r.Use(middleware.Cors())
	privateGroup.Use(middleware.NewLogger(), middleware.Cors(), middleware.GinRecovery(true))
	originRoute.Use(middleware.NewLogger(), middleware.Cors(), middleware.GinRecovery(true))
	ShopRoute(privateGroup)
	ArtworkRoute(privateGroup)
	ArtistRoute(privateGroup)
	ChatRoute(originRoute)
	MealRoute(privateGroup)
	InventoryRoute(privateGroup)
	MeetingRoute(privateGroup)
	AuthRoute(privateGroup)
	CopyrightRoute(privateGroup)
	ChainBrowserRoute(originRoute)
	workMealDayRoute(privateGroup)
	GiteaRoute(originRoute)
	DciRoute(originRoute)
	ExamRoute(privateGroup)
	TurnstileRoute(privateGroup)
	PatentRoute(privateGroup)
	PayBillRoute(privateGroup)
	AiImageRoute(privateGroup)
	//ExhibitionRoute(privateGroup)
	//ExamV2Route(privateGroup)
	OaRoute(privateGroup)
	LoanBillRoute(privateGroup)
	ExamV2Route(privateGroup)
	//ExhibitionRoute(privateGroup)
	LaunchRoute(privateGroup)
	EmployeeFileRoute(privateGroup)
	OcrRoute(privateGroup)
	websocketRoute(privateGroup)
	DomainNameRoute(privateGroup)
	//请求相关信息备份
	//r.Use(middleware.Backup(service.AccountProvider, service.GrpcBackupImpl))
	// r.POST("user/register", account.UserRegister) //用户注册
	// r.POST("user/login", account.UserLogin)       //登录
	// r.POST("user/send", account.SendMsg)

	test := r.Group("test") // grpc 测试路由组
	{
		//test.POST("chain", approval.StoreBaiduChainApprovalFilePost)
		//test.GET("make-up", service.CheckClickIn)
		test.POST("demo", service.Demo)
	}
	//自己生成日志
	//r.Use(middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
	//账号模块
	{
		r.POST("user/login", account.UserLogin) //登录
		r.POST("user/artwork/login", account.ArtworkUserLogin)
		r.POST("user/send", account.SendMsg)
		//r.GET("user/online/infos", account.OnlineInfos)
		r.GET("user/online/info", account.OnlineInfoById)
		r.GET("user/offline", account.Offline)
		r.POST("user/refresh/token", account.RefreshToken) //登出

		acRoute := r.Group("/user")
		acRoute.POST("logout", account.UserLogout) //登出
		acRoute.Use(middleware.CheckLogin(service.AccountProvider), middleware.CheckAuth(service.AccountProvider, service.RuleProvider), middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
		{
			r.POST("user/register", account.UserRegister)             //用户注册
			acRoute.POST("auth", account.Authentication)              //实名授权
			acRoute.POST("check", account.CheckMsg)                   //校验验证码
			acRoute.DELETE("remove", account.Remove)                  //删除
			acRoute.POST("update", account.Update)                    //更新
			acRoute.POST("privacyInfo", account.PrivacyInfo)          //私密的加密信息
			acRoute.POST("info", account.Info)                        //用户信息
			acRoute.POST("now/info", account.NowInfo)                 //用户信息
			acRoute.POST("base", account.BaseInfo)                    //基本信息
			acRoute.POST("list", account.List)                        //用户列表
			acRoute.POST("list/supper", department.SupperUsers)       //用户列表
			acRoute.POST("leader", account.DepartmentLeaderList)      // 部门主管名字
			acRoute.POST("mail/account", account.MailAccount)         // 根据昵称获取mail帐号
			acRoute.POST("map/ip/to/location", account.LocationNowIp) // 定位--ip转化
			acRoute.POST("map/location/query", account.SiteQuery)     // 定位--位置转坐标
			acRoute.POST("ocr/card", account.OcrCardId)               // 用户管理-识别身份证
		}
	}
	//权限模块
	{
		positionRoute := r.Group("/position")
		positionRoute.Use(middleware.CheckLogin(service.AccountProvider), middleware.CheckAuth(service.AccountProvider, service.RuleProvider))
		{
			positionRoute.Use(middleware.WarnLog(service.AccountProvider, service.GrpcLogImpl))
			positionRoute.POST("create", position.Create)
			positionRoute.DELETE("remove", position.Remove)
			positionRoute.POST("update", position.Update)
			positionRoute.POST("detail", position.Detail)
			positionRoute.POST("list", position.List)
		}

		departmentHeadRoute := r.Group("/department/head")
		{
			departmentHeadRoute.POST("create", department.CreateDepartmentHead)
			departmentHeadRoute.POST("update", department.UpdateDepartmentHead)
			departmentHeadRoute.POST("list",
				middleware.CheckLogin(service.AccountProvider),
				middleware.CheckAuth(service.AccountProvider, service.RuleProvider),
				department.FindDepartmentHeadList)

		}
		departmentRoute := r.Group("/department")
		departmentRoute.Use(middleware.CheckLogin(service.AccountProvider), middleware.CheckAuth(service.AccountProvider, service.RuleProvider))
		{

			departmentRoute.Use(middleware.WarnLog(service.AccountProvider, service.GrpcLogImpl))
			departmentRoute.POST("create", department.Create)
			departmentRoute.DELETE("remove", department.Remove)
			departmentRoute.POST("update", department.Update)
			departmentRoute.POST("detail", department.Detail)
			departmentRoute.POST("details", department.Details)
			departmentRoute.POST("user/remove", department.RemoveUser)
			departmentRoute.POST("list", department.List)
			departmentRoute.POST("list/all", department.ListAll)
			departmentRoute.POST("all", department.All)
			departmentRoute.POST("users", department.Users)
			departmentRoute.POST("users/v2", department.SupperUsers)
			departmentRoute.POST("base/list", department.BaseList)
			departmentRoute.POST("base/user", department.BaseDepartmentList)
		}

		SupplyInfoRoute := r.Group("/supplyinfo")
		SupplyInfoRoute.Use(middleware.CheckLogin(service.AccountProvider), middleware.CheckAuth(service.AccountProvider, service.RuleProvider), middleware.WarnLog(service.AccountProvider, service.GrpcLogImpl))
		{
			SupplyInfoRoute.POST("/createsupplyinfo", supplyinfo.CreateSupplyInfo)
			SupplyInfoRoute.POST("/getsupplyinfolist", supplyinfo.GetSupplyInfoList)
			SupplyInfoRoute.POST("/getsupplyinfo", supplyinfo.GetSupplyInfo)
			SupplyInfoRoute.POST("/applyartwork", supplyinfo.ApplyArtwork)
			SupplyInfoRoute.POST("/getexam", supplyinfo.GetExam)
			SupplyInfoRoute.POST("/getexamlist", supplyinfo.GetExamList)
			SupplyInfoRoute.POST("/applyvideo", supplyinfo.ApplyExhVideo)
			SupplyInfoRoute.POST("/getvideolist", supplyinfo.GetVideoList)
			SupplyInfoRoute.POST("/applyexam", supplyinfo.ApplyExhExam)
			SupplyInfoRoute.POST("/applyartistinfo", supplyinfo.ApplyArtistInfo)
			SupplyInfoRoute.POST("/getartistinfolist", supplyinfo.GetArtistInfoList)
			SupplyInfoRoute.POST("/importartistsupplylist", supplyinfo.ImportArtistSupplyList)
			SupplyInfoRoute.POST("/importartworksupplylist", supplyinfo.ImportArtworkSupplyList)
		}

		ruleRoute := r.Group("/rule")
		ruleRoute.Use(middleware.CheckLogin(service.AccountProvider), middleware.CheckAuth(service.AccountProvider, service.RuleProvider))
		{
			ruleRoute.Use(middleware.WarnLog(service.AccountProvider, service.GrpcLogImpl))
			ruleRoute.POST("create", rule.Create)
			ruleRoute.DELETE("remove", rule.Remove)
			ruleRoute.POST("update", rule.Update)
			ruleRoute.POST("detail", rule.Detail)
			ruleRoute.POST("list", rule.List)
			ruleRoute.POST("rules", rule.Rules)
			ruleRoute.POST("interface", rule.InterfaceAuth)

			ruleRoute.POST("tree/list", rule.TreeList) //顶级菜单带上子菜单
			ruleRoute.POST("menu/top", rule.TopList)   //顶层
		}

		ruleLogRoute := r.Group("/rule_log")
		ruleLogRoute.Use(middleware.CheckLogin(service.AccountProvider), middleware.CheckAuth(service.AccountProvider, service.RuleProvider))
		{
			ruleLogRoute.Use(middleware.WarnLog(service.AccountProvider, service.GrpcLogImpl))
			ruleLogRoute.POST("list", rule_log.List)
		}

	}

	//审核模块
	{
		approvalRoute := r.Group("/approval")
		approvalRoute.POST("detail/ex", approval.DetailEx)
		approvalRoute.POST("list/ex", approval.ListEx)
		approvalRoute.POST("status/ex", approval.SetStatusEx)
		approvalRoute.POST("status/batch-ex", approval.SetStatusBatchEx)
		approvalRoute.POST("v2/status/batch-ex", approval.SetStatusBatchExV2)
		approvalRoute.Use(middleware.CheckLogin(service.AccountProvider), middleware.CheckAuth(service.AccountProvider, service.RuleProvider))
		{
			approvalRoute.Use(middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
			approvalRoute.POST("create", approval.Create)
			approvalRoute.POST("exhibition/num", approval.UsedNumByUser)
			approvalRoute.POST("status", approval.SetStatus)
			approvalRoute.DELETE("remove", approval.Remove)
			approvalRoute.POST("update", approval.Update)
			approvalRoute.POST("detail", approval.Detail)
			approvalRoute.POST("send", approval.Send)
			approvalRoute.POST("revoke", approval.RevokeAllApproval)
			approvalRoute.POST("print/detail", approval.PrintDetail)
			approvalRoute.POST("/my/work", approval.MyWork)
			approvalRoute.POST("/my/submit", approval.MySubmit)
			approvalRoute.POST("/my/approved", approval.Approved) //我审批过的

			//画展包付款单申请下载的权限
			approvalRoute.POST("/check/pay/right", approval.CheckBundlePay)

			approvalRoute.POST("viewed", approval.Viewed)
			approvalRoute.POST("information", approval.Information)
			approvalRoute.POST("setting/update", approval.UpdateSetting)
			approvalRoute.POST("setting/detail", approval.DetailSetting)

			approvalRoute.POST("type/create", approval.CreateType)
			approvalRoute.DELETE("type/remove", approval.RemoveType)
			approvalRoute.POST("type/update", approval.UpdateType)
			approvalRoute.POST("type/detail", approval.DetailType)
			approvalRoute.POST("type/all", approval.AllType)
			approvalRoute.POST("group/all", approval.AllGroup)

			// 财务报表审批
			approvalRoute.POST("financial_form/setting", approval.GetName)
			approvalRoute.POST("financial_form/export", approval.FinancialFormExport)

			// 招聘
			approvalRoute.POST("job/list", approval.JobList)
			approvalRoute.POST("job/close", approval.CloseJob)
			approvalRoute.POST("job/interviewer", approval.CreateInterviewer)
			approvalRoute.POST("job/interviewer-edit", approval.UpdateInterviewer)

			approvalRoute.POST("employ/close", approval.CloseEmploy)
			// 审批抄送人预览
			approvalRoute.POST("setting/preview", approval.MakeApprovalAndUser)

			// 批量催办
			approvalRoute.POST("list-send", approval.ListSend)

			// 批量审批
			approvalRoute.POST("status/batch", approval.SetStatusBatch)

			// 查询画展包相关审批
			approvalRoute.POST("show-list", approval.ArtShowApprovalList)

		}

		{
			approvalRoute.POST("v2/create", approval.CreateV2)
			approvalRoute.POST("v2/type/create", approval.CreateApprovalType)
			approvalRoute.POST("v2/type/update", approval.UpdateApprovalType)
			approvalRoute.POST("v2/type/copy", approval.CopyApprovalType)
			approvalRoute.POST("v2/type/status", approval.UpdateApprovalTypeStatus)
			approvalRoute.POST("v2/type/all", approval.ApprovalTypeList)
			approvalRoute.POST("v3/type/all", approval.ApprovalTypeListV3)
			approvalRoute.POST("v2/group/create", approval.CreateApprovalGroup)
			approvalRoute.POST("v2/group/all", approval.ApprovalGroupList)

			approvalRoute.POST("v2/my/work", approval.MyWorkV2)
			approvalRoute.POST("v2/setting/preview", approval.MakeApprovalAndUserV2)
			approvalRoute.POST("v2/status", approval.SetStatusBatchV2)
			approvalRoute.POST("v2/only-group", approval.OnlyGroupList)              //只查看分组列表
			approvalRoute.POST("v2/approval-his-type", approval.ApprovalHisTypeList) //查询历史审批类型

			approvalRoute.POST("v2/export-approval", approval.ExportApproval)
			//查询是否有重复的录用
			approvalRoute.POST("v2/find-employ", approval.FindEmploy)
			approvalRoute.POST("v2/updates-old", approval.UpdatesOldData)
			approvalRoute.POST("v2/export-detail-setting", approval.ExportDetailSettingV2)
			approvalRoute.POST("v2/create-auth", approval.CreateAuthQuick)           //快速添加发起人
			approvalRoute.POST("v2/find-turnover", approval.FindTurnoverAndHandover) //查询离职相关审批
			approvalRoute.POST("v2/find-super-type", approval.FindSuperType)         //查询离职相关审批
			approvalRoute.POST("v2/create-auth-batch", approval.CreateAuthBatch)     //批量添加发起人

			approvalRoute.POST("v2/search", es_approval.SearchApproval)
			approvalRoute.POST("v2/bulk", es_approval.BulkApproval)

		}

	}

	//工作餐预约
	{
		weekRoute := r.Group("/work/meal")
		weekRoute.Use(middleware.CheckLogin(service.AccountProvider), middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
		{
			weekRoute.POST("week", weekmeal.Week)                      //主列表
			weekRoute.POST("weeks", weekmeal.Weeks)                    //周报详情
			weekRoute.POST("update", weekmeal.Update)                  //更新
			weekRoute.POST("update/weekInfo", weekmeal.UpdateWeekInfo) //修改就餐人数
		}
	}

	//私宴
	{
		selfMealRoute := r.Group("/self/meal")
		selfMealRoute.Use(middleware.CheckLogin(service.AccountProvider), middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
		{
			selfMealRoute.POST("qr/create", weekmeal.CreateQr) //主列表
		}
	}

	//销售宝后端
	{
		orderRoute := r.Group("/report")
		orderRoute.Use(middleware.CheckLogin(service.AccountProvider), middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
		{
			orderRoute.Use(middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
			orderRoute.POST("list", order.ReportsList)            //主列表
			orderRoute.POST("my/list", order.ReportsMyList)       //历史周报
			orderRoute.POST("users", order.ReportUsers)           //周报详情
			orderRoute.GET("down/users", order.DownReportUsers)   //周报详情下载
			orderRoute.POST("create/before", order.BeforeUser)    //未发布之前的业绩报表
			orderRoute.GET("now", order.NowReport)                //获取当前提交的报表
			orderRoute.POST("create", order.ReportCreate)         //生成周报
			orderRoute.POST("publish", order.ReportPublish)       //发布周报
			orderRoute.GET("sell/amount", order.CheckSalesAmount) //查询销售员到期金额
			orderRoute.GET("find/dates", order.FindDates)         //查询周报日期

		}

		sellerRoute := r.Group("/seller")
		sellerRoute.Use(middleware.CheckLogin(service.AccountProvider))
		{
			sellerRoute.Use(middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
			sellerRoute.POST("report/cashier/list", order.CashierReportsList)    //出纳查看列表
			sellerRoute.POST("report/cashier/sure", order.ReportSure)            //出纳确认订单 内控待确认
			sellerRoute.POST("report/cashier/re", order.ReportRe)                //出纳确认订单 内控待确认
			sellerRoute.POST("reportuser/batch/update", order.ReportUsersUpdate) //出纳查看列表

			sellerRoute.GET("order/down", order.DownReportUsers)        //财务下载订单
			sellerRoute.POST("order/cashier/list", order.CashierOrders) //出纳 订单列表
			sellerRoute.POST("order/control/list", order.ControlOrders) //内控 订单列表
			sellerRoute.POST("order/finance/list", order.FinanceOrders) //财务 订单列表

			//出纳增加权限 列表 详情 编辑 发起
			sellerRoute.POST("cashier/transactions", order.Transactions) //管理下的叮当
			sellerRoute.POST("cashier/seller/users", order.Users)        //管理下的销售
			sellerRoute.POST("cashier/publish", order.CashierSure)       //发起订单
			sellerRoute.POST("cashier/order/update", order.SetOrder)     //更新
			sellerRoute.POST("cashier/order/detail", order.OrderDetail)  //
			sellerRoute.POST("cashier/order/list", order.OrderList)      //

			sellerRoute.POST("order/cashier/sure", order.CashierSure)           //出纳确认订单 内控待确认
			sellerRoute.POST("order/control/sure", order.ControlSure)           //内控确认订单 财务待确认
			sellerRoute.POST("order/finance/sure", order.FinanceSure)           //财务确认订单 内控待确认
			sellerRoute.POST("order/finance/stage/pass", order.FinancePass)     //财务确认订单 内控待确认
			sellerRoute.POST("order/finance/stage/refuse", order.FinanceRefuse) //财务确认订单 内控待确认

			sellerRoute.POST("order/finance/sures", order.FinanceSures) //财务确认订单 内控待确认
			sellerRoute.POST("order/finance/statist", order.Statist)    //财务获取统计数据
			sellerRoute.POST("order/batch/update", order.UpdateOrder)   //批量更新
			//sellerRoute.GET("transaction/list", order.DownReportUsers) //买家列表
			sellerRoute.POST("transaction/cashier/list", order.SiteTransactions) //出纳 买家列表
			sellerRoute.POST("transaction/cashier/detail", order.Transaction)    //出纳 买家列表
			sellerRoute.GET("order/finance/down", order.DownOrders)              //财务下载
			sellerRoute.POST("department/bases", order.DepartmentBases)
			sellerRoute.POST("transaction/update", order.TransactionUpdate)
			//委托单
			sellerRoute.POST("entrust/list", order.GetEntrusts)
			sellerRoute.POST("entrust/detail", order.GetEntrust)
			sellerRoute.POST("entrust/delete", order.EntrustDelete)
			sellerRoute.POST("entrust/batch/update", order.UpdateEntrustKeys)
			sellerRoute.POST("entrust/batch/delete", order.EntrustBatchDelete)
			sellerRoute.POST("entrust/import", order.ImportEntrusts)
			//员工汇报
			sellerRoute.POST("weekly/staff/list", order.GetWeeklyStaffList)
			sellerRoute.POST("weekly/staff/info", order.GetWeeklyStaffInfo)
			sellerRoute.POST("weekly/staff/excel", order.GetWeeklyStaffExcel)
			//总监周报
			sellerRoute.POST("weekly/boss/list", order.GetWeeklyBossList)
			sellerRoute.POST("weekly/boss/info", order.GetWeeklyBossInfo)
			sellerRoute.POST("weekly/comment/create", order.CreateWeeklyComment)
			sellerRoute.POST("weekly/comment/info", order.GetWeeklyComment)
			// 账号审核
			sellerRoute.POST("examine/list", account.ExamineList)       //审核列表
			sellerRoute.POST("examine/approve", account.ExamineApprove) //审批
		}
	}

	//需要登陆保护
	{
		taskRoute := r.Group("/task")
		taskRoute.Use(middleware.CheckLogin(service.AccountProvider), middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
		taskRoute.POST("detail", task.Detail)
		taskRoute.POST("create", task.Create)
		taskRoute.POST("update", task.Update)
		taskRoute.POST("remove", task.Remove)
		taskRoute.POST("list", task.List)
		taskRoute.POST("finish", task.Finish)
		taskRoute.POST("statistics", task.DataInfo)
	}

	//日志
	log := r.Group("/log")
	log.Use(middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
	{
		log.POST("/loglist", service.LogList)
		log.POST("/loginfo", service.LogInfo)
		log.POST("/listbyawid", service.LogListByAwId)
	}

	//留言板
	note := r.Group("/note")
	note.Use(middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
	{
		note.POST("/createnote", service.CreateNote)
		note.POST("/noteinfo", service.NoteInfo)
		// note.POST("note/deletenote", service.DeleteNote)
		note.POST("/notelist", service.NoteList)
		note.POST("/notelistbyawid", service.NoteListByAwId)
	}

	//品牌方
	shopbrand := r.Group("/shopbrand")
	shopbrand.Use(middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
	{
		shopbrand.POST("/createbrand", service.CreateBrand)
		shopbrand.POST("/brandlist", service.BrandList)
		shopbrand.POST("/brandinfo", service.BrandInfo)
		shopbrand.POST("/updatebrand", service.UpdateBrand)
		shopbrand.POST("/deletebrand", service.DeleteBrand)
	}

	artShow := r.Group("artshow")
	artShow.Use(middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
	{
		artShow.Use(middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl), middleware.CheckLogin(service.AccountProvider))
		artShow.POST("show/add", service.CreateShow)
		artShow.PUT("show/edit", service.UpdateShow)
		artShow.DELETE("show/remove", service.DelShow)
		artShow.POST("show/list", service.ShowList)
		artShow.POST("show/list-with-rel", service.ShowListWithRel)
		artShow.POST("show/list-apply", service.ShowListApply)
		artShow.POST("show/date-list", service.QueryShowDate)
		artShow.POST("show/list-lately", service.ShowListWithRelLately)
		artShow.POST("show/no-price-num", service.ShowListNoPriceNum)
		artShow.POST("show/artwork-info", service.ShowArtworkInfo)
		artShow.POST("show/detail", service.ShowDetail)
		artShow.POST("show/num", service.ShowStatisticalInfo)
		artShow.POST("show/cancel", service.CancelShow)
		artShow.POST("apply/add", service.CreateApply)
		artShow.PUT("apply/edit", service.UpdateApply)
		artShow.DELETE("apply/remove", service.DelApply)
		artShow.POST("apply/show-list", service.ShowListWithApply)
		artShow.PUT("apply/edit-status", service.UpdateApplyStatus)
		artShow.POST("apply/list", service.ApplyList)
		artShow.POST("apply/rel-info", service.ApplyRelInfoDetail)
		artShow.POST("artwork/price", service.ArtworkPrice)
		artShow.POST("artist/list", service.ShowListForArtwork)

		artShow.POST("show/download-temp", service.ExportShowPrice)
		artShow.POST("show/download", service.ExportShowPrice_withoutCopyright)

		//artShow.POST("show/export-by-artist", service.ExportByArtist)
		artShow.POST("show/export-by-artist", service.ExportByArtist_withoutCopyright)

		artShow.POST("show/show-time-list", service.ShowTimeList)
		artShow.POST("show/all-time-list", service.AllTimeList)
		artShow.POST("show/entertainment-list", service.FindEntertainment)
		artShow.POST("entertainment/edit", service.SaveEntertainment)

		artShow.POST("show/no-price-list", service.FindNoPriceShow)
		artShow.POST("show/no-price-edit", service.UpdateShowPrice)
		artShow.POST("show/price-policy-edit", service.SavePolicy)
		artShow.POST("show/price-policy-info", service.QueryPolicy)
		artShow.POST("entertainment/download", service.ExportEntertainment)

		artShow.POST("show/show-host-edit", service.UpdateArtShowHost)

		//artShow.POST("one-query/price/download", service.ExportShowPriceOneQuery)
		artShow.POST("one-query/price/download", service.ExportShowPriceOneQuery_withoutCopyright)
		artShow.POST("one-query/show-list", service.OnQueryShowList)
		artShow.POST("one-query/show/download", service.ExportShowFieldOneQuery)
		artShow.POST("entertainment/download-png", service.ExportEntertainmentImage)

		artShow.POST("entertainment/group-field", service.FindEntertainmentGroupField)

		artShow.POST("show/can-show", service.CanShow)

		artShow.POST("show/startCheck", service.ShowStartCheck)
		//无底价画展包改版
		artShow.POST("show/artwork-price-list", service.ArtworkPriceListV2)
		artShow.POST("show/update-artwork-price", service.UpdateArtworkPriceV2)
		artShow.POST("show/cal-price", service.CalPriceSingle)

		// 单据
		artShow.POST("invoice/save", service.CreateInvoice)
		artShow.POST("invoice/list", service.ShowInvoiceList)
		artShow.POST("invoice/single", service.ShowInvoice)
		artShow.POST("invoice/num-add", service.UpdateInvoicePrintNum)
		artShow.POST("show/num-add", service.UpdateShowPrintNum)
		//artShow.POST("invoice/price-info", service.QueryPrice)
		artShow.POST("invoice/price-info", service.QueryPrice_withoutCopyright)
		artShow.POST("invoice/status", service.UpdateShowInvoiceStatus)

		// 打印管理
		artShow.POST("print/list", service.ArtShowListForPrint)             // 画展包列表
		artShow.POST("print/preview", service.PrintPreviewFile)             // 打印预览
		artShow.POST("print/detail", service.GetPrintDetail)                // 打印详情
		artShow.POST("print/num-add", service.CreatePrintDetail)            // 添加打印详情
		artShow.POST("print/artistList", service.SearchArtistListByKeyword) // 搜索画家

		// 经纪公司
		artShow.POST("talent-agency/list", service.TalentAgencyList) // 经纪公司列表
	}
	// 无需登录
	noLoginArtShow := r.Group("artshow")
	{
		noLoginArtShow.POST("artwork/price-info", service.ArtworkPriceWholeInfo)
	}

	address := r.Group("address")
	{
		address.Use(middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
		address.POST("addres-list", service.QuerySaleAddress)
	}

	// 打卡机 打卡
	ClockInMachine := r.Group("/api/v1")
	{
		ClockInMachine.POST("record/face", oa.ClockFaceLog)
		ClockInMachine.POST("stranger", oa.Stranger)         //陌生人识别
		ClockInMachine.POST("callPhone", oa.CallPhone2)      //访客呼叫
		ClockInMachine.POST("callManagement", oa.CallPhone3) //访客呼叫管理员
	}

	chain := r.Group("chain")
	chain.Use(middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
	{
		chain.POST("co-chain", service.CoChain)
		chain.POST("create-cert", service.CreateCert)
		chain.POST("auth-img-cochain", service.AuthImgCoChain)
	}

	// 上传
	upload := privateGroup.Group("upload")
	upload.Use(middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
	{
		upload.POST("img", service.UploadImg)
		upload.POST("multi", service.UploadMulti)
		upload.POST("upload-chunk", service.UploadChunk)
	}

	//抽奖
	draw := r.Group("/draw")
	{
		draw.Use(middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
		draw.POST("addAccount", service.AddAccount)           //用户注册并绑定hash
		draw.POST("addprizenum", service.AddPrizeNum)         //奖金累加
		draw.POST("setpool", service.SetPool)                 //设置奖池
		draw.POST("setresult", service.SetResult)             //保存结果密文到百度链
		draw.POST("getaccountprize", service.GetAccountPrize) //查询中奖的账户的信息
		draw.POST("prizelist", service.PrizeList)             //列出奖品信息
		// draw.POST("encrypt", service.Encrypt)                 //加密
		// draw.POST("decrypt", service.Decrypt)                 //解密
	}

	//推送
	pushRoute := r.Group("/push")
	{
		pushRoute.Use(middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
		//空投相关
		pushRoute.POST("putIn", push.PutIn)                     //投放空投
		pushRoute.POST("dropHistortList", push.DropHistortList) //历史空投列表
		pushRoute.POST("dropHistortDel", push.DropHistortDel)   //删除未开始的空投
		pushRoute.POST("user/data", account.MallRandList)       //获取用户数据

		//公告相关
		pushRoute.POST("pushAnnouncement", push.PushAnnouncement)       //推送公告
		pushRoute.POST("announcementHisList", push.AnnouncementHisList) //历史公告列表
		pushRoute.POST("announcementDel", push.AnnouncementDel)         //删除未开始公告
	}

	//销售
	saleRoute := r.Group("/sale")
	{
		saleRoute.Use(middleware.CheckLogin(service.AccountProvider), middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
		saleRoute.POST("series", order.SeriesList)                         //系列列表
		saleRoute.POST("series/collections", order.CollectionsList)        //系列下藏品列表
		saleRoute.POST("pay/detail", order.PayDetail)                      //detail
		saleRoute.POST("pay/express/set", order.PayAgreeExpress)           //修改订单状态
		saleRoute.POST("pay/approval/list", order.PayList)                 //订单列表列表
		saleRoute.POST("pay/approval/info", order.PayHasExpress)           //订单列表列表
		saleRoute.POST("pay/refund", order.Refund)                         //订单列表列表
		saleRoute.POST("pay/offline/list", order.OfflinePayList)           //订单列表列表
		saleRoute.POST("pay/add/tracking/number", order.AddTrackingNumber) //订单添加快递单号
	}

	// 假期
	holidayRoute := r.Group("/holiday")
	{
		holidayRoute.Use(middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
		holidayRoute.POST("single", service.SingleHolidayInfo) // 单日
		holidayRoute.POST("multi", service.MultiHolidayInfo)   // 区间
	}

	// 地图  经纬度 换地址
	addressRoute := r.Group("address")
	{
		addressRoute.POST("address-detail", service.LatLngToAddress)
	}

	//付款单
	paybill := r.Group("/paybill")
	{
		paybill.Use(middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
		paybill.POST("/list", service.PayBillList)
		paybill.POST("/update", service.UpdatePayBill)
		paybill.POST("/excel", service.ExcelBill)
		paybill.POST("/all", service.ArtistAll)
		paybill.POST("/search", service.ArtistBill)
		paybill.POST("/rollback", service.RollbackState)

		paybill.POST("/nocount", service.NoCount)

		paybill.POST("/editadd", service.UpdateAddress)
	}

	//管理系统预约列表
	prebook := r.Group("/prebook")
	{
		prebook.Use(middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
		prebook.POST("prebookList", service.PrebookList)
		prebook.POST("customerList", service.CustomerList)
		prebook.POST("newList", service.NewList)
		prebook.POST("feastList", service.FeastList)
		prebook.POST("addpulg", service.Addpulg)
		prebook.POST("sure", service.SureNo)
		prebook.POST("alter", service.AlterPrice)
		prebook.POST("del", service.DelFeast)
		prebook.POST("feastInfo", service.GetAllFeastInfo)
	}

	//儿童画展报名投票
	childrenCompetition := r.Group("/children/competition")
	{
		childrenCompetition.Use(middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
		childrenCompetition.POST("registration/records", children_competition.GetRegistrationRecordInfo)                     //获取报名记录
		childrenCompetition.POST("participant/registration/records", children_competition.GetParticipantRegistrationRecords) //获取个人报名记录
		childrenCompetition.POST("set/registration/deadline", children_competition.SetRegistrationDeadline)                  //设置报名截止时间
		childrenCompetition.POST("update/work/status", children_competition.UpdateWorkStatus)                                // 更新作品状态
		childrenCompetition.POST("generate/qr/code", children_competition.GenerateQrCode)                                    // 生成投票二维码
		childrenCompetition.POST("batch/generate/qrCode", children_competition.BatchGenerateQrCode)                          // 生成全部投票二维码
		childrenCompetition.POST("works/votes/info", children_competition.GetWorksVotesInfo)                                 // 获取投票结果
		childrenCompetition.POST("works/votes/export/form", children_competition.WorksVotesExportForm)                       // 导出投票结果
	}

	//快递柜
	expressRoute := r.Group("/express")
	{
		expressRoute.Use(middleware.CheckLogin(service.AccountProvider), middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
		expressRoute.POST("singlechip/edit", service.EditSinglechip)            //修改开发板信息 单个
		expressRoute.POST("singlechip/batch/edit", service.BatchEditSinglechip) //修改开发板信息 单个
		expressRoute.POST("circuit/edit", service.EditCircuit)                  //修改柜门信息
		expressRoute.POST("circuit/setStatus", service.EditCircuit)             //柜门修改开闭状态
		expressRoute.POST("circuit/list", service.ListCircuit)                  //快递柜列表
		expressRoute.POST("send", service.Send)                                 //寄件
		expressRoute.POST("history/list", service.HistoryList)                  //寄件历史明细
		expressRoute.POST("singlechip/open", service.Open)                      //开发板开启关闭状态
		expressRoute.POST("singlechip/remove", service.Remove)                  //开发板一键删除
		expressRoute.POST("singlechip/list", service.ExpressList)               //开发板一键删除
		expressRoute.POST("singlechip/create", service.Create)
	}

	//管理系统预约商城
	productRoute := r.Group("/product")
	{
		productRoute.POST("/list", service.ProductList)               //商品列表
		productRoute.POST("/info", service.ProductInfo)               //商品详情
		productRoute.POST("/sell", service.SellList)                  //商品购买记录
		productRoute.POST("/add", service.AddProduct)                 //新增商品
		productRoute.POST("/updatestate", service.UpdateProductState) //更新商品状态
		productRoute.POST("/del", service.DelProduct)                 //删除未上架的商品
		productRoute.POST("/updategoods", service.UpdateProduct)      //修改未上架的商品
		productRoute.POST("/sale", service.SaleLog)                   //没有商品版的售卖记录
		productRoute.POST("/refuse", service.WechatRefunds)           //退款
	}

	healthRoute := r.Group("/health")
	{
		healthRoute.Use(middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
		healthRoute.POST("/list", service.DoctorList)                    //医生信息列表
		healthRoute.POST("/prelist", service.PreList)                    //预约列表
		healthRoute.POST("/info", service.PreInfo)                       //预约详情
		healthRoute.POST("/edit", service.UpdatePre)                     //更新预约信息
		healthRoute.POST("/illmessage", service.SendMedical)             //发送电子病历
		healthRoute.POST("/okmsg", service.SendPremsg)                   //发送电子病历
		healthRoute.POST("/editstreet", service.UpdateStreet)            //更新街道
		healthRoute.POST("/adddr", service.AddDr)                        //增加预约科室医生
		healthRoute.POST("/call", service.CallSign)                      //叫号
		healthRoute.POST("/callup", service.QueueUp)                     //插队
		healthRoute.POST("/drde", service.DrDe)                          //医生和对应科室
		healthRoute.POST("/delex", service.DelExAdd)                     //删除科室
		healthRoute.POST("/appoint/list", service.HealthAppointList)     //删除科室
		healthRoute.POST("/appoint/del", service.DelHealthAppoint)       //删除科室
		healthRoute.POST("/appoint/update", service.UpdateHealthAppoint) //删除科室

	}

	preorderRoute := r.Group("/preorder")
	{
		preorderRoute.Use(middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
		// preorderRoute.POST("/add", service.AddTeamType)      //增加团队类型
		preorderRoute.POST("/list", service.TeamTypeList) //查看团队类型
		preorderRoute.POST("/edit", service.EditTeamType) //修改团队类型
		// preorderRoute.POST("/del", service.DelTeamType)      //删除团队类型
		preorderRoute.POST("/lotset", service.LotSet)     //批量设置可预约日期
		preorderRoute.POST("/setlist", service.SetList)   //根据给的时间展示可展的和不可展的日期
		preorderRoute.POST("/editset", service.EditSet)   //删除当天，把当天变成不可展
		preorderRoute.POST("/timeedit", service.EditTime) //设置时间段（时间段的增删改查）
		preorderRoute.POST("/timelist", service.AllTime)  //时间段列表
	}

	journalRoute := r.Group("/journal")
	{
		journalRoute.Use(middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
		journalRoute.POST("/add", service.AddJournal)        //创建一条日志
		journalRoute.POST("/selflist", service.SelfList)     //自己创建的日志列表
		journalRoute.POST("/acceptlist", service.AcceptList) //接收到的日志
		journalRoute.POST("/info", service.JournalInfo)      //日志详情
		journalRoute.POST("/read", service.ReadJournal)      //日志已读
		journalRoute.POST("/undo", service.UndoJournal)      //日志撤销
	}

	paymentRoute := r.Group("/payment")
	{
		paymentRoute.Use(middleware.CheckLogin(service.AccountProvider), middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
		paymentRoute.POST("/add", order.AddShop)
		paymentRoute.POST("/list", order.ShopList)
		paymentRoute.POST("/edit", order.EditShop)
		paymentRoute.POST("/del", order.DelShop)
		paymentRoute.POST("/addqr", order.AddQr)
		paymentRoute.POST("/addqr64", order.AddQrBase64)
		paymentRoute.POST("/onqr", order.OnQr)
		paymentRoute.POST("/offqr", order.OffQr)
		paymentRoute.POST("/downloadqr", order.Downloadqr)
	}

	// 二维码收款订单管理
	collectionRecordRoute := r.Group("/collection/record")
	{
		collectionRecordRoute.Use(middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
		collectionRecordRoute.POST("/list", order.CollectionRecordList)    //特定条件的收款列表
		collectionRecordRoute.POST("/export/excel", order.ExportForm)      //导出特定条件的收款信息到excel表
		collectionRecordRoute.POST("/modify/remarks", order.ModifyRemarks) //编辑收款订单备注
	}

	// 培训管理
	trainRoute := r.Group("/train")
	{
		trainRoute.Use(middleware.CheckLogin(service.AccountProvider), middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
		trainRoute.POST("/createTrainInfo", train.CreateTrainInfo)                  //新增培训信息
		trainRoute.POST("/updateTrainInfo", train.UpdateTrainInfo)                  //更改培训信息
		trainRoute.POST("/deleteTrainInfo", train.DeleteTrainInfo)                  //删除培训信息
		trainRoute.POST("/trainInfoList", train.TrainInfoList)                      //培训信息列表
		trainRoute.POST("/specific/trainInfo/list", train.SpecificTrainInfoList)    //具体培训下签到人员列表
		trainRoute.GET("/specific/trainInfo/list", train.SpecificTrainInfoListJson) //具体培训下签到人员列表
		trainRoute.POST("/pic/stream", train.GetPicStream)                          //具体培训下签到人员列表
		trainRoute.POST("/personnel/upload", train.UploadPersonnel)                 //人员名单批量上传
	}

	// 招聘管理
	recruitRoute := r.Group("/recruit")
	{
		recruitRoute.Use(middleware.CheckLogin(service.AccountProvider), middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
		recruitRoute.POST("/resume/upload/multi", recruit.UploadMulti)                        // 简历设计业务上传
		recruitRoute.POST("/post/submit", recruit.PostSubmit)                                 // 提交岗位审批
		recruitRoute.POST("/post/up", recruit.PostUp)                                         // 关闭岗位审批
		recruitRoute.POST("/post/cancel/up", recruit.PostCancelUp)                            // 关闭岗位审批
		recruitRoute.POST("/post/pass", recruit.PostPass)                                     // 岗位审批审核通过
		recruitRoute.POST("/post/reject", recruit.PostReject)                                 // 岗位审批审核不通过
		recruitRoute.POST("/post/check", recruit.PostCheck)                                   // 岗位审批抄送已阅
		recruitRoute.POST("/post/close", recruit.PostClose)                                   // 关闭岗位审批
		recruitRoute.POST("/resume/submit", recruit.ResumeSubmit)                             // 简历上传
		recruitRoute.POST("/resume/submit/v2", recruit.ResumeSubmitV2)                        // 简历上传v2 版本携带上传的文件的名称以及附件
		recruitRoute.POST("/resume/pass", recruit.ResumePass)                                 // 简历审核通过
		recruitRoute.POST("/resume/offer/sure", recruit.GiveResumeOffer)                      // 简历给予offer 录用最终一步 创建帐号信息
		recruitRoute.POST("/resume/reject", recruit.ResumeReject)                             // 简历审核不通过
		recruitRoute.POST("/resume/interview/time", recruit.ResumeInterviewTime)              // 预约面试时间
		recruitRoute.POST("/resume/modify/interview/time", recruit.ResumeModifyInterviewTime) // 修改面试时间
		recruitRoute.POST("/resume/modify/info", recruit.ResumeModifyInfo)                    // 修改面试信息
		recruitRoute.POST("/resume/interview/pass", recruit.ResumeInterviewPass)              // 面试通过
		recruitRoute.POST("/resume/interview/reject", recruit.ResumeInterviewReject)          // 面试不通过
		recruitRoute.POST("/resume/terminate", recruit.ResumeTerminate)                       // 简历终止
		recruitRoute.POST("/resume/upload/detail", recruit.ResumeUploadDetail)                // 上传面试信息
		recruitRoute.POST("/offer/submit", recruit.OfferSubmit)                               // 提交入职申请
		recruitRoute.POST("/offer/pass", recruit.OfferPass)                                   // 入职申请审核通过
		recruitRoute.POST("/offer/reject", recruit.OfferReject)                               // 入职申请审核不通过
		recruitRoute.POST("/offer/check", recruit.OfferCheck)                                 // 入职申请已阅
		recruitRoute.POST("/position/query", recruit.PositionQuery)                           // 分页查询岗位申请
		recruitRoute.POST("/resume/page/query", recruit.ResumePageQuery)                      // 分页查询简历
		recruitRoute.POST("/offer/query", recruit.OfferQuery)                                 // 分页查询入职申请
		recruitRoute.POST("/detail/query", recruit.DetailQuery)                               // 分页查询审批详情
		recruitRoute.POST("/copied/query", recruit.CopiedQuery)                               // 分页查询审批抄送
		recruitRoute.POST("/change/record/query", recruit.ChangeRecordQuery)                  // 查询变更记录
		recruitRoute.POST("/status/query", recruit.StatusQuery)                               // 查询状态列表
		recruitRoute.POST("/data/info", recruit.DataInfo)                                     // 查询状态列表
		recruitRoute.POST("/resume/queryphone", recruit.CheckResumeExist)                     // 根据手机号查询，简历是否存在人才招聘系统中，是的话不允许直接发起录用申请
	}

	// 物资管理
	goodsManagementRoute := r.Group("/goods/management")
	{
		goodsManagementRoute.Use(middleware.CheckLogin(service.AccountProvider), middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
		goodsManagementRoute.POST("/add/borrow/goods", goods_management.AddBorrowGoods)             // 增加借用商品
		goodsManagementRoute.POST("/borrow/goods/list", goods_management.BorrowGoodsList)           // 借用商品列表
		goodsManagementRoute.POST("/generate/qr/code", goods_management.GenerateBorrowQrCode)       // 具体借用商品生成二维码
		goodsManagementRoute.POST("/put/down/borrow/goods", goods_management.PutDownBorrowGoods)    // 上下架借用的商品
		goodsManagementRoute.POST("/delete/borrow/goods", goods_management.DeleteBorrowGoods)       // 删除借用的商品
		goodsManagementRoute.POST("/specific/borrow/record", goods_management.SpecificBorrowRecord) // 具体物品的借用记录
	}

	// 站点签到
	SiteSignInRoute := r.Group("/site")
	{
		SiteSignInRoute.Use(middleware.CheckLogin(service.AccountProvider), middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
		SiteSignInRoute.POST("/siteList", service.SiteSignInList)             // 站点列表
		SiteSignInRoute.POST("/creCodeImage", service.CreCodeImage)           // 生成二维码
		SiteSignInRoute.POST("/downloadCodeImage", service.DownloadCodeImage) // 下载二维码
		SiteSignInRoute.POST("/queryByApi", service.QuerySiteByApi)           // 通过API查询经纬度
		SiteSignInRoute.POST("/reverseGeocoding", service.ReverseGeocoding)   // 自选地图点
	}

	// 客服工作台
	customerService := r.Group("/customerService")
	{
		customerService.Use(middleware.CheckLogin(service.AccountProvider), middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
		customerService.POST("/lock/artist/list", customer_service.GetLockArtistList)          // 已经锁定的画家列表
		customerService.POST("/audit/artwork/list", customer_service.GetPassedArtworkList)     // 审核完成的画作列表
		customerService.POST("/notify/artist", customer_service.UpdateNotifyStatus)            // 更新通知画家
		customerService.POST("/notified/artistList", customer_service.GetNotifiedArtistList)   // 已通知画家列表
		customerService.POST("/notified/artworkList", customer_service.GetNotifiedArtworkList) // 已通知画家列表
	}
	// 首页
	homePage := r.Group("/homePage")
	{
		homePage.Use(middleware.CheckLogin(service.AccountProvider), middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
		homePage.POST("/apply/count", home_page.ApplyCount)     // 审核/考勤-申请数量
		homePage.POST("/clockIn/info", home_page.ClockingIn)    // 考勤信息
		homePage.POST("/clockIn/record", home_page.ClockRecord) // 打卡记录
	}
	//新增画家宝审批流相关接口 By JJxu 2023-03-06
	NewAuditRouter(r)
	//静态文件
	r.StaticFS("/static", http.Dir("./runtime"))
	r.NoRoute(func(c *gin.Context) {
		c.JSON(http.StatusNotFound, gin.H{
			"status": 1,
			"msg":    "不存在的路由",
		})
	})
	pprof.Register(r)
	return r
}
