package router

import (
	"github.com/fonchain_enterprise/fonchain-main/pkg/middleware"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/shop"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/task"
	"github.com/gin-gonic/gin"
)

// ShopRoute 商城后台 路由配置
func ShopRoute(Router *gin.RouterGroup) {
	series := Router.Group("series")
	series.Use(middleware.CheckLogin(service.AccountProvider), middleware.CheckAuth(service.AccountProvider, service.RuleProvider))
	{
		series.Use(middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
		series.POST("save", shop.SaveSeries)
		series.POST("detail", shop.SeriesDetail)
		series.POST("update-lang", shop.UpdateSeriesLang)
		series.POST("autoshelf", shop.AutoShelf)
		series.POST("handshelf", shop.HandShelf)
		series.POST("series-list", shop.SeriesList)
		series.POST("del", shop.SeriesDel)
		series.POST("get/series/language/info", shop.GetSeriesLanguageInfo)
	}

	brand := Router.Group("brand")
	brand.Use(middleware.CheckLogin(service.AccountProvider), middleware.CheckAuth(service.AccountProvider, service.RuleProvider))
	{
		brand.Use(middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
		brand.POST("create", shop.CreateBrand)
		brand.POST("info", shop.BrandInfo)
		brand.POST("update", shop.BrandUpdate)
		brand.POST("del", shop.BrandDel)
		brand.POST("list", shop.BrandList)
		brand.POST("allbrand", shop.AllBrand)
		brand.POST("update/language/info", shop.UpdateLanguageInfo)
		brand.POST("get/all/language/info", shop.GetAllLanguageInfo)
	}

	feedback := Router.Group("feedback")
	feedback.Use(middleware.CheckLogin(service.AccountProvider), middleware.CheckAuth(service.AccountProvider, service.RuleProvider))
	{
		feedback.Use(middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
		feedback.POST("create", task.FeedBackCreate)
		feedback.POST("top", task.Topping)
		feedback.POST("list", task.FeedBackList)
	}

	collections := Router.Group("collections")
	collections.Use(middleware.CheckLogin(service.AccountProvider), middleware.CheckAuth(service.AccountProvider, service.RuleProvider))
	{
		collections.Use(middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
		collections.POST("create", shop.AddCollections)
		collections.POST("update", shop.UpdateCollectionsProfile)
		collections.POST("del", shop.UpdateCollectionsProfile)
		collections.POST("list", shop.CollectionsList)
		collections.POST("detail", shop.CollectionsDetail)
		collections.POST("qr", shop.Qr)
		collections.POST("extend", shop.Extend)
		collections.POST("get/language/status", shop.GetCollectionLanguageInfo) // 获取藏品多语言填写状态
	}

	recordRouter := Router.Group("shop/record")
	recordRouter.Use(middleware.CheckLogin(service.AccountProvider), middleware.CheckAuth(service.AccountProvider, service.RuleProvider))
	{
		recordRouter.POST("list", shop.RecordList)
	}

	testRouter := Router.Group("shop/test")
	//testRouter.Use(middleware.CheckLogin(service.AccountProvider))
	{
		testRouter.POST("auto/wiped", shop.MethodAutoWipedOut)
		testRouter.POST("auto/position", shop.MethodAutoPosition)
		testRouter.GET("push/wiped", shop.PushAutoWipedOut)
		testRouter.GET("push/position", shop.PushAutoPosition)
	}
}
