package router

import (
	"github.com/fonchain_enterprise/fonchain-main/pkg/middleware"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/account"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/turnstile"
	"github.com/gin-gonic/gin"
)

// TurnstileRoute 速通门的认证
func TurnstileRoute(r *gin.RouterGroup) {
	turnRoute := r.Group("turnstile")

	turnRoute.GET("ws", turnstile.DeviceWs) //websocket
	//turnRoute.POST("update/account", account.UpdateDeviceAccount) //更新/创建 设备人员帐号
	//turnRoute.POST("del/account", account.DelSingleDeviceAccount) //删除人员

	noSecret := r.Group("/secret")
	{
		//访客
		noSecret.POST("visitor", account.ClockVisitor) // 访客申请
		//noSecret.POST("set/door", account.ClockDeviceWXSetDoor)                 // 控制设备开门/关门
		//noSecret.POST("batch/capture/photo", account.ClockDeviceWXCapturePhoto) // 抓拍现场照片
	}
	clockRoute := r.Group("/clock")
	clockRoute.Use(middleware.CheckLogin(service.AccountProvider), middleware.CheckAuth(service.AccountProvider, service.RuleProvider), middleware.AutoLog(service.AccountProvider, service.GrpcLogImpl))
	{
		clockRoute.POST("create", account.CreateClockDevice)                        // 创建打卡机设备
		clockRoute.POST("update", account.UpdateClockDevice)                        // 更新打卡机设备
		clockRoute.DELETE("remove", account.RemoveClockDevice)                      // 删除打卡机设备
		clockRoute.POST("info", account.ClockDeviceInfo)                            // 查看打卡机详情
		clockRoute.POST("list", account.ClockDeviceList)                            // 查看打卡机列表
		clockRoute.POST("single/untie", account.ClockDeviceSingleUntie)             // 单人解绑打卡机设备
		clockRoute.POST("batch/bind", account.ClockDeviceBatchBind)                 // 批量绑定打卡机
		clockRoute.POST("batch/untie", account.ClockDeviceBatchUntie)               // 批量解绑打卡机
		clockRoute.POST("batch/detection/photo", account.ClockDeviceDetectionPhoto) // 检测图片质量
		clockRoute.POST("batch/capture/photo", account.ClockDeviceCapturePhoto)     // 抓拍现场照片
		//通行规则
		clockRoute.POST("rule/issuance", account.ClockDeviceRuleIssuance) // 人员通行规则下发
		clockRoute.POST("rule/acquire", account.ClockDeviceRuleAcquire)   // 获取人员通行规则
		clockRoute.POST("rule/delete", account.ClockDeviceRuleDelete)     // 删除人员通行规则
		//密码开门
		clockRoute.POST("password/switch", account.ClockDevicePasswordSwitch) // 设置密码开门开关
		clockRoute.POST("password/set", account.ClockDevicePasswordSet)       // 设置设备开门密码
		//读取设备人员
		clockRoute.POST("person/sync", account.ClockDevicePersonSync)   // 通知设备同步人员数据
		clockRoute.POST("person/total", account.ClockDevicePersonTotal) // 查询设备人员数量和id
		clockRoute.POST("person/info", account.ClockDevicePersonInfo)   // 根据userid读取用户数据
		//设备设置
		clockRoute.POST("set/acquire", account.ClockDeviceSetAcquire)                 // 获取设备设置
		clockRoute.POST("set/logo", account.ClockDeviceSetLogo)                       // 设置设备logo
		clockRoute.POST("set/slogan", account.ClockDeviceSetSlogan)                   // 设置广告语
		clockRoute.POST("set/time", account.ClockDeviceSetTime)                       // 设置设备时间
		clockRoute.POST("set/interval", account.ClockDeviceSetInterval)               // 设置设备识别间隔
		clockRoute.POST("set/distance", account.ClockDeviceSetDistance)               // 设置设备识别距离
		clockRoute.POST("set/mask", account.ClockDeviceSetMask)                       // 口罩检测开关
		clockRoute.POST("set/living", account.ClockDeviceSetLiving)                   // 活体检测
		clockRoute.POST("set/living/class", account.ClockDeviceSetLivingClass)        // 活体检测等级
		clockRoute.POST("set/stranger", account.ClockDeviceSetStranger)               // 陌生人识别
		clockRoute.POST("set/volume", account.ClockDeviceSetVolume)                   // 设备音量
		clockRoute.POST("set/username", account.ClockDeviceSetUsername)               // 播放用户名
		clockRoute.POST("set/hint", account.ClockDeviceSetHint)                       // 设置设备提示语音
		clockRoute.POST("set/password", account.ClockDeviceSetPassword)               // 修改进入设置页面密码
		clockRoute.POST("set/restart", account.ClockDeviceSetRestart)                 // 重启设备
		clockRoute.POST("set/relay", account.ClockDeviceSetRelay)                     // 设置继电器状态-常开/正常
		clockRoute.POST("set/capture", account.ClockDeviceSetCapture)                 // 设置设备识别照片抓拍的质量
		clockRoute.POST("set/callout", account.ClockDeviceSetCallout)                 // 访问呼叫开关
		clockRoute.POST("set/printer", account.ClockDeviceSetPrinter)                 // 打印机开关
		clockRoute.POST("set/visitor", account.ClockDeviceSetVisitor)                 // 首页访客申请入库内容
		clockRoute.POST("set/token", account.ClockDeviceSetToken)                     // 设置http请求的token
		clockRoute.POST("set/verify", account.ClockDeviceSetVerify)                   // 设置验证接口超时时长
		clockRoute.POST("set/delay", account.ClockDeviceSetDelay)                     // 设置设备开门延迟
		clockRoute.POST("set/press/time", account.ClockDeviceSetPressTime)            // 进入设置页面长按时间
		clockRoute.POST("set/desensitization", account.ClockDeviceSetDesensitization) // 姓名脱敏显示
		clockRoute.POST("set/section", account.ClockDeviceSetSection)                 // 设置读卡扇区
		clockRoute.POST("set/rail", account.ClockDeviceSetRail)                       // 设置设备位置围栏
		clockRoute.POST("set/door", account.ClockDeviceSetDoor)                       // 控制设备开门/关门
		//屏保
		clockRoute.POST("screensaver/switch", account.ClockScreensaverState)      // 设备节能屏保开关
		clockRoute.POST("screensaver/settings", account.ClockScreensaver)         // 设置设备屏保
		clockRoute.POST("screensaver/delete", account.ClockDelScreensaver)        // 删除屏保
		clockRoute.POST("screensaver/interval", account.ClockScreensaverInterval) // 设置屏保显示时长
		//体温检测
		clockRoute.POST("temperature/detection", account.ClockTemperatureDetection)       // 体温检测开关
		clockRoute.POST("temperature/mode", account.ClockTemperatureMode)                 // 体温检测模式
		clockRoute.POST("temperature/standard", account.ClockTemperatureStandard)         // 初筛模式标准
		clockRoute.POST("temperature/calibration", account.ClockTemperatureCalibration)   // 设置体温校准模式
		clockRoute.POST("temperature/low/pass", account.ClockTemperatureLowPass)          // 低温可通行开关
		clockRoute.POST("temperature/play", account.ClockTemperaturePlay)                 // 播报体温开关
		clockRoute.POST("temperature/reflectivity", account.ClockTemperatureReflectivity) // 设置体温反射率
		clockRoute.POST("temperature/correction", account.ClockTemperatureCorrection)     // 设置体温校准值
		clockRoute.POST("temperature/distance", account.ClockTemperatureDistance)         // 设置设备体温检测距离
		clockRoute.POST("temperature/measure", account.ClockTemperatureMeasure)           // 设置测温时间段
		//设备更新
		clockRoute.POST("version/auto", account.ClockVersionAuto) // 开机自动更新
		clockRoute.POST("version/info", account.ClockVersionInfo) // 获取设备版本信息
		//clockRoute.POST("version/push", account.ClockVersionPush)   // 发送设备版本信息给服务端
		clockRoute.POST("version/check", account.ClockVersionCheck) // 通知设备在线检查更新
		//clockRoute.POST("version/Interval", account.ClockDeviceSetRail) // 发送升级结果给服务端
		//clockRoute.POST("version/Interval", account.ClockDeviceSetRail) // 更新下载进度
		//二维码
		clockRoute.POST("apply/value", account.ClockApplyValue)   // 设置设备首页二维码
		clockRoute.POST("apply/prefix", account.ClockApplyPrefix) // 设置访客二维码前缀
		//全景相机设置
		clockRoute.POST("camera/panoramic", account.ClockCameraPanoramic) // 全景相机开关
		clockRoute.POST("camera/params", account.ClockCameraParams)       // 设置全景相机的参数
		//个性化设置
		clockRoute.POST("recognition/style", account.ClockRecognitionStyle) // 识别结果样式
		//识别后在线验证
		clockRoute.POST("online/verify", account.ClockOnlineVerify)  // 识别后在线验证
		clockRoute.POST("online/card", account.ClockOnlineCard)      // 在线验证卡号
		clockRoute.POST("online/id/card", account.ClockOnlineIdCard) // 在线验证身份证号
		//在线识别
		clockRoute.POST("online/recognition", account.ClockOnline)        // 设置在线识别间隔
		clockRoute.POST("online/state", account.ClockOnlineState)         // 在线识别开关
		clockRoute.POST("online/temporary", account.ClockOnlineTemporary) // 临时启用在线识别
		//识别设置
		clockRoute.POST("discern/settings", account.ClockDiscernSettings) // 设置可通行人员类型
		clockRoute.POST("discern/class", account.ClockDiscernClass)       // 设置设备识别置信度
		//设备禁用
		clockRoute.POST("forbidden/switch", account.ClockForbiddenSwitch) // 开启/关闭设备识别
		clockRoute.POST("forbidden/number", account.ClockForbiddenNumber) // 停用状态通行人数
		//广告
		clockRoute.POST("advertising/add", account.ClockAdvertisingAdd)       // 下发广告 - 图片
		clockRoute.POST("advertising/del", account.ClockAdvertisingDel)       // 删除广告
		clockRoute.POST("advertising/change", account.ClockAdvertisingChange) // 启用停用广告
		//门磁状态
		clockRoute.POST("door/lock", account.ClockDoorLock)         // 获取(上报)门锁状态
		clockRoute.POST("door/listener", account.ClockDoorListener) // 设置门锁实时监听开关
		//打卡记录
		clockRoute.POST("log/list", account.ClockLogList) // 查看打卡记录列表
		clockRoute.POST("log/info", account.ClockLogInfo) // 查看打卡记录详情
	}
}
