package approval

import (
	"fmt"
	api "github.com/fonchain_enterprise/fonchain-main/api/approval"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/login"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	m "github.com/fonchain_enterprise/fonchain-main/pkg/service/approval/model"
	"github.com/getsentry/sentry-go"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
	"strconv"
)

// 审批类型版本
const (
	VersionV1 = 1 // 旧版本
	VersionV2 = 2 // 新版本
)

func CreateApprovalType(c *gin.Context) {
	var req api.CreateApprovalTypeRequest
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.Error, err)
		return
	}

	userInfo := login.GetUserInfoFromC(c)

	req.Operator = userInfo.NickName

	if req.Version == 0 {
		req.Version = VersionV2
	}

	res, err3 := service.ApprovalProvider.CreateApprovalType(c, &req)

	if err3 != nil {
		service.Error(c, e.Error, err3)
		return
	}

	service.Success(c, res)
}

func UpdateApprovalType(c *gin.Context) {
	var req api.CreateApprovalTypeRequest
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.Error, err)
		return
	}

	fmt.Println("========================== Type =====================")
	fmt.Printf("CreateApprovalTypeRequest : %+v", req)
	fmt.Println("========================== Type =====================")
	fmt.Println()

	userInfo := login.GetUserInfoFromC(c)

	req.Operator = userInfo.NickName

	res, err := service.ApprovalProvider.UpdateApprovalType(c, &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
}

func CopyApprovalType(c *gin.Context) {
	var req api.CreateApprovalTypeRequest
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.Error, err)
		return
	}

	fmt.Println("========================== Type =====================")
	fmt.Printf("CreateApprovalTypeRequest : %+v", req)
	fmt.Println("========================== Type =====================")
	fmt.Println()

	userInfo := login.GetUserInfoFromC(c)

	req.Operator = userInfo.NickName

	res, err := service.ApprovalProvider.CopyApprovalType(c, &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
}

func UpdateApprovalTypeStatus(c *gin.Context) {
	var req api.UpdateTypeStatusRequest
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.Error, err)
		return
	}

	fmt.Println("========================== Type =====================")
	fmt.Printf("UpdateTypeStatusRequest : %+v", req)
	fmt.Println("========================== Type =====================")
	fmt.Println()

	res, err := service.ApprovalProvider.UpdateTypeStatus(c, &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
}

func ApprovalGroupList(c *gin.Context) {
	var req api.ApprovalGroupListRequest
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.Error, err)
		return
	}

	fmt.Println("========================== Type =====================")
	fmt.Printf("ApprovalGroupListRequest : %+v", req)
	fmt.Println("========================== Type =====================")
	fmt.Println()
	if req.Domain == "" {
		req.Domain = "0"
	}

	res, err := service.ApprovalProvider.ApprovalGroupList(c, &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	//字段国际化
	if len(res.GroupList) > 0 {
		lang := c.Request.Header.Get("Accept-Language")
		for _, groupOne := range res.GroupList {
			//groupOne.Title = m.GetLanguageType(groupOne.Title)
			for _, typeOne := range groupOne.List {
				typeOne.Title = m.GetLanguageType(lang, typeOne.Title)

			}

		}
	}
	service.Success(c, res)
}

func ApprovalTypeList(c *gin.Context) {
	var req api.ApprovalTypeListRequest
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.Error, err)
		return
	}

	userInfo := login.GetUserInfoFromC(c)
	if req.Positions == nil || len(req.Positions) < 1 {
		req.Positions = make([]*api.ApprovalTypeListRequest_Position, 0)
		for i := 0; i < len(userInfo.PositionUsers); i++ {
			req.Positions = append(req.Positions, &api.ApprovalTypeListRequest_Position{
				PositionUID:   strconv.FormatUint(userInfo.PositionUsers[i].PositionID, 10),
				DepartmentUID: strconv.FormatUint(userInfo.PositionUsers[i].DepartmentId, 10),
			})
		}
	}

	res, err := service.ApprovalProvider.ApprovalTypeList(c, &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
}

func ApprovalTypeListV3(c *gin.Context) {
	var req api.ApprovalTypeListRequest
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.Error, err)
		return
	}
	req.Domain = "0"

	res, err := service.ApprovalProvider.ApprovalTypeList(c, &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	typeList := make([]*api.CreateApprovalTypeRequest, 0)
	if len(res.TypeList) > 0 {
		for _, typeOne := range res.TypeList {
			if m.GetOaType(typeOne.KeyWord) != "" {
				typeList = append(typeList, typeOne)
			}
		}
	}
	res.TypeList = typeList

	service.Success(c, res)
}

func CreateApprovalGroup(c *gin.Context) {
	var req api.CreateApprovalTypeGroupRequest
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.Error, err)
		return
	}

	res, err3 := service.ApprovalProvider.CreateApprovalGroup(c, &req)

	if err3 != nil {
		service.Error(c, e.Error, err3)
		return
	}

	service.Success(c, res)
}

// OnlyGroupList 只查看分组列表
func OnlyGroupList(c *gin.Context) {
	var req api.ApprovalGroupListRequest
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.Error, err)
		return
	}
	if req.Domain == "" {
		req.Domain = "0"
	}
	res, err1 := service.ApprovalProvider.OnlyGroupList(c, &req)

	if err1 != nil {
		sentry.CaptureException(err1)
		service.Error(c, e.Error, err1)
		return
	}

	service.Success(c, res)
}

// ApprovalHisTypeList 查询历史审批类型
func ApprovalHisTypeList(c *gin.Context) {
	var req api.ApprovalGroupListRequest
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.Error, err)
		return
	}

	res, err1 := service.ApprovalProvider.ApprovalHisTypeList(c, &req)

	if err1 != nil {
		sentry.CaptureException(err1)
		service.Error(c, e.Error, err1)
		return
	}

	service.Success(c, res)
}
