package service

import (
	"context"
	"dubbo.apache.org/dubbo-go/v3/common/logger"
	"fmt"
	"github.com/360EntSecGroup-Skylar/excelize"
	artShow "github.com/fonchain_enterprise/fonchain-main/api/artShow"
	"github.com/fonchain_enterprise/fonchain-main/api/artwork_third_party"
	"github.com/fonchain_enterprise/fonchain-main/pkg/config"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/logic"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model"
	"github.com/fonchain_enterprise/fonchain-main/pkg/serializer"
	"github.com/fonchain_enterprise/fonchain-main/pkg/utils"
	"github.com/gin-gonic/gin"
	"github.com/golang/freetype"
	"golang.org/x/image/font"
	"image"
	"image/color"
	"image/draw"
	"image/png"
	"io/ioutil"
	"os"
	"strconv"
	"strings"
	"time"
)

func ExportShowPrice(c *gin.Context) {
	var exportShowPriceReq model.ExportShowPrice
	if err := c.ShouldBind(&exportShowPriceReq); err != nil {
		logger.Errorf("ExportShowPrice ShouldBind err", err)
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if len(exportShowPriceReq.ShowInfos) < 1 {
		logger.Errorf("ExportShowPrice  query params  err :", "画展包 uid 为空")
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   nil,
			Msg:    "画展包 uid 为空",
			Status: e.Failed,
		})
		return
	}

	//  组装 获取 画作 基础信息参数
	artworkDataReq := artwork_third_party.AwDataByShowIdRequest{}
	uids := make([]string, 0)
	for i := 0; i < len(exportShowPriceReq.ShowInfos); i++ {
		uids = append(uids, exportShowPriceReq.ShowInfos[i].UID)
	}
	artworkDataReq.ArtShowIds = uids

	//  获取 画作 基础信息
	resp, err := GrpcArtworkThirdPartyImpl.AwDataByShowId(context.Background(), &artworkDataReq)
	if err != nil {
		logger.Errorf("AwDataByShowId query err", err)
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	if len(resp.Data) < 1 {
		logger.Errorf("ExportShowPrice  query params  err :", "画作信息为空")
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   nil,
			Msg:    "画作信息为空",
			Status: e.Failed,
		})
		return
	}

	// 组装 画作 基础信息
	artworkData := make(map[string]*artwork_third_party.AwDataByShowIdResponse_Info, 0)
	for i := 0; i < len(resp.Data); i++ {
		artworkData[resp.Data[i].ArtworkUuid] = resp.Data[i]
	}

	// 文件的路径 压缩包的名字
	now := strconv.FormatInt(time.Now().Unix(), 10)
	dirPath := "./runtime/artshow/"
	utils.CheckDirPath(dirPath, true)

	zipPathName := now + ".zip"
	zipPath := fmt.Sprintf("./runtime/%s/", now+"_"+exportShowPriceReq.ExportType)
	utils.CheckDirPath(zipPath, true)

	//// 获取 画作价格信息
	//artworkPriceReq := artShow.ShowDetailReq{}
	applyInfos := make([]*model.ApplyInfo, 0)
	//filePaths := make([]string, 0)
	var msg string
	for i := 0; i < len(exportShowPriceReq.ShowInfos); i++ {
		// 获取 画作价格信息
		artworkPriceList := new(artShow.ArtworkPriceListRes)
		// 获取 画作价格信息
		artworkPriceReq := artShow.ShowDetailReq{}
		artworkPriceReq.ShowUID = append(artworkPriceReq.ShowUID, exportShowPriceReq.ShowInfos[i].UID)
		artworkPriceList, err = GrpcArtShowImpl.ArtworkPriceList(context.Background(), &artworkPriceReq)
		if err != nil {
			logger.Errorf("ArtworkPriceList query err", err)
			ResponseMsg(c, e.SUCCESS, serializer.Response{
				Msg:    err.Error(),
				Status: e.Failed,
			})
			return
		}
		// 获取 画展包信息
		showDetailReq := new(artShow.ShowDetailReq)
		showDetailReq.ShowUID = append(showDetailReq.ShowUID, exportShowPriceReq.ShowInfos[i].UID)
		artShowInfo, err := GrpcArtShowImpl.ShowDetail(context.Background(), showDetailReq)
		if err != nil {
			logger.Errorf("ShowDetail query err", err)
			ResponseMsg(c, e.SUCCESS, serializer.Response{
				Msg:    err.Error(),
				Status: e.Failed,
			})
			return
		}
		// 每个画展包创建一个文件夹
		newDirPath := dirPath + artShowInfo.Data[len(artShowInfo.Data)-1].ShowName
		utils.CheckDirPath(newDirPath, true)
		// 生成 表格
		//paths, applyInfo, err := logic.ExportShowPrice(artworkPriceList.Data, artworkData, artShowInfo.Data, exportShowPriceReq.ExportType, newDirPath)
		_, applyInfo, err, m := logic.ExportShowPrice(artworkPriceList.Data, artworkData, artShowInfo.Data[len(artShowInfo.Data)-1], exportShowPriceReq.ExportType, newDirPath)
		if err != nil {
			logger.Errorf("ArtworkPriceList query err", err)
			ResponseMsg(c, e.SUCCESS, serializer.Response{
				Msg:    e.ErrorSaveFile,
				Status: e.Failed,
			})
			return
		}

		if m != "" {
			msg = m
		} else {
			msg = "导出成功"
		}
		//filePaths = append(filePaths, newDirPath+"/")
		// 获取 付款明细数据
		if applyInfo != nil {
			applyInfos = append(applyInfos, applyInfo)
		}
	}

	// 生成 付款明细 表格
	if len(applyInfos) > 0 {
		_, err := logic.ExportShowPricePayInfo(applyInfos, dirPath)
		if err != nil {
			logger.Errorf("ExportShowPricePayInfo query err", err)
			ResponseMsg(c, e.SUCCESS, serializer.Response{
				Msg:    e.ErrorSaveFile,
				Status: e.Failed,
			})
			return
		}
		//filePaths = append(filePaths, path)
	}

	if err := utils.ZipFile(dirPath, zipPath+zipPathName); err != nil {
		logger.Errorf("CompressByFiles  err", err)
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    e.ErrorZip,
			Status: e.Failed,
		})
		return
	}

	if err = os.RemoveAll(dirPath); err != nil {
		logger.Errorf("RemoveAll   err", err)
	}

	var httpType string
	if config.IsHttps {
		httpType = model.HttpsType
	} else {
		httpType = model.HttpType
	}

	var exportUrl = strings.Replace(strings.Join([]string{strings.Replace(zipPath, ".", fmt.Sprintf("%s%s", httpType, c.Request.Host), 1), zipPathName}, ""), "runtime", "static", 1)

	ResponseQuickMsg(c, e.Ok, msg, map[string]string{
		"ExportUrl": exportUrl,
	})

}

func ExportEntertainment(c *gin.Context) {
	req := new(model.ExportEntertainment)
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("ExportEntertainment ShouldBind err", err)
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	// 文件的路径 压缩包的名字
	//now := strconv.FormatInt(time.Now().Unix(), 10)
	dirPath := "./runtime/entertainment"
	utils.CheckDirPath(dirPath, true)

	//zipPathName := now + ".zip"
	//zipPath := fmt.Sprintf("./runtime/%s/", req.ShowDate+"_"+now)
	//utils.CheckDirPath(zipPath, true)

	if len(req.Data) == 0 {
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Data:   nil,
			Msg:    e.ErrorExport,
			Status: e.Ok,
		})
		return
	}

	xlsx := excelize.NewFile()
	index := "Sheet1"

	xlsx.MergeCell("Sheet1", "B1", "I1")

	songTi12LeftBoldStyle, _ := xlsx.NewStyle(`{"alignment":{"horizontal":"left"},"font":{"bold":true,"size":12,"family":"宋体"}}`)
	songTi12CenterBoldStyle, _ := xlsx.NewStyle(`{"alignment":{"horizontal":"center"},"font":{"bold":true,"size":12,"family":"宋体"}}`)
	songTi12LeftStyle, _ := xlsx.NewStyle(`{"alignment":{"horizontal":"left"},"font":{"size":12,"family":"宋体"}}`)

	xlsx.SetCellStyle(index, "A1", "A1", songTi12LeftBoldStyle)
	xlsx.SetCellValue(index, "A1", "画展日期")

	xlsx.MergeCell("Sheet1", "B1", "I1")
	xlsx.SetCellStyle(index, "B1", "B1", songTi12CenterBoldStyle)
	xlsx.SetCellValue(index, "B1", req.ShowDate)

	secondRow := []interface{}{"画家姓名", "画展地址", "联系方式", "随行人员", "到达日期", "到达时间", "出行方式", "车次号", "到达地点", "入住酒店", "接送人", "邀请人", "画展时间"}
	xlsx.SetCellStyle(index, "A2", "M2", songTi12LeftBoldStyle)
	xlsx.SetSheetRow(index, "A2", &secondRow)

	rowIndex := 2
	for i := 0; i < len(req.Data); i++ {
		rowIndex++
		xlsx.SetCellStyle(index, fmt.Sprintf("A%v", rowIndex), fmt.Sprintf("L%v", rowIndex), songTi12LeftStyle)
		xlsx.SetCellValue(index, fmt.Sprintf("A%v", rowIndex), req.Data[i].ArtistName)
		xlsx.SetCellValue(index, fmt.Sprintf("B%v", rowIndex), req.Data[i].Address)
		xlsx.SetCellValue(index, fmt.Sprintf("C%v", rowIndex), req.Data[i].TelNum)
		xlsx.SetCellValue(index, fmt.Sprintf("D%v", rowIndex), req.Data[i].Entourage)
		xlsx.SetCellValue(index, fmt.Sprintf("E%v", rowIndex), req.Data[i].ArrivalDate)
		xlsx.SetCellValue(index, fmt.Sprintf("F%v", rowIndex), req.Data[i].ArrivalTime)
		var travelWay string
		if req.Data[i].TravelWay == 1 {
			travelWay = "自驾"
		} else if req.Data[i].TravelWay == 2 {
			travelWay = "公共交通"
		} else {
			travelWay = ""
		}
		xlsx.SetCellValue(index, fmt.Sprintf("G%v", rowIndex), travelWay)
		xlsx.SetCellValue(index, fmt.Sprintf("H%v", rowIndex), req.Data[i].TrainNum)
		xlsx.SetCellValue(index, fmt.Sprintf("I%v", rowIndex), req.Data[i].ArrivalAddress)
		xlsx.SetCellValue(index, fmt.Sprintf("J%v", rowIndex), req.Data[i].Hotel)
		xlsx.SetCellValue(index, fmt.Sprintf("K%v", rowIndex), req.Data[i].Receiver)
		xlsx.SetCellValue(index, fmt.Sprintf("L%v", rowIndex), req.Data[i].Agent)
		xlsx.SetCellValue(index, fmt.Sprintf("M%v", rowIndex), req.Data[i].ShowTime)
	}

	startTime := strings.Split(req.ShowDate, " - ")[0]
	var fileName string
	if len(strings.Split(startTime, " ")) > 1 {
		fileName = strings.Split(startTime, " ")[0]
	} else {
		fileName = startTime
	}
	path := dirPath + "/" + "画展日期:" + fileName + ".xlsx"
	err := xlsx.SaveAs(path)
	if err != nil {
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Data:   nil,
			Msg:    e.GetMsg(e.ErrorFileCreate),
			Status: e.Ok,
		})
		return
	}

	//if err := utils.ZipFile(dirPath, zipPath+zipPathName); err != nil {
	//	logger.Errorf("CompressByFiles  err", err)
	//	ResponseMsg(c, e.SUCCESS, serializer.Response{
	//		Msg:    err.Error(),
	//		Status: e.Failed,
	//	})
	//	return
	//}

	//if err = os.Remove(dirPath + path); err != nil {
	//	logger.Errorf("Remove   err", err)
	//}

	var httpType string
	if config.IsHttps {
		httpType = model.HttpsType
	} else {
		httpType = model.HttpType
	}

	var exportUrl = strings.Replace(strings.Replace(path, ".", fmt.Sprintf("%s%s", httpType, c.Request.Host), 1), "runtime", "static", 1)
	//var exportUrl = fmt.Sprintf("%s%s/%s", httpType, c.Request.Host, dirPath + path)

	ResponseQuickMsg(c, e.Ok, "导出成功", map[string]string{
		"ExportUrl": exportUrl,
	})

}

func ExportShowPriceOneQuery(c *gin.Context) {
	var exportShowPriceReq model.ExportOneQueryShowPrice
	if err := c.ShouldBind(&exportShowPriceReq); err != nil {
		logger.Errorf("ExportShowPrice ShouldBind err", err)
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if exportShowPriceReq.ExportType != "cp" && exportShowPriceReq.ExportType != "aw" {
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   nil,
			Msg:    e.ErrorExportNotAllowed,
			Status: e.Failed,
		})
		return
	}

	if len(exportShowPriceReq.OneQueryShowInfos) < 1 {
		logger.Errorf("ExportShowPrice  query params  err :", "画展包 uid 为空")
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   nil,
			Msg:    "画展包 uid 为空",
			Status: e.Failed,
		})
		return
	}

	//  组装 获取 画作 基础信息参数
	artworkDataReq := artwork_third_party.AwDataByShowIdRequest{}
	uids := make([]string, 0)
	for i := 0; i < len(exportShowPriceReq.OneQueryShowInfos); i++ {
		uids = append(uids, exportShowPriceReq.OneQueryShowInfos[i].ShowUID)
	}
	artworkDataReq.ArtShowIds = uids

	//  获取 画作 基础信息
	resp, err := GrpcArtworkThirdPartyImpl.AwDataByShowId(context.Background(), &artworkDataReq)
	if err != nil {
		logger.Errorf("AwDataByShowId query err", err)
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	if len(resp.Data) < 1 {
		logger.Errorf("ExportShowPrice  query params  err :", "画作信息为空")
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   nil,
			Msg:    "画作信息为空",
			Status: e.Failed,
		})
		return
	}

	// 组装 画作 基础信息
	artworkData := make(map[string]*artwork_third_party.AwDataByShowIdResponse_Info, 0)
	for i := 0; i < len(resp.Data); i++ {
		artworkData[resp.Data[i].ArtworkUuid] = resp.Data[i]
	}

	// 文件的路径 压缩包的名字
	now := strconv.FormatInt(time.Now().Unix(), 10)
	dirPath := "./runtime/artshow/"
	utils.CheckDirPath(dirPath, true)

	zipPathName := now + ".zip"
	zipPath := fmt.Sprintf("./runtime/%s/", now+"_"+exportShowPriceReq.ExportType)
	utils.CheckDirPath(zipPath, true)

	// 获取 画作价格信息
	artworkPriceReq := artShow.ShowDetailReq{}

	var retPath string
	var msg string
	for i := 0; i < len(exportShowPriceReq.OneQueryShowInfos); i++ {
		// 获取 画作价格信息
		artworkPriceList := new(artShow.ArtworkPriceListRes)
		artworkPriceReq.ShowUID = append(artworkPriceReq.ShowUID, exportShowPriceReq.OneQueryShowInfos[i].ShowUID)
		artworkPriceList, err = GrpcArtShowImpl.ArtworkPriceList(context.Background(), &artworkPriceReq)
		if err != nil {
			logger.Errorf("ArtworkPriceList query err", err)
			ResponseMsg(c, e.SUCCESS, serializer.Response{
				Msg:    err.Error(),
				Status: e.Failed,
			})
			return
		}

		if len(artworkPriceList.Data) == 0 {
			ResponseMsg(c, e.SUCCESS, serializer.Response{
				Msg:    e.ErrorNotFoundArtwork,
				Status: e.Failed,
			})
			return
		}
		// 获取 画展包信息
		showDetailReq := new(artShow.ShowDetailReq)
		showDetailReq.ShowUID = append(showDetailReq.ShowUID, exportShowPriceReq.OneQueryShowInfos[i].ShowUID)
		artShowInfo, err := GrpcArtShowImpl.ShowDetail(context.Background(), showDetailReq)
		if err != nil {
			logger.Errorf("ShowDetail query err", err)
			ResponseMsg(c, e.SUCCESS, serializer.Response{
				Msg:    err.Error(),
				Status: e.Failed,
			})
			return
		}
		// 每个画展包创建一个文件夹
		newDirPath := dirPath + artShowInfo.Data[len(artShowInfo.Data)-1].ShowName
		utils.CheckDirPath(newDirPath, true)
		// 生成 表格
		path, err, m := logic.OneQueryExportShowPrice(artworkPriceList.Data, artworkData, artShowInfo.Data[len(artShowInfo.Data)-1], exportShowPriceReq.ExportType, newDirPath)
		if err != nil {
			logger.Errorf("OneQueryShowPrice query err", err)
			ResponseMsg(c, e.SUCCESS, serializer.Response{
				Msg:    e.ErrorSaveFile,
				Status: e.Failed,
			})
			return
		}

		if len(path) == 1 {
			retPath = path[0]
		}

		if m != "" {
			msg = m
		} else {
			msg = "导出成功"
		}
	}

	var httpType string
	if config.IsHttps {
		httpType = model.HttpsType
	} else {
		httpType = model.HttpType
	}

	if retPath != "" {
		retPath = strings.Replace(retPath, ".", fmt.Sprintf("%s%s", httpType, c.Request.Host), 1)
	}

	if len(exportShowPriceReq.OneQueryShowInfos) > 1 {
		if err = utils.ZipFile(dirPath, zipPath+zipPathName); err != nil {
			logger.Errorf("CompressByFiles  err", err)
			ResponseMsg(c, e.SUCCESS, serializer.Response{
				Msg:    err.Error(),
				Status: e.Failed,
			})
			return
		}
		if err = os.RemoveAll(dirPath); err != nil {
			logger.Errorf("RemoveAll   err", err)
		}
		retPath = strings.Join([]string{strings.Replace(zipPath, ".", fmt.Sprintf("%s%s", httpType, c.Request.Host), 1), zipPathName}, "")
	}

	var exportUrl = strings.Replace(retPath, "runtime", "static", 1)

	ResponseQuickMsg(c, e.Ok, msg, map[string]string{
		"ExportUrl": exportUrl,
	})

}

func ExportShowFieldOneQuery(c *gin.Context) {
	oneQueryShowListReq := new(artShow.OneQueryShowListReq)
	if err := c.ShouldBind(&oneQueryShowListReq); err != nil {
		logger.Errorf("oneQueryShowListReq ShouldBind err", err)
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	showListRes, err := GrpcArtShowImpl.OneQueryShowList(context.Background(), oneQueryShowListReq)
	if err != nil {
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	var fileName string
	if len(showListRes.Data) > 0 {
		columns := []string{"画展包名称", "画家姓名", "时间", "地点"}
		var data []interface{}
		for _, v := range showListRes.Data {
			var temp []string
			temp = append(temp, v.ShowName)
			temp = append(temp, v.ArtistName)
			temp = append(temp, v.ShowTime)
			temp = append(temp, v.Address)
			data = append(data, &temp)
		}
		fileName = fmt.Sprintf("画展包数据%v.xlsx", time.Now().Unix())
		_, err = utils.ToExcelByType(columns, data, "slice", "./runtime/"+fileName)
		if err != nil {
			ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		}
	}
	var httpType string
	if config.IsHttps {
		httpType = model.HttpsType
	} else {
		httpType = model.HttpType
	}
	var exportUrl string = fmt.Sprintf("%s%s/static/%s", httpType, c.Request.Host, fileName)
	ResponseQuickMsg(c, e.Ok, "导出成功", map[string]string{
		"url": exportUrl,
	})
	return
}

func ExportEntertainmentImage(c *gin.Context) {
	req := new(model.ExportEntertainment)
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("ExportEntertainment ShouldBind err", err)
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	// 文件的路径 压缩包的名字
	//now := strconv.FormatInt(time.Now().Unix(), 10)
	originFile, err := os.Open("./data/entertainmentImage.png")

	if err != nil {
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Data:   nil,
			Msg:    e.ErrorReadTemp,
			Status: e.Ok,
		})
		return
	}
	defer originFile.Close()

	dirPath := "./runtime/entertainment"
	utils.CheckDirPath(dirPath, true)

	if len(req.Data) == 0 {
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Data:   nil,
			Msg:    e.ErrorExport,
			Status: e.Ok,
		})
		return
	}

	originImg, err := png.Decode(originFile) // 解码
	if err != nil {
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Data:   nil,
			Msg:    e.ErrorEncodeImage,
			Status: e.Ok,
		})
		return
	}

	startTime := strings.Split(req.ShowDate, " - ")[0]
	var fileName string
	if len(strings.Split(startTime, " ")) > 1 {
		fileName = strings.Split(startTime, " ")[0]
	} else {
		fileName = startTime
	}
	path := dirPath + "/" + "画展日期-" + fileName + ".png"
	saveFile, err := os.Create(path)
	if err != nil {
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Data:   nil,
			Msg:    e.GetMsg(e.ErrorFileCreate),
			Status: e.Ok,
		})
		return
	}
	defer saveFile.Close()

	fontPath := "./data/font1716.ttf"

	var showDateSize float64 = 24
	outImage, _ := addLabel(originImg, req.ShowDate, 650, 125, color.RGBA{0, 0, 0, 255}, showDateSize, fontPath)
	var worldSize float64 = 14
	y := 177 + int(worldSize)
	for i := 0; i < len(req.Data); i++ {
		lineX := 30
		if i > 0 {
			if i > 4 {
				y = y + int(worldSize) + 18
			} else {
				y = y + int(worldSize) + 18 - i
			}
		}

		outImage, _ = addLabel(outImage, req.Data[i].ArtistName, lineX, y, color.RGBA{0, 0, 0, 255}, worldSize, fontPath)

		lineX += 80
		outImage, _ = addLabel(outImage, req.Data[i].Address, lineX, y, color.RGBA{0, 0, 0, 255}, worldSize, fontPath)

		lineX += 98
		outImage, _ = addLabel(outImage, req.Data[i].TelNum, lineX, y, color.RGBA{0, 0, 0, 255}, worldSize, fontPath)

		lineX += 120
		outImage, _ = addLabel(outImage, req.Data[i].Entourage, lineX, y, color.RGBA{0, 0, 0, 255}, worldSize, fontPath)

		lineX += 80
		outImage, _ = addLabel(outImage, req.Data[i].ArrivalDate, lineX, y, color.RGBA{0, 0, 0, 255}, worldSize, fontPath)

		lineX += 115
		outImage, _ = addLabel(outImage, req.Data[i].ArrivalTime, lineX, y, color.RGBA{0, 0, 0, 255}, worldSize, fontPath)

		var travelWay string
		if req.Data[i].TravelWay == 1 {
			travelWay = "自驾"
		} else if req.Data[i].TravelWay == 2 {
			travelWay = "公共交通"
		} else {
			travelWay = ""
		}
		lineX += 100
		outImage, _ = addLabel(outImage, travelWay, lineX, y, color.RGBA{0, 0, 0, 255}, worldSize, fontPath)

		lineX += 100
		outImage, _ = addLabel(outImage, req.Data[i].TrainNum, lineX, y, color.RGBA{0, 0, 0, 255}, worldSize, fontPath)

		lineX += 110
		outImage, _ = addLabel(outImage, req.Data[i].ArrivalAddress, lineX, y, color.RGBA{0, 0, 0, 255}, worldSize, fontPath)

		lineX += 132
		outImage, _ = addLabel(outImage, req.Data[i].Hotel, lineX, y, color.RGBA{0, 0, 0, 255}, worldSize, fontPath)

		lineX += 350
		outImage, _ = addLabel(outImage, req.Data[i].Receiver, lineX, y, color.RGBA{0, 0, 0, 255}, worldSize, fontPath)

		lineX += 210
		var agent string
		if strings.Count(req.Data[i].Agent, "丰链") > 0 {
			agent = "公司"
		} else if strings.Count(req.Data[i].Agent, "泰丰") > 0 {
			agent = "公司"
		} else {
			agent = req.Data[i].Agent
		}

		outImage, _ = addLabel(outImage, agent, lineX, y, color.RGBA{0, 0, 0, 255}, worldSize, fontPath)

		lineX += 160
		outImage, _ = addLabel(outImage, req.Data[i].ShowTime, lineX, y, color.RGBA{0, 0, 0, 255}, worldSize, fontPath)

	}

	err = png.Encode(saveFile, outImage)
	if err != nil {
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Data:   nil,
			Msg:    e.GetMsg(e.ErrorFileCreate),
			Status: e.Ok,
		})
		return
	}

	var httpType string
	if config.IsHttps {
		httpType = model.HttpsType
	} else {
		httpType = model.HttpType
	}

	var exportUrl = strings.Replace(strings.Replace(path, ".", fmt.Sprintf("%s%s", httpType, c.Request.Host), 1), "runtime", "static", 1)
	//var exportUrl = fmt.Sprintf("%s%s/%s", httpType, c.Request.Host, dirPath + path)

	ResponseQuickMsg(c, e.Ok, "导出成功", map[string]string{
		"ExportUrl": exportUrl,
	})

}

func addLabel(img image.Image, label string, x, y int, fontColor color.Color, size float64, fontPath string) (image.Image, error) {
	bound := img.Bounds()
	// 创建一个新的图片
	rgba := image.NewRGBA(image.Rect(0, 0, bound.Dx(), bound.Dy()))
	// 读取字体
	fontBytes, err := ioutil.ReadFile(fontPath)
	if err != nil {
		return rgba, err
	}
	myFont, err := freetype.ParseFont(fontBytes)
	if err != nil {
		return rgba, err
	}

	draw.Draw(rgba, rgba.Bounds(), img, bound.Min, draw.Src)
	c := freetype.NewContext()
	c.SetDPI(72)
	c.SetFont(myFont)
	c.SetFontSize(size)
	c.SetClip(rgba.Bounds())
	c.SetDst(rgba)
	uni := image.NewUniform(fontColor)
	c.SetSrc(uni)
	c.SetHinting(font.HintingNone)

	// 在指定的位置显示
	pt := freetype.Pt(x, y+int(c.PointToFixed(size)>>6))
	if _, err := c.DrawString(label, pt); err != nil {
		return rgba, err
	}

	return rgba, nil
}

func ExportByArtist(c *gin.Context) {
	var exportByArtistReq model.ExportShowPrice
	if err := c.ShouldBind(&exportByArtistReq); err != nil {
		logger.Errorf("ExportShowPrice ShouldBind err", err)
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if len(exportByArtistReq.ShowInfos) < 1 {
		logger.Errorf("ExportShowPrice  query params  err :", "画展包 uid 为空")
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   nil,
			Msg:    "画展包 uid 为空",
			Status: e.Failed,
		})
		return
	}

	//  组装 获取 画作 基础信息参数
	artworkDataReq := artwork_third_party.AwDataByShowIdRequest{}
	uids := make([]string, 0)
	for i := 0; i < len(exportByArtistReq.ShowInfos); i++ {
		uids = append(uids, exportByArtistReq.ShowInfos[i].UID)
	}
	artworkDataReq.ArtShowIds = uids
	//  获取 画作 基础信息
	resp, err := GrpcArtworkThirdPartyImpl.AwDataByShowId(context.Background(), &artworkDataReq)
	if err != nil {
		logger.Errorf("AwDataByShowId query err", err)
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	if len(resp.Data) < 1 {
		logger.Errorf("ExportShowPrice  query params  err :", "画作信息为空")
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   nil,
			Msg:    "画作信息为空",
			Status: e.Failed,
		})
		return
	}

	// 组装 画作 基础信息
	artworkData := make(map[string]*artwork_third_party.AwDataByShowIdResponse_Info, 0)
	for i := 0; i < len(resp.Data); i++ {
		artworkData[resp.Data[i].ArtworkUuid] = resp.Data[i]
	}

	// 文件的路径 压缩包的名字
	now := strconv.FormatInt(time.Now().Unix(), 10)
	dirPath := "./runtime/artshow/"
	utils.CheckDirPath(dirPath, true)

	zipPathName := now + ".zip"
	zipPath := fmt.Sprintf("./runtime/%s/", now+"_"+exportByArtistReq.ExportType)
	utils.CheckDirPath(zipPath, true)

	// 获取 画作价格信息

	artworkPriceReq := artShow.ShowDetailReq{}
	applyInfos := make([]*model.ApplyInfo, 0)

	var msg string
	// 获取 画作价格信息
	artworkPriceList := new(artShow.ArtworkPriceListRes)
	artworkPriceReq.ShowUID = uids
	artworkPriceList, err = GrpcArtShowImpl.ArtworkPriceList(context.Background(), &artworkPriceReq)
	if err != nil {
		logger.Errorf("ArtworkPriceList query err", err)
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	// 获取 画展包信息
	showDetailReq := new(artShow.ShowDetailReq)
	showDetailReq.ShowUID = uids
	artShowInfos, err := GrpcArtShowImpl.ShowDetail(context.Background(), showDetailReq)
	if err != nil {
		logger.Errorf("ShowDetail query err", err)
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	//用画家名创建文件夹
	newDirPath := dirPath + artShowInfos.Data[0].ArtistName
	utils.CheckDirPath(newDirPath, true)

	// 生成 表格
	_, applyInfo, err, m := logic.ExportByArtist(artworkPriceList.Data, artworkData, artShowInfos.Data, exportByArtistReq.ExportType, newDirPath)
	if err != nil {
		logger.Errorf("ArtworkPriceList query err", err)
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if m != "" {
		msg = m
	} else {
		msg = "导出成功"
	}
	// 获取 付款明细数据
	if applyInfo != nil {
		applyInfos = append(applyInfos, applyInfo)
	}

	// 生成 付款明细 表格
	if len(applyInfos) > 0 {
		_, err := logic.ExportShowPricePayInfo(applyInfos, dirPath)
		if err != nil {
			logger.Errorf("ExportShowPricePayInfo query err", err)
			ResponseMsg(c, e.SUCCESS, serializer.Response{
				Msg:    e.ErrorSaveFile,
				Status: e.Failed,
			})
			return
		}
		//filePaths = append(filePaths, path)
	}

	if err := utils.ZipFile(dirPath, zipPath+zipPathName); err != nil {
		logger.Errorf("CompressByFiles  err", err)
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    e.ErrorZip,
			Status: e.Failed,
		})
		return
	}

	if err = os.RemoveAll(dirPath); err != nil {
		logger.Errorf("RemoveAll   err", err)
	}

	var httpType string
	if config.IsHttps {
		httpType = model.HttpsType
	} else {
		httpType = model.HttpType
	}

	var exportUrl = strings.Replace(strings.Join([]string{strings.Replace(zipPath, ".", fmt.Sprintf("%s%s", httpType, c.Request.Host), 1), zipPathName}, ""), "runtime", "static", 1)

	ResponseQuickMsg(c, e.Ok, msg, map[string]string{
		"ExportUrl": exportUrl,
	})

}
