package service

import (
	"context"
	"dubbo.apache.org/dubbo-go/v3/common/logger"
	"fmt"
	artShow "github.com/fonchain_enterprise/fonchain-main/api/artShow"
	"github.com/fonchain_enterprise/fonchain-main/api/artwork_third_party"
	"github.com/fonchain_enterprise/fonchain-main/pkg/config"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/logic"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model"
	"github.com/fonchain_enterprise/fonchain-main/pkg/serializer"
	"github.com/fonchain_enterprise/fonchain-main/pkg/utils"
	"github.com/gin-gonic/gin"
	"os"
	"strconv"
	"strings"
	"time"
)

func ExportShowPrice_withoutCopyright(c *gin.Context) {
	var exportShowPriceReq model.ExportShowPrice
	if err := c.ShouldBind(&exportShowPriceReq); err != nil {
		logger.Errorf("ExportShowPrice ShouldBind err", err)
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if len(exportShowPriceReq.ShowInfos) < 1 {
		logger.Errorf("ExportShowPrice  query params  err :", "画展包 uid 为空")
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   nil,
			Msg:    "画展包 uid 为空",
			Status: e.Failed,
		})
		return
	}

	//  组装 获取 画作 基础信息参数
	artworkDataReq := artwork_third_party.AwDataByShowIdRequest{}
	uids := make([]string, 0)
	for i := 0; i < len(exportShowPriceReq.ShowInfos); i++ {
		uids = append(uids, exportShowPriceReq.ShowInfos[i].UID)
	}
	artworkDataReq.ArtShowIds = uids

	//  获取 画作 基础信息
	resp, err := GrpcArtworkThirdPartyImpl.AwDataByShowId(context.Background(), &artworkDataReq)
	if err != nil {
		logger.Errorf("AwDataByShowId query err", err)
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	if len(resp.Data) < 1 {
		logger.Errorf("ExportShowPrice  query params  err :", "画作信息为空")
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   nil,
			Msg:    "画作信息为空",
			Status: e.Failed,
		})
		return
	}

	// 组装 画作 基础信息
	artworkData := make(map[string]*artwork_third_party.AwDataByShowIdResponse_Info, 0)
	for i := 0; i < len(resp.Data); i++ {
		artworkData[resp.Data[i].ArtworkUuid] = resp.Data[i]
	}

	// 文件的路径 压缩包的名字
	now := strconv.FormatInt(time.Now().Unix(), 10)
	dirPath := "./runtime/artshow/"
	utils.CheckDirPath(dirPath, true)

	zipPathName := now + ".zip"
	zipPath := fmt.Sprintf("./runtime/%s/", now+"_"+exportShowPriceReq.ExportType)
	utils.CheckDirPath(zipPath, true)

	//// 获取 画作价格信息
	//artworkPriceReq := artShow.ShowDetailReq{}
	applyInfos := make([]*model.ApplyInfo, 0)
	//filePaths := make([]string, 0)
	var msg string
	for i := 0; i < len(exportShowPriceReq.ShowInfos); i++ {
		// 获取 画作价格信息
		artworkPriceList := new(artShow.ArtworkPriceListRes)
		// 获取 画作价格信息
		artworkPriceReq := artShow.ShowDetailReq{}
		artworkPriceReq.ShowUID = append(artworkPriceReq.ShowUID, exportShowPriceReq.ShowInfos[i].UID)
		artworkPriceList, err = GrpcArtShowImpl.ArtworkPriceList(context.Background(), &artworkPriceReq)
		if err != nil {
			logger.Errorf("ArtworkPriceList query err", err)
			ResponseMsg(c, e.SUCCESS, serializer.Response{
				Msg:    err.Error(),
				Status: e.Failed,
			})
			return
		}
		// 获取 画展包信息
		showDetailReq := new(artShow.ShowDetailReq)
		showDetailReq.ShowUID = append(showDetailReq.ShowUID, exportShowPriceReq.ShowInfos[i].UID)
		artShowInfo, err := GrpcArtShowImpl.ShowDetail(context.Background(), showDetailReq)
		if err != nil {
			logger.Errorf("ShowDetail query err", err)
			ResponseMsg(c, e.SUCCESS, serializer.Response{
				Msg:    err.Error(),
				Status: e.Failed,
			})
			return
		}
		// 每个画展包创建一个文件夹
		newDirPath := dirPath + artShowInfo.Data[len(artShowInfo.Data)-1].ShowName
		utils.CheckDirPath(newDirPath, true)
		// 生成 表格
		//paths, applyInfo, err := logic.ExportShowPrice(artworkPriceList.Data, artworkData, artShowInfo.Data, exportShowPriceReq.ExportType, newDirPath)
		_, applyInfo, err, m := logic.ExportShowPrice_withoutCopyright(artworkPriceList.Data, artworkData, artShowInfo.Data[len(artShowInfo.Data)-1], exportShowPriceReq.ExportType, newDirPath)
		if err != nil {
			logger.Errorf("ArtworkPriceList query err", err)
			ResponseMsg(c, e.SUCCESS, serializer.Response{
				Msg:    e.ErrorSaveFile,
				Status: e.Failed,
			})
			return
		}

		if m != "" {
			msg = m
		} else {
			msg = "导出成功"
		}
		//filePaths = append(filePaths, newDirPath+"/")
		// 获取 付款明细数据
		if applyInfo != nil {
			applyInfos = append(applyInfos, applyInfo)
		}
	}

	// 生成 付款明细 表格
	if len(applyInfos) > 0 {
		_, err := logic.ExportShowPricePayInfo_withoutCopyright(applyInfos, dirPath)
		if err != nil {
			logger.Errorf("ExportShowPricePayInfo query err", err)
			ResponseMsg(c, e.SUCCESS, serializer.Response{
				Msg:    e.ErrorSaveFile,
				Status: e.Failed,
			})
			return
		}
		//filePaths = append(filePaths, path)
	}

	if err := utils.ZipFile(dirPath, zipPath+zipPathName); err != nil {
		logger.Errorf("CompressByFiles  err", err)
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    e.ErrorZip,
			Status: e.Failed,
		})
		return
	}

	if err = os.RemoveAll(dirPath); err != nil {
		logger.Errorf("RemoveAll   err", err)
	}

	var httpType string
	if config.IsHttps {
		httpType = model.HttpsType
	} else {
		httpType = model.HttpType
	}

	var exportUrl = strings.Replace(strings.Join([]string{strings.Replace(zipPath, ".", fmt.Sprintf("%s%s", httpType, c.Request.Host), 1), zipPathName}, ""), "runtime", "static", 1)

	ResponseQuickMsg(c, e.Ok, msg, map[string]string{
		"ExportUrl": exportUrl,
	})

}

func ExportShowPriceOneQuery_withoutCopyright(c *gin.Context) {
	var exportShowPriceReq model.ExportOneQueryShowPrice
	if err := c.ShouldBind(&exportShowPriceReq); err != nil {
		logger.Errorf("ExportShowPrice ShouldBind err", err)
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if exportShowPriceReq.ExportType != "cp" && exportShowPriceReq.ExportType != "aw" {
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   nil,
			Msg:    e.ErrorExportNotAllowed,
			Status: e.Failed,
		})
		return
	}

	if len(exportShowPriceReq.OneQueryShowInfos) < 1 {
		logger.Errorf("ExportShowPrice  query params  err :", "画展包 uid 为空")
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   nil,
			Msg:    "画展包 uid 为空",
			Status: e.Failed,
		})
		return
	}

	//  组装 获取 画作 基础信息参数
	artworkDataReq := artwork_third_party.AwDataByShowIdRequest{}
	uids := make([]string, 0)
	for i := 0; i < len(exportShowPriceReq.OneQueryShowInfos); i++ {
		uids = append(uids, exportShowPriceReq.OneQueryShowInfos[i].ShowUID)
	}
	artworkDataReq.ArtShowIds = uids

	//  获取 画作 基础信息
	resp, err := GrpcArtworkThirdPartyImpl.AwDataByShowId(context.Background(), &artworkDataReq)
	if err != nil {
		logger.Errorf("AwDataByShowId query err", err)
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	if len(resp.Data) < 1 {
		logger.Errorf("ExportShowPrice  query params  err :", "画作信息为空")
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   nil,
			Msg:    "画作信息为空",
			Status: e.Failed,
		})
		return
	}

	// 组装 画作 基础信息
	artworkData := make(map[string]*artwork_third_party.AwDataByShowIdResponse_Info, 0)
	for i := 0; i < len(resp.Data); i++ {
		artworkData[resp.Data[i].ArtworkUuid] = resp.Data[i]
	}

	// 文件的路径 压缩包的名字
	now := strconv.FormatInt(time.Now().Unix(), 10)
	dirPath := "./runtime/artshow/"
	utils.CheckDirPath(dirPath, true)

	zipPathName := now + ".zip"
	zipPath := fmt.Sprintf("./runtime/%s/", now+"_"+exportShowPriceReq.ExportType)
	utils.CheckDirPath(zipPath, true)

	// 获取 画作价格信息
	artworkPriceReq := artShow.ShowDetailReq{}

	var retPath string
	var msg string
	for i := 0; i < len(exportShowPriceReq.OneQueryShowInfos); i++ {
		// 获取 画作价格信息
		artworkPriceList := new(artShow.ArtworkPriceListRes)
		artworkPriceReq.ShowUID = append(artworkPriceReq.ShowUID, exportShowPriceReq.OneQueryShowInfos[i].ShowUID)
		artworkPriceList, err = GrpcArtShowImpl.ArtworkPriceList(context.Background(), &artworkPriceReq)
		if err != nil {
			logger.Errorf("ArtworkPriceList query err", err)
			ResponseMsg(c, e.SUCCESS, serializer.Response{
				Msg:    err.Error(),
				Status: e.Failed,
			})
			return
		}

		if len(artworkPriceList.Data) == 0 {
			ResponseMsg(c, e.SUCCESS, serializer.Response{
				Msg:    e.ErrorNotFoundArtwork,
				Status: e.Failed,
			})
			return
		}
		// 获取 画展包信息
		showDetailReq := new(artShow.ShowDetailReq)
		showDetailReq.ShowUID = append(showDetailReq.ShowUID, exportShowPriceReq.OneQueryShowInfos[i].ShowUID)
		artShowInfo, err := GrpcArtShowImpl.ShowDetail(context.Background(), showDetailReq)
		if err != nil {
			logger.Errorf("ShowDetail query err", err)
			ResponseMsg(c, e.SUCCESS, serializer.Response{
				Msg:    err.Error(),
				Status: e.Failed,
			})
			return
		}
		// 每个画展包创建一个文件夹
		newDirPath := dirPath + artShowInfo.Data[len(artShowInfo.Data)-1].ShowName
		utils.CheckDirPath(newDirPath, true)
		// 生成 表格
		path, err, m := logic.OneQueryExportShowPrice_withoutCopyright(artworkPriceList.Data, artworkData, artShowInfo.Data[len(artShowInfo.Data)-1], exportShowPriceReq.ExportType, newDirPath)
		if err != nil {
			logger.Errorf("OneQueryShowPrice query err", err)
			ResponseMsg(c, e.SUCCESS, serializer.Response{
				Msg:    e.ErrorSaveFile,
				Status: e.Failed,
			})
			return
		}

		if len(path) == 1 {
			retPath = path[0]
		}

		if m != "" {
			msg = m
		} else {
			msg = "导出成功"
		}
	}

	var httpType string
	if config.IsHttps {
		httpType = model.HttpsType
	} else {
		httpType = model.HttpType
	}

	if retPath != "" {
		retPath = strings.Replace(retPath, ".", fmt.Sprintf("%s%s", httpType, c.Request.Host), 1)
	}

	if len(exportShowPriceReq.OneQueryShowInfos) > 1 {
		if err = utils.ZipFile(dirPath, zipPath+zipPathName); err != nil {
			logger.Errorf("CompressByFiles  err", err)
			ResponseMsg(c, e.SUCCESS, serializer.Response{
				Msg:    err.Error(),
				Status: e.Failed,
			})
			return
		}
		if err = os.RemoveAll(dirPath); err != nil {
			logger.Errorf("RemoveAll   err", err)
		}
		retPath = strings.Join([]string{strings.Replace(zipPath, ".", fmt.Sprintf("%s%s", httpType, c.Request.Host), 1), zipPathName}, "")
	}

	var exportUrl = strings.Replace(retPath, "runtime", "static", 1)

	ResponseQuickMsg(c, e.Ok, msg, map[string]string{
		"ExportUrl": exportUrl,
	})

}

func ExportByArtist_withoutCopyright(c *gin.Context) {
	var exportByArtistReq model.ExportShowPrice
	if err := c.ShouldBind(&exportByArtistReq); err != nil {
		logger.Errorf("ExportShowPrice ShouldBind err", err)
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if len(exportByArtistReq.ShowInfos) < 1 {
		logger.Errorf("ExportShowPrice  query params  err :", "画展包 uid 为空")
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   nil,
			Msg:    "画展包 uid 为空",
			Status: e.Failed,
		})
		return
	}

	//  组装 获取 画作 基础信息参数
	artworkDataReq := artwork_third_party.AwDataByShowIdRequest{}
	uids := make([]string, 0)
	for i := 0; i < len(exportByArtistReq.ShowInfos); i++ {
		uids = append(uids, exportByArtistReq.ShowInfos[i].UID)
	}
	artworkDataReq.ArtShowIds = uids
	//  获取 画作 基础信息
	resp, err := GrpcArtworkThirdPartyImpl.AwDataByShowId(context.Background(), &artworkDataReq)
	if err != nil {
		logger.Errorf("AwDataByShowId query err", err)
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	if len(resp.Data) < 1 {
		logger.Errorf("ExportShowPrice  query params  err :", "画作信息为空")
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   nil,
			Msg:    "画作信息为空",
			Status: e.Failed,
		})
		return
	}

	// 组装 画作 基础信息
	artworkData := make(map[string]*artwork_third_party.AwDataByShowIdResponse_Info, 0)
	for i := 0; i < len(resp.Data); i++ {
		artworkData[resp.Data[i].ArtworkUuid] = resp.Data[i]
	}

	// 文件的路径 压缩包的名字
	now := strconv.FormatInt(time.Now().Unix(), 10)
	dirPath := "./runtime/artshow/"
	utils.CheckDirPath(dirPath, true)

	zipPathName := now + ".zip"
	zipPath := fmt.Sprintf("./runtime/%s/", now+"_"+exportByArtistReq.ExportType)
	utils.CheckDirPath(zipPath, true)

	// 获取 画作价格信息

	artworkPriceReq := artShow.ShowDetailReq{}
	applyInfos := make([]*model.ApplyInfo, 0)

	var msg string
	// 获取 画作价格信息
	artworkPriceList := new(artShow.ArtworkPriceListRes)
	artworkPriceReq.ShowUID = uids
	artworkPriceList, err = GrpcArtShowImpl.ArtworkPriceList(context.Background(), &artworkPriceReq)
	if err != nil {
		logger.Errorf("ArtworkPriceList query err", err)
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	// 获取 画展包信息
	showDetailReq := new(artShow.ShowDetailReq)
	showDetailReq.ShowUID = uids
	artShowInfos, err := GrpcArtShowImpl.ShowDetail(context.Background(), showDetailReq)
	if err != nil {
		logger.Errorf("ShowDetail query err", err)
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	//用画家名创建文件夹
	newDirPath := dirPath + artShowInfos.Data[0].ArtistName
	utils.CheckDirPath(newDirPath, true)

	// 生成 表格
	_, applyInfo, err, m := logic.ExportByArtist_withoutCopyright(artworkPriceList.Data, artworkData, artShowInfos.Data, exportByArtistReq.ExportType, newDirPath)
	if err != nil {
		logger.Errorf("ArtworkPriceList query err", err)
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if m != "" {
		msg = m
	} else {
		msg = "导出成功"
	}
	// 获取 付款明细数据
	if applyInfo != nil {
		applyInfos = append(applyInfos, applyInfo)
	}

	// 生成 付款明细 表格
	if len(applyInfos) > 0 {
		_, err := logic.ExportShowPricePayInfo_withoutCopyright(applyInfos, dirPath)
		if err != nil {
			logger.Errorf("ExportShowPricePayInfo query err", err)
			ResponseMsg(c, e.SUCCESS, serializer.Response{
				Msg:    e.ErrorSaveFile,
				Status: e.Failed,
			})
			return
		}
		//filePaths = append(filePaths, path)
	}

	if err := utils.ZipFile(dirPath, zipPath+zipPathName); err != nil {
		logger.Errorf("CompressByFiles  err", err)
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    e.ErrorZip,
			Status: e.Failed,
		})
		return
	}

	if err = os.RemoveAll(dirPath); err != nil {
		logger.Errorf("RemoveAll   err", err)
	}

	var httpType string
	if config.IsHttps {
		httpType = model.HttpsType
	} else {
		httpType = model.HttpType
	}

	var exportUrl = strings.Replace(strings.Join([]string{strings.Replace(zipPath, ".", fmt.Sprintf("%s%s", httpType, c.Request.Host), 1), zipPathName}, ""), "runtime", "static", 1)

	ResponseQuickMsg(c, e.Ok, msg, map[string]string{
		"ExportUrl": exportUrl,
	})

}
