package artist

import (
	"encoding/json"
	"fmt"
	"github.com/fonchain_enterprise/fonchain-main/api/artist"
	"github.com/fonchain_enterprise/fonchain-main/pkg/cache"
	projectConf "github.com/fonchain_enterprise/fonchain-main/pkg/config"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model"
	pkdRequest "github.com/fonchain_enterprise/fonchain-main/pkg/model/request"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/response"
	service "github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/gin-gonic/gin"
	"github.com/google/uuid"
	"github.com/jinzhu/copier"
	"time"
)

func EditLimit(ctx *gin.Context) {
	var listReq pkdRequest.EditLimitReq
	if err := ctx.ShouldBind(&listReq); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	if listReq.TagName == "" {
		service.ResponseQuickMsg(ctx, e.Failed, e.GetMsg(e.InvalidParams), nil)
		return
	}
	var userInfo model.LoginInfo
	if mLoginInfoAny, exists := ctx.Get("mLoginInfo"); exists {
		userInfo = mLoginInfoAny.(model.LoginInfo)
	}
	if projectConf.Env == "dev" {
		userInfo.ID = 1
		userInfo.NickName = "admin"
	}
	var limitInfoResponse response.EditLimitInfo
	_ = copier.Copy(&limitInfoResponse, &userInfo)
	limitInfoResponse.AdminID = userInfo.ID
	limitInfoResponse.Uuid = uuid.New().String()
	limitInfoResponse.TagName = listReq.TagName
	limitInfoByte, _ := json.Marshal(limitInfoResponse)
	cacheKey := fmt.Sprintf("editLimit_%s_%s", listReq.TagName, listReq.ArtistUuid)
	reply := cache.RedisClient.SetNX(cacheKey, string(limitInfoByte), time.Minute*10)
	if !reply.Val() {
		limitVal := cache.RedisClient.Get(cacheKey)
		_ = json.Unmarshal([]byte(limitVal.Val()), &limitInfoResponse)
		if limitInfoResponse.AdminID != userInfo.ID {
			service.ResponseQuickMsg(ctx, e.Failed, e.ERROR_EDIT_LIMIT, limitInfoResponse)
			return
		}
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), limitInfoResponse)
	return
}

func UpdateContract(ctx *gin.Context) {
	var req artist.UpdateArtistContractReq
	if err := ctx.ShouldBind(&req); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	resp, err := service.GrpcArtistImpl.UpdateArtistContract(ctx, &req)
	if err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.Success), resp)
	return
}
