package artist

import (
	"github.com/fonchain_enterprise/fonchain-main/api/artist"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/utils/stime"
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
	"time"
)

func BlackList(ctx *gin.Context) {
	req := artist.BlackListReq{}
	if err := ctx.ShouldBind(&req); err != nil {
		zap.L().Error("BlackList ShouldBind err", zap.Error(err))
		service.ResponseQuickMsg(ctx, e.Failed, "", nil)
		return
	}
	resp, _err := service.GrpcArtistImpl.BlackList(ctx, &req)
	if _err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, _err.Error(), nil)
		return
	}
	for _, v := range resp.Data {
		if v.BlackListStatus == 2 {
			listTimes, _ := stime.DatetimeToTimes(v.BlackListTime, stime.Format_Normal_YMDhms)
			if listTimes > 0 && (int64(listTimes)+86400*365) < time.Now().Unix() {
				v.BlackListStatus = 100
			}
		}
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.Success), resp)
	return
}

func BlackListUpdate(ctx *gin.Context) {
	req := artist.BlackListUpdateReq{}
	if err := ctx.ShouldBind(&req); err != nil {
		zap.L().Error("BlackListUpdate ShouldBind err", zap.Error(err))
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	resp, _err := service.GrpcArtistImpl.BlackListUpdate(ctx, &req)
	if _err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, _err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.Success), resp)
	return
}
