// Package area -----------------------------
// @file      : handle.go
// @author    : JJXu
// @contact   : wavingbear@163.com
// @time      : 2024/7/22 下午1:17
// -------------------------------------------
package area

import (
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/gin-gonic/gin"
)

func GetAreaTreeHandler(c *gin.Context) {
	t, err := GetChinaAreaTree()
	if err != nil {
		service.Error(c, e.Failed, err)
		return
	}
	service.Success(c, &t)
}

type GetBankListRequest struct {
	Province  string `json:"province"`
	City      string `json:"city"`
	BankName  string `json:"bankName"`
	LBankName string `json:"lBankName"`
}

func GetBankListHandler(c *gin.Context) {
	var request GetBankListRequest
	if err := c.ShouldBindJSON(&request); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}
	if request.Province == "" {
		service.Error(c, e.InvalidParams, nil, "省份不能为空")
		return
	}
	if request.City == "" {
		service.Error(c, e.InvalidParams, nil, "城市不能为空")
		return
	}
	if request.BankName == "" {
		service.Error(c, e.InvalidParams, nil, "银行总行名称不能为空")
		return
	}
	if len([]rune(request.BankName)) > 10 {
		service.Error(c, e.InvalidParams, nil, "开户行请填写总行名称")
		return
	}
	data, err := new(BankService).QueryBanInfoList(request.Province, request.City, request.BankName, request.LBankName)
	if err != nil {
		service.Error(c, e.Failed, err)
		return
	}
	service.ResponseList(c, data, service.OptionPage(1, len(data), len(data)))
}

func GetMainBankListHandler(c *gin.Context) {
	data := new(BankService).QueryMainBanInfoList()
	service.ResponseList(c, data)
}
