package asArtshow

import (
	"dubbo.apache.org/dubbo-go/v3/common/logger"
	"github.com/fonchain_enterprise/fonchain-main/api/artistInfoUser"
	"github.com/fonchain_enterprise/fonchain-main/api/artistinfoArtshow"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/artistInfo/asPush"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/artistInfo/asUser"
	"github.com/gin-gonic/gin"
)

var BrokerSupplement = new(brokerSupplement)

type brokerSupplement struct{}

func (b brokerSupplement) GetItemList(c *gin.Context) {
	var req = artistinfoArtshow.GetBrokerItemListRequest{}
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, err.Error())
		return
	}

	datas, err := service.GrpcArtistInfoArtshowImpl.GetBrokerItemList(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GetItemList: GrpcArtistInfoArtshowImpl.GetBrokerItemList Error ", err.Error())
		return
	} else {
		service.ResponseList(c, datas.List, service.OptionPage(datas.Page.Page, datas.Page.PageSize, datas.Page.Total), service.OptionMsg("查询成功"))
	}
}

func (b brokerSupplement) CreateBrokerSupplementRecord(c *gin.Context) {
	var req = artistinfoArtshow.GenerateBrokerRecordRequest{}
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, err.Error())
		return
	}

	data, err := service.GrpcArtistInfoArtshowImpl.GenerateBrokerRecord(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GetItemList: GrpcArtistInfoArtshowImpl.GenerateBrokerRecord Error ", err.Error())
		return
	} else {
		service.ResponseData(c, data)
	}
}

func (b brokerSupplement) UpdateBrokerSupplementRecord(c *gin.Context) {
	var req = artistinfoArtshow.UpdateBrokerRecordRequest{}
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, err.Error())
		return
	}
	defer func() {
		artshowDetail, err := service.GrpcArtistInfoArtshowImpl.GetBrokerDetail(c, &artistinfoArtshow.GetBrokerDetailRequest{Id: req.Id})
		if err == nil {
			userInfo, errs := service.GrpcArtistInfoUserImpl.FindUser(c, &artistInfoUser.FindUserRequest{MgmtArtistUid: artshowDetail.ArtistUid})
			if errs == nil {
				//if errs = asUser.ArtistInfoPushNotice.ArtistArtshowSupplementAuditNotice(userInfo.Id, asUser.AuditType_BrokerSupplement); errs != nil {
				//	logger.Error("UpdateBrokerSupplementRecord,发送app通知失败:" + errs.Error())
				//}
				pusher := asPush.NewPusher()
				if errs = pusher.ArtistArtshowSupplementAuditNotice(userInfo.Id, asUser.AuditType_BrokerSupplement); errs != nil {
					logger.Error("UpdateBrokerSupplementRecord,发送app通知失败:" + errs.Error())
				}
			}
		}
	}()
	data, err := service.GrpcArtistInfoArtshowImpl.UpdateBrokerRecord(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GetItemList: GrpcArtistInfoArtshowImpl.GenerateBrokerRecord Error ", err.Error())
		return
	} else {
		service.ResponseData(c, data)
	}
}

func (b brokerSupplement) GetSupplementDetail(c *gin.Context) {
	var req = artistinfoArtshow.GetBrokerDetailRequest{}
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, err.Error())
		return
	}
	if req.ArtistUid == "" && req.Id == 0 {
		userInfo, err, code := asUser.GetUserInfoWithToken(c)
		if err != nil {
			service.Error(c, code, err, err.Error())
			return
		}
		req.ArtistUid = userInfo.MgmtArtistUid
	}
	data, err := service.GrpcArtistInfoArtshowImpl.GetBrokerDetail(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GetSupplementDetail: .GrpcArtistInfoArtshowImpl.GetBrokerDetail Error ", err.Error())
		return
	} else {
		service.ResponseData(c, data)
	}
}

func (b brokerSupplement) BindAndGetArtPackageAndBrokerSupplement(c *gin.Context) {
	var req = artistinfoArtshow.BindAndGetArtPackageRequest{}
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, err.Error())
		return
	}
	data, err := service.GrpcArtistInfoArtshowImpl.BindAndGetArtPackageAndBrokerSupplement(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "BindAndGetArtPackageAndBrokerSupplement: GrpcArtistInfoArtshowImpl.BindAndGetArtPackageA... Error ", err.Error())
		return
	} else {
		service.ResponseData(c, data.Data)
	}
}

func (b brokerSupplement) GetSupplementList(c *gin.Context) {
	var req = artistinfoArtshow.GetBrokerSupplementListRequest{}
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, err.Error())
		return
	}
	data, err := service.GrpcArtistInfoArtshowImpl.GetBrokerSupplementList(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GetSupplementList: GrpcArtistInfoArtshowImpl.GetBrokerSupplementList Error ", err.Error())
		return
	} else {
		service.ResponseList(c, data.List, service.OptionPage(data.Page, data.Page, data.Total), service.OptionMsg("查询成功"))
	}
}
