// Package artshowVideo -----------------------------
// @file      : dto.go
// @author    : JJXu
// @contact   : wavingbear@163.com
// @time      : 2023/3/2 9:12
// -------------------------------------------
package asArtshow

import (
	"context"
	"github.com/fonchain_enterprise/fonchain-main/api/artistInfoUser"
	"github.com/fonchain_enterprise/fonchain-main/api/artistinfoArtshow"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/artistInfo/asArtwork"
)

type RequestGenerateArtshowRecord struct {
	ArtistUids []string `json:"artistUids"`
}

// if err != nil {
// service.ErrorWithMark(c, e.Failed, err, "GenerateArtshowVideoRecord: GrpcArtistInfoUserImpl.FindUsers Error", "创建失败")
// }
func (r *RequestGenerateArtshowRecord) ConvertGrpcRequest() (requestData []*artistinfoArtshow.ArtshowVideoInfo, err error) {
	userInfos, err := service.GrpcArtistInfoUserImpl.FindUsers(context.Background(), &artistInfoUser.FindUsersRequest{
		MgmtArtistUids: r.ArtistUids,
	})
	if err != nil {
		return nil, err
	}
	//service.GrpcArtistInfoArtworkImpl.GetArtworkLockRecords(c)
	for _, v := range userInfos.Data {
		realName := ""
		if v.RealName != nil {
			realName = v.RealName.Name
		}
		// 注释掉:默认用户已锁定
		// 判断用户锁定状态
		//if !v.IsLock{
		//	return errors.New("")
		//}

		requestData = append(requestData, &artistinfoArtshow.ArtshowVideoInfo{
			ArtistUid:   v.MgmtArtistUid,
			LockTime:    v.LatestLockTime,
			ArtistName:  realName,
			AuditStatus: int64(asArtwork.AuditType_Supplemented),
			Status:      2, //锁定
		})
	}
	return
}

type RequestGetArtshowList struct {
	ArtistName string `json:"artistName"`
	Page       int    `json:"page"`
	PageSize   int    `json:"pageSize"`
}

type ResponseArtshowInfo struct {
	ArtistUid   string `json:"artistUid"`
	LockTime    string `json:"lockTime"`
	AuditStatus int32  `json:"auditStatus"`
	ArtistName  string `json:"artistName"`
	VideoUrl    string `json:"videoUrl"`
	CreatedAt   int64  `json:"createdAt"`
}

type ResponseArtshowList struct {
	Data []ResponseArtshowList
}

type ArtshowVideoInfo struct {
	Id          int64  `json:"id"`
	ArtistUid   string `json:"artistUid"`
	LockTime    string `json:"lockTime"`
	VideoUrl    string `json:"videoUrl"`
	AuditStatus int64  `json:"auditStatus"`
	AuditMark1  string `json:"auditMark1"`
	AuditMark2  string `json:"auditMark2"`
	CreatedAt   string `json:"createdAt"`
	UpdatedAt   string `json:"updatedAt"`
	DeletedAt   int64  `json:"deletedAt"`
	ArtistName  string `json:"artistName"`
	Status      int64  `json:"status"` //锁定状态
}

type RequestGetArtshowVideoListAtUserSide struct {
	Page      int64  `json:"page"`
	PageSize  int64  `json:"pageSize"`
	ArtistUid string `json:"artistUid"`
}

type ResponseVideoDetail struct {
	Id          int64  `json:"id"`
	ArtistUid   string `json:"artistUid"`
	LockTime    string `json:"lockTime"`
	VideoUrl    string `json:"videoUrl"`
	AuditStatus int64  `json:"auditStatus"`
	AuditMark1  string `json:"auditMark1"`
	AuditMark2  string `json:"auditMark2"`
	CreatedAt   string `json:"createdAt"`
	UpdatedAt   string `json:"updatedAt"`
	DeletedAt   int64  `json:"deletedAt"`
	ArtistName  string `json:"artistName"`
	Status      int64  `json:"status"` //锁定状态
	Editable    bool   `json:"editable"`
}

func (e *ResponseVideoDetail) ConvertResponse(data *artistinfoArtshow.ArtshowVideoInfo) {
	if data == nil {
		return
	}
	e.Id = data.Id
	e.ArtistUid = data.ArtistUid
	e.LockTime = data.LockTime
	e.VideoUrl = data.VideoUrl
	e.AuditStatus = data.AuditStatus
	e.AuditMark1 = data.AuditMark1
	e.AuditMark2 = data.AuditMark2
	e.CreatedAt = data.CreatedAt //time.Unix(data.CreatedAt, 0).Format(stime.Format_Normal_YMDhms)
	e.UpdatedAt = data.UpdatedAt //time.Unix(data.UpdatedAt, 0).Format(stime.Format_Normal_YMDhms)
	e.DeletedAt = data.DeletedAt
	e.ArtistName = data.ArtistName
	e.Status = data.Status
	e.Editable = data.Editable
}

type TimeGroup struct {
	Time                  string                   `json:"time"`
	ArtworkSupplementList []*ArtworkSupplementData `json:"artworkSupplementList"`
	ArtistSupplementList  []*ArtistSupplementData  `json:"artistSupplementList"`
	ArtistIndexList       []*ArtistIndexData       `json:"artistIndexList"`
	ArtistVideoList       []*ArtistVideoData       `json:"artistVideoList"`
}
type ArtworkSupplementData struct {
	ArtistUid      string `json:"artistUid"`
	Status         int64  `json:"status"`
	LockTime       string `json:"lockTime"`
	AuditStatus    int64  `json:"auditStatus"`
	AuditMark1     string `json:"auditMark1"`
	AuditMark2     string `json:"auditMark2"`
	ArtworkName    string `json:"artworkName"`
	CreatedDate    string `json:"createdDate"`
	CreatedAddress string `json:"createdAddress"`
	CreatedAt      string `json:"createdAt"`
	UpdatedAt      string `json:"updatedAt"`
	DeletedAt      int64  `json:"deletedAt"`
	ArtworkUuid    string `json:"ArtworkUuid"`
	Length         int32  `json:"Length"`
	Width          int32  `json:"Width"`
	Ruler          int32  `json:"Ruler"`
	HdPic          string `json:"HdPic"`
	ArtistPhoto    string `json:"ArtistPhoto"`
}

type ArtistSupplementData struct {
	ArtistUid       string `json:"artistUid"`
	Status          int64  `json:"status"`
	LockTime        string `json:"lockTime"`
	AuditStatus     int64  `json:"auditStatus"`
	AuditMark1      string `json:"auditMark1"`
	AuditMark2      string `json:"auditMark2"`
	ArtistName      string `json:"artistName"`
	ArtistProfile   string `json:"artistProfile"`
	CountryArtLevel int64  `json:"countryArtLevel"`
	ArtistCertPic   string `json:"artistCertPic"`
	BankNum         string `json:"bankNum"`
	BankName        string `json:"bankName"`
	Id              int64  `json:"id"`
	CreatedAt       string `json:"createdAt"`
	UpdatedAt       string `json:"updatedAt"`
	DeletedAt       string `json:"deletedAt"`
}

type ArtistIndexData struct {
	ArtistUid   string `json:"artistUid"`
	Title       string `json:"title"`
	Class       string `json:"class"`
	TitleScore  int64  `json:"titleScore"`
	Score       string `json:"score"`
	Types       string `json:"types"`
	Status      int64  `json:"status"`
	LockTime    string `json:"lockTime"`
	AuditMark1  string `json:"auditMark1"`
	AuditMark2  string `json:"auditMark2"`
	AuditStatus int64  `json:"auditStatus"`
	Id          int64  `json:"id"`
	CreatedAt   string `json:"createdAt"`
	UpdatedAt   string `json:"updatedAt"`
	DeletedAt   string `json:"deletedAt"`
}

type ArtistVideoData struct {
	Id          int64  `json:"id"`
	ArtistUid   string `json:"artistUid"`
	LockTime    string `json:"lockTime"`
	VideoUrl    string `json:"videoUrl"`
	AuditStatus int64  `json:"auditStatus"`
	AuditMark1  string `json:"auditMark1"`
	AuditMark2  string `json:"auditMark2"`
	ArtistName  string `json:"artistName"`
}

type AuditArtshowVideoRequest struct {
	ArtshowVideoIds []int64 `json:"artshowVideoIds"`
	AuditStatus     int64   `json:"auditStatus"`
	AuditMark1      string  `json:"auditMark"`
	AuditMark2      string  `json:"auditMark2"`
}

type GetViewArtistIndexStaticListRequest struct {
	Page     int64 `json:"page"`
	PageSize int64 `json:"pageSize"`
	artistinfoArtshow.ViewArtistIndexStaticData
}
