// Package asArtshow -----------------------------
// @file      : dto_ArtistSupplement.go
// @author    : JJXu
// @contact   : wavingbear@163.com
// @time      : 2023/3/3 8:28
// -------------------------------------------
package asArtshow

import (
	"github.com/fonchain_enterprise/fonchain-main/api/artistInfoUser"
	"github.com/fonchain_enterprise/fonchain-main/api/artistinfoArtshow"
)

type ArtistSupplementInfo struct {
	Id        int64  `gorm:"primarykey;"  json:"id" form:"id"`
	CreatedAt string `gorm:"column:created_at" json:"createdAt"`
	UpdatedAt string `gorm:"column:updated_at" json:"updatedAt"`
	DeletedAt int64  `gorm:"column:deleted_at;type:bigint" json:"deletedAt"`
	//通过这两个字段弱关联 artwork_lock_record表中对应的画作
	ArtistUid string `json:"artistUid" gorm:"column:artist_uid;comment:"`
	Status    int64  `json:"status" gorm:"column:status;default:2;comment:2=锁定 3=解锁"` //跟随用户的锁定和解锁状态,用于控制数据的展示
	LockTime  string `json:"lockTime" gorm:"column:lock_time;comment:"`

	//审批字段
	AuditStatus int64  `json:"auditStatus" gorm:"column:audit_status;comment:审核状态：2= 待审核,3= 审核失败,4= 审核通过,5= 待补充"`
	AuditMark1  string `json:"auditMark1" gorm:"column:audit_mark1;comment:审核备注1"`
	AuditMark2  string `json:"auditMark2" gorm:"column:audit_mark2;comment:审核备注2"`

	ArtistName      string `json:"artistName" gorm:"column:artist_name;comment:"`
	ArtistProfile   string `json:"artistProfile" gorm:"column:artist_profile;comment:个人简介"`
	CountryArtLevel int64  `json:"countryArtLevel" gorm:"column:country_art_level;comment:国家美术师级别: 1=无  2=1级 3=2级"`
	ArtistCertPic   string `json:"artistCertPic" gorm:"column:artist_cert_pic;comment:国家美术师证书"`
	BankNum         string `json:"bankNum" gorm:"column:bank_num;comment:开户行"`
	BankName        string `json:"bankName" gorm:"column:bank_name;comment:银行卡账号"`
	Tnum            string `json:"tnum" gorm:"column:tnum;comment:"`
	Honor           string `json:"honor"`
}

type RequestGetArtistSupplementListAtUserSide struct {
	Page      int64  `json:"page"`
	PageSize  int64  `json:"pageSize"`
	ArtistUid string `json:"artistUid"`
}

type RequestGenerateArtistSupplementRecord struct {
	ArtistUids []string `json:"artistUids"`
}

type SupplementArtistUserInfo struct {
	ArtistUid     string `json:"artistUid"`
	ArtistName    string `json:"artistName"`
	DataCreatedAt string `json:"dataCreatedAt"`
	DataUpdatedAt string `json:"dataUpdateAt"`
}

type ResponseGetLockedSupplementArtistList struct {
	Data []SupplementArtistUserInfo
}

type ArtistSupplementDetail struct {
	Id              int64  `json:"id"`
	ArtistUid       string `json:"artistUid"`
	Status          int64  `json:"status"`
	LockTime        string `json:"lockTime"`
	AuditStatus     int64  `json:"auditStatus"`
	AuditMark1      string `json:"auditMark1"`
	AuditMark2      string `json:"auditMark2"`
	ArtistName      string `json:"artistName"`
	ArtistProfile   string `json:"artistProfile"`
	CountryArtLevel int64  `json:"countryArtLevel"`
	ArtistCertPic   string `json:"artistCertPic"`
	BankNum         string `json:"bankNum"`
	BankName        string `json:"bankName"`
	CreatedAt       string `json:"createdAt"`
	UpdatedAt       string `json:"updatedAt"`
	DeletedAt       int64  `json:"deletedAt"`
	Editable        bool   `json:"editable"` //是否用户可编辑
}

func (e *ArtistSupplementDetail) ConvertResponse(data *artistinfoArtshow.ArtistSupplementInfo) {
	e.Id = data.Id
	e.ArtistUid = data.ArtistUid
	e.Status = data.Status
	e.LockTime = data.LockTime
	e.AuditStatus = data.AuditStatus
	e.AuditMark1 = data.AuditMark1
	e.AuditMark2 = data.AuditMark2
	e.ArtistName = data.ArtistName
	e.ArtistProfile = data.ArtistProfile
	e.CountryArtLevel = data.CountryArtLevel
	e.ArtistCertPic = data.ArtistCertPic
	e.BankNum = data.BankNum
	e.BankName = data.BankName
	e.CreatedAt = data.CreatedAt
	e.UpdatedAt = data.UpdatedAt
	e.DeletedAt = data.DeletedAt
	e.Editable = data.Editable
}

type GetArtistSupplementListRequest struct {
	artistinfoArtshow.GetArtistSupplementListRequest
	History bool `json:"history"` //是否查询历史记录
}

type UserRelationArtistSupplement struct {
	UserInfo    *artistInfoUser.UserInfo
	ArtistIndex *artistinfoArtshow.ArtistIndexInfo
}

type ArtshowArtistSupplementInfoV3 struct {
	Id            int64  `gorm:"primarykey;"  json:"id" form:"id"`
	CreatedAt     string `gorm:"column:created_at" json:"createdAt"`
	UpdatedAt     string `gorm:"column:updated_at" json:"updatedAt"`
	DeletedAt     int64  `gorm:"column:deleted_at;type:bigint" json:"deletedAt"`
	ArtistUid     string `json:"artistUid" gorm:"column:artist_uid;comment:"`
	ArtistProfile string `json:"artistProfile" gorm:"column:artist_profile;comment:个人简介"`
	ArtistCertPic string `json:"artistCertPic" gorm:"column:artist_cert_pic;comment:国家美术师证书"`
	BankNum       string `json:"bankNum" gorm:"column:bank_num;comment:开户行"`
	BankName      string `json:"bankName" gorm:"column:bank_name;comment:银行卡账号"`
	LBankName     string `json:"lBankName"`
	HonorType
	DataUpdatedAt string       `json:"dataUpdatedAt"`
	EditStatus    int64        `json:"editStatus"`    // 编辑状态
	ExhibitionTip string       `json:"exhibitionTip"` // 画展经历提示词
	BankInfo      BankInfoType `json:"bankInfo"`      // 银行卡信息
}

func (a *ArtshowArtistSupplementInfoV3) IsEmpty() bool {
	if a.ArtistCertPic == "" && a.BankNum == "" && a.BankName == "" && len(a.HonorType.Title) == 0 && len(a.HonorType.Award) == 0 && len(a.HonorType.Exhibition) == 0 && len(a.HonorType.Published) == 0 {
		return true
	}
	return false
}

type PushArtistSupplementDataV3 struct {
	ArtistUid     string `json:"artistUid"`
	ArtistProfile string `json:"artistProfile"`
	//ArtistCertPic string `json:"artistCertPic"`
	BankNum     string `json:"bankNum"`
	BankName    string `json:"bankName"`
	ArtistHonor HonorType
}

type ImportArtistSupplementRequest struct {
	ArtistUid string `json:"artistUid"`
}
