package asArtwork

import (
	"strings"
	"time"

	"github.com/fonchain_enterprise/fonchain-main/api/artistInfoArtwork"
	"github.com/fonchain_enterprise/fonchain-main/api/artwork"
	"github.com/fonchain_enterprise/fonchain-main/api/artwork_query"
	"github.com/fonchain_enterprise/fonchain-main/pkg/utils/stime"
)

type AuditStatus int64

const (
	AuditType_preSave      AuditStatus = 1 //1= 暂存
	AuditType_Pending      AuditStatus = 2 //2= 待审核
	AuditType_Failed       AuditStatus = 3 //3= 审核失败
	AuditType_Pass         AuditStatus = 4 //4= 审核通过
	AuditType_Supplemented AuditStatus = 5 //5= 待补充
)

var auditStatusMaper = map[AuditStatus]string{
	AuditType_preSave:      "暂存",
	AuditType_Pending:      "待审核",
	AuditType_Failed:       "审核失败",
	AuditType_Pass:         "审核通过",
	AuditType_Supplemented: "待补充",
}

func (a AuditStatus) String() string {
	if a == 0 {
		return "无"
	}
	str, ok := auditStatusMaper[a]
	if !ok {
		return "未知"
	}
	return str
}

type AddArtWorkRequest struct {
	ArtistUuid     string   `json:"artistUid"`      //画家uuid
	ArtworkName    string   `json:"artworkName"`    //画作名字
	ArtworkUuid    string   `json:"artworkUid"`     //画作uuid
	ArtCondition   int32    `json:"artCondition"`   //画作品相 1 完好 2 破损 3 残缺
	Mountmode      int32    `json:"mountmode"`      //装裱方式 1 画心
	ArtHorizontal  int32    `json:"artHorizontal"`  //幅式 请求 catlist 接口
	Size           int32    `json:"size"`           //尺寸 1 大 2中  3小
	Length         int32    `json:"length"`         //长度
	Width          int32    `json:"width"`          //宽度
	Ruler          int32    `json:"ruler"`          //平尺
	InscribeDate   string   `json:"inscribeDate"`   //落款时间
	CreatedDate    string   `json:"createdDate"`    //创作日期
	CreatedAddress []string `json:"createdAddress"` //创作地点
	Abstract       string   `json:"abstract"`       //简介
	Belong         int32    `json:"belong"`         //归属状态  1 泰丰 2丰链
	FlowState      int32    `json:"flowState"`      //流程状态 1 丰链 2 托裱 3 泰丰
	ArtQuality     int32    `json:"artQuality"`     //画作状态 1 完好   2 破损  3 残缺
	ArtworkType    int32    `json:"artworkType"`    //画作类型  1-优秀画作，2-赠画，3-卷轴，4-普通画作
	IncompletePic  []string `json:"incompletePic"`  ///画作状态图片
	Signpic        string   `json:"signpic"`        //落款图
	Sealpic        string   `json:"sealpic"`        //人名章图
	ArtistPhoto    string   `json:"artistPhoto"`    //画家画作合照
	PhotoPic       string   `json:"photoPic"`       //手机拍摄
	HdPic          string   `json:"hdPic"`          //高清图
	Material       int32    `json:"material"`       //画作材质  1 宣纸  2 绢  3 水彩纸
	ArtistName     string   `json:"artistName"`     //画家名字
	CreateSource   int32    `json:"createSource"`   //1 后台 2 画家宝
	PriceRun       float32  `json:"priceRun"`       //润格
	Signdate       string   `json:"signdate"`       //签约时间
	PriceRuler     float32  `json:"priceRuler"`     //平尺单价
	PriceCopyright float32  `json:"priceCopyright"` //版权单价
	PriceArtwork   float32  `json:"price_artwork"`  //画作单价
	PriceMarket    float32  `json:"price_market"`   //市场单价
	TreasureName   string   `json:"treasureName"`   //画家宝画作名
	FilterState    int32    `json:"filter_state"`   //筛选状态1 通过 2 不通过
	StorageStatus  int32    `json:"storageStatus"`  //库存状态 0=未入库  1=仓库  2 =数字化中 3 =鉴证中  4=出库
	SaleStatus     int32    `json:"saleStatus"`     //销售状态 0=不在仓库  1=在仓库  2=数字化中  3=鉴证中  4=数字化完  5=鉴证完成  6=待销售
}

func (a AddArtWorkRequest) ProtoRequest() *artwork.CreArtProRequest {
	return &artwork.CreArtProRequest{
		ArtistUuid:     a.ArtistUuid,
		ArtistName:     a.ArtistName,
		ArtworkName:    a.ArtworkName,
		ArtCondition:   a.ArtCondition,
		Mountmode:      a.Mountmode,
		ArtHorizontal:  a.ArtHorizontal,
		Size:           a.Size,
		Length:         a.Length,
		Width:          a.Width,
		Ruler:          a.Ruler,
		InscribeDate:   a.InscribeDate,
		CreatedDate:    a.CreatedDate,
		CreatedAddress: strings.Join(a.CreatedAddress, "，"),
		Abstract:       a.Abstract,
		PriceRuler:     a.PriceRuler,
		PriceCopyright: a.PriceCopyright,
		PriceArtwork:   a.PriceArtwork,
		PriceMarket:    a.PriceMarket,
		Belong:         a.Belong,
		FlowState:      a.FlowState,
		ArtQuality:     a.ArtQuality,
		IncompletePic:  a.IncompletePic,
		Signpic:        a.Signpic,
		Sealpic:        a.Sealpic,
		ArtistPhoto:    a.ArtistPhoto,
		PhotoPic:       a.PhotoPic,
		HdPic:          a.HdPic,
		Material:       a.Material,
		ArtworkUuid:    a.ArtworkUuid,
		ArtworkType:    a.ArtworkType,
		CreateSource:   a.CreateSource,
		TreasureName:   a.TreasureName,
		FilterState:    a.FilterState,
		PriceRun:       a.PriceRun,
		StorageStatus:  a.StorageStatus,
		SaleStatus:     a.SaleStatus,
		Signdate:       a.Signdate,
	}
}

// type ArtworkListRequest struct {
// 	Keyword       string `json:"keyword" form:"keyword"`
// 	Page          int32  `json:"page" form:"page"`
// 	PageSize      int32  `json:"pageSize" form:"pageSize"`
// 	StorageStatus int32  `json:"storageStatus" form:"storageStatus"`
// 	IsOver        int32  `json:"isOver" form:"isOver"`
// 	AdminId       int32  `json:"adminId" form:"adminId"`
// 	ArtistUid     string `json:"artistUid" form:"artistUid"`
// 	InArtShow     int32  `json:"inArtShow" form:"inArtShow"` // 是否在画家宝
// }

// func (a ArtworkListRequest) GrRpcequest() *artwork_query.ArtworkListRequest {
// 	var storageStatus = wrapperspb.IntpValue{
// 		Value: a.StorageStatus,
// 	}
// 	return &artwork_query.ArtworkListRequest{
// 		Keyword:       a.Keyword,
// 		Page:          a.Page,
// 		PageSize:      a.PageSize,
// 		StorageStatus: &storageStatus,
// 		IsOver:        a.IsOver,
// 		AdminId:       a.AdminId,
// 		ArtistUid:     a.ArtistUid,
// 		InArtShow:     a.InArtShow,
// 	}
// }

type DelAwRequest struct {
	ArtworkUuid string `json:"artworkUid"`
}

func (d DelAwRequest) ProtoRequest() *artwork_query.DelAwRequest {
	return &artwork_query.DelAwRequest{
		ArtworkUuid: d.ArtworkUuid,
	}
}

type ArtworkDetailRequest struct {
	Type        string `json:"type"`
	ArtworkUuid string `json:"artworkUid"`
}

func (d ArtworkDetailRequest) ProtoRequest() *artwork.ArtworkDetailRequest {
	return &artwork.ArtworkDetailRequest{
		Type:        d.Type,
		ArtworkUuid: d.ArtworkUuid,
	}
}

type ArtworkListResponse struct {
	// Id              int     `json:"id"`
	// ArtworkUuid     string  `json:"artworkUid"`
	// ArtistName      string  `json:"artistName"`
	// ArtworkName     string  `json:"artworkName"`
	// Length          int     `json:"length"`
	// Width           int     `json:"width"`
	// Ruler           int     `json:"ruler"`
	// Num             string  `json:"num"`
	// HdPic           string  `json:"hdPic"`
	// InStorageTime   string  `json:"inStorageTime"`
	// WtState         int     `json:"wtState"`
	// Changchainstate int     `json:"changchainstate"`
	// BaiduState      int     `json:"baiduState"`
	// PhotoPic        string  `json:"photoPic"`
	// PriceRun        float64 `json:"priceRun"`
	CreatedAt string `json:"createdAt"`
	UpdatedAt string `json:"updatedAt"`
	// DeletedAt       int64   `json:"deletedAt"`
	ArtistUid      string `json:"artistUid"`
	ArtworkName    string `json:"artworkName"`
	Length         int32  `json:"length"`
	Width          int32  `json:"width"`
	Ruler          int32  `json:"ruler"`
	CreatedAddress string `json:"createdAddress"`
	ArtistPhoto    string `json:"artistPhoto"`
	HdPic          string `json:"hdPic"`
	ArtworkUid     string `json:"artworkUid"`
	CreateDate     string `json:"createDate"`
	InscribeDate   string `json:"inscribeDate"`
	FolwIndexAtNow string `json:"folwIndexAtNow"` //当前画作审批流位置
	AuditStatusStr string `json:"auditStatusStr"` //画作补充信息审批状态
	FirstState     int32  `json:"firstState"`     //2、6合同    1：未生成合同 2：已生成合同
	SecondState    int32  `json:"secondState"`    //3合同    1：未生成合同 2：已生成合同
	ThirdState     int32  `json:"thirdState"`     //5合同    1：未生成合同 2：已生成合同
	Tfnum          string `json:"tfnum"`
}
type ArtworkDetailResponse struct {
	ArtworkUuid string `json:"artworkUid"`
	ProfileInfo struct {
		ArtworkName    string   `json:"artworkName"`
		ArtistName     string   `json:"artistName"`
		ArtCondition   int32    `json:"artCondition"`
		Mountmode      int32    `json:"mountmode"`
		ArtHorizontal  int32    `json:"artHorizontal"`
		Size           int32    `json:"size"`
		Length         int32    `json:"length"`
		Width          int32    `json:"width"`
		Ruler          int32    `json:"ruler"`
		InscribeDate   string   `json:"inscribeDate"`
		CreatedDate    string   `json:"createdDate"`
		CreatedAddress []string `json:"createdAddress"`
		Abstract       string   `json:"abstract"`
		Belong         int32    `json:"belong"`
		FlowState      int32    `json:"flowState"`
		ArtQuality     int32    `json:"artQuality"`
		IncompletePic  []string `json:"incompletePic"`
		Signpic        string   `json:"signpic"`
		Sealpic        string   `json:"sealpic"`
		ArtistPhoto    string   `json:"artistPhoto"`
		PhotoPic       string   `json:"photoPic"`
		HdPic          string   `json:"hdPic"`
		Material       int32    `json:"material"`
		ArtworkUuid    string   `json:"artworkUid"`
		ArtistUuid     string   `json:"artistUuid"`
		ArtworkType    int32    `json:"artworkType"`
		CreateSource   int32    `json:"createSource"`
		FilterState    int32    `json:"filterState"`
		PriceRun       float32  `json:"priceRun"`
		Signdate       string   `json:"signdate"`
		//AuditStatus    int64    `json:"auditStatus"`
		//AuditMark      string   `json:"auditMark"`
		//AuditMark2     string   `json:"auditMark2"`
		CreatedAt int64 `json:"createdAt"`
		UpdatedAt int64 `json:"updatedAt"`
		DeletedAt int64 `json:"deletedAt"`

		BaseAuditStatus       int32  `json:"baseAuditStatus"`
		BaseAuditMark         string `json:"baseAuditMark"`
		BaseAuditMark2        string `json:"baseAuditMark2"`
		SupplementAuditStatus int32  `json:"supplementAuditStatus"`
		SupplementAuditMark   string `json:"supplementAuditMark"`
		SupplementAuditMark2  string `json:"supplementAuditMark2"`
		AuditFlowIndex        int32  `json:"auditFlowIndex"`
	} `json:"profileInfo"`
}

func (a *ArtworkDetailResponse) Convert(data *artwork.ArtworkDetailResponse, lockData *artistInfoArtwork.ArtworkLockInfo) {
	a.ProfileInfo.ArtworkName = data.ProfileInfo.ArtworkName
	a.ProfileInfo.ArtistName = data.ProfileInfo.ArtistName
	a.ProfileInfo.ArtCondition = data.ProfileInfo.ArtCondition
	a.ProfileInfo.Mountmode = data.ProfileInfo.Mountmode
	a.ProfileInfo.ArtHorizontal = data.ProfileInfo.ArtHorizontal
	a.ProfileInfo.Size = data.ProfileInfo.Size
	a.ProfileInfo.Length = data.ProfileInfo.Length
	a.ProfileInfo.Width = data.ProfileInfo.Width
	a.ProfileInfo.Ruler = data.ProfileInfo.Ruler
	a.ProfileInfo.InscribeDate = data.ProfileInfo.InscribeDate
	a.ProfileInfo.CreatedDate = data.ProfileInfo.CreatedDate
	a.ProfileInfo.CreatedAddress = strings.Split(data.ProfileInfo.CreatedAddress, "，")
	a.ProfileInfo.Abstract = data.ProfileInfo.Abstract
	a.ProfileInfo.Belong = data.ProfileInfo.Belong
	a.ProfileInfo.FlowState = data.ProfileInfo.FlowState
	a.ProfileInfo.ArtQuality = data.ProfileInfo.ArtQuality
	a.ProfileInfo.IncompletePic = data.ProfileInfo.IncompletePic
	a.ProfileInfo.Signpic = data.ProfileInfo.Signpic
	a.ProfileInfo.Sealpic = data.ProfileInfo.Sealpic
	a.ProfileInfo.ArtistPhoto = data.ProfileInfo.ArtistPhoto
	a.ProfileInfo.PhotoPic = data.ProfileInfo.PhotoPic
	a.ProfileInfo.HdPic = data.ProfileInfo.HdPic
	a.ProfileInfo.Material = data.ProfileInfo.Material
	a.ProfileInfo.ArtworkUuid = data.ProfileInfo.ArtworkUuid
	a.ProfileInfo.ArtistUuid = data.ProfileInfo.ArtistUuid
	a.ProfileInfo.ArtworkType = data.ProfileInfo.ArtworkType
	a.ProfileInfo.CreateSource = data.ProfileInfo.CreateSource
	a.ProfileInfo.FilterState = data.ProfileInfo.FilterState
	a.ProfileInfo.PriceRun = data.ProfileInfo.PriceRun
	a.ProfileInfo.Signdate = data.ProfileInfo.Signdate
	//a.ProfileInfo.AuditStatus = lockData.AuditStatus
	//a.ProfileInfo.AuditMark = lockData.AuditMark
	//a.ProfileInfo.AuditMark2 = lockData.AuditMark2
	a.ProfileInfo.CreatedAt = lockData.CreatedAt
	a.ProfileInfo.UpdatedAt = lockData.UpdatedAt
	a.ProfileInfo.DeletedAt = lockData.DeletedAt
	a.ProfileInfo.BaseAuditStatus = int32(lockData.BaseAuditStatus)
	a.ProfileInfo.BaseAuditMark = lockData.BaseAuditMark
	a.ProfileInfo.BaseAuditMark2 = lockData.BaseAuditMark2
	a.ProfileInfo.SupplementAuditStatus = lockData.SupplementAuditStatus
	a.ProfileInfo.SupplementAuditMark = lockData.SupplementAuditMark
	a.ProfileInfo.SupplementAuditMark2 = lockData.SupplementAuditMark2
	a.ProfileInfo.AuditFlowIndex = lockData.AuditFlowIndex
}

//type ArtworkPreviewResponse struct {
//	ID            int         `json:"ID"`
//	CreatedAt     time.Time   `json:"CreatedAt"`
//	UpdatedAt     time.Time   `json:"UpdatedAt"`
//	DeletedAt     interface{} `json:"DeletedAt"`
//	Id            int         `json:"id"`
//	ArtistId      int         `json:"artistId"`
//	Name          string      `json:"name"`
//	ArtworkId     string      `json:"artworkId"`
//	ModelYear     string      `json:"modelYear"`
//	Photo         string      `json:"photo"`
//	BatchId       int         `json:"batchId"`
//	ArtistPhoto   string      `json:"artistPhoto"`
//	CreateAddress string      `json:"createAddress"`
//	Width         int         `json:"width"`
//	Height        int         `json:"height"`
//	Ruler         int         `json:"ruler"`
//	AgeOfCreation string      `json:"ageOfCreation"`
//	CreateTime    string      `json:"createTime"`
//	Introduct     string      `json:"introduct"`
//	NetworkTrace  bool        `json:"networkTrace"`
//	FlowState     int         `json:"FlowState"`
//	Url           string      `json:"url"`
//	Remark        string      `json:"remark"`
//	Remark2       string      `json:"remark2"`
//	State         int         `json:"state"`
//}

type ArtworkPreviewResponse struct {
	ArtistUuid     string   `json:"artistUid"`
	ArtworkName    string   `json:"artworkName"`
	Length         int32    `json:"length"`
	Width          int32    `json:"width"`
	Ruler          int32    `json:"ruler"`
	CreatedAddress []string `json:"createdAddress"`
	ArtistPhoto    string   `json:"artistPhoto"`
	HdPic          string   `json:"hdPic"`
	ArtworkUid     string   `json:"artworkUid"`
	CreatedDate    string   `json:"createdDate"`
	LockStatus     int64    `json:"lockStatus"`
	//AuditStatus    int64    `json:"auditStatus"`
	//AuditMark      string   `json:"auditMark"`
	//AuditMark2     string   `json:"auditMark2"`
	CreatedAt             string `json:"createdAt"`
	UpdatedAt             string `json:"updatedAt"`
	DeletedAt             int64  `json:"deletedAt"`
	BaseAuditStatus       int32  `json:"baseAuditStatus"`
	BaseAuditMark         string `json:"baseAuditMark"`
	BaseAuditMark2        string `json:"baseAuditMark2"`
	SupplementAuditStatus int32  `json:"supplementAuditStatus"`
	SupplementAuditMark   string `json:"supplementAuditMark"`
	SupplementAuditMark2  string `json:"supplementAuditMark2"`
	AuditFlowIndex        int32  `json:"auditFlowIndex"`
	Insource              int32  `json:"insource"`
	NationalExhibition    int64  `json:"nationalExhibition"` //是否入选国家画展 1=无(默认) 2=入围 3=入选
	TfNum                 string `json:"tfNum"`
	ArtworkType           int32  `json:"artworkType"`
	ArtistName            string `json:"artistName"`
	PenName               string `json:"penName"`
	//操作记录相关
	BaseInfoArtistUpdateAt string `json:"baseInfoArtistUpdateAt"`

	BaseInfoLatestOperator        int32  `json:"baseInfoLatestOperator"`
	BaseInfoLatestOperationTime   string `json:"baseInfoLatestOperationTime"`
	SupplyInfoArtistUpdateAt      string `json:"supplyInfoArtistUpdateAt"`
	SupplyInfoLatestOperator      int32  `json:"supplyInfoLatestOperator"`
	SupplyInfoLatestOperationTime string `json:"supplyInfoLatestOperationTime"`
	Id                            int32  `json:"id"`
	NationalExhibitionProof       string `json:"nationalExhibitionProof"`
}

type ArtworkPreviewExport struct {
	ArtistUuid  string `json:"artistUid"`
	ArtworkName string `json:"artworkName"`
	Length      int32  `json:"length"`
	Width       int32  `json:"width"`
	Ruler       int32  `json:"ruler"`
	ArtistPhoto string `json:"artistPhoto"`
	HdPic       string `json:"hdPic"`
	ArtworkUid  string `json:"artworkUid"`
	CreatedDate string `json:"createdDate"`
	LockStatus  int64  `json:"lockStatus"`
	//AuditStatus    int64    `json:"auditStatus"`
	//AuditMark      string   `json:"auditMark"`
	//AuditMark2     string   `json:"auditMark2"`
	CreatedAt             string `json:"createdAt"`
	UpdatedAt             string `json:"updatedAt"`
	DeletedAt             int64  `json:"deletedAt"`
	BaseAuditStatus       int32  `json:"baseAuditStatus"`
	BaseAuditMark         string `json:"baseAuditMark"`
	BaseAuditMark2        string `json:"baseAuditMark2"`
	SupplementAuditStatus int32  `json:"supplementAuditStatus"`
	SupplementAuditMark   string `json:"supplementAuditMark"`
	SupplementAuditMark2  string `json:"supplementAuditMark2"`
	AuditFlowIndex        int32  `json:"auditFlowIndex"`
	Insource              int32  `json:"insource"`
	NationalExhibition    int64  `json:"nationalExhibition"` //是否入选国家画展 1=无(默认) 2=入围 3=入选
	TfNum                 string `json:"tfNum"`
	ArtworkType           int32  `json:"artworkType"`
}

type ArtworkPreviewListResponse struct {
	Data []*ArtworkPreviewResponse
}

//func (a *ArtworkPreviewResponse) ConvertData() []*artistInfoArtwork.ArtworkPreviewInfo {
//
//}

type GetArtworkHistoryListRequest struct {
}

//	type GetArtworkHistoryResponse struct {
//		LockTime string                   `json:"lockTime"`
//		List     []ArtworkPreviewResponse `json:"list"`
//	}
//
// type GetArtworkHistoryListResponse []GetArtworkHistoryResponse
//
//	func (g *GetArtworkHistoryResponse) Clean(records []*ArtworkPreviewResponse) {
//		if len(records) == 0 {
//			return
//		}
//		var group = make(map[string][]ArtworkPreviewResponse, 0)
//		for _, record := range records {
//			if record == nil {
//				continue
//			}
//			group[record.LockTime] = append(group[record.LockTime], ArtworkPreviewResponse{
//				ArtistUuid:     record.ArtistUid,
//				ArtworkName:    record.ArtworkName,
//				Length:         record.Length,
//				Width:          record.Width,
//				Ruler:          record.Ruler,
//				CreatedAddress: record.CreatedAddress,
//				ArtistPhoto:    record.ArtistPhoto,
//				HdPic:          record.HdPic,
//				ArtworkUid:     record.ArtworkUid,
//				CreatedDate:    record.CreatedDate,
//			})
//		}
//		if len(group) == 0 {
//			return
//		}
//		for lockTime, datas := range group {
//			g = append(g, GetArtworkHistoryResponse{
//				LockTime: "",
//				List:     datas,
//			})
//		}
//	}
type ArtworkLockRecord struct {
	LockGroup string                    `json:"lockGroup,omitempty"`
	DataList  []*ArtworkPreviewResponse `json:"dataList,omitempty"`
}
type ArtworkLockRecordList struct {
	Data []*ArtworkLockRecord
}

func (a *ArtworkLockRecordList) ConvertData(records []*artistInfoArtwork.ArtworkLockRecord) {
	for _, v := range records {
		var dataList = []*ArtworkPreviewResponse{}
		for _, dl := range v.DataList {
			dataList = append(dataList, &ArtworkPreviewResponse{
				ArtistUuid:     dl.ArtistUuid,
				ArtworkName:    dl.ArtworkName,
				Length:         dl.Length,
				Width:          dl.Width,
				Ruler:          dl.Ruler,
				CreatedAddress: dl.CreatedAddress,
				ArtistPhoto:    dl.ArtistPhoto,
				HdPic:          dl.HdPic,
				ArtworkUid:     dl.ArtworkUid,
				CreatedDate:    dl.CreatedDate,
				LockStatus:     int64(dl.LockStatus),
				//AuditStatus:    dl.AuditStatus,
				//AuditMark:      dl.AuditMark,
				//AuditMark2:     dl.AuditMark2,
				CreatedAt:             time.Unix(dl.CreatedAt, 0).Format(stime.Format_Normal_YMDhms),
				UpdatedAt:             time.Unix(dl.UpdatedAt, 0).Format(stime.Format_Normal_YMDhms),
				DeletedAt:             dl.DeletedAt,
				BaseAuditStatus:       int32(dl.BaseAuditStatus),
				BaseAuditMark:         dl.BaseAuditMark,
				BaseAuditMark2:        dl.BaseAuditMark2,
				SupplementAuditStatus: dl.SupplementAuditStatus,
				SupplementAuditMark:   dl.SupplementAuditMark,
				SupplementAuditMark2:  dl.SupplementAuditMark2,
				AuditFlowIndex:        dl.AuditFlowIndex,
			})
		}
		a.Data = append(a.Data, &ArtworkLockRecord{
			LockGroup: v.LockGroup,
			DataList:  dataList,
		})
	}
}

type ArtworkSupplementPreviewList struct {
	ArtistUuid  string `json:"artistUuid"`
	ArtworkName string `json:"artworkName"`
	// 补充信息审批状态
	SupplementAuditStatus int32  `protobuf:"varint,15,opt,name=supplementAuditStatus,proto3" json:"supplementAuditStatus,omitempty"`
	SupplementAuditMark   string `protobuf:"bytes,16,opt,name=supplementAuditMark,proto3" json:"supplementAuditMark,omitempty"`
	SupplementAuditMark2  string `protobuf:"bytes,17,opt,name=supplementAuditMark2,proto3" json:"supplementAuditMark2,omitempty"`
	// 当前审批流位置
	AuditFlowIndex int32 `protobuf:"varint,18,opt,name=auditFlowIndex,proto3" json:"auditFlowIndex,omitempty"`
}

type ImportArtowrkRequest struct {
	ArtworkUid  string   `json:"artworkUid"`
	ArtworkUids []string `json:"artworkUids"`
}

type GetArtworkPreviewListReq struct {
	artwork_query.ArtworkPreviewListRequest
	History               bool   `json:"history"`
	BaseAuditStatus       int64  `json:"baseAuditStatus"`       //画作基本信息审核状态 1:暂存 2:待审核 3：审核不通过 4：审核通过
	SupplementAuditStatus int32  `json:"supplementAuditStatus"` //画作补充信息审核状态 1:暂存 2:待审核 3：审核不通过 4：审核通过
	ArtworkName           string `json:"artworkName"`
	//单数据排序
	SortField string `json:"sortField"` //排序字段
	SortOrder string `json:"sortOrder"` //排序方式
	LockTime  string `json:"lockTime"`  // 画家最新锁定时间

	ArtistName string `json:"artistName"` //为数据导出功能提供画家名字模糊查询
	Tfnum      string `json:"tfnum"`      //画作编号
	//区间
	StartBaseInfoArtistUpdateAt string `json:"startBaseInfoArtistUpdateAt"` //更新时间起始
	EndBaseInfoArtistUpdateAt   string `json:"endBaseInfoArtistUpdateAt"`   //更新时间结束
	QueryHaveProof              int    `json:"queryHaveProof"`              //拓展查询 查询有未完成审批画作的画家  0不查询 1查询
	QueryUnfinishedAudit        int    `json:"queryUnfinishedAudit"`        //拓展查询  查询有入选凭证的画家 0不查询 1查询
	NationalExhibition          int    `json:"nationalExhibition"`          //国展入选情况
	RulerStart                  *int   `json:"rulerStart"`                  //平尺数区间起始值
	RulerEnd                    *int   `json:"rulerEnd"`                    //平尺数区间结束值
}
type ArtworkViewDetailListType struct {
	artistInfoArtwork.ArtworkViewDetail
	CommentData string `json:"commentData"`
}
type GetArtworkPreviewDetailReq struct {
	ArtworkUid string `json:"artworkUid"`
}

type GetArtworkPreviewExportReq struct {
	artwork_query.ArtworkPreviewListRequest
	History               bool   `json:"history"`
	BaseAuditStatus       int32  `json:"baseAuditStatus"`       //画作基本信息审核状态 1:暂存 2:待审核 3：审核不通过 4：审核通过
	SupplementAuditStatus int32  `json:"supplementAuditStatus"` //画作补充信息审核状态 1:暂存 2:待审核 3：审核不通过 4：审核通过
	ArtworkName           string `json:"artworkName"`
	//单数据排序
	SortField                   string   `json:"sortField"`    //排序字段
	SortOrder                   string   `json:"sortOrder"`    //排序方式
	ArtistName                  string   `json:"artistName"`   //画家名字
	PenName                     string   `json:"penName"`      //笔名
	ColumnId                    string   `json:"columnId"`     //字段集合
	ArtworkUuids                []string `json:"artworkUuids"` //画作ID
	ExportType                  int32    `json:"exportType"`   //操作类型
	StartBaseInfoArtistUpdateAt string   `json:"startBaseInfoArtistUpdateAt"`
	EndBaseInfoArtistUpdateAt   string   `json:"endBaseInfoArtistUpdateAt"`
}

type ExportData struct {
	ExportUrl string `json:"exportUrl"`
}
type GetArtworkListRequest struct {
	Keyword    string `json:"keyword"`
	Page       int64  `json:"page"`
	PageSize   int64  `json:"pageSize"`
	ArtistUid  string `json:"artistUid"`
	SearchMode int    `json:"searchMode"` //0=所有画作，1=锁定状态画作 2=解锁的画作（历史数据）
}
