package asUser

import (
	"github.com/fonchain_enterprise/fonchain-main/api/artistInfoUser"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/artistInfo/asUtil"
	"github.com/gin-gonic/gin"
	"github.com/jinzhu/copier"
)

var Handler = &ArtistInfoUserHandler{}

type ArtistInfoUserHandler struct {
}

// 创建ExportField
func (a *ArtistInfoUserHandler) CreateExportField(c *gin.Context) {
	var req artistInfoUser.ExportFieldData
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	_, err := service.GrpcArtistInfoUserImpl.CreateExportField(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GrpcArtistInfoUserImpl.CreateExportField Err", "创建失败")
		return
	}
	service.Success(c)
}

// 删除ExportField
func (a *ArtistInfoUserHandler) DeleteExportField(c *gin.Context) {
	var req artistInfoUser.DeleteExportFieldRequest
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	_, err := service.GrpcArtistInfoUserImpl.DeleteExportField(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "DeleteExportField Err", "删除失败")
		return
	}
	service.Success(c)
}

// 更新ExportField
func (a *ArtistInfoUserHandler) UpdateExportField(c *gin.Context) {
	var req artistInfoUser.ExportFieldData
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	_, err := service.GrpcArtistInfoUserImpl.UpdateExportField(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GrpcArtistInfoUserImpl.UpdateExportField Err", "更新失败")
		return
	}
	service.Success(c)
}

// 使用id查询ExportField
func (a *ArtistInfoUserHandler) GetExportFieldDetail(c *gin.Context) {
	var req artistInfoUser.GetExportFieldByIdRequest
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	resp, err := service.GrpcArtistInfoUserImpl.GetExportFieldDetail(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GrpcArtistInfoUserImpl.GetExportFieldDetail Err", "查询失败")
		return
	}
	service.Success(c, resp)
}

// 批量查询ExportField
func (a *ArtistInfoUserHandler) GetExportFieldList(c *gin.Context) {
	var req GetExportFieldListRequest
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	var protoReq artistInfoUser.GetExportFieldListRequest
	asUtil.RequestDataConvert(&req, &protoReq)
	resp, err := service.GrpcArtistInfoUserImpl.GetExportFieldList(c, &protoReq)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GetExportFieldList Err", "查询失败")
		return
	}
	var returnData []SimpleExportFieldInfo
	copier.Copy(&returnData, resp.List)
	service.ResponseList(c, returnData, service.OptionPage(resp.Page, resp.PageSize, resp.Total), service.OptionMsg("查询成功"))
}
