package asUser

import (
	"github.com/fonchain_enterprise/fonchain-main/api/artistInfoUser"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/gin-gonic/gin"
)

// 创建User
func (a *ArtistInfoUserHandler) CreateUser(c *gin.Context) {
	var req artistInfoUser.UserData
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	_, err := service.GrpcArtistInfoUserImpl.CreateUser(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GrpcArtistInfoUserImpl.CreateUser Err", "创建失败")
		return
	}
	service.Success(c)
}

// 删除User
func (a *ArtistInfoUserHandler) DeleteUser(c *gin.Context) {
	var req artistInfoUser.DeleteUserRequest
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	_, err := service.GrpcArtistInfoUserImpl.DeleteUser(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "DeleteUser Err", "删除失败")
		return
	}
	service.Success(c)
}

// 更新User
func (a *ArtistInfoUserHandler) UpdateUser(c *gin.Context) {
	var req artistInfoUser.UserData
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	_, err := service.GrpcArtistInfoUserImpl.UpdateUser(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GrpcArtistInfoUserImpl.UpdateUser Err", "更新失败")
		return
	}
	//循环引包
	//go asUser.DynamicUserService.UpdateUserWistArtistUid(req.MgmtArtistUid)
	service.Success(c)
}

// 使用id查询User
func (a *ArtistInfoUserHandler) GetUserDetail(c *gin.Context) {
	var req artistInfoUser.GetUserByIdRequest
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	resp, err := service.GrpcArtistInfoUserImpl.GetUserDetail(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GrpcArtistInfoUserImpl.GetUserDetail Err", "查询失败")
		return
	}
	service.Success(c, resp)
}

// 批量查询User
func (a *ArtistInfoUserHandler) GetUserList(c *gin.Context) {
	var req artistInfoUser.GetUserListRequest
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	resp, err := service.GrpcArtistInfoUserImpl.GetUserList(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GetUserList Err", "查询失败")
		return
	}
	service.ResponseList(c, resp.List, service.OptionPage(resp.Page, resp.PageSize, resp.Total), service.OptionMsg("查询成功"))
}
