// Package blackList -----------------------------
// @file      : balckList.go
// @author    : JJXu
// @contact   : wavingbear@163.com
// @time      : 2023/7/7 13:45
// -------------------------------------------
package blackList

import (
	"github.com/fonchain_enterprise/fonchain-main/api/artistInfoUser"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/utils/idcard"
	"github.com/gin-gonic/gin"
)

var Service = &BlackListService{}

type BlackListService struct {
}

func (r *BlackListService) Create(c *gin.Context) {
	var req artistInfoUser.AddToBlackListRequest
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	if req.ValueType == artistInfoUser.BL_ValueType_Nonnn {
		service.Error(c, e.InvalidParams, nil, "数据类型不能为空")
		return
	} else if req.ValueType == artistInfoUser.BL_ValueType_IdCard {
		if !idcard.Validate(req.Value) {
			service.Error(c, e.InvalidParams, nil, "身份证号码有误")
			return
		}
	}
	if req.Value == "" {
		service.Error(c, e.InvalidParams, nil, "值不能为空")
		return
	}
	_, err := service.GrpcArtistInfoUserImpl.AddToBlackList(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GrpcArtistInfoUserImpl.AddToBlackList Err", err.Error())
		return
	}
	service.Success(c)
}
func (r *BlackListService) Delete(c *gin.Context) {
	var req artistInfoUser.DeleteBlackListValuesRequest
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	_, err := service.GrpcArtistInfoUserImpl.DeleteBlackListValues(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GrpcArtistInfoUserImpl.DeleteBlackListValues Err", err.Error())
		return
	}
	service.Success(c)
}
func (r *BlackListService) Update(c *gin.Context) {
	var req artistInfoUser.UpdateBlackListValuesRequest
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	if req.Id == 0 {
		service.Error(c, e.InvalidParams, nil, "数据id不能为空")
		return
	}
	if req.ValueType == artistInfoUser.BL_ValueType_IdCard {
		if !idcard.Validate(req.Value) {
			service.Error(c, e.InvalidParams, nil, "身份证号码有误")
			return
		}
	}
	_, err := service.GrpcArtistInfoUserImpl.UpdateBlackListValue(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GrpcArtistInfoUserImpl.UpdateBlackListValue Err", err.Error())
		return
	}
	service.Success(c)

}
func (r *BlackListService) Search(c *gin.Context) {
	var req struct {
		Id        int64                       `json:"id"`
		ValueType artistInfoUser.BL_ValueType `json:"valueType"`
		Value     string                      `json:"value"`
		Remark    string                      `json:"remark"`
		Page      int64                       `json:"page"`
		PageSize  int64                       `json:"pageSize"`
	}
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	var micReq = artistInfoUser.GetBlackListValuesRequest{
		Param: &artistInfoUser.AddToBlackListRequest{
			Id:        req.Id,
			ValueType: req.ValueType,
			Value:     req.Value,
			Remark:    req.Remark,
		},
		Page:     req.Page,
		PageSize: req.PageSize,
	}
	res, err := service.GrpcArtistInfoUserImpl.GetBlackListValues(c, &micReq)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GrpcArtistInfoUserImpl.UpdateBlackListValue Err", err.Error())
		return
	}
	service.ResponseList(c, res.Datas, service.OptionPage(req.Page, req.PageSize, res.Total), service.OptionMsg("查询成功"))
}
