// Package inCircleArticle -----------------------------
// @file      : comment.go
// @author    : JJXu
// @contact   : wavingbear@163.com
// @time      : 2023/8/16 19:49
// -------------------------------------------
package inCircleArticle

import (
	"github.com/fonchain_enterprise/fonchain-main/api/inCircleArticle"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/artistInfo/asUtil"
	"github.com/fonchain_enterprise/utils/ipAddrQuery"
	"github.com/gin-gonic/gin"
)

// 创建Comment
func (a *InCircleArticleHandler) CreateComment(c *gin.Context) {
	var req inCircleArticle.CommentData
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	//获取城市位置
	clientIp := c.ClientIP()
	address, errs := ipAddrQuery.NewIpAddrQuery().GetAddress(clientIp)
	if errs != nil {
		req.Location = "未知"
	}
	req.Location = address.City
	_, err := service.GrpcInCircleArticleImpl.CreateComment(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GrpcInCircleArticleImpl.CreateComment Err", "创建失败")
		return
	}

	service.Success(c)
}

// 删除Comment
func (a *InCircleArticleHandler) DeleteComment(c *gin.Context) {
	var req inCircleArticle.DeleteCommentRequest
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	_, err := service.GrpcInCircleArticleImpl.DeleteComment(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "DeleteComment Err", "删除失败")
		return
	}
	service.Success(c)
}

// 更新Comment
func (a *InCircleArticleHandler) UpdateComment(c *gin.Context) {
	var req inCircleArticle.CommentData
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	_, err := service.GrpcInCircleArticleImpl.UpdateComment(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GrpcInCircleArticleImpl.UpdateComment Err", "更新失败")
		return
	}
	service.Success(c)
}

// 使用id查询Comment
func (a *InCircleArticleHandler) GetCommentDetail(c *gin.Context) {
	var req inCircleArticle.GetCommentByIdRequest
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	resp, err := service.GrpcInCircleArticleImpl.GetCommentDetail(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GrpcInCircleArticleImpl.GetCommentDetail Err", "查询失败")
		return
	}
	service.Success(c, resp)
}

// 批量查询Comment
func (a *InCircleArticleHandler) GetCommentList(c *gin.Context) {
	var req GetCommentListRequest
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	var protoReq inCircleArticle.GetCommentListRequest
	asUtil.RequestDataConvert(&req, &protoReq)
	resp, err := service.GrpcInCircleArticleImpl.GetCommentList(c, &protoReq)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GetCommentList Err", "查询失败")
		return
	}
	service.ResponseList(c, resp.List, service.OptionPage(resp.Page, resp.PageSize, resp.Total), service.OptionMsg("查询成功"))
}
