package inCircleArticle

import (
	"github.com/fonchain_enterprise/fonchain-main/api/inCircleArticle"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/artistInfo/asUser"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/artistInfo/asUtil"
	"github.com/gin-gonic/gin"
)

// 创建Swiper
func (a *InCircleArticleHandler) CreateSwiper(c *gin.Context) {
	var req inCircleArticle.SwiperData
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	//获取更新者信息
	jwt, err := asUser.GetJwtInfo(c)
	if err == nil && jwt != nil {
		req.CreatedBy = int64(jwt.ID)
	}
	_, err = service.GrpcInCircleArticleImpl.CreateSwiper(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GrpcInCircleArticleImpl.CreateSwiper Err", "创建失败")
		return
	}
	service.Success(c)
}

// 删除Swiper
func (a *InCircleArticleHandler) DeleteSwiper(c *gin.Context) {
	var req inCircleArticle.DeleteSwiperRequest
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	_, err := service.GrpcInCircleArticleImpl.DeleteSwiper(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "DeleteSwiper Err", "删除失败")
		return
	}
	service.Success(c)
}

// 更新Swiper
func (a *InCircleArticleHandler) UpdateSwiper(c *gin.Context) {
	var req inCircleArticle.SwiperData
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	//获取更新者信息
	jwt, err := asUser.GetJwtInfo(c)
	if err == nil && jwt != nil {
		req.UpdateBy = int64(jwt.ID)
	}
	_, err = service.GrpcInCircleArticleImpl.UpdateSwiper(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GrpcInCircleArticleImpl.UpdateSwiper Err", "更新失败")
		return
	}
	service.Success(c)
}

// 使用id查询Swiper
func (a *InCircleArticleHandler) GetSwiperDetail(c *gin.Context) {
	var req inCircleArticle.GetSwiperByIdRequest
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	resp, err := service.GrpcInCircleArticleImpl.GetSwiperDetail(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GrpcInCircleArticleImpl.GetSwiperDetail Err", "查询失败")
		return
	}
	service.Success(c, resp)
}

// 批量查询Swiper
func (a *InCircleArticleHandler) GetSwiperList(c *gin.Context) {
	var req GetSwiperListRequest
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	var protoReq inCircleArticle.GetSwiperListRequest
	asUtil.RequestDataConvert(&req, &protoReq)
	resp, err := service.GrpcInCircleArticleImpl.GetSwiperList(c, &protoReq)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GetSwiperList Err", "查询失败")
		return
	}
	service.ResponseList(c, resp.List, service.OptionPage(resp.Page, resp.PageSize, resp.Total), service.OptionMsg("查询成功"))
}
