// Package inCircleArticle -----------------------------
// @file      : inCricleUser.go
// @author    : JJXu
// @contact   : wavingbear@163.com
// @time      : 2023/8/16 19:49
// -------------------------------------------
package inCircleArticle

import (
	"github.com/fonchain_enterprise/fonchain-main/api/inCircleArticle"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/artistInfo/asUtil"
	"github.com/gin-gonic/gin"
)

// 创建InCircleUser
func (a *InCircleArticleHandler) CreateInCircleUser(c *gin.Context) {
	var req inCircleArticle.InCircleUserData
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	_, err := service.GrpcInCircleArticleImpl.CreateInCircleUser(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GrpcInCircleArticleImpl.CreateInCircleUser Err", err.Error())
		return
	}
	service.Success(c)
}

// 删除InCircleUser
func (a *InCircleArticleHandler) DeleteInCircleUser(c *gin.Context) {
	var req inCircleArticle.DeleteInCircleUserRequest
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	_, err := service.GrpcInCircleArticleImpl.DeleteInCircleUser(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "DeleteInCircleUser Err", err.Error())
		return
	}
	service.Success(c)
}

// 更新InCircleUser
func (a *InCircleArticleHandler) UpdateInCircleUser(c *gin.Context) {
	var req inCircleArticle.InCircleUserData
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	_, err := service.GrpcInCircleArticleImpl.UpdateInCircleUser(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GrpcInCircleArticleImpl.UpdateInCircleUser Err", err.Error())
		return
	}
	service.Success(c)
}

// 使用id查询InCircleUser
func (a *InCircleArticleHandler) GetInCircleUserDetail(c *gin.Context) {
	var req inCircleArticle.GetInCircleUserByIdRequest
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	resp, err := service.GrpcInCircleArticleImpl.GetInCircleUserDetail(c, &req)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GrpcInCircleArticleImpl.GetInCircleUserDetail Err", err.Error())
		return
	}
	service.Success(c, resp)
}

// 批量查询InCircleUser
func (a *InCircleArticleHandler) GetInCircleUserList(c *gin.Context) {
	var req GetInCircleUserListRequest
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}

	var protoReq inCircleArticle.GetInCircleUserListRequest
	asUtil.RequestDataConvert(&req, &protoReq)
	resp, err := service.GrpcInCircleArticleImpl.GetInCircleUserList(c, &protoReq)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GetInCircleUserList Err", err.Error())
		return
	}
	service.ResponseList(c, resp.List, service.OptionPage(resp.Page, resp.PageSize, resp.Total), service.OptionMsg("查询成功"))
}
