package mailArtwork

import (
	"github.com/fonchain_enterprise/fonchain-main/api/mailArtwork"
	"github.com/fonchain_enterprise/fonchain-main/pkg/utils/stime"
	"time"
)

type GetMailArtworkListRequest struct {
	Page     int64 `json:"page"`
	PageSize int64 `json:"pageSize"`
	mailArtwork.MailArtworkData
}
type GetViewMailArtworkDataListRequest struct {
	Page     int64 `json:"page"`
	PageSize int64 `json:"pageSize"`
	mailArtwork.ViewMailArtworkData
}
type MailDetail struct {
	ID                int64         `json:"ID"`
	CreatedAt         string        `json:"createdAt"`
	TrackingNo        string        `json:"trackingNo"`
	CourierType       int32         `json:"courierType"`
	Company           string        `json:"company"`
	LatestStatus      string        `json:"latestStatus"`
	LatestCourierTime string        `json:"latestCourierTime"`
	Remark            string        `json:"remark"`
	History           []MailHistory `json:"history"`
}

func (m *MailDetail) Convert(info *mailArtwork.CourierInfoData, history []*mailArtwork.CourierHistoryData) {
	if info == nil {
		return
	}
	m.ID = info.ID
	m.CreatedAt = info.CreatedAt
	m.TrackingNo = info.TrackingNo
	m.CourierType = info.CourierType
	m.Company = info.Company
	m.LatestStatus = info.LatestStatus
	m.LatestCourierTime = info.LatestCourierTime
	m.Remark = info.Remark
	m.History = []MailHistory{}
	for i := range history {
		m.History = append(m.History, MailHistory{
			ID:              history[i].ID,
			CourierId:       history[i].CourierId,
			AreaName:        history[i].AreaName,
			LogisticsStatus: history[i].LogisticsStatus,
			Desc:            history[i].Desc,
			CourierTimeDesc: time.UnixMilli(history[i].CourierTime).Format(stime.Format_Normal_YMDhms),
		})
	}
}

type MailHistory struct {
	ID              int64  `json:"ID"`
	CourierId       int64  `json:"courierId"`
	AreaName        string `json:"areaName"`
	LogisticsStatus string `json:"logisticsStatus"`
	Desc            string `json:"desc"`
	CourierTimeDesc string `json:"courierTimeDesc"`
}
type PutInStorageRequest struct {
	ArtworkUuid string `json:"artworkUid"`
	Detail      string `json:"detail"`
}

type ConfirmMailArtworkRequest struct {
	ArtistUid  string   `json:"artistUid"`
	ArtworkUid []string `json:"artworkUid"`
}
type ArtworkCourierGroupRequest struct {
	ArtworkUid []string `json:"artworkUid"`
}
type GetCanSendMailArtworkListRequest struct {
	mailArtwork.CanSendMailArtworkData
	Page     int64  `json:"page"`
	PageSize int64  `json:"pageSize"`
	Order    string `json:"order"`
	Where    string `json:"where"`
}

type ShortUrlTestRequest struct {
	Url string `json:"url"`
}
