package mailArtwork

import (
	"github.com/fonchain_enterprise/fonchain-main/api/mailArtwork"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/artistInfo/asUtil"
	"github.com/gin-gonic/gin"
)

// 批量查询ArtistReturnStatData
// 和徐嘉鸿沟通：1. 一级列表中筛选条件--确认方式 ，功能用处不大，应此去掉。
//  2. 一级列表和二级列表中的筛选条件--状态，应该要统一下拉列表中的名称的，并且要联动的，但是徐嘉鸿说不改。
func (a *MailArtworkHandler) GetArtistReturnStatDataList(c *gin.Context) {
	var req GetArtistReturnStatDataListRequest
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	var protoReq = mailArtwork.GetArtistReturnStatListRequest{Query: &mailArtwork.ArtistReturnStatData{}}
	asUtil.RequestDataConvert(&req, &protoReq)
	resp, err := service.GrpcMailArtworkImpl.GetArtistReturnStatList(c, &protoReq)
	if err != nil {
		service.Error(c, e.Failed, err)
		return
	}
	service.ResponseList(c, resp.List, service.OptionPage(resp.Page, resp.PageSize, resp.Total), service.OptionMsg("查询成功"))
}

// 批量查询 GetCanReturnArtworkList
func (a *MailArtworkHandler) GetCanReturnArtworkList(c *gin.Context) {
	var req GetCanReturnArtworkListRequest
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	//所有参数都为空则返回空值
	if req.ArtworkUid == "" &&
		req.ArtistUid == "" &&
		req.Tfnum == "" &&
		req.ArtworkName == "" &&
		req.ArtworkType == 0 &&
		req.Length == 0 &&
		req.Width == 0 &&
		req.Ruler == 0 &&
		req.HdPic == "" &&
		req.ArtistPhoto == "" &&
		req.Sealpic == "" &&
		req.NationalExhibition == 0 &&
		req.NationalExhibitionProof == "" &&
		req.ArtistName == "" {
		service.ResponseList(c, []struct{}{}, service.OptionPage(req.Page, req.PageSize, 0), service.OptionMsg("查询成功"))
		return
	}
	var protoReq = mailArtwork.GetCanReturnArtworkListRequest{Query: &mailArtwork.CanReturnArtworkData{}}
	asUtil.RequestDataConvert(&req, &protoReq)
	resp, err := service.GrpcMailArtworkImpl.GetCanReturnArtworkList(c, &protoReq)
	if err != nil {
		service.Error(c, e.Failed, err)
		return
	}
	service.ResponseList(c, resp.Data, service.OptionPage(resp.Page, resp.PageSize, resp.Total), service.OptionMsg("查询成功"))
}

// 批量查询 ViewReturnArtworkData
func (a *MailArtworkHandler) GetViewReturnArtworkDataList(c *gin.Context) {
	var req GetViewReturnArtworkDataListRequest
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	var protoReq = mailArtwork.GetViewReturnArtworkListRequest{Query: &mailArtwork.ViewReturnArtworkData{}}
	asUtil.RequestDataConvert(&req, &protoReq)
	resp, err := service.GrpcMailArtworkImpl.GetViewReturnArtworkList(c, &protoReq)
	if err != nil {
		service.Error(c, e.Failed, err)
		return
	}
	for i, v := range resp.Data {
		if v.AddressConfirm == 1 {
			resp.Data[i].Address = "待画家短信确认"
		}
	}
	service.ResponseList(c, resp.Data, service.OptionPage(resp.Page, resp.PageSize, resp.Total), service.OptionMsg("查询成功"))
}
