// Package service -----------------------------
// @file      : artistOneQuery.go
// @author    : JJXu
// @contact   : wavingbear@163.com
// @time      : 2024/3/9 13:21
// -------------------------------------------
package service

import (
	artShow "github.com/fonchain_enterprise/fonchain-main/api/artShow"
	"github.com/fonchain_enterprise/fonchain-main/api/artist"
	"github.com/fonchain_enterprise/fonchain-main/api/artwork_query"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model"
	"github.com/gin-gonic/gin"
	"github.com/jinzhu/copier"
	"time"
)

func ArtistOneQueryListOldLogic(c *gin.Context, reqModel model.OneQueryReq) {
	var req artist.OneQueryReq
	var err error
	// 如果没有时间默认当月
	if reqModel.ShowTime == "" {
		reqModel.ShowTime = time.Now().Format("2006-01")
	}
	_ = copier.Copy(&req, reqModel)
	var artistResp *artShow.ArtistWithShowTimeResp
	if artistResp, err = GrpcArtShowImpl.ArtistWithShowTime(c, &artShow.ArtistWithShowTimeReq{ShowTime: reqModel.ShowTime}); err != nil {
		ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	if len(artistResp.ArtistUids) > 0 {
		req.ShowTimeArtistUuids = artistResp.ArtistUids
	}

	if req.Permission == model.PermissionTraining {
		passResp, errS := GrpcArtworkQueryImpl.GetPassArtist(c, &artwork_query.GetPassArtistReq{})
		if errS != nil {
			ResponseQuickMsg(c, e.Failed, errS.Error(), nil)
			return
		}
		req.PassArtistUuids = passResp.ArtistUuids
	}
	// 如果出库时间不为空 查询满足出库时间的画家
	if req.StockOutTime != "" {
		stockResp, _err := GrpcArtworkQueryImpl.StockOutArtist(c, &artwork_query.StockOutArtistReq{StockOutTime: req.StockOutTime})
		if _err != nil {
			ResponseQuickMsg(c, e.Failed, _err.Error(), nil)
			return
		}
		req.StockOutArtistUuids = stockResp.ArtistUuids
		if len(req.StockOutArtistUuids) == 0 {
			req.StockOutArtistUuids = []string{"-1"}
		}
	}
	resp, errs := GrpcArtistImpl.OneQueryOld(c, &req)
	if errs != nil {
		ResponseQuickMsg(c, e.Failed, errs.Error(), nil)
		return
	}
	ResponseQuickMsg(c, e.Ok, resp.Msg, resp)
	return
}
