package artwork

import (
	"fmt"
	"github.com/fonchain_enterprise/fonchain-main/api/artwork_third_party"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/logic"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
)

func RepairList(c *gin.Context) {
	var req artwork_third_party.RepairListReq
	if err := c.ShouldBind(&req); err != nil {
		zap.L().Error("RepairList ShouldBind err", zap.Error(err))
		service.ResponseQuickMsg(c, e.Failed, e.GetMsg(e.InvalidParams), nil)
		return
	}
	listResp, _err := service.GrpcArtworkThirdPartyImpl.RepairList(c, &req)
	if _err != nil {
		service.ResponseQuickMsg(c, e.Failed, _err.Error(), nil)
		return
	}
	for _, v := range listResp.Data {
		if v.BlackListStatus == 2 {
			v.ArtistName = fmt.Sprintf("#%s", v.ArtistName)
		}
	}
	service.ResponseQuickMsg(c, e.Ok, e.GetMsg(e.Success), listResp)
	return
}

func RepairUpdate(c *gin.Context) {
	var req artwork_third_party.RepairUpdateReq
	if err := c.ShouldBind(&req); err != nil {
		zap.L().Error("RepairUpdate ShouldBind err", zap.Error(err))
		service.ResponseQuickMsg(c, e.Failed, e.GetMsg(e.InvalidParams), nil)
		return
	}
	listResp, _err := service.GrpcArtworkThirdPartyImpl.RepairUpdate(c, &req)
	if _err != nil {
		service.ResponseQuickMsg(c, e.Failed, _err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(c, e.Ok, e.GetMsg(e.Success), listResp)
	return
}

func RepairDel(c *gin.Context) {
	var req artwork_third_party.RepairDelReq
	if err := c.ShouldBind(&req); err != nil {
		zap.L().Error("RepairDel ShouldBind err", zap.Error(err))
		service.ResponseQuickMsg(c, e.Failed, e.GetMsg(e.InvalidParams), nil)
		return
	}
	listResp, _err := service.GrpcArtworkThirdPartyImpl.RepairDel(c, &req)
	if _err != nil {
		service.ResponseQuickMsg(c, e.Failed, _err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(c, e.Ok, e.GetMsg(e.Success), listResp)
	return
}

func RepairDetail(c *gin.Context) {
	var req artwork_third_party.RepairDetailReq
	if err := c.ShouldBind(&req); err != nil {
		zap.L().Error("RepairDetail ShouldBind err", zap.Error(err))
		service.ResponseQuickMsg(c, e.Failed, e.GetMsg(e.InvalidParams), nil)
		return
	}
	listResp, _err := service.GrpcArtworkThirdPartyImpl.RepairDetail(c, &req)
	if _err != nil {
		service.ResponseQuickMsg(c, e.Failed, _err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(c, e.Ok, e.GetMsg(e.Success), listResp)
	return
}

func RepairExport(c *gin.Context) {
	var req artwork_third_party.RepairListReq
	if err := c.ShouldBind(&req); err != nil {
		zap.L().Error("RepairDetail ShouldBind err", zap.Error(err))
		service.ResponseQuickMsg(c, e.Failed, e.GetMsg(e.InvalidParams), nil)
		return
	}
	req.Page = 1
	req.PageSize = 999
	listResp, _err := service.GrpcArtworkThirdPartyImpl.RepairList(c, &req)
	if _err != nil {
		service.ResponseQuickMsg(c, e.Failed, _err.Error(), nil)
		return
	}
	//组装数据
	if len(listResp.Data) == 0 {
		listResp.Data = []*artwork_third_party.RepairListResp_Info{}
	}

	exportUrl, err := logic.RegroupArtworkRepairData(c, listResp.Data)
	if err != nil {
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(c, e.Ok, e.GetMsg(e.Success), map[string]string{
		"ExportUrl": exportUrl,
	})
	return
}
