package artwork

import (
	"errors"
	"fmt"
	"github.com/fonchain_enterprise/fonchain-main/api/artwork"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/logic"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/utils"
	"github.com/gin-gonic/gin"
	uuid "github.com/satori/go.uuid"
	"os"
	"strings"
)

func GenerateSupplement(ctx *gin.Context) {
	req := &artwork.EditSupplementReq{}
	if err := ctx.ShouldBind(&req); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	req.Supplement = strings.TrimSpace(req.Supplement)
	if req.Supplement == "" {
		service.ResponseQuickMsg(ctx, e.Failed, e.ErrorNotEmpty, nil)
		return
	}
	var err error
	var cdnUrl string
	// 生成pdf
	var logicPdf = new(logic.GoPdf)
	uuidObj, _ := uuid.NewV4()
	fileName := fmt.Sprintf("%s_%s.pdf", req.ArtworkUuid, uuidObj.String())
	_, _ = utils.CheckDirPath("./runtime/artwork", true)
	pdfPath := fmt.Sprintf("./runtime/artwork/%s", fileName)
	if err = logicPdf.TextToPdf(req.Supplement, 14, 50, 50, pdfPath); err != nil {
		return
	}
	file, _err := os.Stat(pdfPath)
	if _err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, _err.Error(), nil)
		return
	}
	fmt.Println(file.Size())
	if file.Size() >= 1024*1024*10 {
		err = errors.New(e.ErrorFileOverSize)
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	if cdnUrl, err = service.PutBosWithName(pdfPath, true,
		"/artwork/supplement/"+fileName); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	req.Supplement = cdnUrl
	_, err = service.GrpcArtworkImpl.EditSupplement(ctx, req)
	if err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.Success), nil)
	return
}
