package service

import (
	"context"
	"dubbo.apache.org/dubbo-go/v3/common/logger"
	"github.com/fonchain_enterprise/fonchain-main/api/artwork_chain"
	"github.com/fonchain_enterprise/fonchain-main/pkg/common"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/gin-gonic/gin"
)

func CoChain(c *gin.Context) {
	var req artwork_chain.CoChainRequest
	var err error
	if err = c.ShouldBind(&req); err != nil {
		logger.Errorf("CoChain ShouldBind err", err)
		ResponseQuickMsg(c, e.Ok, err.Error(), nil)
		return
	}
	if err = req.Validate(); err != nil {
		err = common.SubstrError(err)
		ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	resp, err := GrpcArtworkChainImpl.CoChain(context.Background(), &req)
	if err != nil {
		ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	ResponseQuickMsg(c, e.Ok, e.GetMsg(e.Success), resp)
	return
}

func CreateCert(c *gin.Context) {
	var req artwork_chain.CreateCertRequest
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("CreateCert ShouldBind err", err)
		ResponseQuickMsg(c, e.Ok, err.Error(), nil)
		return
	}
	if err := req.Validate(); err != nil {
		err = common.SubstrError(err)
		ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	resp, err := GrpcArtworkChainImpl.CreateCert(context.Background(), &req)
	if err != nil {
		ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	ResponseQuickMsg(c, e.Ok, e.GetMsg(e.Success), resp)
	return
}

func AuthImgCoChain(c *gin.Context) {
	var req artwork_chain.AuthImgCoChainReq
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("CreateCert ShouldBind err", err)
		ResponseQuickMsg(c, e.Ok, err.Error(), nil)
		return
	}
	if err := req.Validate(); err != nil {
		err = common.SubstrError(err)
		ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	resp, err := GrpcArtworkChainImpl.AuthImgCoChain(context.Background(), &req)
	if err != nil {
		ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	ResponseQuickMsg(c, e.Ok, e.GetMsg(e.Success), resp)
	return
}
