package chainbrowser

import (
	"context"
	"dubbo.apache.org/dubbo-go/v3/common/logger"
	"errors"
	"fmt"
	"github.com/fonchain_enterprise/fonchain-main/api/account"
	"github.com/fonchain_enterprise/fonchain-main/api/artwork_query"
	"github.com/fonchain_enterprise/fonchain-main/api/chain"
	"github.com/fonchain_enterprise/fonchain-main/api/chainbrowser"
	"github.com/fonchain_enterprise/fonchain-main/api/equity"
	"github.com/fonchain_enterprise/fonchain-main/api/order"
	"github.com/fonchain_enterprise/fonchain-main/pkg/common"
	"github.com/fonchain_enterprise/fonchain-main/pkg/common/untils/fileutils"
	"github.com/fonchain_enterprise/fonchain-main/pkg/config"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	logic_browser "github.com/fonchain_enterprise/fonchain-main/pkg/logic/chainbrowser"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model"
	"github.com/fonchain_enterprise/fonchain-main/pkg/serializer"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/utils"
	"github.com/fonchain_enterprise/utils/pay"
	"github.com/gin-gonic/gin"
	"github.com/jinzhu/copier"
	"go.uber.org/zap"
	"google.golang.org/protobuf/types/known/emptypb"
	"io/ioutil"
	"strconv"
	"strings"
	"time"
)

// HomeData 首页数据
func HomeData(ctx *gin.Context) {
	//var req chainbrowser.WtBlockDetailReq
	//if err := ctx.ShouldBind(&req); err != nil {
	//	logger.Errorf("BlockHashDetail err req %+v", err)
	//	service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
	//	return
	//}
	resp, err := service.GrpcChainBrowserImpl.HomeData(context.Background(), &emptypb.Empty{})
	if err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	chainListResp, _ := service.GrpcArtworkQueryImpl.ChainList(context.Background(), &artwork_query.ChainListReq{
		ChainType: "baidu",
		Page:      1,
		PageSize:  1,
	})
	for k, v := range resp.ChainInfo {
		if v.ChainType == "baidu" {
			resp.ChainInfo[k].TransactionNum = chainListResp.Count
		}
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.Success), resp)
	return
}

func HashDetail(ctx *gin.Context) {
	var req model.HashDetailReq
	var (
		err           error
		cxDetailResp  *chainbrowser.ChengXinDetailResp
		chainInfoResp *artwork_query.ChainInfoByHashResp
		//baiduXuperResp    *chainbrowser.BaiduXuperDetailResp
		wtTransactionResp *chainbrowser.WtTransactionDetailResp
	)
	if err = ctx.ShouldBindJSON(&req); err != nil {
		msg := utils.ValidatorErrorMsg(err)
		service.ResponseQuickMsg(ctx, e.Failed, msg, nil)
		return
	}
	var hashDetailResp model.HashDetailResp
	hashDetailResp.Status = model.ChainStatusOk
	switch req.ChainType {
	case model.ChainTypeWt:
		wtTransactionResp, err = service.GrpcChainBrowserImpl.WtTransactionDetail(context.Background(), &chainbrowser.WtTransactionDetailReq{TransactionHash: req.Hash})
		if err != nil {
			service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
			return
		}
		//hashDetailResp.TransactionHash = wtTransactionResp.TxsHash
		//hashDetailResp.FileHash = wtTransactionResp.BlockHash
		//hashDetailResp.Height = wtTransactionResp.Height
		//hashDetailResp.CreateTime = wtTransactionResp.Timestamp
		if !strings.Contains(wtTransactionResp.Data, "交易站点") {
			chainInfoResp, err = service.GrpcArtworkQueryImpl.ChainInfoByHash(context.Background(), &artwork_query.ChainInfoByHashReq{WtTransactionHash: []string{req.Hash}})
			if err != nil {
				service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
				return
			}
			if chainInfoResp != nil {
				wtTransactionResp.Data = chainInfoResp.Data[0].Basicinfoofartwork
			}
		}
		service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.Success), wtTransactionResp)
		return
	case model.ChainTypeCx:
		cxDetailResp, err = service.GrpcChainBrowserImpl.ChengXinDetail(context.Background(), &chainbrowser.ChengXinDetailReq{CxTransactionHash: req.Hash})
		if err != nil {
			service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
			return
		}
		_ = copier.Copy(&hashDetailResp, cxDetailResp)
	case model.ChainTypeBaidu:
		chainInfoResp, err = service.GrpcArtworkQueryImpl.ChainInfoByHash(context.Background(), &artwork_query.ChainInfoByHashReq{BaiduTransactionHash: []string{req.Hash}})
		if err != nil {
			service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
			return
		}
		if chainInfoResp.Data[0].RequestId1 == "" {
			service.ResponseQuickMsg(ctx, e.Failed, e.GetMsg(e.NOTDATA), nil)
			return
		}
		//baiduXuperResp, err = service.GrpcChainBrowserImpl.BaiduXuperDetail(context.Background(), &chainbrowser.BaiduXuperDetailReq{RequestId: chainInfoResp.Data})
		//if err != nil {
		//	service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		//	return
		//}
		//_ = copier.Copy(&hashDetailResp, baiduXuperResp)
		//hashDetailResp.ChainData = chainInfoResp.Basicinfoofartwork
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.Success), hashDetailResp)
	return
}

func BlockHashDetail(ctx *gin.Context) {
	var req chainbrowser.BlockDetailReq
	if err := ctx.ShouldBind(&req); err != nil {
		logger.Errorf("BlockDetailReq err req %+v", err)
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	resp, err := service.GrpcChainBrowserImpl.BlockDetail(context.Background(), &req)
	if err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	if req.ChainType == "wt" {
		if resp.TxList != nil {
			var logicBrowserArtwork = new(logic_browser.Artwork)
			var hashList []string
			for _, val := range resp.TxList {
				hashList = append(hashList, val.Hash)
			}
			hashInfoReq := artwork_query.ChainInfoByHashReq{WtTransactionHash: hashList}
			chainInfoResp, _err := service.GrpcArtworkQueryImpl.ChainInfoByHash(context.Background(), &hashInfoReq)
			if _err != nil {
				service.ResponseQuickMsg(ctx, e.Failed, _err.Error(), nil)
				return
			}
			for key, val := range resp.TxList {
				for _, v := range chainInfoResp.Data {
					if val.Hash == v.WtTransactionHash {
						windupTime := logicBrowserArtwork.CheckSpecialChain(v.Tfnum)
						if windupTime != "" {
							resp.TxList[key].CreateDate = windupTime
						}
						resp.TxList[key].Tfnum = v.Tfnum
					}
				}
			}
		}
	}

	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.Success), resp)
	return
}

func BlockList(ctx *gin.Context) {
	var req chainbrowser.BlockListReq
	if err := ctx.ShouldBind(&req); err != nil {
		logger.Errorf("BlockList err req %+v", err)
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	resp, err := service.GrpcChainBrowserImpl.BlockList(context.Background(), &req)
	if err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.Success), resp)
	return
}

func TxList(ctx *gin.Context) {
	var (
		req           *chainbrowser.TransactionListReq
		resp          *chainbrowser.TransactionListResp
		chainListResp *artwork_query.ChainListResp
		err           error
	)
	resp = &chainbrowser.TransactionListResp{}
	if err = ctx.ShouldBind(&req); err != nil {
		logger.Errorf("TxList err req %+v", err)
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	switch req.ChainType {
	case "wt", "cx":
		resp, err = service.GrpcChainBrowserImpl.TransactionList(context.Background(), req)
		if err != nil {
			service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
			return
		}
	case "baidu":
		chainListResp, err = service.GrpcArtworkQueryImpl.ChainList(context.Background(), &artwork_query.ChainListReq{
			ChainType: "baidu",
			Page:      req.Page,
			PageSize:  req.PageSize,
			Tfnum:     strings.Replace(req.BusinessId, "Art_", "", 1),
		})
		if err != nil {
			service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
			return
		}
		if chainListResp != nil && chainListResp.Data != nil {
			resp.Count = chainListResp.Count
			for _, v := range chainListResp.Data {
				var temp chainbrowser.TransactionListResp_Info
				temp.Hash = v.Hash
				temp.CreateDate = v.CreateDate
				resp.Data = append(resp.Data, &temp)
			}
		}
	}
	if req.ChainType == "wt" {
		if resp.Data != nil {
			var logicBrowserArtwork = new(logic_browser.Artwork)
			var hashList []string
			for _, val := range resp.Data {
				hashList = append(hashList, val.Hash)
			}
			hashInfoReq := artwork_query.ChainInfoByHashReq{WtTransactionHash: hashList}
			chainInfoResp, _err := service.GrpcArtworkQueryImpl.ChainInfoByHash(context.Background(), &hashInfoReq)
			if _err != nil {
				service.ResponseQuickMsg(ctx, e.Failed, _err.Error(), nil)
				return
			}

			for key, val := range resp.Data {
				for _, v := range chainInfoResp.Data {
					if val.Hash == v.WtTransactionHash {
						windupTime := logicBrowserArtwork.CheckSpecialChain(v.Tfnum)
						if windupTime != "" {
							resp.Data[key].CreateDate = windupTime
						}
						resp.Data[key].Tfnum = v.Tfnum
					}
				}
			}
		}
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.Success), resp)
	return
}

func TxHashDetail(ctx *gin.Context) {
	var (
		req           chainbrowser.TxHashDetailReq
		chainInfoResp *artwork_query.ChainInfoByHashResp
	)
	if err := ctx.ShouldBind(&req); err != nil {
		logger.Errorf("TxHashDetail err req %+v", err)
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	resp, err := service.GrpcChainBrowserImpl.TxHashDetail(context.Background(), &req)
	if err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	hashInfoReq := &artwork_query.ChainInfoByHashReq{}
	switch req.ChainType {
	case "wt":
		hashInfoReq.WtTransactionHash = append(hashInfoReq.WtTransactionHash, req.Hash)
	case "cx":
		hashInfoReq.Changtransactionhash = append(hashInfoReq.WtTransactionHash, req.Hash)
	case "baidu":
		hashInfoReq.BaiduTransactionHash = append(hashInfoReq.WtTransactionHash, req.Hash)
	}
	if chainInfoResp, err = service.GrpcArtworkQueryImpl.ChainInfoByHash(context.Background(), hashInfoReq); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	if chainInfoResp.Data != nil {
		chainInfoRespInfo := chainInfoResp.Data[0]
		switch req.ChainType {
		case "baidu":
			resp.TxsHash = chainInfoRespInfo.Transactionhash
			resp.Height = chainInfoRespInfo.BaiduHeight
			resp.CreatedDate = chainInfoRespInfo.Winduptime
		}
		resp.CertUrl = chainInfoRespInfo.Certtmpicossaddr
		resp.FileUrl = chainInfoRespInfo.AuthImg
		if resp.ExtData == "" {
			resp.ExtData = chainInfoRespInfo.Basicinfoofartwork
		}
		resp.BusinessId = "Art_" + chainInfoRespInfo.Tfnum
		var logicBrowserArtwork = new(logic_browser.Artwork)
		if req.ChainType == "wt" {
			windupTime := logicBrowserArtwork.CheckSpecialChain(chainInfoRespInfo.Tfnum)
			if windupTime != "" {
				resp.CreatedDate = windupTime
			}
		}
	}

	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.Success), resp)
	return
}

func FileHashList(ctx *gin.Context) {
	var req chainbrowser.FileHashListReq
	if err := ctx.ShouldBind(&req); err != nil {
		logger.Errorf("FileHashList err req %+v", err)
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	resp, err := service.GrpcChainBrowserImpl.FileHashList(context.Background(), &req)
	if err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.Success), resp)
	return
}

func FileAddChain(ctx *gin.Context) {
	var (
		req              *chainbrowser.FileAddChainReq
		fileAddChainResp *chainbrowser.FileAddChainResp
		err              error
		fileId           string
		requestId        string
		fileByte         []byte
	)
	if err = ctx.ShouldBind(&req); err != nil {
		logger.Errorf("FileAddChain err req %+v", err)
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	fileByte, err = fileutils.ReadUrlFile(req.FileUrl)
	if err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	req.FileHash = fileutils.SHA256V(fileByte)
	if err = req.Validate(); err != nil {
		err = common.SubstrError(err)
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	if fileAddChainResp, err = service.GrpcChainBrowserImpl.FileAddChain(context.Background(), req); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	fileId = fileAddChainResp.ID
	if config.AppMode == "prod" || true {
		baiduChainReq := chain.BaiduReq{
			FileHash: req.FileHash,
			FileName: req.FileName,
		}
		fmt.Println(&baiduChainReq)
		baiduResp, _err := service.ChainProvider.CoBaiduChain(context.Background(), &baiduChainReq)
		if _err != nil {
			service.ResponseQuickMsg(ctx, e.Failed, _err.Error(), nil)
			return
		}
		requestId = baiduResp.RequestId
		if _, err = service.GrpcChainBrowserImpl.UpdateFileChain(context.Background(), &chainbrowser.UpdateFileChainReq{
			FileId:    fileId,
			RequestId: requestId,
		}); err != nil {
			service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
			return
		}
		//fmt.Println(baiduResp)
		var logicArtwork = new(logic_browser.Artwork)
		go func() {
			for {
				zap.L().Info("queryChainRes info 01:request_id:" + requestId)
				time.Sleep(10 * time.Second)
				baiduInfoResp, _ := logicArtwork.RefreshBaiduInfo(requestId, fileId)
				if baiduInfoResp.BosUrl != "" {
					if _, err = service.GrpcChainBrowserImpl.UpdateFileChain(context.Background(), &chainbrowser.UpdateFileChainReq{
						FileId:     fileId,
						TxId:       baiduInfoResp.TxId,
						BlockId:    baiduInfoResp.BlockId,
						CertUrl:    baiduInfoResp.BosUrl,
						CreateTime: baiduInfoResp.CreateTime,
						Height:     int64(baiduInfoResp.Height),
					}); err != nil {
						service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
					}
					break
				}
			}
		}()
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.Success), map[string]string{
		"requestId": requestId,
	})
	return
}

func FileHashDetail(ctx *gin.Context) {
	var req chainbrowser.FileChainDetailReq
	if err := ctx.ShouldBind(&req); err != nil {
		logger.Errorf("FileHashList err req %+v", err)
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	resp, err := service.GrpcChainBrowserImpl.FileChainDetail(context.Background(), &req)
	if err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.Success), resp)
	return
}

func RefreshFileChain(ctx *gin.Context) {
	var req chainbrowser.FileChainDetailReq
	var err error
	if err = ctx.ShouldBind(&req); err != nil {
		logger.Errorf("RefreshFileChain err req %+v", err)
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	var logicArtwork = new(logic_browser.Artwork)
	resp, _err := service.GrpcChainBrowserImpl.FileChainDetail(context.Background(), &chainbrowser.FileChainDetailReq{FileId: req.FileId})
	if _err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, _err.Error(), nil)
		return
	}
	if resp.RequestId == "" {
		service.ResponseQuickMsg(ctx, e.Failed, e.GetMsg(e.NOTDATA), nil)
		return
	}
	if resp.CertUrl != "" {
		service.ResponseQuickMsg(ctx, e.Failed, e.ErrorBaiduChainAlreadyDone, nil)
		return
	}
	baiduInfoResp, _ := logicArtwork.RefreshBaiduInfo(resp.RequestId, req.FileId)
	if baiduInfoResp.BosUrl != "" {
		if _, err = service.GrpcChainBrowserImpl.UpdateFileChain(context.Background(), &chainbrowser.UpdateFileChainReq{
			FileId:     req.FileId,
			TxId:       baiduInfoResp.TxId,
			BlockId:    baiduInfoResp.BlockId,
			CertUrl:    baiduInfoResp.BosUrl,
			CreateTime: baiduInfoResp.CreateTime,
			Height:     int64(baiduInfoResp.Height),
		}); err != nil {
			service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		}
		service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.Success), nil)
		return
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.ErrorBaiduChaining, nil)
	return
}

func UpdateFreeResources(c *gin.Context) {
	var updateFreeResourcesRequest equity.UpdateFreeResourcesRequest
	if err := c.ShouldBind(&updateFreeResourcesRequest); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	updateFreeResourcesRequest.UpdateType = 1 // 微服务那边，多余的进行删除

	rep, err := service.EquityProvider.UpdateFreeResources(context.Background(), &updateFreeResourcesRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, rep)

	return

}

func GetAllFreeResources(c *gin.Context) {
	var getAllFreeResourcesRequest equity.GetAllFreeResourcesRequest
	if err := c.ShouldBind(&getAllFreeResourcesRequest); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	rep, err := service.EquityProvider.GetAllFreeResources(context.Background(), &getAllFreeResourcesRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, rep)

	return

}

func UpdateConsumeResources(c *gin.Context) {
	var updateConsumeResourcesRequest equity.UpdateConsumeResourcesRequest
	if err := c.ShouldBind(&updateConsumeResourcesRequest); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	rep, err := service.EquityProvider.UpdateConsumeResources(context.Background(), &updateConsumeResourcesRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, rep)

	return

}

func GetAllConsumeResources(c *gin.Context) {
	var getAllConsumeResourcesRequest equity.GetAllConsumeResourcesRequest
	if err := c.ShouldBind(&getAllConsumeResourcesRequest); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	rep, err := service.EquityProvider.GetAllConsumeResources(context.Background(), &getAllConsumeResourcesRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, rep)

	return

}

func UpdateConversionRatio(c *gin.Context) {
	var updateConversionRatioRequest equity.UpdateConversionRatioRequest
	if err := c.ShouldBind(&updateConversionRatioRequest); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	rep, err := service.EquityProvider.UpdateConversionRatio(context.Background(), &updateConversionRatioRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, rep)

	return

}

func GetAllRefillCardStatus(c *gin.Context) {
	var getAllRefillCardStatusRequest equity.GetAllRefillCardStatusRequest
	if err := c.ShouldBind(&getAllRefillCardStatusRequest); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	rep, err := service.EquityProvider.GetAllRefillCardStatus(context.Background(), &getAllRefillCardStatusRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, rep)

	return

}

func CreateRefillCardAmountInfo(c *gin.Context) {
	var createRefillCardAmountInfoRequest equity.CreateRefillCardAmountInfoRequest
	if err := c.ShouldBind(&createRefillCardAmountInfoRequest); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	rep, err := service.EquityProvider.CreateRefillCardAmountInfo(context.Background(), &createRefillCardAmountInfoRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, rep)

	return

}

func GetRefillCardAmountInfo(c *gin.Context) {
	var getRefillCardAmountInfoRequest equity.GetRefillCardAmountInfoRequest
	if err := c.ShouldBind(&getRefillCardAmountInfoRequest); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	rep, err := service.EquityProvider.GetRefillCardAmountInfo(context.Background(), &getRefillCardAmountInfoRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, rep)

	return

}

func StopRefillCardAmountInfo(c *gin.Context) {
	var stopRefillCardAmountInfoRequest equity.StopRefillCardAmountInfoRequest
	if err := c.ShouldBind(&stopRefillCardAmountInfoRequest); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	rep, err := service.EquityProvider.StopRefillCardAmountInfo(context.Background(), &stopRefillCardAmountInfoRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, rep)

	return

}

func OpenRefillCardAmountInfo(c *gin.Context) {
	var openRefillCardAmountInfoRequest equity.OpenRefillCardAmountInfoRequest
	if err := c.ShouldBind(&openRefillCardAmountInfoRequest); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	rep, err := service.EquityProvider.OpenRefillCardAmountInfo(context.Background(), &openRefillCardAmountInfoRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, rep)

	return

}

func DeleteRefillCardAmountInfo(c *gin.Context) {
	var deleteRefillCardAmountInfoRequest equity.DeleteRefillCardAmountInfoRequest
	if err := c.ShouldBind(&deleteRefillCardAmountInfoRequest); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	rep, err := service.EquityProvider.DeleteRefillCardAmountInfo(context.Background(), &deleteRefillCardAmountInfoRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, rep)

	return

}

func GetAllPayStatus(c *gin.Context) {
	var getAllPayStatusRequest equity.GetAllPayStatusRequest
	if err := c.ShouldBind(&getAllPayStatusRequest); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	rep, err := service.EquityProvider.GetAllPayStatus(context.Background(), &getAllPayStatusRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, rep)

	return

}

type NameAndPhone struct {
	Name  string
	Phone string
}

func RechargeRecordManage(c *gin.Context) {
	var rechargeRecordManageRequest equity.RechargeRecordManageRequest
	if err := c.ShouldBind(&rechargeRecordManageRequest); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	idMap := make(map[int32]NameAndPhone)
	var id []int32

	//根据姓名进行模糊插叙
	listRequest := account.ListRequest{
		Domain: "blockchain",
		Key:    rechargeRecordManageRequest.Name,
		TelNum: rechargeRecordManageRequest.Phone,
	}
	repList, err := service.AccountProvider.List(context.Background(), &listRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	if len(repList.Data) == 0 {
		fmt.Println("测试节点1111")
		rep := &equity.RechargeRecordManageRespond{}
		service.Success(c, rep)
		return
	}

	for _, v := range repList.Data {
		nameAndPhone := NameAndPhone{
			Name:  v.NickName,
			Phone: v.TelNum,
		}

		idMap[int32(v.ID)] = nameAndPhone
		id = append(id, int32(v.ID))
	}

	rechargeRecordManageRequest.UserID = id

	rep, err := service.EquityProvider.RechargeRecordManage(context.Background(), &rechargeRecordManageRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	if len(rep.RechargeRecordManage) != 0 {
		for k, v := range rep.RechargeRecordManage {
			nameAndPhone := idMap[v.UserID]
			rep.RechargeRecordManage[k].Name = nameAndPhone.Name
			rep.RechargeRecordManage[k].Phone = nameAndPhone.Phone
		}
	}

	service.Success(c, rep)

	return

}

func RechargeRecordManageExportForm(c *gin.Context) {
	var rechargeRecordManageRequest equity.RechargeRecordManageRequest
	if err := c.ShouldBind(&rechargeRecordManageRequest); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	idMap := make(map[int32]NameAndPhone)
	var id []int32
	//根据姓名进行模糊插叙
	listRequest := account.ListRequest{
		Domain: "blockchain",
		Key:    rechargeRecordManageRequest.Name,
		TelNum: rechargeRecordManageRequest.Phone,
	}
	repList, err := service.AccountProvider.List(context.Background(), &listRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	if len(repList.Data) == 0 {
		rep := &equity.RechargeRecordManageRespond{}
		service.Success(c, rep)
		return
	}

	for _, v := range repList.Data {
		nameAndPhone := NameAndPhone{
			Name:  v.NickName,
			Phone: v.TelNum,
		}

		idMap[int32(v.ID)] = nameAndPhone
		id = append(id, int32(v.ID))
	}

	rechargeRecordManageRequest.UserID = id
	rechargeRecordManageRequest.Page = 1
	rechargeRecordManageRequest.PageSize = 9999

	rep, err := service.EquityProvider.RechargeRecordManage(context.Background(), &rechargeRecordManageRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	if len(rep.RechargeRecordManage) != 0 {
		for k, v := range rep.RechargeRecordManage {
			nameAndPhone := idMap[v.UserID]
			rep.RechargeRecordManage[k].Name = nameAndPhone.Name
			rep.RechargeRecordManage[k].Phone = nameAndPhone.Phone
		}
	} else {
		service.ResponseQuickMsg(c, e.Failed, errors.New("所选条件无可导出数据").Error(), nil)
		return
	}

	columns := []string{"订单uid", "用户id", "付款人", "手机号", "充值卡号", "充值卡面额", "实际充值金额", "状态", "账号", "交易渠道", "创建订单时间", "交易流水号", "金额(总)"}
	var totalPrice float32

	var data []interface{}
	// 添加数据行
	for k, rechargeRecord := range rep.RechargeRecordManage {
		var info []string
		info = append(info, rechargeRecord.Uid)                        // 订单uid
		info = append(info, fmt.Sprintf("%d", rechargeRecord.UserID))  // 用户id
		info = append(info, rechargeRecord.Name)                       // 付款人
		info = append(info, rechargeRecord.Phone)                      // 手机号
		info = append(info, rechargeRecord.RefillCardNumber)           // 充值卡号
		info = append(info, fmt.Sprintf("%.2f", rechargeRecord.Money)) // 充值卡面额
		info = append(info, fmt.Sprintf("%.2f", rechargeRecord.Money)) // 实际充值金额
		if rechargeRecord.PayStatus == 1 {
			info = append(info, "待付款") // 状态
		} else if rechargeRecord.PayStatus == 2 {
			info = append(info, "支付成功")
		} else if rechargeRecord.PayStatus == 3 {
			info = append(info, "支付失败")
		} else {
			info = append(info, "未知状态")
		}

		info = append(info, rechargeRecord.OpenID) // 账号

		if rechargeRecord.PaymentPlatform == 1 { // 交易渠道
			info = append(info, "微信支付")
		} else if rechargeRecord.PaymentPlatform == 2 {
			info = append(info, "支付宝支付")
		} else {
			info = append(info, "未知支付平台")
		}

		info = append(info, rechargeRecord.OrderTimeFormat) // 创建订单时间
		info = append(info, rechargeRecord.PaySerialNumber) // 交易流水号

		totalPrice = totalPrice + rechargeRecord.Money
		if k == len(rep.RechargeRecordManage)-1 {
			info = append(info, fmt.Sprintf("%.2f", totalPrice)) //总价 以元为单位
		}
		data = append(data, &info)
	}

	filePath := "./runtime/充值卡订管理表.xlsx"
	_, err = utils.ToExcelByType(columns, data, "slice", filePath)
	if err != nil {
		fmt.Println("err", err)
		service.ResponseQuickMsg(c, e.Failed, errors.New("转换为excel类型出错").Error(), nil)
		return
	}

	var httpType string
	if config.IsHttps {
		httpType = model.HttpsType
	} else {
		httpType = model.HttpType
	}
	var exportUrl = fmt.Sprintf("%s%s/static/充值卡订管理表.xlsx", httpType, c.Request.Host)

	repEF := &equity.RechargeRecordManageExportFormRespond{
		ExportUrl: exportUrl,
	}

	service.Success(c, repEF)

	return

}

func GetAllRefundStatus(c *gin.Context) {
	var getAllRefundStatusRequest equity.GetAllRefundStatusRequest
	if err := c.ShouldBind(&getAllRefundStatusRequest); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	rep, err := service.EquityProvider.GetAllRefundStatus(context.Background(), &getAllRefundStatusRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, rep)

	return

}

func RefundApplyManage(c *gin.Context) {
	var refundApplyManageRequest equity.RefundApplyManageRequest
	if err := c.ShouldBind(&refundApplyManageRequest); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	idMap := make(map[int32]NameAndPhone)
	var id []int32
	//根据姓名进行模糊插叙
	listRequest := account.ListRequest{
		Domain: "blockchain",
		Key:    refundApplyManageRequest.Name,
		TelNum: refundApplyManageRequest.Phone,
	}
	repList, err := service.AccountProvider.List(context.Background(), &listRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	if len(repList.Data) == 0 {
		rep := &equity.RefundApplyManageRespond{}
		service.Success(c, rep)
		return
	}

	for _, v := range repList.Data {
		nameAndPhone := NameAndPhone{
			Name:  v.NickName,
			Phone: v.TelNum,
		}

		idMap[int32(v.ID)] = nameAndPhone
		id = append(id, int32(v.ID))
	}

	refundApplyManageRequest.UserID = id

	rep, err := service.EquityProvider.RefundApplyManage(context.Background(), &refundApplyManageRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	if len(rep.RefundApplyManage) != 0 {
		for k, v := range rep.RefundApplyManage {
			nameAndPhone := idMap[v.UserID]
			rep.RefundApplyManage[k].Name = nameAndPhone.Name
			rep.RefundApplyManage[k].Phone = nameAndPhone.Phone
		}
	}

	service.Success(c, rep)

	return

}

func RefundApplyManageExportForm(c *gin.Context) {
	var refundApplyManageRequest equity.RefundApplyManageRequest
	if err := c.ShouldBind(&refundApplyManageRequest); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	idMap := make(map[int32]NameAndPhone)
	var id []int32

	//根据姓名进行模糊插叙
	listRequest := account.ListRequest{
		Domain: "blockchain",
		Key:    refundApplyManageRequest.Name,
		TelNum: refundApplyManageRequest.Phone,
	}
	repList, err := service.AccountProvider.List(context.Background(), &listRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	if len(repList.Data) == 0 {
		rep := &equity.RefundApplyManageRespond{}
		service.Success(c, rep)
		return
	}

	for _, v := range repList.Data {
		nameAndPhone := NameAndPhone{
			Name:  v.NickName,
			Phone: v.TelNum,
		}

		idMap[int32(v.ID)] = nameAndPhone
		id = append(id, int32(v.ID))
	}

	refundApplyManageRequest.UserID = id
	refundApplyManageRequest.Page = 1
	refundApplyManageRequest.PageSize = 9999

	rep, err := service.EquityProvider.RefundApplyManage(context.Background(), &refundApplyManageRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	if len(rep.RefundApplyManage) != 0 {
		for k, v := range rep.RefundApplyManage {
			nameAndPhone := idMap[v.UserID]
			rep.RefundApplyManage[k].Name = nameAndPhone.Name
			rep.RefundApplyManage[k].Phone = nameAndPhone.Phone
		}
	} else {
		service.ResponseQuickMsg(c, e.Failed, errors.New("所选条件无可导出数据").Error(), nil)
		return
	}

	columns := []string{"订单uid", "用户id", "申请人", "手机号", "充值卡号", "退款金额", "状态", "账号", "交易渠道", "申请时间", "交易流水号", "备注", "金额(总)"}
	var totalPrice float32

	var data []interface{}
	// 添加数据行
	for k, refundApplyManage := range rep.RefundApplyManage {
		var info []string
		info = append(info, refundApplyManage.Uid)                        // 订单uid
		info = append(info, fmt.Sprintf("%d", refundApplyManage.UserID))  // 用户id
		info = append(info, refundApplyManage.Name)                       // 申请人
		info = append(info, refundApplyManage.Phone)                      // 手机号
		info = append(info, refundApplyManage.RefillCardNumber)           // 充值卡号
		info = append(info, fmt.Sprintf("%.2f", refundApplyManage.Money)) // 退款金额
		if refundApplyManage.RefundStatus == 2 {                          // 状态
			info = append(info, "退款中")
		} else if refundApplyManage.RefundStatus == 3 {
			info = append(info, "已退款")
		} else if refundApplyManage.RefundStatus == 4 {
			info = append(info, "已驳回")
		} else {
			info = append(info, "未知状态")
		}

		info = append(info, refundApplyManage.OpenID) // 账号

		if refundApplyManage.PaymentPlatform == 1 { // 交易渠道
			info = append(info, "微信支付")
		} else if refundApplyManage.PaymentPlatform == 2 {
			info = append(info, "支付宝支付")
		} else {
			info = append(info, "未知支付平台")
		}

		info = append(info, refundApplyManage.RefundTimeFormat)   // 申请时间
		info = append(info, refundApplyManage.RefundSerialNumber) // 交易流水号
		info = append(info, refundApplyManage.Remark)             // 备注

		totalPrice = totalPrice + refundApplyManage.Money
		if k == len(rep.RefundApplyManage)-1 {
			info = append(info, fmt.Sprintf("%.2f", totalPrice)) //总价 以元为单位
		}
		data = append(data, &info)
	}

	filePath := "./runtime/充值卡退款申请订单表.xlsx"
	_, err = utils.ToExcelByType(columns, data, "slice", filePath)
	if err != nil {
		fmt.Println("err", err)
		service.ResponseQuickMsg(c, e.Failed, errors.New("转换为excel类型出错").Error(), nil)
		return
	}

	var httpType string
	if config.IsHttps {
		httpType = model.HttpsType
	} else {
		httpType = model.HttpType
	}
	var exportUrl = fmt.Sprintf("%s%s/static/充值卡退款申请订单表.xlsx", httpType, c.Request.Host)

	repEF := &equity.RefundApplyManageExportFormRespond{
		ExportUrl: exportUrl,
	}

	service.Success(c, repEF)

	return

}

func RefundAllow(c *gin.Context) {
	var refundAllowRequest equity.RefundAllowRequest
	if err := c.ShouldBind(&refundAllowRequest); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	// 获取操作人员姓名
	if mLoginInfoAny, exists := c.Get("mLoginInfo"); exists {
		userInfo := mLoginInfoAny.(model.LoginInfo)
		refundAllowRequest.UserID = int32(userInfo.ID)
	} else {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   nil,
			Msg:    errors.New("未获取到用户信息").Error(),
			Status: e.Failed,
		})
		return
	}

	//refundAllowRequest.UserID = 88

	rep, err := service.EquityProvider.RefundAllow(context.Background(), &refundAllowRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	// 退款
	if rep.PaymentPlatform == 1 { // 微信退款
		req := order.WechatJsApiRefundsRequest{
			Reason:      "用户申请退款",                                                        // 选填 退款原因
			OutTradeNo:  rep.OutTradNo,                                                   // 商户订单号  （商户订单号和商户退款单号相同）
			OutRefundNo: rep.OutRefundNo,                                                 // 商户退款单号
			NotifyUrl:   "https://appointtest2.szjixun.cn/appointment/pay/SNskAPyadfk22", //选填 退款结果回调url
			Cent:        int64(rep.Cent),                                                 // 退款金额
			Total:       int64(rep.Cent),                                                 // 原订单金额
		}

		res, err := service.OrderProvider.WechatJsApiRefunds(context.Background(), &req)
		if err != nil {
			service.Error(c, e.Error, err)
			return
		}

		service.Success(c, res)

		return

	} else if rep.PaymentPlatform == 2 { // 支付宝退款

		//支付宝创建订单
		payCreateObj := &order.AliReFundRequest{
			OutTradeNo:  rep.OutTradNo, // 商户网站唯一订单号，用以标识一次支付请求
			TotalAmount: rep.Cent,      // 支付金额 (分)
		}

		res, err := service.OrderProvider.AliReFund(c, payCreateObj)

		if err != nil {
			service.Error(c, e.Error, err)
			return
		}

		service.Success(c, res)
		return

	} else {
		service.Error(c, e.InvalidParams, errors.New("未获取到支付类型"))
	}

	return

}

func AdminTopUpResource(c *gin.Context) {
	var adminTopUpResourceRequest equity.AdminTopUpResourceRequest
	if err := c.ShouldBind(&adminTopUpResourceRequest); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	//先进行权益修改
	freeResources := &equity.FreeResources{FreeType: 3, Resource: adminTopUpResourceRequest.Resource}

	_, err := service.EquityProvider.UpdateFreeResources(context.Background(), &equity.UpdateFreeResourcesRequest{UpdateType: 2, FreeResources: []*equity.FreeResources{freeResources}})
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	adminTopUpResourceRespond := &equity.AdminTopUpResourceRespond{}
	var errDesc string
	for _, v := range adminTopUpResourceRequest.UserID {
		rep, err := service.EquityProvider.UserGetResource(context.Background(), &equity.UserGetResourceRequest{UserID: v, ResourcesType: 3})
		if err != nil {
			// 根据用户id获取到用户姓名
			//根据姓名进行模糊插叙
			infoRequest := account.InfoRequest{
				Domain: "blockchain",
				ID:     uint64(v),
			}
			repList, err := service.AccountProvider.Info(context.Background(), &infoRequest)
			if err != nil {
				errDesc += fmt.Sprintf("用户id" + strconv.Itoa(int(v)) + ":" + err.Error() + "  ")
			} else {
				errDesc += fmt.Sprintf("用户姓名" + repList.Info.NickName + ":" + err.Error() + "  ")
			}

		} else if rep != nil {
			adminTopUpResource := &equity.AdminTopUpResource{
				Uid:           rep.Uid,
				GetResource:   rep.GetResource,
				ExistResource: rep.ExistResource,
			}

			adminTopUpResourceRespond.AdminTopUpResource = append(adminTopUpResourceRespond.AdminTopUpResource, adminTopUpResource)
		}
	}

	if errDesc == "" {
		service.Success(c, adminTopUpResourceRespond)
	} else {
		service.Error(c, e.Error, errors.New(errDesc))
	}

	return
}

func WxRefund(c *gin.Context) {

	transaction, err := pay.WechatNotify(c)
	if err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	if *transaction.TradeState != "SUCCESS" { // 是否成功
		service.Success(c, nil)
		return
	}

	fmt.Println("OutTradeNo", *transaction.OutTradeNo)

	rep, err := service.EquityProvider.WxRefund(context.Background(), &equity.WxRefundRequest{OutTradNo: *transaction.OutTradeNo})
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}
	service.Success(c, rep)

}

func AliRefund(c *gin.Context) {

	body, _ := ioutil.ReadAll(c.Request.Body)

	req := &order.AliNotifyRequest{
		RawQuery: string(body),
	}

	detail, err := service.OrderProvider.AliNotify(c, req)

	fmt.Println(detail, err)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	if detail.TradeStatus != "TRADE_SUCCESS" { //支付不是成功状态
		service.Error(c, e.Error, err)
		return
	}

	rep, err := service.EquityProvider.AliRefund(context.Background(), &equity.AliRefundRequest{OutTradNo: detail.OutTradeNo})
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}
	service.Success(c, rep)

}

func RefundRefuse(c *gin.Context) {
	var refundRefuseRequest equity.RefundRefuseRequest
	if err := c.ShouldBind(&refundRefuseRequest); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	rep, err := service.EquityProvider.RefundRefuse(context.Background(), &refundRefuseRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, rep)

	return

}

func GetAllAddType(c *gin.Context) {
	var getAllAddTypeRequest equity.GetAllAddTypeRequest
	if err := c.ShouldBind(&getAllAddTypeRequest); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	rep, err := service.EquityProvider.GetAllAddType(context.Background(), &getAllAddTypeRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, rep)

	return

}

func ResourceUsageDetailAddManage(c *gin.Context) {
	var resourceUsageDetailAddManageRequest equity.ResourceUsageDetailAddManageRequest
	if err := c.ShouldBind(&resourceUsageDetailAddManageRequest); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	idMap := make(map[int32]NameAndPhone)
	var id []int32

	//根据姓名进行模糊插叙
	listRequest := account.ListRequest{
		Domain: "blockchain",
		Key:    resourceUsageDetailAddManageRequest.Name,
	}
	repList, err := service.AccountProvider.List(context.Background(), &listRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	if len(repList.Data) == 0 {
		rep := &equity.ResourceUsageDetailAddManageRespond{}
		service.Success(c, rep)
		return
	}

	for _, v := range repList.Data {
		nameAndPhone := NameAndPhone{
			Name:  v.NickName,
			Phone: v.TelNum,
		}

		idMap[int32(v.ID)] = nameAndPhone
		id = append(id, int32(v.ID))
	}

	resourceUsageDetailAddManageRequest.UserID = id

	rep, err := service.EquityProvider.ResourceUsageDetailAddManage(context.Background(), &resourceUsageDetailAddManageRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	if len(rep.ResourceEquityAdd) != 0 {
		for k, v := range rep.ResourceEquityAdd {
			nameAndPhone := idMap[v.UserID]
			rep.ResourceEquityAdd[k].Name = nameAndPhone.Name
			rep.ResourceEquityAdd[k].Phone = nameAndPhone.Phone
		}
	}

	service.Success(c, rep)

	return

}

func ResourceUsageDetailAddManageExportForm(c *gin.Context) {
	var resourceUsageDetailAddManageRequest equity.ResourceUsageDetailAddManageRequest
	if err := c.ShouldBind(&resourceUsageDetailAddManageRequest); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	idMap := make(map[int32]NameAndPhone)
	var id []int32

	//根据姓名进行模糊插叙
	listRequest := account.ListRequest{
		Domain: "blockchain",
		Key:    resourceUsageDetailAddManageRequest.Name,
	}
	repList, err := service.AccountProvider.List(context.Background(), &listRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	if len(repList.Data) == 0 {
		rep := &equity.ResourceUsageDetailAddManageRespond{}
		service.Success(c, rep)
		return
	}

	for _, v := range repList.Data {
		nameAndPhone := NameAndPhone{
			Name:  v.NickName,
			Phone: v.TelNum,
		}

		idMap[int32(v.ID)] = nameAndPhone
		id = append(id, int32(v.ID))
	}

	resourceUsageDetailAddManageRequest.UserID = id
	resourceUsageDetailAddManageRequest.Page = 1
	resourceUsageDetailAddManageRequest.PageSize = 9999

	rep, err := service.EquityProvider.ResourceUsageDetailAddManage(context.Background(), &resourceUsageDetailAddManageRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	if len(rep.ResourceEquityAdd) != 0 {
		for k, v := range rep.ResourceEquityAdd {
			nameAndPhone := idMap[v.UserID]
			rep.ResourceEquityAdd[k].Name = nameAndPhone.Name
			rep.ResourceEquityAdd[k].Phone = nameAndPhone.Phone
		}
	} else {
		service.ResponseQuickMsg(c, e.Failed, errors.New("所选条件无可导出数据").Error(), nil)
		return
	}

	columns := []string{"订单uid", "用户id", "付款人", "手机号", "兑换流水号", "充值卡号", "兑换消耗消费卡", "兑换类型", "所得权益", "兑换时间", "总金额", "总权益值"}
	var totalPrice float32
	var totalResource int32

	var data []interface{}
	// 添加数据行
	for k, resourceEquityAdd := range rep.ResourceEquityAdd {
		var info []string
		info = append(info, resourceEquityAdd.Uid)                        // 订单uid
		info = append(info, fmt.Sprintf("%d", resourceEquityAdd.UserID))  // 用户id
		info = append(info, resourceEquityAdd.Name)                       // 付款人
		info = append(info, resourceEquityAdd.Phone)                      // 手机号
		info = append(info, resourceEquityAdd.SerialNumber)               // 兑换流水号
		info = append(info, resourceEquityAdd.RefillCardNumber)           // 充值卡号
		info = append(info, fmt.Sprintf("%.2f", resourceEquityAdd.Money)) // 兑换消耗消费卡
		if resourceEquityAdd.ExchangeType == 6 {                          // 兑换类型
			info = append(info, "手动兑换")
		} else if resourceEquityAdd.ExchangeType == 5 {
			info = append(info, "自动兑换")
		} else if resourceEquityAdd.ExchangeType == 4 {
			info = append(info, "系统赠送(实名认证)")
		} else if resourceEquityAdd.ExchangeType == 3 {
			info = append(info, "系统赠送(注册)")
		} else {
			info = append(info, "未知状态")
		}

		info = append(info, fmt.Sprintf("%d", resourceEquityAdd.Resource)) // 所得权益
		info = append(info, resourceEquityAdd.ExchangeTimeFormat)          // 兑换时间

		totalPrice = totalPrice + resourceEquityAdd.Money
		totalResource = totalResource + resourceEquityAdd.Resource
		if k == len(rep.ResourceEquityAdd)-1 {
			info = append(info, fmt.Sprintf("%.2f", totalPrice))  //总金额 以元为单位
			info = append(info, fmt.Sprintf("%d", totalResource)) //总权益值
		}
		data = append(data, &info)
	}

	filePath := "./runtime/资源权益管理表.xlsx"
	_, err = utils.ToExcelByType(columns, data, "slice", filePath)
	if err != nil {
		fmt.Println("err", err)
		service.ResponseQuickMsg(c, e.Failed, errors.New("转换为excel类型出错").Error(), nil)
		return
	}

	var httpType string
	if config.IsHttps {
		httpType = model.HttpsType
	} else {
		httpType = model.HttpType
	}
	var exportUrl = fmt.Sprintf("%s%s/static/资源权益管理表.xlsx", httpType, c.Request.Host)

	repEF := &equity.RefundApplyManageExportFormRespond{
		ExportUrl: exportUrl,
	}

	service.Success(c, repEF)

	return

}

func GetAllReduceType(c *gin.Context) {
	var getAllReduceTypeRequest equity.GetAllReduceTypeRequest
	if err := c.ShouldBind(&getAllReduceTypeRequest); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	rep, err := service.EquityProvider.GetAllReduceType(context.Background(), &getAllReduceTypeRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, rep)

	return

}

func ResourceUsageDetailReduceManage(c *gin.Context) {
	var resourceUsageDetailReduceManageRequest equity.ResourceUsageDetailReduceManageRequest
	if err := c.ShouldBind(&resourceUsageDetailReduceManageRequest); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	idMap := make(map[int32]NameAndPhone)
	var id []int32

	//根据姓名进行模糊插叙
	listRequest := account.ListRequest{
		Domain: "blockchain",
		Key:    resourceUsageDetailReduceManageRequest.Name,
	}
	repList, err := service.AccountProvider.List(context.Background(), &listRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	if len(repList.Data) == 0 {
		rep := &equity.ResourceUsageDetailReduceManageRespond{}
		service.Success(c, rep)
		return
	}

	for _, v := range repList.Data {
		nameAndPhone := NameAndPhone{
			Name:  v.NickName,
			Phone: v.TelNum,
		}

		idMap[int32(v.ID)] = nameAndPhone
		id = append(id, int32(v.ID))
	}

	resourceUsageDetailReduceManageRequest.UserID = id

	rep, err := service.EquityProvider.ResourceUsageDetailReduceManage(context.Background(), &resourceUsageDetailReduceManageRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	if len(rep.ResourceReduceEquity) != 0 {
		for k, v := range rep.ResourceReduceEquity {
			nameAndPhone := idMap[v.UserID]
			rep.ResourceReduceEquity[k].Name = nameAndPhone.Name
			rep.ResourceReduceEquity[k].Phone = nameAndPhone.Phone
		}
	}

	service.Success(c, rep)

	return

}

func ResourceUsageDetailReduceManageExportForm(c *gin.Context) {
	var resourceUsageDetailReduceManageRequest equity.ResourceUsageDetailReduceManageRequest
	if err := c.ShouldBind(&resourceUsageDetailReduceManageRequest); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	idMap := make(map[int32]NameAndPhone)
	var id []int32

	//根据姓名进行模糊插叙
	listRequest := account.ListRequest{
		Domain: "blockchain",
		Key:    resourceUsageDetailReduceManageRequest.Name,
	}
	repList, err := service.AccountProvider.List(context.Background(), &listRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	if len(repList.Data) == 0 {
		rep := &equity.ResourceUsageDetailReduceManageRespond{}
		service.Success(c, rep)
		return
	}

	for _, v := range repList.Data {
		nameAndPhone := NameAndPhone{
			Name:  v.NickName,
			Phone: v.TelNum,
		}

		idMap[int32(v.ID)] = nameAndPhone
		id = append(id, int32(v.ID))
	}

	resourceUsageDetailReduceManageRequest.UserID = id
	resourceUsageDetailReduceManageRequest.Page = 1
	resourceUsageDetailReduceManageRequest.PageSize = 9999

	rep, err := service.EquityProvider.ResourceUsageDetailReduceManage(context.Background(), &resourceUsageDetailReduceManageRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	if len(rep.ResourceReduceEquity) != 0 {
		for k, v := range rep.ResourceReduceEquity {
			nameAndPhone := idMap[v.UserID]
			rep.ResourceReduceEquity[k].Name = nameAndPhone.Name
			rep.ResourceReduceEquity[k].Phone = nameAndPhone.Phone
		}
	} else {
		service.ResponseQuickMsg(c, e.Failed, errors.New("所选条件无可导出数据").Error(), nil)
		return
	}

	columns := []string{"订单uid", "用户id", "付款人", "手机号", "消耗流水号", "资源权益消耗数（丰）", "类型", "存证hash", "操作时间", "总资源权益消耗数（丰）"}
	var totalResource int32

	var data []interface{}
	// 添加数据行
	for k, resourceReduceEquity := range rep.ResourceReduceEquity {
		var info []string
		info = append(info, resourceReduceEquity.Uid)                         // 订单uid
		info = append(info, fmt.Sprintf("%d", resourceReduceEquity.UserID))   // 用户id
		info = append(info, resourceReduceEquity.Name)                        // 付款人
		info = append(info, resourceReduceEquity.Phone)                       // 手机号
		info = append(info, resourceReduceEquity.SerialNumber)                // 消耗流水号
		info = append(info, fmt.Sprintf("%d", resourceReduceEquity.Resource)) // 资源权益消耗数（丰）
		if resourceReduceEquity.ReduceType == 1 {                             // 类型
			info = append(info, "普通存证")
		} else if resourceReduceEquity.ReduceType == 2 {
			info = append(info, "隐私存证")
		} else {
			info = append(info, "未知状态")
		}

		info = append(info, resourceReduceEquity.TxHash)          // 存证hash
		info = append(info, resourceReduceEquity.UsageTimeFormat) // 操作时间

		totalResource = totalResource + resourceReduceEquity.Resource
		if k == len(rep.ResourceReduceEquity)-1 {
			info = append(info, fmt.Sprintf("%d", totalResource)) //总权益值
		}
		data = append(data, &info)
	}

	filePath := "./runtime/权益资源使用明细表.xlsx"
	_, err = utils.ToExcelByType(columns, data, "slice", filePath)
	if err != nil {
		fmt.Println("err", err)
		service.ResponseQuickMsg(c, e.Failed, errors.New("转换为excel类型出错").Error(), nil)
		return
	}

	var httpType string
	if config.IsHttps {
		httpType = model.HttpsType
	} else {
		httpType = model.HttpType
	}
	var exportUrl = fmt.Sprintf("%s%s/static/权益资源使用明细表.xlsx", httpType, c.Request.Host)

	repEF := &equity.ResourceUsageDetailReduceManageExportFormRespond{
		ExportUrl: exportUrl,
	}

	service.Success(c, repEF)

	return

}

func GetResourcesByUserIds(c *gin.Context) {
	var getResourcesByUserIdsRequest equity.GetResourcesByUserIdsRequest
	if err := c.ShouldBind(&getResourcesByUserIdsRequest); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	rep, err := service.EquityProvider.GetResourcesByUserIds(context.Background(), &getResourcesByUserIdsRequest)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, rep)

	return

}
