package service_copyright

import (
	"context"
	"github.com/fonchain_enterprise/fonchain-main/api/dci"
	"github.com/fonchain_enterprise/fonchain-main/api/digital_copyright"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	loigc_copyright "github.com/fonchain_enterprise/fonchain-main/pkg/logic/copyright"
	model_copyright "github.com/fonchain_enterprise/fonchain-main/pkg/model/copyright"
	pkgRequest "github.com/fonchain_enterprise/fonchain-main/pkg/request"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
	"time"
)

func UpdateUser(ctx *gin.Context) {
	var (
		err         error
		req         pkgRequest.DciUpdateUser
		updateResp  *dci.UpdateDciUserResponse
		dciUserInfo *digital_copyright.DciUserInfoResp
		//queryDciInfo         *dci.QueryDciUserResponse
		cardFaceWordFileId   string
		cardNationWordFileId string
	)
	var newDciUserInfo = &digital_copyright.AddDciUserRequest{}
	if err = ctx.ShouldBind(&req); err != nil {
		zap.L().Error("UpdateUser err ShouldBind", zap.Error(err))
		service.ResponseQuickMsg(ctx, e.Failed, e.GetMsg(e.InvalidParams), nil)
		return
	}
	if req.ArtistUuid == "" || req.CardFaceUrl == "" || req.CardNationalUrl == "" {
		service.ResponseQuickMsg(ctx, e.Failed, e.GetMsg(e.InvalidParams), nil)
		return
	}
	if dciUserInfo, err = service.GrpcCopyrightImpl.DciUserInfo(context.Background(), &digital_copyright.DciUserInfoReq{
		ArtistUuid: req.ArtistUuid,
	}); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	if dciUserInfo.DciUserId == "" {
		service.ResponseQuickMsg(ctx, e.Failed, e.ErrorDciUserNoReg, nil)
		return
	}
	defer func() {
		go UpdateDciUserInfo(newDciUserInfo)
	}()
	newDciUserInfo.ArtistUuid = req.ArtistUuid
	newDciUserInfo.CertificateNumber = dciUserInfo.CertificateNumber
	newDciUserInfo.Phone = dciUserInfo.Phone
	if dciUserInfo.CardFaceUrl == req.CardFaceUrl && dciUserInfo.CardNationUrl == req.CardNationalUrl {
		service.ResponseQuickMsg(ctx, e.Failed, e.ErrorDciNoNeedUpdate, nil)
		return
	}
	var logicDciUser = new(loigc_copyright.DciUser)
	if cardFaceWordFileId, err = logicDciUser.DciUploadUrl(req.CardFaceUrl); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	if cardNationWordFileId, err = logicDciUser.DciUploadUrl(req.CardNationalUrl); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	dciUpdateReq := dci.UpdateDciUserRequest{
		DciUserId:       dciUserInfo.DciUserId,
		CertFrontFileId: cardFaceWordFileId,
		CertBackFileId:  cardNationWordFileId,
	}
	//if queryDciInfo, err = service.GrpcDciImpl.QueryDciUser(context.Background(), &dci.QueryDciUserRequest{
	//	CertificateType:   "IDENTITY_CARD",
	//	CertificateNumber: dciUserInfo.CertificateNumber,
	//	Phone:             dciUserInfo.Phone,
	//}); err != nil {
	//	service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
	//	return
	//}
	//fmt.Println(queryDciInfo)
	if updateResp, err = service.GrpcDciImpl.UpdateDciUser(context.Background(), &dciUpdateReq); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	if updateResp.ResultCode != "OK" {
		service.ResponseQuickMsg(ctx, e.Failed, model_copyright.DciUserResultMsg[updateResp.ResultMsg], nil)
		return
	}
	newDciUserInfo.CardNationUrl = req.CardNationalUrl
	newDciUserInfo.CardFaceUrl = req.CardFaceUrl
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), nil)
	return
}

func DciUserInfo(ctx *gin.Context) {
	artistUuid := ctx.PostForm("artistUuid")
	userInfo, err := service.GrpcCopyrightImpl.DciUserInfo(context.Background(), &digital_copyright.DciUserInfoReq{
		ArtistUuid: artistUuid,
	})
	if err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	dciResp, _err := service.GrpcDciImpl.QueryDciUser(context.Background(), &dci.QueryDciUserRequest{
		CertificateType:   "IDENTITY_CARD",
		CertificateNumber: userInfo.CertificateNumber,
		Phone:             userInfo.Phone,
	})
	if _err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, _err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), dciResp)
	return
}

func UpdateDciUserInfo(dciUserInfo *digital_copyright.AddDciUserRequest) {
	time.Sleep(time.Second * 2)
	if dciUserInfo.ArtistUuid == "" || dciUserInfo.CertificateNumber == "" || dciUserInfo.Phone == "" {
		return
	}
	dciUserResp, err := service.GrpcDciImpl.QueryDciUser(context.Background(), &dci.QueryDciUserRequest{
		CertificateType:   "IDENTITY_CARD",
		CertificateNumber: dciUserInfo.CertificateNumber,
		Phone:             dciUserInfo.Phone,
	})
	var dciUserStatus string
	if err == nil && dciUserResp.ResultCode == "OK" {
		dciUserStatus = dciUserResp.DciUserStatus
	}
	if _, err = service.GrpcCopyrightImpl.UpdateDciUser(context.Background(), &digital_copyright.AddDciUserRequest{
		ArtistUuid:    dciUserInfo.ArtistUuid,
		CardFaceUrl:   dciUserInfo.CardFaceUrl,
		CardNationUrl: dciUserInfo.CardNationUrl,
		DciUserStatus: dciUserStatus,
	}); err != nil {
		return
	}
}
