package service_copyright

import (
	"context"
	"encoding/base64"
	"encoding/json"
	"fmt"
	"github.com/fonchain_enterprise/fonchain-main/pkg/cache"
	"github.com/google/uuid"
	"net/url"
	"sort"
	"strings"
	"sync"
	"time"

	"github.com/fonchain_enterprise/fonchain-main/api/dci"
	"github.com/fonchain_enterprise/fonchain-main/api/digital_copyright"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	loigc_copyright "github.com/fonchain_enterprise/fonchain-main/pkg/logic/copyright"
	model_copyright "github.com/fonchain_enterprise/fonchain-main/pkg/model/copyright"
	dciModel "github.com/fonchain_enterprise/fonchain-main/pkg/model/dci"
	pkgRequest "github.com/fonchain_enterprise/fonchain-main/pkg/request"
	pkgResponse "github.com/fonchain_enterprise/fonchain-main/pkg/response"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/gin-gonic/gin"
	"github.com/jinzhu/copier"
	"go.uber.org/zap"
	"google.golang.org/protobuf/types/known/emptypb"
)

// 数登申请
func DciRegistration(ctx *gin.Context) {
	var (
		err        error
		createReq  pkgRequest.CopyrightIdAll
		createResp *dci.CreateDciRegistrationResponse
	)
	var logicDigital = new(loigc_copyright.Digital)
	if err = ctx.ShouldBind(&createReq); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	if createResp, err = logicDigital.DciRegistration(createReq.CopyrightId); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), createResp)
	return
}

// 查询数登
func DciQueryRegistration(ctx *gin.Context) {
	var (
		err              error
		idAll            pkgRequest.CopyrightIdAll
		digitalQueryResp *dci.QueryDciRegistrationResponse
	)
	var logicDigital = new(loigc_copyright.Digital)
	if err = ctx.ShouldBind(&idAll); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	if digitalQueryResp, err = logicDigital.DciQueryRegistration(idAll, model_copyright.CopyrightInfo{}); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), digitalQueryResp)
	return
}

func DciGetPayUrl(ctx *gin.Context) {
	var (
		err        error
		createReq  pkgRequest.CopyrightIdAll
		payUrlResp *dci.GetDciPayUrlResponse
	)
	var logicDigital = new(loigc_copyright.Digital)
	if err = ctx.ShouldBind(&createReq); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	zap.L().Info("DciGetPayUrl 3")
	if payUrlResp, err = logicDigital.DciGetPayUrl(createReq); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	if payUrlResp.ResultCode != "OK" {
		msg := model_copyright.DciUserResultMsg[payUrlResp.ResultMsg]
		if msg == "" {
			msg = e.ErrorUnknown
		}
		service.ResponseQuickMsg(ctx, e.Failed, msg, map[string]string{
			"payUrl": "",
		})
		return
	}
	var payUrl string = payUrlResp.PayUrl
	if payUrl == "" {
		service.ResponseQuickMsg(ctx, e.Failed, e.ErrorGetPayUrl, map[string]string{
			"payUrl": payUrl,
		})
		return
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), map[string]string{
		"payUrl": payUrl,
	})
	return
}

func DciGetPayQuery(ctx *gin.Context) {
	var (
		err          error
		createReq    pkgRequest.CopyrightIdAll
		queryPayResp *dci.QueryDciPayResponse
	)
	var logicDigital = new(loigc_copyright.Digital)
	if err = ctx.ShouldBind(&createReq); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	if queryPayResp, err = logicDigital.DciGetPayQuery(createReq); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), queryPayResp)
	return
}

func DciGetRegistrationCert(ctx *gin.Context) {
	var (
		err       error
		url1      string
		url2      string
		createReq pkgRequest.CopyrightIdAll
		//certResp  *dci.GetDciRegistrationcertResponse
	)
	var logicDigital = new(loigc_copyright.Digital)
	if err = ctx.ShouldBind(&createReq); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	var copyrightInfo model_copyright.CopyrightInfo
	_ = copier.Copy(&copyrightInfo, createReq)
	if url1, url2, err = logicDigital.DciGetRegistrationCert(copyrightInfo, false); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), map[string]string{
		"url1": url1,
		"url2": url2,
	})
	return
}

func RetryDigitalRegister(ctx *gin.Context) {
	var (
		err error
		req *digital_copyright.RetryReq
	)
	var logicDigital = new(loigc_copyright.Digital)
	if err = ctx.ShouldBind(&req); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	if err = logicDigital.RetryDigitalRegister(ctx, req); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), nil)
	return
}

func CloseDigitalRegister(ctx *gin.Context) {
	var (
		err error
		req *dci.CloseDciRegistrationRequest
	)
	var logicDigital = new(loigc_copyright.Digital)
	if err = ctx.ShouldBind(&req); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	if err = logicDigital.CloseDigitalRegister(req); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), nil)
	return
}

func UnpackCallback(ctx *gin.Context) {
	var Info struct {
		ReturnAll string `json:"returnAll"`
	}
	var decodedURL string
	info := &Info
	_ = ctx.BindJSON(&info)
	decoded, err := base64.StdEncoding.DecodeString(info.ReturnAll)
	if err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), info)
		return
	}
	restoredMap := new(dciModel.Callback)
	// 反序列化为 map[string]interface{}
	//var restoredMap map[string]interface{}
	decodedURL, err = url.QueryUnescape(string(decoded))
	err = json.Unmarshal([]byte(decodedURL), &restoredMap)
	if err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), info)
		return
	}
	service.ResponseQuickMsg(ctx, e.Ok, "", info)
	return
}

func DataSummary(ctx *gin.Context) {
	resp, err := service.GrpcCopyrightImpl.DataSummary(ctx, &digital_copyright.DataSummaryReq{})
	if err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	//查询转让
	transferResp, _err := service.GrpcArtworkQueryImpl.TotalTransfer(context.Background(), &emptypb.Empty{})
	if _err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, _err.Error(), nil)
		return
	}
	resp.CrTransferDoneNum = transferResp.TotalNum
	resp.CrRegisterInfo.OfflineTotalNum = transferResp.TotalOfflineCopyrightNum

	type StatusInfo struct {
		Status     string `json:"status"`
		PrecessNum uint8  `json:"precessNum"`
		Redirect   string `json:"redirect"`
	}
	var agentStatusData []StatusInfo
	var statusSlice []string = []string{
		"WAIT_SUBMIT",
		"COPYRIGHT_CHECK",
		"DIGITAL_CHECK",
		"DIGITAL_WAIT_PAY",
		"DIGITAL_REVIEW",
		//"COMPLETED",
		"ALL_AMEND",
		//"ALL_END"
	}
	for _, status := range statusSlice {
		var statusInfo1 StatusInfo
		statusInfo1.Status = status
		statusInfo1.PrecessNum = 0
		statusInfo1.Redirect = status
		agentStatusData = append(agentStatusData, statusInfo1)
	}
	respStatus, _err := service.GrpcCopyrightImpl.StatusInfo(context.Background(), &digital_copyright.StatusInfoReq{CopyrightId: ""})
	if _err != nil {
		_ = copier.Copy(&resp.AgentStatusData, agentStatusData)
		_ = copier.Copy(&resp.ArtistStatusData, agentStatusData)
		return
	}
	for _, v := range respStatus.Data {
		var temp = digital_copyright.DataSummaryResp_StatusInfo{}
		if strings.Contains("COMPLETED,ALL_END", v.Status) {
			continue
		}
		temp.PrecessNum = v.PrecessNum
		temp.Status = v.Status
		temp.Redirect = v.Status
		resp.AgentStatusData = append(resp.AgentStatusData, &temp)
	}
	//err = copier.CopyWithOption(&resp.AgentStatusData, respStatus.Data, opt)
	//_ = copier.Copy(&resp.AgentStatusData, respStatus.Data)
	fmt.Println(err)
	_ = copier.Copy(&resp.ArtistStatusData, agentStatusData)
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), resp)
	return
}

func CategoryDataSummary(ctx *gin.Context) {
	var req = digital_copyright.CategoryDataSummaryReq{}
	if err := ctx.ShouldBind(&req); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	resp, err := service.GrpcCopyrightImpl.CategoryDataSummary(ctx, &req)
	if err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	type Info struct {
		WorkCategory string `json:"workCategory"`
		AgentNum     int32  `json:"agentNum"`
		ArtistNum    int32  `json:"artistNum"`
	}
	var data []*Info
	var mm map[string]*Info = map[string]*Info{}
	for _, v := range resp.Data {
		c, ok := mm[v.WorkCategory]
		if !ok {
			mm[v.WorkCategory] = &Info{}
			c = &Info{}
		}
		c.WorkCategory = v.WorkCategory
		if v.Source == 1 {
			c.AgentNum = v.DoneNum
		}
		if v.Source == 2 {
			c.ArtistNum = v.DoneNum
		}
		mm[v.WorkCategory] = c
	}
	for _, v := range mm {
		data = append(data, v)
	}
	sort.Slice(data, func(i, j int) bool {
		return data[i].WorkCategory < data[j].WorkCategory
	})
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), data)
	return
}

func dateRange(startDate, endDate string, dateFormat string) ([]string, error) {
	start, err := time.Parse(dateFormat, startDate)
	if err != nil {
		return nil, err
	}

	end, err := time.Parse(dateFormat, endDate)
	if err != nil {
		return nil, err
	}

	dates := []string{}

	for d := start; !d.After(end); d = d.AddDate(0, 0, 1) {
		dates = append(dates, d.Format(dateFormat))
	}

	return dates, nil
}

func CategoryDaySummary(ctx *gin.Context) {
	var req = digital_copyright.CategoryDaySummaryReq{}
	if err := ctx.ShouldBind(&req); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	resp, err := service.GrpcCopyrightImpl.CategoryDaySummary(ctx, &req)
	if err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}

	dates, _ := dateRange(req.StartDate, req.EndDate, "2006-01-02")
	for _, date := range dates {
		var exists bool
		for _, v := range resp.Data {
			if v.DayDate == date {
				exists = true
				break
			}
		}
		if !exists {
			var info = digital_copyright.CategoryDaySummaryResp_Info{}
			info.DayDate = date
			info.WorkCategory = req.WorkCategory
			info.DoneNum = 0
			resp.Data = append(resp.Data, &info)
		}
	}
	sort.Slice(resp.Data, func(i, j int) bool {
		return resp.Data[i].DayDate < resp.Data[j].DayDate
	})
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), resp.Data)
	return
}

func AllStatusInfo(ctx *gin.Context) {
	type StatusInfo struct {
		Status     string `json:"status"`
		PrecessNum uint8  `json:"precessNum"`
	}
	var data = make(map[string]interface{}, 2)
	var agentStatusData []StatusInfo
	var statusSlice []string = []string{
		"WAIT_SUBMIT",
		"COPYRIGHT_CHECK",
		"DIGITAL_CHECK",
		"DIGITAL_WAIT_PAY",
		"DIGITAL_REVIEW",
		"COMPLETED",
		"ALL_AMEND",
		"ALL_END"}
	for _, status := range statusSlice {
		var statusInfo1 StatusInfo
		statusInfo1.Status = status
		statusInfo1.PrecessNum = 0
		agentStatusData = append(agentStatusData, statusInfo1)
	}
	resp, err := service.GrpcCopyrightImpl.StatusInfo(context.Background(), &digital_copyright.StatusInfoReq{CopyrightId: ""})
	if err != nil {
		data["agentData"] = agentStatusData
		return
	}
	data["agentData"] = resp.Data
	data["artistData"] = agentStatusData
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), data)
	return
}

func BatchDci(ctx *gin.Context) {
	var (
		err      error
		batchReq *pkgRequest.BatchDciRequest
	)
	if err = ctx.ShouldBind(&batchReq); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, e.GetMsg(e.InvalidParams), nil)
		return
	}
	if batchReq.ArtworkUuids == nil || len(batchReq.ArtworkUuids) == 0 {
		service.ResponseQuickMsg(ctx, e.Failed, e.GetMsg(e.InvalidParams), nil)
		return
	}
	var resultChan = make(chan pkgResponse.DciCopyrightResult, len(batchReq.ArtworkUuids))
	var logicCopyright = new(loigc_copyright.Copyright)
	var failData []pkgResponse.DciCopyrightResult
	zap.L().Info("DciCopyright 100")
	newUuid := uuid.New().String()
	cacheKey := fmt.Sprintf("bacthDciCopyright_%s", newUuid)
	cacheAllKey := "batchDciCopyright"
	cache.RedisClient.SAdd(cacheAllKey, cacheKey)
	defer cache.RedisClient.Del(cacheKey)
	var data = map[string]interface{}{}
	if err = logicCopyright.CheckDciUserInfo(ctx, batchReq.ArtworkUuids); err != nil {
		failData = append(failData, pkgResponse.DciCopyrightResult{
			ArtworkName: "",
			Tfnum:       "",
			FailReason:  err.Error(),
		})
		data = map[string]interface{}{
			"failData": failData,
		}
		zap.L().Info("DciCopyright", zap.Any("failData", failData))
		zap.L().Error("DciCopyright", zap.Error(err))
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), data)
		return
	}
	zap.L().Info("DciCopyright 110", zap.Any("batchReq.ArtworkUuids", batchReq.ArtworkUuids))
	var wgp sync.WaitGroup
	for _, artworkUuid := range batchReq.ArtworkUuids {
		wgp.Add(1)
		go func(artworkUuid string) {
			defer wgp.Done()
			resultInfo := logicCopyright.DciCopyright(artworkUuid, ctx)
			resultChan <- resultInfo
		}(artworkUuid)
	}
	go func() {
		wgp.Wait()
		close(resultChan)
	}()
	zap.L().Info("DciCopyright 120", zap.Any("batchReq.ArtworkUuids", batchReq.ArtworkUuids))
	cache.RedisClient.SAdd(cacheKey, batchReq.ArtworkUuids)
	cache.RedisClient.Expire(cacheKey, time.Second*5*60)
	for resultInfo := range resultChan {
		zap.L().Info("DciCopyright 200", zap.Any("resultInfo", resultInfo))
		if resultInfo.FailReason != "" {
			failData = append(failData, resultInfo)
		}
	}

	zap.L().Info("DciCopyright 200")
	data = map[string]interface{}{
		"failData": failData,
	}
	if len(failData) == 0 {
		service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), data)
		return
	}
	service.ResponseQuickMsg(ctx, e.Failed, e.GetMsg(e.Failed), data)
	return
}

func Test(ctx *gin.Context) {
	return
}
