package service_copyright

import (
	"context"
	"github.com/fonchain_enterprise/fonchain-main/api/software_copyright"
	"github.com/fonchain_enterprise/fonchain-main/pkg/config"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
)

func SaveSoftwareCopyright(ctx *gin.Context) {
	var (
		saveReq *software_copyright.CopyrightInfo
		saveRes *software_copyright.SaveSoftwareCopyrightRes
		err     error
	)
	if err = ctx.ShouldBind(&saveReq); err != nil {
		zap.L().Error("SaveSoftwareCopyright err ShouldBind", zap.Error(err))
		service.ResponseQuickMsg(ctx, e.Failed, e.GetMsg(e.InvalidParams), nil)
		return
	}
	if config.Env != "dev" {
		if mLoginInfoAny, exists := ctx.Get("mLoginInfo"); exists {
			userInfo := mLoginInfoAny.(model.LoginInfo)
			saveReq.CreatedBy = userInfo.NickName
		}
	}
	if saveRes, err = service.GrpcSoftwareCopyrightImpl.SaveSoftwareCopyright(context.Background(), saveReq); err != nil {
		zap.L().Error("SaveSoftwareCopyright err Save", zap.Error(err))
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), saveRes)
	return
}

func SoftwareCopyrightList(ctx *gin.Context) {
	var (
		listReq *software_copyright.CopyrightListReq
		listRes *software_copyright.CopyrightListRes
		err     error
	)
	if err = ctx.ShouldBind(&listReq); err != nil {
		zap.L().Error("SoftwareCopyrightList err ShouldBind", zap.Error(err))
		service.ResponseQuickMsg(ctx, e.Failed, e.GetMsg(e.InvalidParams), nil)
		return
	}
	if listRes, err = service.GrpcSoftwareCopyrightImpl.SoftwareCopyrightList(context.Background(), listReq); err != nil {
		zap.L().Error("SoftwareCopyrightList err List", zap.Error(err))
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), listRes)
	return
}

func DelSoftwareCopyright(ctx *gin.Context) {
	var (
		delReq *software_copyright.DelReq
		delRes *software_copyright.DelRes
		err    error
	)
	if err = ctx.ShouldBind(&delReq); err != nil {
		zap.L().Error("DelSoftwareCopyright err ShouldBind", zap.Error(err))
		service.ResponseQuickMsg(ctx, e.Failed, e.GetMsg(e.InvalidParams), nil)
		return
	}
	if delRes, err = service.GrpcSoftwareCopyrightImpl.DelCopyright(context.Background(), delReq); err != nil {
		zap.L().Error("DelSoftwareCopyright err Delete", zap.Error(err))
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), delRes)
	return
}
