package service_copyright

import (
	"fmt"
	"github.com/fonchain_enterprise/fonchain-main/api/artist"
	"github.com/fonchain_enterprise/fonchain-main/api/artistInfoUser"
	"github.com/fonchain_enterprise/fonchain-main/api/digital_copyright"
	appConfig "github.com/fonchain_enterprise/fonchain-main/pkg/config"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	model_copyright "github.com/fonchain_enterprise/fonchain-main/pkg/model/copyright"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/gin-gonic/gin"
)

func UpdateSupplement(ctx *gin.Context) {
	var req = digital_copyright.UpdateSupplementReq{}
	if err := ctx.ShouldBind(&req); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	// 先去版权查询画家ID
	if req.CopyrightId != "" {
		copyrightResp, err := service.GrpcCopyrightImpl.CopyrightDetail(ctx, &digital_copyright.CopyrightDetailReq{CopyrightId: req.CopyrightId})
		if err != nil {
			service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
			return
		}
		if appConfig.Env != "dev" {
			// 判断画家是否在画家宝
			findUserResp, _err := service.GrpcArtistInfoUserImpl.FindUser(ctx, &artistInfoUser.FindUserRequest{MgmtArtistUid: copyrightResp.BaseInfo.ArtistUuid})
			if _err != nil {
				service.ResponseQuickMsg(ctx, e.Failed, _err.Error(), nil)
				return
			}
			if findUserResp.MgmtArtistUid == "" {
				service.ResponseQuickMsg(ctx, e.Failed, e.COPYRIGHT_NOT_IN_ARTIST_INFO_USER, nil)
				return
			}
		}
		req.OriginWorkFileUrl = copyrightResp.WorkInfo.WorkFileUrl
	}
	resp, err := service.GrpcCopyrightImpl.UpdateSupplement(ctx, &req)
	if err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), resp)
	return
}

func CopyrightInfo(ctx *gin.Context) {
	var req = model_copyright.CopyrightReq{}
	if err := ctx.ShouldBind(&req); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	detailResp, err := service.GrpcCopyrightImpl.CopyrightDetail(ctx, &digital_copyright.CopyrightDetailReq{CopyrightId: req.CopyrightId})
	if err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	// 判断画家是否在画家宝
	//if appConfig.Env != "dev" {
	findUserResp, _err := service.GrpcArtistInfoUserImpl.FindUser(ctx, &artistInfoUser.FindUserRequest{MgmtArtistUid: detailResp.BaseInfo.ArtistUuid})
	if _err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, _err.Error(), nil)
		return
	}
	if findUserResp.MgmtArtistUid == "" {
		service.ResponseQuickMsg(ctx, e.Failed, e.COPYRIGHT_NOT_IN_ARTIST_INFO_USER, nil)
		return
	}
	//}
	var tnum string
	resp := &digital_copyright.SupplementDetailResp{}
	if detailResp.BaseInfo.ArtistUuid != "" {
		artistInfoResp, _ := service.GrpcArtistImpl.ArtistInfo(ctx, &artist.ArtistInfoRequest{Uid: detailResp.BaseInfo.ArtistUuid})
		if artistInfoResp != nil && artistInfoResp.ProfileInfo != nil {
			tnum = artistInfoResp.ProfileInfo.Tnum
		}
	}
	resp = &digital_copyright.SupplementDetailResp{
		WorkName:    detailResp.BaseInfo.WorkName,
		Tfnum:       detailResp.WorkInfo.Tfnum,
		AuthorName:  detailResp.WorkInfo.AuthorName,
		Tnum:        tnum,
		WorkFileUrl: detailResp.WorkInfo.WorkFileUrl,
		CopyrightId: req.CopyrightId,
		ArtistUuid:  detailResp.BaseInfo.ArtistUuid,
		ArtworkUuid: detailResp.BaseInfo.ArtworkUuid,
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), resp)
	return
}

func SupplementDetail(ctx *gin.Context) {
	var req = digital_copyright.SupplementDetailReq{}
	if err := ctx.ShouldBind(&req); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	resp, err := service.GrpcCopyrightImpl.SupplementDetail(ctx, &req)
	if err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	if resp.ArtistUuid != "" {
		artistInfoResp, _ := service.GrpcArtistImpl.ArtistInfo(ctx, &artist.ArtistInfoRequest{Uid: resp.ArtistUuid})
		if artistInfoResp != nil && artistInfoResp.ProfileInfo != nil {
			resp.Tnum = artistInfoResp.ProfileInfo.Tnum
		}
	}
	fmt.Println(resp)
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), resp)
	return
}

func SupplementList(ctx *gin.Context) {
	var req = digital_copyright.SupplementListReq{}
	if err := ctx.ShouldBind(&req); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	resp, err := service.GrpcCopyrightImpl.SupplementList(ctx, &req)
	if err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), resp)
	return
}
