package service_copyright

import (
	"context"
	"github.com/fonchain_enterprise/fonchain-main/api/trademark"
	"github.com/fonchain_enterprise/fonchain-main/pkg/config"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
)

func SaveTrademark(ctx *gin.Context) {
	var (
		saveReq *trademark.TrademarkInfo
		saveRes *trademark.SaveTrademarkRes
		err     error
	)
	if err = ctx.ShouldBind(&saveReq); err != nil {
		zap.L().Error("SaveTrademark err ShouldBind", zap.Error(err))
		service.ResponseQuickMsg(ctx, e.Failed, e.GetMsg(e.InvalidParams), nil)
		return
	}
	if config.Env != "dev" {
		if mLoginInfoAny, exists := ctx.Get("mLoginInfo"); exists {
			userInfo := mLoginInfoAny.(model.LoginInfo)
			saveReq.CreatedBy = userInfo.NickName
		}
	}
	if saveRes, err = service.GrpcTrademarkImpl.SaveTrademark(context.Background(), saveReq); err != nil {
		zap.L().Error("SaveTrademark err Save", zap.Error(err))
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), saveRes)
	return
}

func TrademarkList(ctx *gin.Context) {
	var (
		listReq *trademark.TrademarkListReq
		listRes *trademark.TrademarkListRes
		err     error
	)
	if err = ctx.ShouldBind(&listReq); err != nil {
		zap.L().Error("TrademarkList err ShouldBind", zap.Error(err))
		service.ResponseQuickMsg(ctx, e.Failed, e.GetMsg(e.InvalidParams), nil)
		return
	}
	if listRes, err = service.GrpcTrademarkImpl.TrademarkList(context.Background(), listReq); err != nil {
		zap.L().Error("TrademarkList err List", zap.Error(err))
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), listRes)
	return
}

func DelTrademark(ctx *gin.Context) {
	var (
		delReq *trademark.DelReq
		delRes *trademark.DelRes
		err    error
	)
	if err = ctx.ShouldBind(&delReq); err != nil {
		zap.L().Error("DelTrademark err ShouldBind", zap.Error(err))
		service.ResponseQuickMsg(ctx, e.Failed, e.GetMsg(e.InvalidParams), nil)
		return
	}
	if delRes, err = service.GrpcTrademarkImpl.DelTrademark(context.Background(), delReq); err != nil {
		zap.L().Error("DelTrademark err Delete", zap.Error(err))
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), delRes)
	return
}
