/**
 * @Author: sun
 * @Company: 苏州天和汇工业
 * @Description:
 * @File:  customer_service
 * @Version: 1.0.0
 * @Date: 2024/10/24 9:19
 */

package customer_service

import (
	"fmt"
	"github.com/fonchain_enterprise/fonchain-main/api/artistInfoArtwork"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/artistInfo/asArtist"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/artistInfo/asArtwork"
	"github.com/gin-gonic/gin"
	"strconv"
	"time"
)

func GetLockArtistList(c *gin.Context) {
	var req asArtist.ArtistArtworkStaticRequest
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, err.Error())
		return
	}
	var artistStaticRequest = &artistInfoArtwork.GetLockArtistListRequest{
		Page:            req.Page,
		PageSize:        req.PageSize,
		ArtistName:      req.ArtistName,
		Tnum:            req.Tnum,
		Tfnum:           req.Tfnum,
		ArtworkName:     req.ArtworkName,
		BaseAuditStatus: int32(req.BaseAuditStatus),
	}
	data, err := service.GrpcArtistInfoArtworkImpl.GetLockArtistList(c, artistStaticRequest)
	var resp []*model.CustomerArtistListRes
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GrpcArtistInfoArtworkImpl.GetLockArtistList Err", err.Error())
	}
	if len(data.Data) == 0 {
		service.ResponseList(c, []model.CustomerArtistListRes{}, service.OptionPage(data.Page, data.PageSize, data.Total),
			service.OptionAddField_ArtshowCount("artistUid"))
		return
	}
	for _, v := range data.Data {
		tmp := &model.CustomerArtistListRes{}
		if len(v.CaaCertNum) > 0 {
			tmp.IsMaster = true
		}
		tmp.ArtistUid = v.ArtistUid
		tmp.ArtworkTotal = v.ArtworkTotal
		tmp.CaaCertNum = v.CaaCertNum
		tmp.IsLock = v.IsLock
		tmp.LatestLockTime = v.LatestLockTime
		tmp.RealName = v.RealName
		tmp.PasswdArtworkTotal = v.PassedArtworkTotal
		tmp.PenName = v.PenName
		tmp.IdNum = v.IdNum
		tmp.TNum = v.Tnum
		tmp.ArtistShowCount = fmt.Sprintf("%s(%d)", v.RealName, v.PassedArtworkTotal)
		tmp.PassedRulerTotal = v.PassedRulerTotal
		tmp.RulerTotal = v.RulerTotal
		tmp.IsNew = v.IsNew
		resp = append(resp, tmp)
	}
	// //查询inSource字段值
	service.ResponseList(c, resp, service.OptionPage(data.Page, data.PageSize, data.Total),
		service.OptionAddField_ArtshowCount("artistUid"))
}

func GetPassedArtworkList(c *gin.Context) {
	var req asArtwork.GetArtworkPreviewListReq
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	var tmpReq = &artistInfoArtwork.GetNotifiedArtworkListRequest{
		ArtistUid:       req.ArtistUid,
		Tnum:            req.LockTime,
		Tfnum:           req.Tfnum,
		ArtworkName:     req.ArtworkName,
		ArtistName:      req.ArtistName,
		IsNotify:        1,
		BaseAuditStatus: req.BaseAuditStatus,
	}
	if req.RulerStart != nil {
		tmpReq.Where = fmt.Sprintf("ap.ruler >= %d", *req.RulerStart)
	}
	if req.RulerEnd != nil {
		tmpReq.Where = fmt.Sprintf("ap.ruler <= %d", *req.RulerEnd)
	}
	if req.RulerStart != nil && req.RulerEnd != nil {
		tmpReq.Where = fmt.Sprintf("ap.ruler >= %d and ap.ruler <= %d", *req.RulerStart, *req.RulerEnd)
	}
	var resp []*model.CustomerArtworkListRes

	data, err := service.GrpcArtistInfoArtworkImpl.GetNotifiedArtworkList(c, tmpReq)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GrpcArtistInfoArtworkImpl.GetArtworkViewList Err", err.Error())
		return
	}
	if len(data.Datas) == 0 {
		service.ResponseList(c, resp, service.OptionPage(data.Page, data.PageSize, data.Total),
			service.OptionAddField_ArtshowCount("artistUid"))
		return
	}
	for _, v := range data.Datas {
		tmp := &model.CustomerArtworkListRes{}
		tmp.ArtworkName = v.ArtworkName
		tmp.BaseAuditStatus = v.BaseAuditStatus
		tmp.Tfnum = v.Tfnum
		tmp.ArtworkType = v.ArtworkType
		tmp.Width = v.Width
		tmp.Length = v.Length
		tmp.Ruler = v.Ruler
		tmp.HdPic = v.HdPic
		tmp.ApplyAt = parseTime(v.CreatedAt)
		tmp.SealPic = v.SealPic
		tmp.ArtistPhoto = v.ArtistPhoto
		tmp.BaseAuditMark2 = v.BaseAuditMark2
		tmp.BaseAuditMark = v.BaseAuditMark
		resp = append(resp, tmp)
	}
	service.ResponseList(c, resp,
		service.OptionPage(req.Page, req.PageSize, data.Total), service.OptionMsg("查询成功"),
	)
}

func UpdateNotifyStatus(c *gin.Context) {
	var req model.UpdateNotifyStatusReq
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	if _, err := service.GrpcArtistInfoArtworkImpl.UpdateNotifyArtist(c, &artistInfoArtwork.UpdateNotifyRequest{
		ArtistUid: req.ArtistUid,
	}); err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GrpcArtistInfoArtworkImpl.GetArtworkViewList Err", err.Error())
	}
	service.Success(c)
}

func GetNotifiedArtistList(c *gin.Context) {
	var req asArtist.ArtistArtworkStaticRequest
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	var artistStaticRequest = &artistInfoArtwork.GetLockArtistListRequest{
		Page:            req.Page,
		PageSize:        req.PageSize,
		Tnum:            req.Tnum,
		Tfnum:           req.Tfnum,
		ArtworkName:     req.ArtworkName,
		ArtistName:      req.ArtistName,
		BaseAuditStatus: int32(req.BaseAuditStatus),
	}
	var resp []*model.CustomerArtistListRes

	data, err := service.GrpcArtistInfoArtworkImpl.GetNotifiedArtistList(c, artistStaticRequest)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "ArtistsArtworkStaticList:  service.GrpcArtistInfoArtworkImpl.ArtistArtworkStatic", "查询失败")
		return
	}
	if len(data.Data) == 0 {
		service.ResponseList(c, resp,
			service.OptionPage(req.Page, req.PageSize, data.Total), service.OptionMsg("查询成功"),
			service.OptionAddField_ArtshowCount("artistUid"),
		)
		return
	}
	for _, v := range data.Data {
		tmp := &model.CustomerArtistListRes{}
		if len(v.CaaCertNum) > 0 {
			tmp.IsMaster = true
		}
		tmp.ArtistUid = v.ArtistUid
		tmp.ArtworkTotal = v.ArtworkTotal
		tmp.CaaCertNum = v.CaaCertNum
		tmp.IsLock = v.IsLock
		tmp.LatestLockTime = v.LatestLockTime
		tmp.RealName = v.RealName
		tmp.PasswdArtworkTotal = v.PassedArtworkTotal
		tmp.PenName = v.PenName
		tmp.IdNum = v.IdNum
		tmp.TNum = v.Tnum
		tmp.ArtistShowCount = fmt.Sprintf("%s(%d)", v.RealName, v.PassedArtworkTotal)
		tmp.PassedRulerTotal = v.PassedRulerTotal
		tmp.RulerTotal = v.RulerTotal
		tmp.IsNew = v.IsNew
		tmp.NotifyAt = parseTimeV2(v.NotifyAt)
		resp = append(resp, tmp)
	}

	service.ResponseList(c, resp,
		service.OptionPage(req.Page, req.PageSize, data.Total), service.OptionMsg("查询成功"),
		service.OptionAddField_ArtshowCount("artistUid"),
	)
}

func GetNotifiedArtworkList(c *gin.Context) {
	var req asArtwork.GetArtworkPreviewListReq
	if err := c.ShouldBindJSON(&req); err != nil {
		service.Error(c, e.InvalidParams, err, "无效参数")
		return
	}
	var tmpReq = &artistInfoArtwork.GetNotifiedArtworkListRequest{
		ArtistUid:       req.ArtistUid,
		Tnum:            req.LockTime,
		Tfnum:           req.Tfnum,
		ArtworkName:     req.ArtworkName,
		ArtistName:      req.ArtistName,
		IsNotify:        2,
		BaseAuditStatus: req.BaseAuditStatus,
	}
	if req.RulerStart != nil {
		tmpReq.Where = fmt.Sprintf("ap.ruler >= %d", *req.RulerStart)
	}
	if req.RulerEnd != nil {
		tmpReq.Where = fmt.Sprintf("ap.ruler <= %d", *req.RulerEnd)
	}
	if req.RulerStart != nil && req.RulerEnd != nil {
		tmpReq.Where = fmt.Sprintf("ap.ruler >= %d and ap.ruler <= %d", *req.RulerStart, *req.RulerEnd)
	}
	var resp []*model.CustomerArtworkListRes

	data, err := service.GrpcArtistInfoArtworkImpl.GetNotifiedArtworkList(c, tmpReq)
	if err != nil {
		service.ErrorWithMark(c, e.Failed, err, "GrpcArtistInfoArtworkImpl.GetArtworkViewList Err", err.Error())
		return
	}
	if len(data.Datas) == 0 {
		service.ResponseList(c, resp,
			service.OptionPage(req.Page, req.PageSize, data.Total), service.OptionMsg("查询成功"),
		)
		return
	}

	for _, v := range data.Datas {
		tmp := &model.CustomerArtworkListRes{}
		tmp.ArtworkName = v.ArtworkName
		tmp.BaseAuditStatus = v.BaseAuditStatus
		tmp.Tfnum = v.Tfnum
		tmp.ArtworkType = v.ArtworkType
		tmp.Width = v.Width
		tmp.Length = v.Length
		tmp.Ruler = v.Ruler
		tmp.HdPic = v.HdPic
		tmp.ApplyAt = parseTime(v.CreatedAt)
		tmp.SealPic = v.SealPic
		tmp.ArtistPhoto = v.ArtistPhoto
		tmp.NotifyAt = parseTimeV2(v.UpdatedAt)
		tmp.BaseAuditMark2 = v.BaseAuditMark2
		tmp.BaseAuditMark = v.BaseAuditMark
		resp = append(resp, tmp)
	}
	service.ResponseList(c, resp,
		service.OptionPage(req.Page, req.PageSize, data.Total), service.OptionMsg("查询成功"),
	)
}

func parseTime(in string) (out string) {
	tmp, _ := strconv.Atoi(in)

	// 将时间戳转换为 time.Time 类型
	out = time.Unix(int64(tmp), 0).Format("2006-01-02 15:04:05")

	// 格式化时间
	//formattedTime := t.Format("2006-01-02 15:04:05")
	return
}

func parseTimeV2(in string) (out string) {
	// 解析原始时间字符串
	const (
		layoutISO    = "2006-01-02T15:04:05+08:00"
		layoutCustom = "2006-01-02 15:04:05"
	)
	t, err := time.Parse(layoutISO, in)
	if err != nil {
		fmt.Println("解析时间错误:", err)
		return
	}
	// 格式化时间为所需的格式
	out = t.Format(layoutCustom)
	return
}
