package dci

import (
	"context"
	"dubbo.apache.org/dubbo-go/v3/common/logger"
	"fmt"
	"github.com/fonchain_enterprise/fonchain-main/api/dci"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	dciErr "github.com/fonchain_enterprise/fonchain-main/pkg/e/dci"
	"github.com/fonchain_enterprise/fonchain-main/pkg/serializer"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
)

/*
 * @api {post} /dci/file/upload 获取授权访问OSS链接
 * @apiVersion 1.0.0
 * @apiName GetUploadFile
 */
func GetUploadFile(c *gin.Context) {
	req := new(dci.GetUploadUrlRequest)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("GetUploadUrlRequest ShouldBind err", err)
		service.ResponseMsg(c, e.InvalidParams, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	res, err := service.GrpcDciImpl.GetUploadUrl(context.Background(), req)

	zap.L().Info(" dci 文件上传 :", zap.Reflect("GetUploadUrl :", res))

	fmt.Println("======  GetUploadUrl  res ===============================================")
	fmt.Printf("%+v\n", res)
	fmt.Println("======  GetUploadUrl  res ===============================================")
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if res.ResultCode != "OK" {
		res.ResultCode = dciErr.ResultCode[res.ResultCode]
		res.ResultMsg = dciErr.ResultMsg[res.ResultMsg]
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   res,
			Msg:    res.ResultMsg,
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})

}
