package dci

import (
	"context"
	"dubbo.apache.org/dubbo-go/v3/common/logger"
	"fmt"
	"github.com/fonchain_enterprise/fonchain-main/api/dci"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	dciErr "github.com/fonchain_enterprise/fonchain-main/pkg/e/dci"
	"github.com/fonchain_enterprise/fonchain-main/pkg/serializer"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
)

/*
 * @api {post} /dci/user/add 著作权人 申领 DCI信息
 * @apiVersion 1.0.0
 * @apiName AddDciUser
 */
func AddDciUser(c *gin.Context) {
	req := new(dci.AddDciUserRequest)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("AddDciUserRequest ShouldBind err", err)
		service.ResponseMsg(c, e.InvalidParams, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	res, err := service.GrpcDciImpl.AddDciUser(context.Background(), req)

	zap.L().Info(" 著作权人 申领 DCI信息 :", zap.Reflect("AddDciUser :", res))

	fmt.Println("======  AddDciUser  res ===============================================")
	fmt.Printf("%+v\n", res)
	fmt.Println("======  AddDciUser  res ===============================================")
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if res.ResultCode != "OK" {
		res.ResultCode = dciErr.ResultCode[res.ResultCode]
		res.ResultMsg = dciErr.ResultMsg[res.ResultMsg]
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   res,
			Msg:    res.ResultMsg,
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})
}

/*
 * @api {post} /dci/user/update 著作权人 更新 DCI信息
 * @apiVersion 1.0.0
 * @apiName UpdateDciUser
 */
func UpdateDciUser(c *gin.Context) {
	req := new(dci.UpdateDciUserRequest)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("UpdateDciUserRequest ShouldBind err", err)
		service.ResponseMsg(c, e.InvalidParams, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	res, err := service.GrpcDciImpl.UpdateDciUser(context.Background(), req)

	zap.L().Info(" 著作权人 更新 DCI信息 :", zap.Reflect("UpdateDciUser :", res))

	fmt.Println("======  UpdateDciUser  res ===============================================")
	fmt.Printf("%+v\n", res)
	fmt.Println("======  UpdateDciUser  res ===============================================")
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if res.ResultCode != "OK" {
		res.ResultCode = dciErr.ResultCode[res.ResultCode]
		res.ResultMsg = dciErr.ResultMsg[res.ResultMsg]
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   res,
			Msg:    res.ResultMsg,
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})
}

/*
 * @api {post} /dci/user/info 著作权人 查询 DCI信息
 * @apiVersion 1.0.0
 * @apiName QueryDciUser
 */
func QueryDciUser(c *gin.Context) {
	req := new(dci.QueryDciUserRequest)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("QueryDciUserRequest ShouldBind err", err)
		service.ResponseMsg(c, e.InvalidParams, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	res, err := service.GrpcDciImpl.QueryDciUser(context.Background(), req)

	zap.L().Info(" 著作权人 查询 DCI信息 :", zap.Reflect("QueryDciUser :", res))

	fmt.Println("======  QueryDciUser  res ===============================================")
	fmt.Printf("%+v\n", res)
	fmt.Println("======  QueryDciUser  res ===============================================")
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if res.ResultCode != "OK" {
		res.ResultCode = dciErr.ResultCode[res.ResultCode]
		res.ResultMsg = dciErr.ResultMsg[res.ResultMsg]
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   res,
			Msg:    res.ResultMsg,
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})
}
