package dci

import (
	"bytes"
	"context"
	"dubbo.apache.org/dubbo-go/v3/common/logger"
	"encoding/json"
	"fmt"
	"github.com/fonchain_enterprise/fonchain-main/api/dci"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	dciErr "github.com/fonchain_enterprise/fonchain-main/pkg/e/dci"
	loigc_copyright "github.com/fonchain_enterprise/fonchain-main/pkg/logic/copyright"
	model_copyright "github.com/fonchain_enterprise/fonchain-main/pkg/model/copyright"
	dciModel "github.com/fonchain_enterprise/fonchain-main/pkg/model/dci"
	pkgRequest "github.com/fonchain_enterprise/fonchain-main/pkg/request"
	"github.com/fonchain_enterprise/fonchain-main/pkg/serializer"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
	"io"
	"net/http"
	"net/url"
)

// CreateDciPreregistration
/*
 * @api {post} /dci/reg/create 申领DCI
 * @apiVersion 1.0.0
 * @apiName CreateDciPreregistration
 */
func CreateDciPreregistration(c *gin.Context) {
	req := new(dci.CreateDciPreregistrationRequest)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("CreateDciPreregistrationRequest ShouldBind err", err)
		service.ResponseMsg(c, e.InvalidParams, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	res, err := service.GrpcDciImpl.CreateDciPreregistration(context.Background(), req)

	zap.L().Info(" 作品 申领 DCI :", zap.Reflect("CreateDciPreregistration :", res))

	fmt.Println("======  CreateDciPreregistration  res ===============================================")
	fmt.Printf("%+v\n", res)
	fmt.Println("======  CreateDciPreregistration  res ===============================================")
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if res.ResultCode != "OK" {
		res.ResultCode = dciErr.ResultCode[res.ResultCode]
		res.ResultMsg = dciErr.ResultMsg[res.ResultMsg]
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   res,
			Msg:    res.ResultMsg,
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})
}

/*
 * @api {post} /dci/reg/info DCI申领查询
 * @apiVersion 1.0.0
 * @apiName QueryDciPreregistration
 */
func QueryDciPreregistration(c *gin.Context) {
	req := new(dci.QueryDciPreregistrationRequest)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("QueryDciPreregistrationRequest ShouldBind err", err)
		service.ResponseMsg(c, e.InvalidParams, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	res, err := service.GrpcDciImpl.QueryDciPreregistration(context.Background(), req)

	zap.L().Info(" 作品 查询 DCI :", zap.Reflect("QueryDciPreregistration :", res))

	fmt.Println("======  QueryDciPreregistration  res ===============================================")
	fmt.Printf("%+v\n", res)
	fmt.Println("======  QueryDciPreregistration  res ===============================================")
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if res.ResultCode != "OK" {
		res.ResultCode = dciErr.ResultCode[res.ResultCode]
		res.ResultMsg = dciErr.ResultMsg[res.ResultMsg]
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   res,
			Msg:    res.ResultMsg,
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})
}

/*
 * @api {post} /dci/work/create 数字登记
 * @apiVersion 1.0.0
 * @apiName CreateDciRegistration
 */
func CreateDciRegistration(c *gin.Context) {
	req := new(dci.CreateDciRegistrationRequest)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("CreateDciRegistrationRequest ShouldBind err", err)
		service.ResponseMsg(c, e.InvalidParams, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	res, err := service.GrpcDciImpl.CreateDciRegistration(context.Background(), req)

	zap.L().Info(" 发起数登申请 :", zap.Reflect("CreateDciRegistration :", res))

	fmt.Println("======  CreateDciRegistration  res ===============================================")
	fmt.Printf("%+v\n", res)
	fmt.Println("======  CreateDciRegistration  res ===============================================")
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if res.ResultCode != "OK" {
		res.ResultCode = dciErr.ResultCode[res.ResultCode]
		res.ResultMsg = dciErr.ResultMsg[res.ResultMsg]
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   res,
			Msg:    res.ResultMsg,
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})
}

/*
 * @api {post} /dci/work/info 数字登记查询
 * @apiVersion 1.0.0
 * @apiName QueryDciRegistration
 */
func QueryDciRegistration(c *gin.Context) {
	req := new(dci.QueryDciRegistrationRequest)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("QueryDciRegistrationRequest ShouldBind err", err)
		service.ResponseMsg(c, e.InvalidParams, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	res, err := service.GrpcDciImpl.QueryDciRegistration(context.Background(), req)

	zap.L().Info(" 查询数登申请 :", zap.Reflect("QueryDciRegistration :", res))

	fmt.Println("======  QueryDciRegistration  res ===============================================")
	fmt.Printf("%+v\n", res)
	fmt.Println("======  QueryDciRegistration  res ===============================================")
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if res.ResultCode != "OK" {
		res.ResultCode = dciErr.ResultCode[res.ResultCode]
		res.ResultMsg = dciErr.ResultMsg[res.ResultMsg]
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   res,
			Msg:    res.ResultMsg,
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})
}

/*
 * @api {post} /dci/work/pay-url 数登支付链接获取
 * @apiVersion 1.0.0
 * @apiName GetDciPayUrl
 */
func GetDciPayUrl(c *gin.Context) {
	req := new(dci.GetDciPayUrlRequest)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("GetDciPayUrlRequest ShouldBind err", err)
		service.ResponseMsg(c, e.InvalidParams, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	res, err := service.GrpcDciImpl.GetDciPayUrl(context.Background(), req)

	zap.L().Info(" 数登支付链接获取 :", zap.Reflect("GetDciPayUrl :", res))

	fmt.Println("======  GetDciPayUrl  res ===============================================")
	fmt.Printf("%+v\n", res)
	fmt.Println("======  GetDciPayUrl  res ===============================================")
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if res.ResultCode != "OK" {
		res.ResultCode = dciErr.ResultCode[res.ResultCode]
		res.ResultMsg = dciErr.ResultMsg[res.ResultMsg]
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   res,
			Msg:    res.ResultMsg,
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})
}

/*
 * @api {post} /dci/work/pay-info 数登支付查询
 * @apiVersion 1.0.0
 * @apiName QueryDciPay
 */
func QueryDciPay(c *gin.Context) {
	req := new(dci.QueryDciPayRequest)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("QueryDciPayRequest ShouldBind err", err)
		service.ResponseMsg(c, e.InvalidParams, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	res, err := service.GrpcDciImpl.QueryDciPay(context.Background(), req)

	zap.L().Info(" 数登支付查询 :", zap.Reflect("QueryDciPay :", res))

	fmt.Println("======  QueryDciPay  res ===============================================")
	fmt.Printf("%+v\n", res)
	fmt.Println("======  QueryDciPay  res ===============================================")
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if res.ResultCode != "OK" {
		res.ResultCode = dciErr.ResultCode[res.ResultCode]
		res.ResultMsg = dciErr.ResultMsg[res.ResultMsg]
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   res,
			Msg:    res.ResultMsg,
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})
}

/*
 * @api {post} /dci/work/cert-info 获取数登证书下载
 * @apiVersion 1.0.0
 * @apiName GetDciRegistrationcert
 */
func GetDciRegistrationcert(c *gin.Context) {
	req := new(dci.GetDciRegistrationcertRequest)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("GetDciRegistrationcertRequest ShouldBind err", err)
		service.ResponseMsg(c, e.InvalidParams, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	res, err := service.GrpcDciImpl.GetDciRegistrationcert(context.Background(), req)

	zap.L().Info(" 获取数登证书下载 :", zap.Reflect("GetDciRegistrationcert :", res))

	fmt.Println("======  GetDciRegistrationcert  res ===============================================")
	fmt.Printf("%+v\n", res)
	fmt.Println("======  GetDciRegistrationcert  res ===============================================")
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if res.ResultCode != "OK" {
		res.ResultCode = dciErr.ResultCode[res.ResultCode]
		res.ResultMsg = dciErr.ResultMsg[res.ResultMsg]
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   res,
			Msg:    res.ResultMsg,
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})
}

/*
 * @api {post} /dci/work/retry 补正申请
 * @apiVersion 1.0.0
 * @apiName RetryDciRegistration
 */
func RetryDciRegistration(c *gin.Context) {
	req := new(dci.RetryDciRegistrationRequest)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("RetryDciRegistrationRequest ShouldBind err", err)
		service.ResponseMsg(c, e.InvalidParams, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	res, err := service.GrpcDciImpl.RetryDciRegistration(context.Background(), req)

	zap.L().Info(" 补正申请 :", zap.Reflect("RetryDciRegistration :", res))

	fmt.Println("======  RetryDciRegistration  res ===============================================")
	fmt.Printf("%+v\n", res)
	fmt.Println("======  RetryDciRegistration  res ===============================================")
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if res.ResultCode != "OK" {
		res.ResultCode = dciErr.ResultCode[res.ResultCode]
		res.ResultMsg = dciErr.ResultMsg[res.ResultMsg]
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   res,
			Msg:    res.ResultMsg,
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})
}

/*
 * @api {post} /dci/work/close 数登停止申请
 * @apiVersion 1.0.0
 * @apiName CloseDciRegistration
 */
func CloseDciRegistration(c *gin.Context) {
	req := new(dci.CloseDciRegistrationRequest)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("CloseDciRegistrationRequest ShouldBind err", err)
		service.ResponseMsg(c, e.InvalidParams, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	res, err := service.GrpcDciImpl.CloseDciRegistration(context.Background(), req)

	zap.L().Info(" 数登停止申请 :", zap.Reflect("CloseDciRegistration :", res))

	fmt.Println("======  CloseDciRegistration  res ===============================================")
	fmt.Printf("%+v\n", res)
	fmt.Println("======  CloseDciRegistration  res ===============================================")
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if res.ResultCode != "OK" {
		res.ResultCode = dciErr.ResultCode[res.ResultCode]
		res.ResultMsg = dciErr.ResultMsg[res.ResultMsg]
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   res,
			Msg:    res.ResultMsg,
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})
}

/*
 * @api {post} /dci/callback 数登消息回调
 * @apiVersion 1.0.0
 * @apiName Callback
 */
func Callback(c *gin.Context) {
	req := new(dciModel.Callback)
	req = &dciModel.Callback{}
	res := new(dciModel.CallbackResp)
	var bizContent dciModel.BizContent
	returnAllBytes, _ := io.ReadAll(c.Request.Body)
	c.Request.Body = io.NopCloser(bytes.NewBuffer(returnAllBytes))
	returnAllString := string(returnAllBytes)
	zap.L().Info("Callback", zap.Any("returnAllString", returnAllString))
	postBizContent := c.PostForm("biz_content")
	productEnv := c.PostForm("product_env")
	zap.L().Info("Callback ", zap.Any("postBizContent", postBizContent))
	zap.L().Info("Callback ", zap.Any("productEnv", productEnv))
	switch productEnv {
	case "SIT":
		apiUrl := "http://114.218.158.24:9020/dci/callback"
		data := url.Values{}
		data.Set("product_env", "TRANSPOND")
		data.Set("biz_content", postBizContent)
		resp, _ := http.PostForm(apiUrl, data)
		defer resp.Body.Close()
	case "PRD", "PROD", "TRANSPOND":
		err := json.Unmarshal([]byte(postBizContent), &bizContent)
		if err != nil {
			return
		}
		zap.L().Info("bizContent ", zap.Any("bizContent", bizContent))
		var logicDigital = new(loigc_copyright.Digital)
		_, _ = logicDigital.DciQueryRegistration(pkgRequest.CopyrightIdAll{
			DigitalRegisterId: bizContent.DigitalRegisterId,
		}, model_copyright.CopyrightInfo{DigitalRegisterId: bizContent.DigitalRegisterId})
	}
	_ = c.ShouldBindJSON(&req)
	zap.L().Info("Callback ", zap.Any("ShouldBindJSON req", req))
	_ = json.Unmarshal(returnAllBytes, &req)
	zap.L().Info("Callback", zap.Any("req", req))
	res.ResultCode = "OK"
	c.JSON(e.SUCCESS, res)
	return
}
