package service

import (
	"context"
	"net/http"

	"dubbo.apache.org/dubbo-go/v3/common/logger"
	"github.com/fonchain_enterprise/fonchain-main/api/draw"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/serializer"
	"github.com/gin-gonic/gin"
)

func AddAccount(c *gin.Context) {
	var addAccountRequest draw.AddAccountRequest
	if err := c.ShouldBind(&addAccountRequest); err == nil {
		logger.Errorf("addAccountRequest ShouldBind err", err)
		rep, err := GrpcDrawImpl.AddAccount(context.Background(), &addAccountRequest)
		if err != nil {
			logger.Error(err)
			c.JSON(e.ERROR, serializer.Response{
				Status: e.Failed,
				Code:   e.ERROR,
				Msg:    err.Error(),
				Error:  err,
			})
			return
		}
		c.JSON(http.StatusOK, serializer.Response{
			Status: e.Ok,
			Code:   e.SUCCESS,
			Data:   rep,
		})
		return
	} else {
		logger.Error(err)
		c.JSON(http.StatusOK, serializer.Response{
			Status: e.Failed,
			Code:   e.InvalidParams,
			Msg:    e.GetMsg(e.InvalidParams),
			Error:  err,
		})
		return
	}
}

func AddPrizeNum(c *gin.Context) {
	var addPrizeNumRequest draw.AddPrizeNumRequest
	if err := c.ShouldBind(&addPrizeNumRequest); err == nil {
		logger.Errorf("AddPrizeNumRequest ShouldBind err", err)
		rep, err := GrpcDrawImpl.AddPrizeNum(context.Background(), &addPrizeNumRequest)
		if err != nil {
			logger.Error(err)
			c.JSON(e.ERROR, serializer.Response{
				Status: e.Failed,
				Code:   e.ERROR,
				Msg:    err.Error(),
				Error:  err,
			})
			return
		}
		c.JSON(http.StatusOK, serializer.Response{
			Status: e.Ok,
			Code:   e.SUCCESS,
			Data:   rep,
		})
		return
	} else {
		logger.Error(err)
		c.JSON(http.StatusOK, serializer.Response{
			Status: e.Failed,
			Code:   e.InvalidParams,
			Msg:    e.GetMsg(e.InvalidParams),
			Error:  err,
		})
		return
	}
}

func SetPool(c *gin.Context) {
	var setPoolRequest draw.SetPoolRequest
	if err := c.ShouldBind(&setPoolRequest); err == nil {
		logger.Errorf("SetPoolRequest ShouldBind err", err)
		rep, err := GrpcDrawImpl.SetPool(context.Background(), &setPoolRequest)
		if err != nil {
			logger.Error(err)
			c.JSON(e.ERROR, serializer.Response{
				Status: e.Failed,
				Code:   e.ERROR,
				Msg:    err.Error(),
				Error:  err,
			})
			return
		}
		c.JSON(http.StatusOK, serializer.Response{
			Status: e.Ok,
			Code:   e.SUCCESS,
			Data:   rep,
		})
		return
	} else {
		logger.Error(err)
		c.JSON(http.StatusOK, serializer.Response{
			Status: e.Failed,
			Code:   e.InvalidParams,
			Msg:    e.GetMsg(e.InvalidParams),
			Error:  err,
		})
		return
	}
}

func SetResult(c *gin.Context) {
	var setResultRequest draw.SetResultRequest
	if err := c.ShouldBind(&setResultRequest); err == nil {
		logger.Errorf("SetResultRequest ShouldBind err", err)
		rep, err := GrpcDrawImpl.SetResult(context.Background(), &setResultRequest)
		if err != nil {
			logger.Error(err)
			c.JSON(e.ERROR, serializer.Response{
				Status: e.Failed,
				Code:   e.ERROR,
				Msg:    err.Error(),
				Error:  err,
			})
			return
		}
		c.JSON(http.StatusOK, serializer.Response{
			Status: e.Ok,
			Code:   e.SUCCESS,
			Data:   rep,
		})
		return
	} else {
		logger.Error(err)
		c.JSON(http.StatusOK, serializer.Response{
			Status: e.Failed,
			Code:   e.InvalidParams,
			Msg:    e.GetMsg(e.InvalidParams),
			Error:  err,
		})
		return
	}
}

func GetAccountPrize(c *gin.Context) {
	var getAccountPrizeRequest draw.GetAccountPrizeRequest
	if err := c.ShouldBind(&getAccountPrizeRequest); err == nil {
		logger.Errorf("GetAccountPrizeRequest ShouldBind err", err)
		rep, err := GrpcDrawImpl.GetAccountPrize(context.Background(), &getAccountPrizeRequest)
		if err != nil {
			logger.Error(err)
			c.JSON(e.ERROR, serializer.Response{
				Status: e.Failed,
				Code:   e.ERROR,
				Msg:    err.Error(),
				Error:  err,
			})
			return
		}
		c.JSON(http.StatusOK, serializer.Response{
			Status: e.Ok,
			Code:   e.SUCCESS,
			Data:   rep,
		})
		return
	} else {
		logger.Error(err)
		c.JSON(http.StatusOK, serializer.Response{
			Status: e.Failed,
			Code:   e.InvalidParams,
			Msg:    e.GetMsg(e.InvalidParams),
			Error:  err,
		})
		return
	}
}

func PrizeList(c *gin.Context) {
	var prizeListRequest draw.PrizeListRequest
	if err := c.ShouldBind(&prizeListRequest); err == nil {
		logger.Errorf("PrizeListRequest ShouldBind err", err)
		rep, err := GrpcDrawImpl.PrizeList(context.Background(), &prizeListRequest)
		if err != nil {
			logger.Error(err)
			c.JSON(e.ERROR, serializer.Response{
				Status: e.Failed,
				Code:   e.ERROR,
				Msg:    err.Error(),
				Error:  err,
			})
			return
		}
		c.JSON(http.StatusOK, serializer.Response{
			Status: e.Ok,
			Code:   e.SUCCESS,
			Data:   rep,
		})
		return
	} else {
		logger.Error(err)
		c.JSON(http.StatusOK, serializer.Response{
			Status: e.Failed,
			Code:   e.InvalidParams,
			Msg:    e.GetMsg(e.InvalidParams),
			Error:  err,
		})
		return
	}
}

// func Encrypt(c *gin.Context) {
// 	rep, err := GrpcDrawImpl.Encrypt(context.Background(), new(draw.EncryptRequest))
// 	if err != nil {
// 		logger.Error(err)
// 		c.JSON(e.ERROR, serializer.Response{
// 			Status: e.Failed,
// 			Code:   e.ERROR,
// 			Msg:    err.Error(),
// 			Error:  err,
// 		})
// 		return
// 	}
// 	c.JSON(http.StatusOK, serializer.Response{
// 		Status: e.Ok,
// 		Code:   e.SUCCESS,
// 		Data:   rep,
// 	})
// }

// func Decrypt(c *gin.Context) {
// 	var decryptRequest draw.DecryptRequest
// 	if err := c.ShouldBind(&decryptRequest); err == nil {
// 		logger.Errorf("DecryptRequest ShouldBind err", err)
// 		rep, err := GrpcDrawImpl.Decrypt(context.Background(), &decryptRequest)
// 		if err != nil {
// 			logger.Error(err)
// 			c.JSON(e.ERROR, serializer.Response{
// 				Status: e.Failed,
// 				Code:   e.ERROR,
// 				Msg:    err.Error(),
// 				Error:  err,
// 			})
// 			return
// 		}
// 		c.JSON(http.StatusOK, serializer.Response{
// 			Status: e.Ok,
// 			Code:   e.SUCCESS,
// 			Data:   rep,
// 		})
// 		return
// 	} else {
// 		logger.Error(err)
// 		c.JSON(http.StatusOK, serializer.Response{
// 			Status: e.Failed,
// 			Code:   e.InvalidParams,
// 			Msg:    e.GetMsg(e.InvalidParams),
// 			Error:  err,
// 		})
// 		return
// 	}
// }
