package model

type EsApproval struct {
	ID             uint64         `json:"ID"` // Assuming the field names in JSON are the same as in the mapping
	Domain         string         `json:"Domain"`
	Status         uint64         `json:"Status"`
	Type           string         `json:"Type"`
	SubmitterID    uint64         `json:"SubmitterID"`
	SubmitterName  string         `json:"SubmitterName"`
	CopyUsers      []CopyUser     `json:"CopyUsers"`
	Content        string         `json:"Content"`
	Reply          string         `json:"Reply"`
	CanView        bool           `json:"CanView"`
	Work           Work           `json:"Work"`
	Show           []ShowDetail   `json:"Show"`
	Exhibition     Exhibition     `json:"Exhibition"`
	Bundle         Bundle         `json:"Bundle"`
	ApprovalUsers  []ApprovalUser `json:"ApprovalUsers"`
	CanApproval    bool           `json:"CanApproval"`
	WorkFlows      []WorkFlow     `json:"WorkFlows"`
	AllStatus      uint64         `json:"AllStatus"`
	NowUserId      uint64         `json:"NowUserId"`
	NowUserName    string         `json:"NowUserName"`
	Level          uint64         `json:"Level"`
	NowLevel       uint64         `json:"NowLevel"`
	CreatedAt      string         `json:"CreatedAt"`
	CustomizeInfo  []KeyInfo      `json:"CustomizeInfo"`
	IsCustom       uint64         `json:"IsCustom"`
	TypeName       string         `json:"TypeName"`
	GroupName      string         `json:"GroupName"`
	BundlePayPrice BundlePayPrice `json:"BundlePayPrice"`
	ApprovalOA     ApprovalOA     `json:"ApprovalOA"`
	FinancialForm  FinancialForm  `json:"FinancialForm"`
	DepartmentID   int64          `json:"DepartmentID"`
	PositionID     int64          `json:"PositionID"`
	ExhibitionHis  ExhibitionHis  `json:"ExhibitionHis"`
	Promotion      Promotion      `json:"Promotion"`
	StaffSupply    StaffSupply    `json:"StaffSupply"`
	Employ         Employ         `json:"Employ"`
	IsLeader       bool           `json:"IsLeader"`
	CheckIn        CheckIn        `json:"CheckIn"`
	//ReturnArtworkInfo []ReturnArtworkInfos `json:"ReturnArtworkInfo"` // Assuming it's a slice based on the mapping
	MenuType int32  `json:"MenuType"`
	Object   string `json:"Object"`
}

type KeyInfo struct {
	ApprovalKey         string
	ApprovalName        string
	ApprovalType        string
	ApprovalValue       string
	ApprovalMark        bool
	ApprovalRequired    bool
	ApprovalPlaceholder string
	SelectList          []string
}

type Bundle struct {
	ID              uint64
	ReturnAt        string
	ReceivedAt      string
	ApplicationsNum uint64
}

type ShowDetail struct {
	ID        uint64
	ArtistNum uint64
	ShowAt    string
	Address   string
	Show      Show
}

type Show struct {
	ID        uint64
	ArtistNum uint64
	ShowAt    string
}

type ApprovalWork struct {
	ID              uint64
	WorkID          uint64
	ApprovalID      uint64
	ArtworkID       uint64
	ArtworkName     string
	ArtistName      string
	ArtworkCover    string
	ArtworkNumber   uint64
	ArtistUID       string
	ShowSeq         string
	ArtistShowCount string
}

type Work struct {
	ID            uint64
	ReturnAt      string
	ReceivedAt    string
	ApprovalWorks []ApprovalWork
	ArtistUID     string
}

type ApprovalExhibition struct {
	ID                  uint64
	ExhibitionID        uint64
	ApprovalID          uint64
	PackageName         string
	PackageID           string
	PackageSize         uint64
	PackageNumber       string
	Address             string
	ShowDate            string
	Artworks            []Artwork
	ArtExhibitionDetail ArtExhibitionDetail
	TotalPrice          int64
}

type Exhibition struct {
	ID                  uint64
	ReceivedAt          string
	Address             string
	PidApprovalID       uint64
	ApprovalExhibitions []ApprovalExhibition
	Num                 uint64
	ApplyID             string
	ExhibitionUID       string
}

type Artwork struct {
	ArtworkPriceUID string
	ShowUID         string
	ArtworkUID      string
	ArtworkName     string
	ArtistName      string
	Length          uint64
	Width           uint64
	Ruler           uint64
	SmallPic        string
	SaleAddress     string
	Reward          string
	Tfnum           string
	SaleAddressID   string
	ArtistShowCount string
}

type ArtExhibitionDetail struct {
	ShowUID         string
	ShowSeq         string
	ShowName        string
	ArtistName      string
	ArtistUID       string
	ArtworkNum      uint64
	Ruler           uint64
	Price           uint64
	Reward          uint64
	CreateTime      string
	Operator        string
	IsShow          uint64
	ShowTime        string
	Address         string
	Agent           string
	ArtistShowCount string
	ExhibitionType  int32
	IsInvite        int32
}

type BundlePayPrice struct {
	ShowUid         string
	ApprovalID      uint64
	ShowName        string
	ReceivedAt      string
	ShowDate        string
	ShowLowPrice    string
	ArtworkNum      uint64
	ArtworkSize     string
	ShowAddress     string
	ID              uint64
	Status          uint64
	ArtistName      string
	Rollback        int32
	OperationTimes  int32
	ExhibitionType  uint64
	ArtistShowCount string
}

type WorkFlow struct {
	ID             uint64
	UserID         uint64
	Name           string
	DepartmentUID  string
	DepartmentName string
	PositionUID    string
	PositionName   string
	IsDirect       uint64
	Level          uint64
	BalanceFillIn  uint64
	Signature      string
	Status         uint64
	Tips           string
	Reply          string
	OperatedAt     string
	IsLive         uint64
}

type CopyUser struct {
	ID             uint64
	Name           string
	DepartmentUID  string
	DepartmentName string
	PositionUID    string
	PositionName   string
	Level          uint64
	IsLive         uint64
	Type           string
	IsViewed       bool
}

type ApprovalUser struct {
	ID             uint64
	Name           string
	DepartmentUID  string
	DepartmentName string
	PositionUID    string
	PositionName   string
	IsDirect       uint64
	Level          uint64
	BalanceFillIn  uint64
	Signature      string
	IsLive         uint64
	Type           string
	IsDesignate    uint64
	Tier           uint64
}

type ApplyTime struct {
	Date string
	Hour string
	M    string
}

type ApprovalOA struct {
	ID           uint64
	DeletedAt    uint64
	CreatedAt    string
	UpdatedAt    string
	ApprovalID   uint64
	LeaveApply   LeaveApply
	OutWork      OutWorkApply
	MakeUp       MakeUpApply
	Turnover     TurnoverApply
	OverTime     OverTimeApply
	Leave        Leave
	RevokeReason string
	GoOut        Leave
	BusinessTrip BusinessTripApply
}

type LeaveApply struct {
	UUID          string
	StaffUID      uint64
	StaffNum      string
	StaffName     string
	DepartmentUID string
	ApplyType     string
	ActionTime    string
	ApprovalID    string
	Status        int32
	Reason        string
	VerifyFile    []string
	CopyUsers     []CopyUser
	ApprovalUsers []ApprovalUser
	LeaveBalance  float32
}

type OutWorkApply struct {
	UUID           string
	StaffUID       uint64
	StaffNum       string
	StaffName      string
	DepartmentUID  string
	ApplyType      string
	ActionTime     string
	ApprovalID     string
	Status         int32
	ApplyTimes     []ApplyTime
	Reason         string
	Hours          float32
	OutWorkAddress string
	Vehicle        string
	CopyUsers      []CopyUser
	ApprovalUsers  []ApprovalUser
}

type MakeUpApply struct {
	UUID          string
	StaffUID      uint64
	StaffNum      string
	StaffName     string
	DepartmentUID string
	ApplyType     string
	ActionTime    string
	ApprovalID    string
	Status        int32
	ApplyTimes    []ApplyTime
	Reason        string
	CopyUsers     []CopyUser
	ApprovalUsers []ApprovalUser
}

type TurnoverApply struct {
	UUID          string
	StaffUID      uint64
	StaffNum      string
	StaffName     string
	DepartmentUID string
	ApplyType     string
	ActionTime    string
	ApprovalID    string
	Status        int32
	HandoverUID   uint64
	HandoverName  string
	ApplyTimes    []ApplyTime
	Reason        string
	CopyUsers     []CopyUser
	ApprovalUsers []ApprovalUser
	Positions     []Position
	JoinTime      string
	LeftTime      string
	ContactInfo   string
	HandoverNum   string
	Address       string
	SuperviseUID  uint64
	SuperviseName string
	SuperviseNum  string
	WorkItems     string
	FinanceItems  string
}

type Position struct {
	DepartmentName string
	DepartmentID   int64
	PositionName   string
	PositionID     int64
}

type OverTimeApply struct {
	UUID          string
	StaffUID      uint64
	StaffNum      string
	StaffName     string
	DepartmentUID string
	ApplyType     string
	ActionTime    string
	ApprovalID    string
	Status        int32
	ApplyTimes    []ApplyTime
	Reason        string
	Hours         float32
	CopyUsers     []CopyUser
	ApprovalUsers []ApprovalUser
}

type Leave struct {
	UUID          string
	StaffUID      uint64
	StaffNum      string
	StaffName     string
	DepartmentUID string
	ApplyType     string
	ActionTime    string
	ApprovalID    string
	Status        int32
	ApplyTimes    []ApplyTime
	Reason        string
	Hours         float32
	Days          float32
	CopyUsers     []CopyUser
	ApprovalUsers []ApprovalUser
	VerifyFile    []string
	DateOfBrith   string
	Children      int32
	Month         int32
	Dystocia      int32
}

type FinancialForm struct {
	ID                    uint64
	DeletedAt             uint64
	CreatedAt             string
	UpdatedAt             string
	ApprovalID            uint64
	PaymentCompany        string
	TransactionDepartment string
	Payee                 string
	BeneficiaryBank       string
	BankNo                string
	CostInfo              []CostInfo
	PaymentMethod         string
	InvoiceInfo           []InvoiceInfo
}

type InvoiceInfo struct {
	InvoiceDate     string
	InvoiceNo       string
	InvoiceProvider string
	Amount          string
	UseTo           string
	Applicant       string
	PaymentMethod   string
	PaymentObj      string
	InvoiceType     string
	Notes           string
}

type CostInfo struct {
	Description    string
	UnitPrice      string
	CostUnit       string
	CostNum        int64
	TotalPrice     string
	UsedDepartment string
}

type ExhibitionHis struct {
	ID                     uint64
	ReceivedAt             string
	Address                string
	PidApprovalID          uint64
	ApprovalExhibitionHiss []ApprovalExhibitionHis
	Num                    uint64
	ApplyID                string
}

type ApprovalExhibitionHis struct {
	ID                  uint64
	ExhibitionHisID     uint64
	ApprovalID          uint64
	PackageName         string
	PackageID           string
	PackageSize         uint64
	PackageNumber       string
	Address             string
	ShowDate            string
	Artworks            []Artwork
	ArtExhibitionDetail ArtExhibitionDetail
	TotalPrice          int64
}

type Promotion struct {
	ID            uint64
	DeletedAt     uint64
	CreatedAt     string
	UpdatedAt     string
	ApprovalID    uint64
	StaffUID      uint64
	StaffNum      string
	StaffName     string
	FileUrl       string
	FileName      string
	RequestID     string
	TxId          string
	BlockId       string
	BaiduCertUrl  string
	OldJob        string
	NewJob        string
	EffectiveTime string
	IsSend        uint64
}

type StaffSupply struct {
	ID             uint64
	DeletedAt      uint64
	CreatedAt      string
	UpdatedAt      string
	ApprovalID     uint64
	PositionName   string
	Num            uint64
	JobAsk         string
	JoinInTime     string
	DepartmentName string
	PositionId     string
	DepartmentId   string
}

type Employ struct {
	ID               uint64
	DeletedAt        uint64
	CreatedAt        string
	UpdatedAt        string
	ApprovalID       uint64
	ResumeID         string
	Post             string
	BelongDepartment string
	Name             string
	Mobile           string
	ResumeFile       string
	InterviewFile    string
	EmployeeType     string
	EmployeeNo       string
	EntryTime        string
	CloseReason      string
	IsClosed         int32
	PositionId       string
	DepartmentId     string
	ConfirmDate      string
	Remark           string
}

type Job struct {
	ID             uint64
	DeletedAt      uint64
	CreatedAt      string
	UpdatedAt      string
	ApprovalID     uint64
	PositionName   string
	Num            int64
	JobAsk         string
	JoinInTime     string
	Poster         string
	PosterRule     string
	DepartmentName string
	ResidueNum     int64
	InitiationTime string
	Status         int64
	Interviewers   []Interviewer
}

type Interviewer struct {
	ID              uint64
	DeletedAt       uint64
	CreatedAt       string
	UpdatedAt       string
	JobID           uint64
	Seq             string
	Resume          string
	Name            string
	Tel             string
	Type            string
	InterviewTime   string
	ExpectEntryTime string
	JobNum          string
	InfoTable       string
	Status          int64
	UpdateTime      string
	DepartmentName  string
	PositionName    string
}

type CheckIn struct {
	ApprovalID    uint64
	SaleAddress   string
	DetailAddress string
	Longitude     string
	Latitude      string
}

type ReturnInfo struct {
	ArtistName   string
	PackageName  string
	PackageID    string
	ReturnType   uint64
	PostNum      string
	ReturnTime   string
	ReturnReason string
	Artworks     []ArtworkReturns
	ArtistNum    string
}

type ArtworkReturns struct {
	Tfnum       string
	ArtworkUID  string
	ArtworkName string
	ArtistName  string
	Length      uint64
	Width       uint64
	Ruler       uint64
	SmallPic    string
}

type ReturnArtworkInfos struct {
	ID            uint64
	DeletedAt     uint64
	CreatedAt     string
	UpdatedAt     string
	ApprovalID    uint64
	ReturnInfos   []ReturnInfo
	ExhibitionUID string
	ReturnStatus  int32
}

type BusinessTripApply struct {
	UUID          string
	StaffUID      uint64
	StaffNum      string
	StaffName     string
	DepartmentUID string
	ApplyType     string
	ActionTime    string
	ApprovalID    string
	ItineraryList []Itinerary
	TotalDays     float32
	Reason        string
	Status        int32
	CopyUsers     []CopyUser
	ApprovalUsers []ApprovalUser
}

type Itinerary struct {
	Vehicle      string
	StartAddress string
	EndAddress   string
	ApplyTimes   []ApplyTime
	Days         float32
}
