package exam

import (
	"context"
	"dubbo.apache.org/dubbo-go/v3/common/logger"
	"github.com/fonchain_enterprise/fonchain-main/api/exam"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/serializer"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/gin-gonic/gin"
)

func SaveExam(c *gin.Context) {
	req := new(exam.ExamInfo)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("SaveExam ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	_, err := service.GrpcExamClientImpl.SaveExam(context.Background(), req)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Status: e.Ok,
	})
}

func GetExamInfo(c *gin.Context) {
	req := new(exam.InfoRequest)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("GetExamInfo ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	res, err := service.GrpcExamClientImpl.GetExamInfo(context.Background(), req)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})
}

func GetExamInfos(c *gin.Context) {
	req := new(exam.InfoRequest)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("GetExamInfos ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	res, err := service.GrpcExamClientImpl.GetExamInfos(context.Background(), req)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})
}
