package exam

import (
	"context"
	"strconv"

	"dubbo.apache.org/dubbo-go/v3/common/logger"
	"github.com/fonchain_enterprise/fonchain-main/api/exam"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/login"
	"github.com/fonchain_enterprise/fonchain-main/pkg/serializer"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/gin-gonic/gin"
)

func SaveExamPaper(c *gin.Context) {
	req := new(exam.ExamPaperInfo)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("SaveExamPaper ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	userInfo := login.GetUserInfoFromC(c)
	logger.Info("-------------------------------")
	logger.Info("userInfo:", userInfo)
	req.CreaterName = userInfo.NickName
	req.CreaterId = strconv.FormatUint(userInfo.ID, 10)

	res, err := service.GrpcExamClientImpl.SaveExamPaper(context.Background(), req)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
}

func UpdateExamPaper(c *gin.Context) {
	req := new(exam.ExamPaperInfo)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("UpdateExamPaper ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	userInfo := login.GetUserInfoFromC(c)
	logger.Info("-------------------------------")
	logger.Info("userInfo:", userInfo)
	req.UpdaterName = userInfo.NickName
	req.UpdaterID = int32(userInfo.ID)

	res, err := service.GrpcExamClientImpl.UpdateExamPaper(context.Background(), req)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
}

func DeleteExamPaper(c *gin.Context) {
	req := new(exam.ExamPaperInfo)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("DeleteExamPaper ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	res, err := service.GrpcExamClientImpl.DeleteExamPaper(context.Background(), req)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
}

func GetExamPapers(c *gin.Context) {
	req := new(exam.ExamPaperListRequest)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("GetExamPapers ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	res, err := service.GrpcExamClientImpl.GetExamPapers(context.Background(), req)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
}

func DetailExamPaper(c *gin.Context) {
	req := new(exam.ExamPaperIdRequest)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("DetailExamPaper ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	res, err := service.GrpcExamClientImpl.DetailExamPaper(context.Background(), req)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
}
