package exam

import (
	"context"

	"dubbo.apache.org/dubbo-go/v3/common/logger"
	"github.com/fonchain_enterprise/fonchain-main/api/exam"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/login"
	"github.com/fonchain_enterprise/fonchain-main/pkg/serializer"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/gin-gonic/gin"
)

func SaveTrain(c *gin.Context) {
	req := new(exam.TrainInfo)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("SaveTrain ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	userInfo := login.GetUserInfoFromC(c)
	logger.Info("-------------------------------")
	logger.Info("userInfo:", userInfo)
	req.CreaterName = userInfo.NickName
	req.CreaterID = int32(userInfo.ID)

	_, err := service.GrpcExamClientImpl.SaveTrain(context.Background(), req)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Status: e.Ok,
	})
}

func GetTrainInfo(c *gin.Context) {
	req := new(exam.TrainQueryRequest)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("GetTrainInfo ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	res, err := service.GrpcExamClientImpl.GetTrainInfo(context.Background(), req)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})
}

func GetTrainInfos(c *gin.Context) {
	req := new(exam.TrainQueryRequest)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("GetTrainInfos ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	res, err := service.GrpcExamClientImpl.GetTrainInfos(context.Background(), req)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})
}

func UpdateTrain(c *gin.Context) {
	req := new(exam.TrainInfo)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("UpdateTrain ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	userInfo := login.GetUserInfoFromC(c)
	logger.Info("-------------------------------")
	logger.Info("userInfo:", userInfo)
	req.UpdaterName = userInfo.NickName
	req.UpdaterID = int32(userInfo.ID)

	_, err := service.GrpcExamClientImpl.UpdateTrain(context.Background(), req)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Status: e.Ok,
	})
}

func DeleteTrain(c *gin.Context) {
	req := new(exam.TrainInfo)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("DeleteTrain ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	_, err := service.GrpcExamClientImpl.DeleteTrain(context.Background(), req)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Status: e.Ok,
	})
}
