package exam

import (
	"context"
	"fmt"
	"net/url"
	"os"

	"strconv"

	"io/ioutil"

	"time"

	"github.com/fonchain_enterprise/fonchain-main/api/position"

	"dubbo.apache.org/dubbo-go/v3/common/logger"
	"github.com/fonchain_enterprise/fonchain-main/api/account"
	"github.com/fonchain_enterprise/fonchain-main/api/exam"
	"github.com/fonchain_enterprise/fonchain-main/api/rule"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/login"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/vo"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/vo/v2/user"
	"github.com/fonchain_enterprise/fonchain-main/pkg/serializer"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/gin-gonic/gin"
	"github.com/tealeg/xlsx"
)

func SaveUserExam(c *gin.Context) {
	fmt.Println("=============================================")
	req := new(exam.UserExamTaskRecordInfo)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("SaveUserExam ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	userInfo := login.GetUserInfoFromC(c)
	logger.Info("-------------------------------")
	logger.Info("userInfo:", userInfo)
	req.UserName = userInfo.NickName
	req.UserID = strconv.FormatUint(userInfo.ID, 10)

	infoReq := &account.InfoRequest{
		ID: userInfo.ID,
	}
	infoRes, err := service.AccountProvider.Info(c, infoReq)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	req.JobNumber = infoRes.Info.JobNum
	req.Phone = infoRes.Info.TelNum

	depPosReq := &rule.RulesRequest{
		AccountID: userInfo.ID,
	}
	depPosRes, err := service.RuleProvider.UserInfo(c, depPosReq)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}
	fmt.Println("-------------------------------")
	fmt.Println("部门岗位:", depPosRes.PositionUsers)

	// req.UserName = "李娟测试"
	// req.UserID = "3729"
	// req.JobNumber = "FL043"
	// req.Phone = "15250470713"
	req.PositionIds = make([]int64, 0)
	req.PositionNames = make([]string, 0)
	req.DepartmentIds = make([]int64, 0)
	req.DepartmentNames = make([]string, 0)
	// req.PositionIds = append(req.PositionIds, 99)
	// req.PositionNames = append(req.PositionNames, "运维")
	// req.DepartmentIds = append(req.DepartmentIds, 3)
	// req.DepartmentIds = append(req.DepartmentIds, 71)
	// req.DepartmentNames = append(req.DepartmentNames, "总经理室")
	// req.DepartmentNames = append(req.DepartmentNames, "开发测试用部门")
	positionIdsMap := make(map[uint64]bool)
	positionNamesMap := make(map[string]bool)
	departmentIdsMap := make(map[uint64]bool)
	departmentNamesMap := make(map[string]bool)
	for _, value := range depPosRes.PositionUsers {
		if _, exists := positionIdsMap[value.PositionID]; !exists {
			req.PositionIds = append(req.PositionIds, int64(value.PositionID))
			positionIdsMap[value.PositionID] = true
		}

		if _, exists := positionNamesMap[value.PositionName]; !exists {
			req.PositionNames = append(req.PositionNames, value.PositionName)
			positionNamesMap[value.PositionName] = true
		}

		if _, exists := departmentIdsMap[value.DepartmentId]; !exists {
			req.DepartmentIds = append(req.DepartmentIds, int64(value.DepartmentId))
			departmentIdsMap[value.DepartmentId] = true
		}

		if _, exists := departmentNamesMap[value.DepartmentName]; !exists {
			req.DepartmentNames = append(req.DepartmentNames, value.DepartmentName)
			departmentNamesMap[value.DepartmentName] = true
		}
	}

	res, err := service.GrpcExamClientImpl.SaveUserExam(context.Background(), req)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
}

func GetPersonUserExamTaskRecords(c *gin.Context) {
	req := new(exam.PersonUserExamTaskRecordsRequest)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("GetPersonUserExamTaskRecords ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	userInfo := login.GetUserInfoFromC(c)
	req.UserID = strconv.FormatUint(userInfo.ID, 10)

	infoReq := &rule.RulesRequest{
		AccountID: userInfo.ID,
	}
	infoRes, err := service.RuleProvider.UserInfo(c, infoReq)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}
	fmt.Println("-------------------------------")
	fmt.Println("部门岗位:", infoRes)
	fmt.Println("部门岗位:", infoRes.PositionUsers)

	req.PositionIds = make([]int64, 0)
	req.DepartmentIds = make([]int64, 0)
	for _, value := range infoRes.PositionUsers {
		req.PositionIds = append(req.PositionIds, int64(value.PositionID))
		req.DepartmentIds = append(req.DepartmentIds, int64(value.DepartmentId))
	}

	res, err := service.GrpcExamClientImpl.GetPersonUserExamTaskRecords(context.Background(), req)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
}

func GetPersonUserExamSubTaskRecords(c *gin.Context) {
	fmt.Println("=============================================")
	req := new(exam.PersonUserExamSubTaskRecordsRequest)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("GetPersonUserExamSubTaskRecords ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	userInfo := login.GetUserInfoFromC(c)
	req.UserID = strconv.FormatUint(userInfo.ID, 10)

	res, err := service.GrpcExamClientImpl.GetPersonUserExamSubTaskRecords(context.Background(), req)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
}

func GetPersonScore(c *gin.Context) {
	fmt.Println("=============================================")
	req := new(exam.PersonScoreRequest)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("GetPersonScore ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if req.UserID == "" {
		userInfo := login.GetUserInfoFromC(c)
		req.UserID = strconv.FormatUint(userInfo.ID, 10)
	}

	res, err := service.GrpcExamClientImpl.GetPersonScore(context.Background(), req)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
}

func GradingPaper(c *gin.Context) {
	fmt.Println("=============================================")
	req := new(exam.GradingPaperRequest)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("GradingPaper ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	userInfo := login.GetUserInfoFromC(c)
	req.GradeID = int64(userInfo.ID)
	req.GradeName = userInfo.NickName

	infoReq := &account.InfoRequest{
		ID: userInfo.ID,
	}
	infoRes, err := service.AccountProvider.Info(c, infoReq)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}
	req.GradeJobNum = infoRes.Info.JobNum

	res, err := service.GrpcExamClientImpl.GradingPaper(context.Background(), req)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
}

func DataUserTaskDetails(c *gin.Context) {
	fmt.Println("=============================================")
	req := new(exam.DataUserTaskDetailsRequest)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("DataUserTaskDetails ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	res, err := service.GrpcExamClientImpl.DataUserTaskDetails(context.Background(), req)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
}

func DataPersonMonitor(c *gin.Context) {
	fmt.Println("=============================================")
	req := new(exam.DataPersonMonitorRequest)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("DataPersonMonitorRequest ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	positionIds := make([]uint32, 0)
	for _, value := range req.ExamPositionIds {
		positionIds = append(positionIds, uint32(value))
	}
	// 查询部门下人数
	accountReq := account.ListV2Request{
		Page:        1,
		PageSize:    9999,
		PositionIds: positionIds,
	}
	fmt.Println("========== 部门下请求：", accountReq.PositionIds)
	timer := time.Now() // 记录开始时间
	accountRes, accountErr := callAccountForUser(&accountReq)
	if accountErr != nil {
		service.Error(c, e.Error, accountErr)
		return
	}
	// 组装用户数据
	req.AuthUserInfos = make([]*exam.AuthUserInfo, 0)
	if data, ok := accountRes.Data.([]*user.BaseUserResponse); ok {
		for _, jobData := range data {
			userInfo := new(exam.AuthUserInfo)
			userInfo.UserID = int64(jobData.ID)
			userInfo.UserName = jobData.NickName
			userInfo.JobNum = jobData.JobNum
			userInfo.Phone = jobData.TelNum
			userInfo.DepartmentIds = make([]int64, 0)
			userInfo.DepartmentNames = make([]string, 0)
			userInfo.PositionIds = make([]int64, 0)
			userInfo.PositionNames = make([]string, 0)
			for _, depPos := range jobData.DepPositions {
				userInfo.DepartmentIds = append(userInfo.DepartmentIds, int64(depPos.ID))
				userInfo.DepartmentNames = append(userInfo.DepartmentNames, depPos.Name)
			}
			for _, pos := range jobData.Positions {
				userInfo.PositionIds = append(userInfo.PositionIds, int64(pos.ID))
				userInfo.PositionNames = append(userInfo.PositionNames, pos.Name)
			}
			req.AuthUserInfos = append(req.AuthUserInfos, userInfo)
		}
	} else {
		fmt.Println("Data 不是预期的 []*user.BaseUserResponse 类型")
	}
	elapsed := time.Since(timer) // 计算自开始时间以来经过的时间
	fmt.Printf("ListV2 耗时 %v milliseconds\n", elapsed.Milliseconds())

	res, err := service.GrpcExamClientImpl.DataPersonMonitor(context.Background(), req)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
}

func DataHearMonitor(c *gin.Context) {
	fmt.Println("=============================================")
	req := new(exam.DataHearMonitorRequest)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("DataHearMonitorRequest ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	res, err := service.GrpcExamClientImpl.DataHearMonitor(context.Background(), req)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
}

func ExportExcelDataPersonMonitor(c *gin.Context) {
	fmt.Println("=============================================")
	req := new(exam.DataPersonMonitorRequest)
	if err := c.ShouldBind(req); err != nil {
		logger.Errorf("DataPersonMonitorRequest ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	positionIds := make([]uint32, 0)
	for _, value := range req.ExamPositionIds {
		positionIds = append(positionIds, uint32(value))
	}
	// 查询部门下人数
	accountReq := account.ListV2Request{
		Page:        1,
		PageSize:    9999,
		PositionIds: positionIds,
	}
	fmt.Println("========== 部门下请求：", accountReq.PositionIds)
	timer := time.Now() // 记录开始时间
	accountRes, accountErr := callAccountForUser(&accountReq)
	if accountErr != nil {
		service.Error(c, e.Error, accountErr)
		return
	}
	// 组装用户数据
	req.AuthUserInfos = make([]*exam.AuthUserInfo, 0)
	if data, ok := accountRes.Data.([]*user.BaseUserResponse); ok {
		for _, jobData := range data {
			userInfo := new(exam.AuthUserInfo)
			userInfo.UserID = int64(jobData.ID)
			userInfo.UserName = jobData.NickName
			userInfo.Phone = jobData.TelNum
			userInfo.DepartmentIds = make([]int64, 0)
			userInfo.DepartmentNames = make([]string, 0)
			userInfo.PositionIds = make([]int64, 0)
			userInfo.PositionNames = make([]string, 0)
			for _, depPos := range jobData.DepPositions {
				userInfo.DepartmentIds = append(userInfo.DepartmentIds, int64(depPos.ID))
				userInfo.DepartmentNames = append(userInfo.DepartmentNames, depPos.Name)
			}
			for _, pos := range jobData.Positions {
				userInfo.PositionIds = append(userInfo.PositionIds, int64(pos.ID))
				userInfo.PositionNames = append(userInfo.PositionNames, pos.Name)
			}
			req.AuthUserInfos = append(req.AuthUserInfos, userInfo)
		}
	} else {
		fmt.Println("Data 不是预期的 []*user.BaseUserResponse 类型")
	}
	elapsed := time.Since(timer) // 计算自开始时间以来经过的时间
	fmt.Printf("ListV2 耗时 %v milliseconds\n", elapsed.Milliseconds())

	res, err := service.GrpcExamClientImpl.DataPersonMonitor(context.Background(), req)
	fmt.Println("=========================")
	fmt.Println(res)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	// 创建临时文件
	tmpfile, err := ioutil.TempFile("", "DataPersonMonitor_*.xlsx")
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}
	// 将数据写入 Excel
	if req.Type == "monitor" {
		err = CreateDataPersonMonitorExcel(res, tmpfile)
		if err != nil {
			service.Error(c, e.Error, err)
			return
		}
	} else {
		err = CreateDataPersonMonitorSignExcel(res, tmpfile)
		if err != nil {
			service.Error(c, e.Error, err)
			return
		}
	}
	defer os.Remove(tmpfile.Name()) // 删除临时文件

	// 返回文件流
	formattedDate := time.Now().Format("20060102")
	fileName := req.ExamTaskName + formattedDate + ".xlsx"
	escapedFileName := url.QueryEscape(fileName)
	c.Header("Content-Disposition", "attachment; filename*=UTF-8''"+escapedFileName)
	c.Header("Content-Type", "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet")
	c.File(tmpfile.Name())
}

// 将数据写入 Excel 文件，人员考试导出
func CreateDataPersonMonitorExcel(data *exam.DataPersonMonitorListResponse, file *os.File) error {
	// 创建 Excel 文件
	xlsFile := xlsx.NewFile()
	sheet, err := xlsFile.AddSheet("考培数据")
	if err != nil {
		return err
	}
	// 添加表头
	row := sheet.AddRow()
	row.AddCell().SetValue("姓名")
	row.AddCell().SetValue("部门")
	row.AddCell().SetValue("岗位")
	row.AddCell().SetValue("手机号")
	row.AddCell().SetValue("结果")
	row.AddCell().SetValue("获得证书")
	row.AddCell().SetValue("开始时间")
	row.AddCell().SetValue("完成时间")
	// 将数据写入 Excel
	for _, req := range data.DataPersonMonitorResponses {
		row := sheet.AddRow()
		row.AddCell().SetValue(req.UserName)
		row.AddCell().SetValue(req.DepartmentNames)
		row.AddCell().SetValue(req.PositionNames)
		row.AddCell().SetValue(req.Phone)
		row.AddCell().SetValue(changeResultStatus(req.ResultStatus))
		row.AddCell().SetValue(changeCert(req.IsCert))
		row.AddCell().SetValue(req.StartAt)
		row.AddCell().SetValue(req.FinishAt)
	}
	// 保存 Excel 文件
	err = xlsFile.Write(file)
	if err != nil {
		return err
	}

	return nil
}

// 将数据写入 Excel 文件,签到导出
func CreateDataPersonMonitorSignExcel(data *exam.DataPersonMonitorListResponse, file *os.File) error {
	// 创建 Excel 文件
	xlsFile := xlsx.NewFile()
	sheet, err := xlsFile.AddSheet("考培数据")
	if err != nil {
		return err
	}
	// 添加表头
	row := sheet.AddRow()
	row.AddCell().SetValue("姓名")
	row.AddCell().SetValue("部门")
	row.AddCell().SetValue("岗位")
	row.AddCell().SetValue("手机号")
	row.AddCell().SetValue("是否签到")
	row.AddCell().SetValue("实际签到时间")
	row.AddCell().SetValue("实际签到地点")
	// 将数据写入 Excel
	for _, req := range data.DataPersonMonitorResponses {
		row := sheet.AddRow()
		row.AddCell().SetValue(req.UserName)
		row.AddCell().SetValue(req.DepartmentNames)
		row.AddCell().SetValue(req.PositionNames)
		row.AddCell().SetValue(req.Phone)
		row.AddCell().SetValue(changeCert(req.IsSignIn))
		row.AddCell().SetValue(req.SignInTime)
		row.AddCell().SetValue(req.SignInAddress)
	}
	// 保存 Excel 文件
	err = xlsFile.Write(file)
	if err != nil {
		return err
	}

	return nil
}

func changeResultStatus(in int32) (out string) {
	out = ""
	if in == 1 {
		out = "待评定"
	} else if in == 2 {
		out = "通过"
	} else if in == 3 {
		out = "未通过"
	}
	return out
}

func changeCert(in int32) (out string) {
	out = ""
	if in == 2 {
		out = "是"
	} else {
		out = "否"
	}
	return out
}

func callAccountForUser(req *account.ListV2Request) (response vo.PageResponseV2, err error) {
	var userIds []uint32

	if req.EndEnterDate != "" {
		req.EndEnterDate += " 23:59:59"
	}

	response = vo.PageResponseV2{
		Count: 0,
	}

	res, err := service.AccountProvider.ListV2(context.Background(), req)
	if err != nil {
		return response, err
	}

	if res.Count <= 0 {
		return response, nil
	}
	response.Count = res.Count

	for _, v := range res.Data {
		userIds = append(userIds, uint32(v.ID))
	}

	//查询部门
	puRequest := &position.UserInfosV2Request{IDs: userIds}
	puRes, err := service.PositionProvider.UserInfosV2(context.Background(), puRequest)

	if err != nil {
		return response, err
	}

	response.Data = serializer.UserV2(res, puRes, 0)

	return
}
