package goods_management

import (
	"context"
	"dubbo.apache.org/dubbo-go/v3/common/logger"
	"errors"
	"fmt"
	"github.com/fonchain_enterprise/fonchain-main/api/goods_management"
	"github.com/fonchain_enterprise/fonchain-main/pkg/config"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model"
	"github.com/fonchain_enterprise/fonchain-main/pkg/serializer"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/utils"
	"github.com/fonchain_enterprise/utils/objstorage"
	"github.com/gin-gonic/gin"
	"net/http"
)

func AddBorrowGoods(c *gin.Context) {
	var addBorrowGoodsRequest goods_management.AddBorrowGoodsRequest
	if err := c.ShouldBind(&addBorrowGoodsRequest); err != nil {
		logger.Errorf("addBorrowGoodsRequest ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	// 获取操作人员姓名
	if mLoginInfoAny, exists := c.Get("mLoginInfo"); exists {
		userInfo := mLoginInfoAny.(model.LoginInfo)
		addBorrowGoodsRequest.OperatorName = userInfo.NickName
	} else {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   nil,
			Msg:    errors.New("未获取到用户信息").Error(),
			Status: e.Failed,
		})
		return
	}
	//addBorrowGoodsRequest.OperatorName = "大志"

	rep, err := service.GrpcGoodsManagementImpl.AddBorrowGoods(context.Background(), &addBorrowGoodsRequest)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	c.JSON(http.StatusOK, serializer.Response{
		Status: e.Ok,
		Code:   e.SUCCESS,
		Data:   rep,
	})
}

func BorrowGoodsList(c *gin.Context) {
	var borrowGoodsListRequest goods_management.BorrowGoodsListRequest
	if err := c.ShouldBind(&borrowGoodsListRequest); err != nil {
		logger.Errorf("borrowGoodsListRequest ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	rep, err := service.GrpcGoodsManagementImpl.BorrowGoodsList(context.Background(), &borrowGoodsListRequest)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	c.JSON(http.StatusOK, serializer.Response{
		Status: e.Ok,
		Code:   e.SUCCESS,
		Data:   rep,
	})
}

func GenerateBorrowQrCode(c *gin.Context) {
	var generateBorrowQrCodeRequest goods_management.GenerateBorrowQrCodeRequest
	if err := c.ShouldBind(&generateBorrowQrCodeRequest); err != nil {
		logger.Errorf("generateBorrowQrCodeRequest ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	qrContent := config.MaterialHost + "/#/pages/borrow/index?uid=" + generateBorrowQrCodeRequest.BorrowInfoUid
	fmt.Println("AppointUrl", config.MaterialHost)
	fmt.Println("qrcontent", qrContent)
	qrBytes, err := utils.CreateQr(qrContent, "") // Assuming CreateQr returns []byte
	// fmt.Println("qrBytes 是：", qrBytes)

	if err != nil {
		fmt.Println("创建二维码失败:", err)
		service.ResponseQuickMsg(c, e.Failed, "创建二维码失败", nil)
		return
	}

	qrbytess := []byte(qrBytes)

	objectName := fmt.Sprintf("%s/%s/%s.jpg", config.ConfigData.Oss.BaseDir, config.Env, generateBorrowQrCodeRequest.BorrowInfoUid)
	BOSClient, _ := objstorage.NewOSS(config.ConfigData.Oss.AccessKeyId, config.ConfigData.Oss.AccessKeySecret, config.ConfigData.Oss.Endpoint)
	_, err = BOSClient.PutObjectFromBytes(config.ConfigData.Oss.BucketName, objectName, qrbytess)
	fmt.Println("err", err)
	if err != nil {
		logger.Errorf("quickBos err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	url := fmt.Sprintf("%s/%s", config.ConfigData.Oss.CdnHost, objectName)
	fmt.Println("url", url)
	generateBorrowQrCodeRequest.QrUrl = url
	rep, err := service.GrpcGoodsManagementImpl.GenerateBorrowQrCode(context.Background(), &generateBorrowQrCodeRequest)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	c.JSON(http.StatusOK, serializer.Response{
		Status: e.Ok,
		Code:   e.SUCCESS,
		Data:   rep,
	})
}

func PutDownBorrowGoods(c *gin.Context) {
	var putDownBorrowGoodsRequest goods_management.PutDownBorrowGoodsRequest
	if err := c.ShouldBind(&putDownBorrowGoodsRequest); err != nil {
		logger.Errorf("putDownBorrowGoodsRequest ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	rep, err := service.GrpcGoodsManagementImpl.PutDownBorrowGoods(context.Background(), &putDownBorrowGoodsRequest)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	c.JSON(http.StatusOK, serializer.Response{
		Status: e.Ok,
		Code:   e.SUCCESS,
		Data:   rep,
	})
}

func DeleteBorrowGoods(c *gin.Context) {
	var deleteBorrowGoodsRequest goods_management.DeleteBorrowGoodsRequest
	if err := c.ShouldBind(&deleteBorrowGoodsRequest); err != nil {
		logger.Errorf("deleteBorrowGoodsRequest ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	rep, err := service.GrpcGoodsManagementImpl.DeleteBorrowGoods(context.Background(), &deleteBorrowGoodsRequest)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	c.JSON(http.StatusOK, serializer.Response{
		Status: e.Ok,
		Code:   e.SUCCESS,
		Data:   rep,
	})
}

func SpecificBorrowRecord(c *gin.Context) {
	var specificBorrowRecordRequest goods_management.SpecificBorrowRecordRequest
	if err := c.ShouldBind(&specificBorrowRecordRequest); err != nil {
		logger.Errorf("SpecificBorrowRecord ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	rep, err := service.GrpcGoodsManagementImpl.SpecificBorrowRecord(context.Background(), &specificBorrowRecordRequest)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	c.JSON(http.StatusOK, serializer.Response{
		Status: e.Ok,
		Code:   e.SUCCESS,
		Data:   rep,
	})
}
