package service

import (
	"dubbo.apache.org/dubbo-go/v3/common/logger"
	"fmt"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/serializer"
	"github.com/fonchain_enterprise/fonchain-main/pkg/utils/holiday"
	"github.com/gin-gonic/gin"
)

func SingleHolidayInfo(c *gin.Context) {
	var holidayInfoReq holiday.HolidayInfoReq
	if err := c.ShouldBind(&holidayInfoReq); err != nil {
		logger.Errorf("SingleHolidayInfo ShouldBind err", err)
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	holidayInfoReq.BeginDate = holiday.ConvertTime(holidayInfoReq.BeginDate)

	resp, err := holiday.GetSingleData(holidayInfoReq.BeginDate, true)
	if err != nil {
		logger.Errorf("GetSingleData err", err)
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   resp,
		Status: e.Ok,
	})
}

func MultiHolidayInfo(c *gin.Context) {
	var holidayInfoReq holiday.HolidayInfoReq
	if err := c.ShouldBind(&holidayInfoReq); err != nil {
		logger.Errorf("SingleHolidayInfo ShouldBind err", err)
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	holidayInfoReq.BeginDate = holiday.ConvertTime(holidayInfoReq.BeginDate)
	holidayInfoReq.EndDate = holiday.ConvertTime(holidayInfoReq.EndDate)
	fmt.Println("holidayInfoReq.BeginDate =  : ", holidayInfoReq.BeginDate)
	fmt.Println("holidayInfoReq.EndDate =  : ", holidayInfoReq.EndDate)

	resp, err := holiday.GetMultiData(holidayInfoReq.BeginDate, holidayInfoReq.EndDate, true)
	if err != nil {
		logger.Errorf("GetMultiData err", err)
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   resp,
		Status: e.Ok,
	})
}
