package inventory

import (
	"context"
	"fmt"
	"github.com/fonchain_enterprise/fonchain-main/api/inventory"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/login"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
)

func Remove(c *gin.Context) {
	req := inventory.RemoveRequest{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	userInfo := login.GetUserInfoFromC(c)
	req.UserId = uint32(userInfo.ID)
	req.UserName = userInfo.NickName

	res, err := service.InventoryProvider.Remove(context.Background(), &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func List(c *gin.Context) {
	req := inventory.ProductList{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}
	fmt.Println("1----------", req)
	fmt.Println("1----------", service.InventoryProvider)

	res, err := service.InventoryProvider.List(c, &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func DetailBarCode(c *gin.Context) {
	req := inventory.ProductDetail{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	res, err := service.InventoryProvider.DetailByBarCode(context.Background(), &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func Detail(c *gin.Context) {
	req := inventory.ProductDetail{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	res, err := service.InventoryProvider.Detail(context.Background(), &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func Create(c *gin.Context) {
	req := inventory.ProductRequest{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}
	userInfo := login.GetUserInfoFromC(c)
	req.UserId = uint32(userInfo.ID)
	req.UserName = userInfo.NickName

	res, err := service.InventoryProvider.Create(context.Background(), &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func Update(c *gin.Context) {
	req := inventory.ProductRequest{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	userInfo := login.GetUserInfoFromC(c)
	req.UserId = uint32(userInfo.ID)
	req.UserName = userInfo.NickName

	res, err := service.InventoryProvider.Update(context.Background(), &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func ChangeStock(c *gin.Context) {
	req := inventory.ChangeStockRequest{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	userInfo := login.GetUserInfoFromC(c)
	req.UserId = uint32(userInfo.ID)
	req.UserName = userInfo.NickName

	res, err := service.InventoryProvider.ChangeStock(context.Background(), &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func RemoveProductLog(c *gin.Context) {
	req := inventory.ProductLogDetail{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	res, err := service.InventoryProvider.RemoveProductLog(context.Background(), &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func ListProductLog(c *gin.Context) {
	req := inventory.ProductLogList{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	res, err := service.InventoryProvider.ListProductLog(context.Background(), &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func DetailProductLog(c *gin.Context) {
	req := inventory.ProductLogDetail{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	res, err := service.InventoryProvider.DetailProductLog(context.Background(), &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func CreateProductLog(c *gin.Context) {
	req := inventory.ProductLogRequest{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	res, err := service.InventoryProvider.CreateProductLog(context.Background(), &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func UpdateProductLog(c *gin.Context) {
	req := inventory.ProductLogRequest{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	res, err := service.InventoryProvider.UpdateProductLog(context.Background(), &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}
