package inventory

import (
	"context"
	"errors"
	"fmt"
	"github.com/fonchain_enterprise/fonchain-main/api/warehouse"
	"github.com/fonchain_enterprise/fonchain-main/pkg/config"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/vo"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/utils"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
	"github.com/google/uuid"
	"time"
)

const (
	Status_Not_In_House    = iota + 1 //未入库
	Status_In_House                   //已入库
	Status_Near_Time_House            //即将到期
	Status_Over_House                 //超时 延期中
	Status_Out_House                  //已取货
	Status_Not_Pay                    //没支付
	Status_Over_House_Paid            //延期已补款
)

func WareHousePayBill(c *gin.Context) {
	req := warehouse.TransactionListRequest{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	res, err := service.WarehouseProvider.TransactionList(context.Background(), &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func CountPickUpWareHouse(c *gin.Context) {
	req := warehouse.WarehouseOrderList{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}
	fmt.Println(req.PageSize)
	fmt.Println(req.Page)

	res, err := service.WarehouseProvider.CountPickUp(context.Background(), &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func ListWareHouse(c *gin.Context) {
	req := warehouse.WarehouseOrderList{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}
	fmt.Println(req.PageSize)
	fmt.Println(req.Page)

	res, err := service.WarehouseProvider.List(context.Background(), &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func DownWareHouse(c *gin.Context) {

	req := warehouse.WarehouseOrderList{}
	if err := c.ShouldBindQuery(&req); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	req.Page = 1
	req.PageSize = 999999
	fmt.Println("1-------", req)

	res, err := service.WarehouseProvider.List(context.Background(), &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	//下载
	if len(res.Data) == 0 {
		service.Error(c, e.Error, errors.New("所选条件无可导出数据"))
		return
	}

	columns := []string{"ID", "画作编号", "画作名称", "画家姓名", "画作平尺数", "寄存人", "联系方式", "寄存地点", "下单时间", "状态",
		"寄存周期", "支付金额", "寄存开始日期", "寄存结束日期", "入库时间", "出库时间", "库位类型", "库位号"}
	var data []interface{}
	// 添加数据行
	for _, collectionRecord := range res.Data {
		var info []string
		info = append(info, fmt.Sprintf("%d", collectionRecord.ID))
		info = append(info, collectionRecord.ArtworkNum)
		info = append(info, collectionRecord.ArtworkName)
		info = append(info, collectionRecord.ArtistName)
		info = append(info, fmt.Sprintf("%d", collectionRecord.ArtworkSquareSize))
		info = append(info, collectionRecord.SubmitterName)
		info = append(info, collectionRecord.SubmitterTelnum)
		info = append(info, collectionRecord.WarehouseName)
		info = append(info, collectionRecord.CreatedAt)

		if collectionRecord.ArtworkStatus == Status_Not_In_House {
			info = append(info, "未入库")
		} else if collectionRecord.ArtworkStatus == Status_In_House {
			info = append(info, "已入库")
		} else if collectionRecord.ArtworkStatus == Status_Near_Time_House {
			info = append(info, "即将到期")
		} else if collectionRecord.ArtworkStatus == Status_Over_House {
			info = append(info, "延期未补款")
		} else if collectionRecord.ArtworkStatus == Status_Out_House {
			info = append(info, "已取货")
		} else if collectionRecord.ArtworkStatus == Status_Over_House_Paid {
			info = append(info, "延期已补款")
		} else {
			info = append(info, "其他")
		}

		info = append(info, collectionRecord.CycleName)
		info = append(info, collectionRecord.Money)
		info = append(info, collectionRecord.StartAt)
		info = append(info, collectionRecord.EndAt)
		info = append(info, collectionRecord.InTime)
		info = append(info, collectionRecord.PickupTime)
		shelvesName := "其他"
		if collectionRecord.ShelvesType == 1 {
			shelvesName = "货架"
		}
		info = append(info, shelvesName)
		info = append(info, collectionRecord.ShelvesNum)

		data = append(data, &info)
	}

	content := utils.ToExcel(columns, data, "slice")
	utils.ResponseXls(c, content, "集保订单下载"+time.Now().Format(model.DateFormat))
	return
}

func ExportWareHouse(c *gin.Context) {
	req := warehouse.WarehouseOrderList{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}
	req.Page = 1
	req.PageSize = 999999

	res, err := service.WarehouseProvider.List(context.Background(), &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	//下载
	if len(res.Data) == 0 {
		service.Error(c, e.Error, errors.New("所选条件无可导出数据"))
		return
	}

	httpType := model.HttpType
	if config.IsHttps {
		httpType = model.HttpsType
	}

	var exportUrl = fmt.Sprintf("%s%s/api/warehouse/down?uuid=%s&Key=%s&Status=%d&MobileKey=%s&StartCreatedAt=%s&EndCreatedAt=%s&StartEndAt=%s&EndEndAt=%s&OnlyPickUp=%t&ArtworkNum=%s",
		httpType, c.Request.Host, uuid.New().String(), req.Key, req.Status, req.MobileKey, req.StartCreatedAt, req.EndCreatedAt, req.StartEndAt, req.EndEndAt, req.OnlyPickUp, req.ArtworkNum)

	service.Success(c, map[string]string{"exportUrl": exportUrl})
	return
}

func EditWareHouse(c *gin.Context) {
	req := warehouse.WarehouseOrderRequest{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	res, err := service.WarehouseProvider.Update(context.Background(), &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func GetEndAtYmd(c *gin.Context) {

	req := vo.WareHouseDataYmd{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	cycleReq := warehouse.WarehouseCycleDetail{ID: req.CycleID}

	res, err := service.WarehouseProvider.DetailWarehouseCycle(context.Background(), &cycleReq)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	t, err := time.Parse(`2006-01-02`, req.StartAt)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, vo.WareHouseDataYmdRes{EndAt: t.AddDate(0, 0, int(res.Days)).Format("2006-01-02")})
	return
}

func DetailWareHouse(c *gin.Context) {
	req := warehouse.WarehouseOrderDetail{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	res, err := service.WarehouseProvider.Detail(context.Background(), &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func RemoveWareHouse(c *gin.Context) {
	req := warehouse.WarehouseOrderDetail{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	res, err := service.WarehouseProvider.Remove(context.Background(), &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

//GetProductPreInfo 取货前信息
func GetProductPreInfo(c *gin.Context) {
	req := warehouse.WarehouseOrderDetail{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	res, err := service.WarehouseProvider.PreInfoBeforePickUp(context.Background(), &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

//PickUp 取货
func PickUp(c *gin.Context) {
	req := warehouse.WarehouseOrderDetail{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	res, err := service.WarehouseProvider.PickUp(context.Background(), &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}
