package invoice

import (
	"github.com/fonchain_enterprise/fonchain-main/api/account"
	"github.com/fonchain_enterprise/fonchain-main/api/invoice"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/login"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/request"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/gin-gonic/gin"
)

// FindInvoice 发票列表
func FindInvoice(ctx *gin.Context) {
	req := &invoice.FindInvoiceReq{}
	resp := &invoice.FindInvoiceResp{}
	userList := &account.ListResponse{}
	uniqueUserIds := make(map[uint64]bool)
	var err error
	var userIds []uint64
	if err = ctx.ShouldBind(&req); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	if req.ApplicantName != "" {
		userList, err = service.AccountProvider.List(ctx, &account.ListRequest{
			Domain: "blockchain",
			Key:    req.ApplicantName,
		})
		if err != nil || len(userList.Data) == 0 {
			service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.NOTDATA), resp)
			return
		}
		for _, i := range userList.Data {
			req.UserIds = append(req.UserIds, i.ID)
		}
	}
	resp, err = service.InvoiceProvider.FindInvoice(ctx, req)

	if err != nil || len(resp.List) == 0 {
		service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.NOTDATA), resp)
		return
	}
	for _, i := range resp.List {
		uniqueUserIds[i.ApplicantId] = true
		uniqueUserIds[i.ProcessorId] = true
	}
	for id := range uniqueUserIds {
		userIds = append(userIds, id)
	}
	users, err := service.AccountProvider.ListByIDs(ctx, &account.ListByIDsRequest{
		Domain: "blockchain",
		IDs:    userIds,
	})
	for _, user := range users.Data {
		for _, inv := range resp.List {
			if user.ID == inv.ApplicantId {
				inv.ApplicantName = user.NickName
				inv.ApplicantTel = user.TelNum
			}
			if user.ID == inv.ProcessorId {
				inv.ProcessorName = user.NickName
			}
		}
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.Success), resp)
	return
}

// HandelInvoice 发票处理
func HandelInvoice(ctx *gin.Context) {
	req := &request.InvoiceHandleReq{}
	var err error
	if err = ctx.ShouldBind(&req); err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	userInfo := login.GetUserInfoFromC(ctx)
	info, err := service.InvoiceProvider.FindInvoice(ctx, &invoice.FindInvoiceReq{Id: req.Id})
	if err != nil || len(info.List) == 0 {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	if info.List[0].Status != 1 {
		service.ResponseQuickMsg(ctx, e.Failed, e.GetMsg(e.ErrStatus), nil)
		return
	}
	resp, err := service.InvoiceProvider.UpdateInvoice(ctx, &invoice.UpdateInvoiceReq{
		Id:             req.Id,
		AttachmentsUrl: req.Url,
		ProcessorId:    userInfo.ID,
		Status:         2, //已完成
		Instructions:   req.Remark,
	})
	if err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.Success), resp)
	return
}
