package service

import (
	"context"

	"dubbo.apache.org/dubbo-go/v3/common/logger"
	"github.com/fonchain_enterprise/fonchain-main/api/journal"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/gin-gonic/gin"
)

//添加日志
func AddJournal(c *gin.Context) {
	//声明一个变量
	var req journal.AddJournalreq

	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("Add ShouldBind err", err)
		ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}

	resp, err := GrpcJournalImpl.AddJournal(context.Background(), &req)
	if err != nil {
		ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	if len(resp.Data) > 0 {
		//获取第一个元素的唯一标识
		uuid := resp.Data[0].Uuid
		ResponseQuickMsg(c, e.Ok, resp.Msg, map[string]interface{}{
			"Uuid": uuid,
		})
	} else {
		ResponseQuickMsg(c, e.Ok, resp.Msg, nil)
	}
	return
}

//自己写的日志列表
func SelfList(c *gin.Context) {

	//声明一个变量
	var req journal.SelfListreq
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("Select ShouldBind err", err)
		ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}

	resp, err := GrpcJournalImpl.SelfList(context.Background(), &req)
	if err != nil {
		ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	ResponseQuickMsg(c, e.Ok, resp.Msg, resp)
	return
}

//日志接收列表
func AcceptList(c *gin.Context) {

	//声明一个变量
	var req journal.AcceptListreq
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("Select ShouldBind err", err)
		ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}

	resp, err := GrpcJournalImpl.AcceptList(context.Background(), &req)
	if err != nil {
		ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	ResponseQuickMsg(c, e.Ok, resp.Msg, resp)
	return
}

//日志详情
func JournalInfo(c *gin.Context) {
	//声明一个变量
	var req journal.JournalInforeq

	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("Info ShouldBind err", err)
		ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	resp, err := GrpcJournalImpl.JournalInfo(context.Background(), &req)
	if err != nil {
		ResponseQuickMsg(c, e.Ok, resp.Msg, nil)
		return
	}
	ResponseQuickMsg(c, e.Ok, resp.Msg, resp)
	return
}

//日志已读
func ReadJournal(c *gin.Context) {
	//声明一个变量
	var req journal.ReadJournalreq
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("Update ShouldBind err", err)
		ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	resp, err := GrpcJournalImpl.ReadJournal(context.Background(), &req)
	if err != nil {
		ResponseQuickMsg(c, e.Ok, resp.Msg, nil)
		return
	}
	ResponseQuickMsg(c, e.Ok, resp.Msg, nil)
	return
}

//日志撤回
func UndoJournal(c *gin.Context) {

	//声明一个变量
	var req journal.UndoJournalreq
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("Update ShouldBind err", err)
		ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}

	resp, err := GrpcJournalImpl.UndoJournal(context.Background(), &req)
	if err != nil {
		ResponseQuickMsg(c, e.Failed, "有人已经阅读日志,不可撤回", nil)
		return
	}
	ResponseQuickMsg(c, e.Ok, resp.Msg, nil)
	return
}
