package launch

import (
	"github.com/fonchain_enterprise/fonchain-main/api/account"
	"github.com/fonchain_enterprise/fonchain-main/api/conference"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/login"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
)

func CreateConference(c *gin.Context) {
	req := &conference.CreateConferenceRequest{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}
	userInfoAny, _ := c.Get("jwtInfo")
	userInfo := userInfoAny.(login.Info)
	infoReq := &account.InfoRequest{
		ID: userInfo.ID,
	}
	infoRes, err := service.AccountProvider.Info(c, infoReq)
	req.CreatedUid = infoRes.Info.ID
	req.JobNum = infoRes.Info.JobNum
	req.Creator = infoRes.Info.NickName
	res, err := service.GrpcConferenceImpl.CreateConference(c, req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}
func UpdateConference(c *gin.Context) {
	req := &conference.UpdateConferenceRequest{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}
	res, err := service.GrpcConferenceImpl.UpdateConference(c, req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}
func DeletedConference(c *gin.Context) {
	req := &conference.DeletedConferenceRequest{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}
	res, err := service.GrpcConferenceImpl.DeletedConference(c, req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}
	service.Success(c, res)
	return
}
func FindConference(c *gin.Context) {
	req := &conference.FindConferenceRequest{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}
	res, err := service.GrpcConferenceImpl.FindConference(c, req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}
	service.Success(c, res)
	return

}
