package service

import (
	"context"

	"github.com/fonchain_enterprise/fonchain-main/api/log"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/serializer"
	"github.com/gin-gonic/gin"
	"github.com/nacos-group/nacos-sdk-go/common/logger"
)

func LogList(c *gin.Context) {
	//声明一个对象
	var LogListService log.LogListreq

	if err := c.ShouldBind(&LogListService); err != nil {
		logger.Errorf("LogList ShouldBind err", err)
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Msg:  err.Error(),
			Code: e.Failed,
		})
		return
	}
	resp, err := GrpcLogImpl.LogList(context.Background(), &LogListService)
	if err != nil {
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Msg:  err.Error(),
			Code: e.Failed,
		})
		return
	}
	ResponseMsg(c, e.SUCCESS, serializer.Response{
		Msg:  resp.Msg,
		Code: e.Ok,
		Data: resp,
	})
}

func LogListByAwId(c *gin.Context) {
	//声明一个对象
	var LogListByAwIdService log.LogListreq

	if err := c.ShouldBind(&LogListByAwIdService); err != nil {
		logger.Errorf("LogListByAwId ShouldBind err", err)
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Msg:  err.Error(),
			Code: e.Failed,
		})
		return
	}
	resp, err := GrpcLogImpl.LogListByAwId(context.Background(), &LogListByAwIdService)
	if err != nil {
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Msg:  err.Error(),
			Code: e.Failed,
		})
		return
	}
	ResponseMsg(c, e.SUCCESS, serializer.Response{
		Msg:  resp.Msg,
		Code: e.Ok,
		Data: resp,
	})
}

func LogInfo(c *gin.Context) {
	//声明一个对象
	var LogInfoService log.LogInforeq
	if err := c.ShouldBind(&LogInfoService); err != nil {
		logger.Errorf("LogInfo ShouldBind err", err)
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Msg:  err.Error(),
			Code: e.Failed,
		})
		return
	}
	resp, err := GrpcLogImpl.LogInfo(context.Background(), &LogInfoService)
	if err != nil {
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Msg:  err.Error(),
			Code: e.Failed,
		})
		return
	}
	ResponseMsg(c, e.UpdatePasswordSuccess, serializer.Response{
		Msg:  resp.Msg,
		Code: e.Ok,
		Data: resp,
	})
}
