package oa

import (
	"context"
	"dubbo.apache.org/dubbo-go/v3/common/logger"
	oaApi "github.com/fonchain_enterprise/fonchain-main/api/oa"
	"github.com/fonchain_enterprise/fonchain-main/api/rule"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/login"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/oa_model"
	"github.com/fonchain_enterprise/fonchain-main/pkg/serializer"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/gin-gonic/gin"
	"github.com/jinzhu/copier"
)

func QueryLeaveApplyInfo(c *gin.Context) {
	req := oaApi.LeaveApplyInfoReq{}
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("LeaveApplyInfoReq ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if req.StaffUID == 0 {
		userInfo := login.GetUserInfoFromC(c)

		//提交人
		req.StaffUID = userInfo.ID
	}

	//req.SubmitterID = userInfo.ID

	req.LeaveApplyType = e.CheckLeaveApplyMap[req.LeaveApplyType]
	if req.LeaveApplyType == "" {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    e.NotNeedUseLeave,
			Status: e.Ok,
		})
		return
	}

	leaveApplyInfo, err := service.GrpcOAImpl.QueryLeaveApplyInfo(context.Background(), &req)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Ok,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Msg:    leaveApplyInfo.Msg,
		Data:   leaveApplyInfo,
		Status: e.Ok,
	})

}

func QueryOaApply(c *gin.Context) {
	req := new(oaApi.ApplyRecordReq)
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("QueryOaApply ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	//if req.StaffUID == 0 {
	//	userInfo := login.GetUserInfoFromC(c)
	//
	//	//提交人
	//	req.StaffUID = userInfo.ID
	//}

	queryOaApplyRes := new(oaApi.ApplyRecordRes)

	queryOaApplyRes, err := service.GrpcOAImpl.QueryOaApply(context.Background(), req)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Data:   queryOaApplyRes,
			Status: e.Ok,
		})
		return
	}
	//
	//fmt.Println()
	//fmt.Println("=============================  queryOaApplyRes =============================")
	//fmt.Printf("%+v\n", queryOaApplyRes)
	//fmt.Println("=============================  queryOaApplyRes =============================")
	//fmt.Println()

	oaApply := new(oa_model.OaApplyRes)
	oaApply.Data = make([]*oa_model.OaApply, 0)
	copier.CopyWithOption(&oaApply, queryOaApplyRes, copier.Option{DeepCopy: true})
	for i := 0; i < len(oaApply.Data); i++ {
		oaApply.Data[i].ApplyTypeZhCN = oa_model.TypeZhCN[oaApply.Data[i].ApplyType]
		rulesRequest := new(rule.RulesRequest)
		rulesRequest.AccountID = oaApply.Data[i].StaffUID
		rulesRes, err := service.RuleProvider.UserInfo(context.Background(), rulesRequest)
		if err == nil {
			for j := 0; j < len(rulesRes.PositionUsers); j++ {
				if j == 0 {
					oaApply.Data[i].PositionName = rulesRes.PositionUsers[j].PositionName
				} else {
					oaApply.Data[i].PositionName = oaApply.Data[i].PositionName + "," + rulesRes.PositionUsers[j].PositionName
				}
			}
		}
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Msg:    queryOaApplyRes.Msg,
		Data:   oaApply,
		Status: e.Ok,
	})
}
