package oa

import (
	"dubbo.apache.org/dubbo-go/v3/common/logger"
	oaApi "github.com/fonchain_enterprise/fonchain-main/api/oa"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/serializer"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/gin-gonic/gin"
	"golang.org/x/net/context"
)

func SaveOaSetting(c *gin.Context) {
	req := oaApi.OaSetting{}
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("OaSetting ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if req.Value == nil || req.Keyword == "" {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    e.GetMsg(e.InvalidParams),
			Status: e.Ok,
		})
		return
	}

	res, err := service.GrpcOAImpl.CreateOaSetting(context.Background(), &req)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Ok,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Msg:    res.Msg,
		Data:   res,
		Status: e.Ok,
	})

}

func QueryOaSetting(c *gin.Context) {
	req := oaApi.OaSettingReq{}
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("OaSettingReq ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	res, err := service.GrpcOAImpl.QueryOaSetting(context.Background(), &req)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Ok,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Msg:    res.Msg,
		Data:   res,
		Status: e.Ok,
	})

}

func DeleteOaSetting(c *gin.Context) {
	req := oaApi.DeleteOaSettingReq{}
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("DeleteOaSettingReq ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	if req.UUID == "" {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    e.GetMsg(e.InvalidParams),
			Status: e.Ok,
		})
		return
	}

	res, err := service.GrpcOAImpl.DeleteOaSetting(context.Background(), &req)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Ok,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Msg:    res.Msg,
		Status: e.Ok,
	})
}
