package oa

import (
	"context"
	"dubbo.apache.org/dubbo-go/v3/common/logger"
	"fmt"
	"github.com/fonchain_enterprise/fonchain-main/api/oa"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/login"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/oa_model"
	"github.com/fonchain_enterprise/fonchain-main/pkg/serializer"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/gin-gonic/gin"
	"time"
)

// 检测是否需要记录 去外勤 和 回公司
func OutWorkCheck(c *gin.Context) {
	/*
		外勤时间包含 考勤时间 则无需记录
		去外勤 和  回公司  单独判断  可能只会有一个
	*/

	req := new(oa.OutWorkCheckReq)

	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("OutWorkCheck ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	userInfo := login.GetUserInfoFromC(c)

	outWorkActionTypeRes, _, err := outWorkCheck(userInfo, req)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   outWorkActionTypeRes,
		Status: e.Ok,
	})

}

func outWorkCheck(userInfo login.Info, req *oa.OutWorkCheckReq) (outWorkActionTypeRes *oa_model.OutWorkActionType, applyTimes []*oa.ApplyTime, err error) {
	outWorkActionTypeRes = new(oa_model.OutWorkActionType)
	//  获取 记录
	checkRecordRes, err := service.GrpcOAImpl.QueryCheckRecord(context.Background(), req)
	if err != nil {
		//service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		//	Msg:    err.Error(),
		//	Status: e.Failed,
		//})
		return outWorkActionTypeRes, nil, err
	}

	applyTimes = make([]*oa.ApplyTime, 0)
	currentOutOrBack := ""

	if checkRecordRes.Data != nil {
		applyTimes = checkRecordRes.Data[len(checkRecordRes.Data)-len(checkRecordRes.Data)].ApplyTimes

		for i := 0; i < len(checkRecordRes.Data); i++ {
			currentOutOrBack = checkRecordRes.Data[i].ActionType
		}

		if currentOutOrBack == e.OutWorkBack {
			outWorkActionTypeRes.IsNeed = false
			//service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			//	Data:   outWorkActionTypeRes,
			//	Status: e.Ok,
			//})
			return outWorkActionTypeRes, applyTimes, nil
		}
	}
	fmt.Println("=== outWorkCheck 1 =================================================")
	fmt.Printf("++ applyTime %+v \n", applyTimes)
	fmt.Printf("++ userInfo %+v \n", userInfo)
	fmt.Printf("++ currentOutOrBack %+v \n", currentOutOrBack)
	fmt.Println("=== outWorkCheck 1 =================================================")

	if len(applyTimes) == 0 {
		// 获取 外勤请假
		oaApplyByApplyUuidReq := new(oa.OaApplyByApplyUuidReq)
		oaApplyByApplyUuidReq.ApplyUUID = req.ApplyUUID
		apply, err := service.GrpcOAImpl.QueryOaApplyByApplyUuid(context.Background(), oaApplyByApplyUuidReq)
		fmt.Println("=== outWorkCheck 2 =================================================")
		fmt.Printf("++ apply %+v \n", apply)
		fmt.Println("=== outWorkCheck 2 =================================================")
		if err != nil {
			//service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			//	Msg:    err.Error(),
			//	Status: e.Failed,
			//})
			return outWorkActionTypeRes, applyTimes, err
		}

		applyTimes = apply.ApplyTimes
	}

	fmt.Println("=== outWorkCheck 3 =================================================")
	fmt.Printf("++ applyTime %+v \n", applyTimes)
	fmt.Printf("++ userInfo %+v \n", userInfo)
	fmt.Printf("++ currentOutOrBack %+v \n", currentOutOrBack)
	fmt.Println("=== outWorkCheck 3 =================================================")

	outWorkActionTypeRes.CurrentActionType, outWorkActionTypeRes.IsNeed, err = service.OutWorkCheckVerify(applyTimes, userInfo, currentOutOrBack)
	if err != nil {
		//service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		//	Msg:    err.Error(),
		//	Status: e.Failed,
		//})
		return outWorkActionTypeRes, applyTimes, err
	}
	return
}

func CreateOutWorkCheck(c *gin.Context) {
	req := new(oa.OutWorkCheckReq)
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("OutWorkCheck ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	userInfo := login.GetUserInfoFromC(c)
	outWorkActionTypeRes, applyTimes, err := outWorkCheck(userInfo, req)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	msg := ""

	if outWorkActionTypeRes.IsNeed == true {
		createOutWorkCheckReq := new(oa.OutWorkCheck)
		createOutWorkCheckReq.StaffUID = userInfo.ID
		createOutWorkCheckReq.ActionType = outWorkActionTypeRes.CurrentActionType
		createOutWorkCheckReq.ApplyUUID = req.ApplyUUID
		createOutWorkCheckReq.ActionTime = time.Now().Format("2006-01-02 15:04")
		createOutWorkCheckReq.ApplyTimes = applyTimes

		res, err := service.GrpcOAImpl.CreateOutWorkCheck(context.Background(), createOutWorkCheckReq)
		if err != nil {
			service.ResponseMsg(c, e.SUCCESS, serializer.Response{
				Msg:    err.Error(),
				Status: e.Failed,
			})
			return
		}
		msg = res.Msg
	} else {
		msg = e.ErrNotNeedRecordOutworkcheck
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Msg:    msg,
		Status: e.Ok,
	})
}
