package service

import (
	"context"
	"errors"
	"fmt"
	"github.com/fonchain_enterprise/fonchain-main/api/account"
	"github.com/fonchain_enterprise/fonchain-main/api/oa"
	"github.com/fonchain_enterprise/fonchain-main/api/rule"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/logic/oa_logic"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/oa_model"
	"github.com/fonchain_enterprise/fonchain-main/pkg/utils/holiday"
	"github.com/jinzhu/copier"
	"math"
	"strconv"
	"strings"
	"time"
)

func AttendanceCollectionHandleV1(collectionRes *oa_model.AttendanceCollectionRes, month string, dates []string) error {
	// 获取 员工 列表
	accountReq := new(account.ListRequest)
	accountReq.Domain = "fontree"
	accountReq.Page = 1
	accountReq.PageSize = 9999
	accountRes, err := AccountProvider.List(context.Background(), accountReq)
	if err != nil {
		//ResponseMsg(c, e.SUCCESS, serializer.Response{
		//	Msg:    err.Error(),
		//	Status: e.Failed,
		//})
		return err
	}

	//查询部门
	puRequest := &rule.PositionUserListRequest{}
	puRes, err := RuleProvider.PositionUserList(context.Background(), puRequest)

	if err != nil {
		//Error(c, e.Error, err)
		return err
	}

	for i := 0; i < len(accountRes.Data); i++ {
		for j := 0; j < len(puRes.Data); j++ {
			if accountRes.Data[i].ID == puRes.Data[j].UserId {
				accountRes.Data[i].Positions = append(accountRes.Data[i].Positions, &account.PositionUser{
					PositionID:     puRes.Data[j].PositionID,
					PositionName:   puRes.Data[j].PositionName,
					DepartmentId:   puRes.Data[j].DepartmentId,
					DepartmentCode: puRes.Data[j].DepartmentCode,
					DepartmentName: puRes.Data[j].DepartmentName,
					UserId:         puRes.Data[j].UserId,
					UserName:       puRes.Data[j].UserName,
				})
			}
		}
	}

	//holidayRes := make([]*holiday.RespData, 0)

	// 获取 休息天数    总天数 - 工作天数 (注：周一到周五 法定调休 均为 工作日)
	holidayRes, err := holiday.GetMultiData(holiday.ConvertTime(dates[0]), holiday.ConvertTime(dates[len(dates)-1]), true)
	if err != nil {
		return err
	}

	fmt.Println("step ==============================  01  ============================== ")

	outWorkDates := holiday.SplitMonthV1(month, "2006-01-02")

	// 获取 每位员工的  信息
	for i := 0; i < len(accountRes.Data); i++ {

		//  计算 休息日 需要结合 考勤班次中的 week
		queryTimeOptions := make([]oa_model.QueryTimeOption, 0)
		for k := 0; k < len(accountRes.Data[i].Positions); k++ {
			queryTimeOption := oa_model.QueryTimeOption{
				PositionUID:   strconv.FormatUint(accountRes.Data[i].Positions[k].PositionID, 10),
				DepartmentUID: strconv.FormatUint(accountRes.Data[i].Positions[k].DepartmentId, 10),
			}
			queryTimeOptions = append(queryTimeOptions, queryTimeOption)
		}
		startDay, endDay, err := WorkingTimeWeekBest(queryTimeOptions)
		if err != nil {
			fmt.Println("WorkDays err is :", err.Error())
			return err
		}

		isWeek := make(map[int]int, 0)

		for i := startDay; i <= endDay; i++ {
			isWeek[i] = i
		}

		//commonWorkDays := holiday.GetCommonWorkDays(holidayRes, isWeek)

		restDays := 0

		for j := 0; j < len(holidayRes); j++ {
			if !holidayRes[j].IsToday {
				if holidayRes[j].Type != 0 {
					restDays++
				}
			} else {
				break
			}
		}

		collection := new(oa_model.StaffCollection)
		outWorkStatus := []int32{e.ApprovalWorkStatusOk}
		collection, err = StaffAttendanceCollectionHandleV2(accountRes.Data[i].ID, month, dates, restDays, outWorkStatus, outWorkDates)
		if err != nil {
			return err
		}

		if collection.OutWork != 0 {
			collectionRes.OutWorkTotal += collection.OutWork
		}

		if collection.MakeUp != 0 {
			collectionRes.MakeUpTotal += collection.MakeUp
		}

		//  放到  StaffAttendanceCollectionHandleV2 里面计算
		//  算旷工时  会把 休息日算上  所以 要 减去 休息日(基础)
		//collection.MissDay = collection.MissDay - int32(restDays)

		collection.StaffName = accountRes.Data[i].NickName

		positionNames := make([]string, 0)
		departmentNames := make([]string, 0)

		for j := 0; j < len(accountRes.Data[i].Positions); j++ {
			positionNames = append(positionNames, accountRes.Data[i].Positions[j].PositionName)
		}

		for j := 0; j < len(accountRes.Data[i].Positions); j++ {
			departmentNames = append(departmentNames, accountRes.Data[i].Positions[j].DepartmentName)
		}

		collection.PositionName = strings.Join(positionNames, ",")
		collection.DepartmentName = strings.Join(departmentNames, ",")

		fmt.Printf("前   判断 collection info  内容： %+v", collection)
		fmt.Println()

		if collection.Miss == 0 && collection.Late == 0 && collection.Before == 0 && collection.MissDay == 0 {
			fmt.Printf("后   判断 collection info  内容： %+v", collection)
			fmt.Println()
			collection.Status = true
			collectionRes.FullAttendance += 1
		} else {
			collection.Status = false
			collectionRes.Abnormal += 1
		}

		collectionRes.Staffs = append(collectionRes.Staffs, *collection)
		fmt.Println("step ==============================  03  ============================== ")
		fmt.Printf(" 请假 collection is  : %+v \n", collection)
		fmt.Println("step ==============================  03  ============================== ")
	}

	fmt.Println("step ==============================  04  ============================== ")

	return nil
}

func StaffAttendanceCollectionHandleV1(staffUID uint64, month string, dates []string, restDays int, outWorkStatus []int32) (collection *oa_model.StaffCollection, err error) {
	collection = new(oa_model.StaffCollection)
	collection, err = StaffCollectionClickInV2(staffUID, month, dates)
	if err != nil {
		return nil, err
	}

	fmt.Println("step ==============================  02  ============================== ")
	fmt.Printf(" 打卡  collection is  : %+v \n", collection)
	fmt.Println("step ==============================  02  ============================== ")

	collection.RestDays = int32(restDays)

	// oa 申请  请假  加班  调休
	applyReq := new(oa.ApplyRecordReq)
	applyReq.StaffUID = collection.StaffUID
	applyReq.ApplyType = []string{
		oa_model.TypeLeave, // 请假
		oa_model.TypeSick,
		oa_model.TypeAnnualLeave,
		oa_model.TypeMaritalLeave,
		oa_model.TypeMatingCheckLeave,
		oa_model.TypeMaternityLeave,
		oa_model.TypePaternityLeave,
		oa_model.TypeParentalLeave,
		oa_model.TypeNursingLeave,
		oa_model.TypeFuneralLeave,
		oa_model.TypeBreastFeedingLeave,
		oa_model.TypeAbortLeave,
		oa_model.TypeDayOff,   // 调休
		oa_model.TypeOverTime, // 加班
		//oa_model.TypeOutWork,  // 外勤
		oa_model.TypeMakeUp, // 补卡
	}
	applyReq.ApplyStatus = []int32{e.ApprovalWorkStatusOk}
	applyReq.BeginTime = dates[0]
	applyReq.EndTime = dates[len(dates)-1]
	applyReq.Page = 1
	applyReq.PageSize = 1000
	applyRes, err := GrpcOAImpl.QueryOaApply(context.Background(), applyReq)
	if err != nil {
		return nil, err
	}

	for i := 0; i < len(applyRes.Data); i++ {
		if applyRes.Data[i].ApplyType == oa_model.TypeLeave ||
			applyRes.Data[i].ApplyType == oa_model.TypeSick ||
			applyRes.Data[i].ApplyType == oa_model.TypeAnnualLeave ||
			applyRes.Data[i].ApplyType == oa_model.TypeMaritalLeave ||
			applyRes.Data[i].ApplyType == oa_model.TypeMatingCheckLeave ||
			applyRes.Data[i].ApplyType == oa_model.TypeMaternityLeave ||
			applyRes.Data[i].ApplyType == oa_model.TypePaternityLeave ||
			applyRes.Data[i].ApplyType == oa_model.TypeParentalLeave ||
			applyRes.Data[i].ApplyType == oa_model.TypeNursingLeave ||
			applyRes.Data[i].ApplyType == oa_model.TypeBreastFeedingLeave ||
			applyRes.Data[i].ApplyType == oa_model.TypeAbortLeave ||
			applyRes.Data[i].ApplyType == oa_model.TypeFuneralLeave {
			collection.Leave += 1
		}

		if applyRes.Data[i].ApplyType == oa_model.TypeDayOff {
			collection.DayOff += 1
		}
		if applyRes.Data[i].ApplyType == oa_model.TypeOverTime {
			if CheckDateInMonth(applyRes.Data[i].ApplyTimes[0].Date, dates) {
				collection.OverWork += applyRes.Data[i].Hours
			}

			// 判断 加班时间时候是 周末  如果是   则需要 在 工作日 +1  休息日 -1
			//applyDate, err := holiday.GetSingleData(holiday.ConvertTime(applyRes.Data[i].ApplyTimes[0].Date), false)
			//if err != nil {
			//	return nil, err
			//}
			//
			//if applyDate.Type != 0 {
			//	collection.RestDays -= 1
			//	collection.WorkDays += 1
			//}
		}
		//if applyRes.Data[i].ApplyType == oa_model.TypeOutWork {
		//	collection.OutWork += 1
		//}
		if applyRes.Data[i].ApplyType == oa_model.TypeMakeUp {
			collection.MakeUp += 1
		}
	}

	// 外勤申请 (e.ApprovalWorkStatusDoing , e.ApprovalWorkStatusOk , e.ApprovalWorkStatusFail)
	outWorkApplyReq := new(oa.ApplyRecordReq)
	outWorkApplyReq.StaffUID = collection.StaffUID
	outWorkApplyReq.ApplyType = []string{
		oa_model.TypeOutWork, // 外勤
		//oa_model.TypeMakeUp,   // 补卡
	}
	outWorkApplyReq.ApplyStatus = outWorkStatus
	outWorkApplyReq.BeginTime = dates[0]
	outWorkApplyReq.EndTime = dates[len(dates)-1]
	outWorkApplyReq.Page = 1
	outWorkApplyReq.PageSize = 1000
	outWorkApplyRes, err := GrpcOAImpl.QueryOaApply(context.Background(), outWorkApplyReq)
	if err != nil {
		return nil, err
	}
	collection.OutWork = int32(outWorkApplyRes.Total)

	fmt.Println("step ==============================  03  ============================== ")
	fmt.Printf(" 审批  collection is  : %+v \n", collection)
	fmt.Println("step ==============================  03  ============================== ")

	return collection, nil
}

func StaffAttendanceCollectionHandleV2(staffUID uint64, month string, dates []string, restDays int, outWorkStatus []int32, outWorkDates []string) (collection *oa_model.StaffCollection, err error) {
	collection = new(oa_model.StaffCollection)
	collection, err = StaffCollectionClickInV2(staffUID, month, dates)
	if err != nil {
		return nil, err
	}

	fmt.Println("step ==============================  02  ============================== ")
	fmt.Printf(" 打卡  collection is  : %+v \n", collection)
	fmt.Println("step ==============================  02  ============================== ")

	// 休息日 处理 需要 去除 休息日加班 情况
	// restDays 是 根据 考勤规则的 week 和 实际的 假期 结合起来 算出来的
	collection.RestDays = int32(restDays) + collection.RestDays

	// oa 申请  请假  加班  调休
	applyReq := new(oa.ApplyRecordReq)
	applyReq.StaffUID = collection.StaffUID
	applyReq.ApplyType = []string{
		oa_model.TypeLeave, // 请假
		oa_model.TypeSick,
		oa_model.TypeAnnualLeave,
		oa_model.TypeMaritalLeave,
		oa_model.TypeMatingCheckLeave,
		oa_model.TypeMaternityLeave,
		oa_model.TypePaternityLeave,
		oa_model.TypeParentalLeave,
		oa_model.TypeNursingLeave,
		oa_model.TypeFuneralLeave,
		oa_model.TypeBreastFeedingLeave,
		oa_model.TypeAbortLeave,
		oa_model.TypeDayOff,   // 调休
		oa_model.TypeOverTime, // 加班
		oa_model.TypeOutWork,  // 外勤
		oa_model.TypeMakeUp,   // 补卡
	}
	applyReq.ApplyStatus = []int32{e.ApprovalWorkStatusOk}
	applyReq.BeginTime = outWorkDates[0]
	applyReq.EndTime = outWorkDates[len(outWorkDates)-1]
	applyReq.Page = 1
	applyReq.PageSize = 1000
	applyRes, err := GrpcOAImpl.QueryOaApply(context.Background(), applyReq)
	if err != nil {
		return nil, err
	}

	for i := 0; i < len(applyRes.Data); i++ {
		if applyRes.Data[i].ApplyType == oa_model.TypeLeave ||
			applyRes.Data[i].ApplyType == oa_model.TypeSick ||
			applyRes.Data[i].ApplyType == oa_model.TypeAnnualLeave ||
			applyRes.Data[i].ApplyType == oa_model.TypeMaritalLeave ||
			applyRes.Data[i].ApplyType == oa_model.TypeMatingCheckLeave ||
			applyRes.Data[i].ApplyType == oa_model.TypeMaternityLeave ||
			applyRes.Data[i].ApplyType == oa_model.TypePaternityLeave ||
			applyRes.Data[i].ApplyType == oa_model.TypeParentalLeave ||
			applyRes.Data[i].ApplyType == oa_model.TypeNursingLeave ||
			applyRes.Data[i].ApplyType == oa_model.TypeAbortLeave ||
			applyRes.Data[i].ApplyType == oa_model.TypeBreastFeedingLeave ||
			applyRes.Data[i].ApplyType == oa_model.TypeFuneralLeave {
			collection.Leave += 1
		}

		if applyRes.Data[i].ApplyType == oa_model.TypeDayOff {
			collection.DayOff += 1
		}
		if applyRes.Data[i].ApplyType == oa_model.TypeOutWork {
			collection.OutWork += 1
		}

		// 加班  有考勤记录 无需 对 workDay 增减
		if applyRes.Data[i].ApplyType == oa_model.TypeOverTime {
			if CheckDateInMonth(applyRes.Data[i].ApplyTimes[0].Date, dates) {
				collection.OverWork += applyRes.Data[i].Hours
			}
		}
		if applyRes.Data[i].ApplyType == oa_model.TypeMakeUp {
			collection.MakeUp += 1
		}
	}

	/*	// 外勤申请 (e.ApprovalWorkStatusDoing , e.ApprovalWorkStatusOk , e.ApprovalWorkStatusFail)
		outWorkApplyReq := new(oa.ApplyRecordReq)
		outWorkApplyReq.StaffUID = collection.StaffUID
		outWorkApplyReq.ApplyType = []string{
			oa_model.TypeOutWork, // 外勤
			//oa_model.TypeMakeUp,   // 补卡
		}
		outWorkApplyReq.ApplyStatus = outWorkStatus
		outWorkApplyReq.BeginTime = outWorkDates[0]
		outWorkApplyReq.EndTime = outWorkDates[len(outWorkDates)-1]
		outWorkApplyReq.Page = 1
		outWorkApplyReq.PageSize = 1000
		outWorkApplyRes, err := GrpcOAImpl.QueryOaApply(context.Background(), outWorkApplyReq)
		if err != nil {
			return nil, err
		}
		collection.OutWork = int32(outWorkApplyRes.Total)

		// 调休
		dayOffApplyReq := new(oa.ApplyRecordReq)
		dayOffApplyReq.StaffUID = collection.StaffUID
		dayOffApplyReq.ApplyType = []string{
			oa_model.TypeDayOff, // 外勤
			//oa_model.TypeMakeUp,   // 补卡
		}
		dayOffApplyReq.ApplyStatus = outWorkStatus
		dayOffApplyReq.BeginTime = outWorkDates[0]
		dayOffApplyReq.EndTime = outWorkDates[len(outWorkDates)-1]
		dayOffApplyReq.Page = 1
		dayOffApplyReq.PageSize = 1000
		dayOffApplyRes, err := GrpcOAImpl.QueryOaApply(context.Background(), dayOffApplyReq)
		if err != nil {
			return nil, err
		}
		collection.OutWork = int32(dayOffApplyRes.Total)

		// 补卡申请 (e.ApprovalWorkStatusDoing , e.ApprovalWorkStatusOk , e.ApprovalWorkStatusFail)
		makeUpApplyReq := new(oa.ApplyRecordReq)
		makeUpApplyReq.StaffUID = collection.StaffUID
		makeUpApplyReq.ApplyType = []string{
			//oa_model.TypeOutWork, // 外勤
			oa_model.TypeMakeUp, // 补卡
		}
		//makeUpApplyReq.ApplyStatus = []int32{e.ApprovalWorkStatusDoing, e.ApprovalWorkStatusOk, e.ApprovalWorkStatusFail}
		makeUpApplyReq.ApplyStatus = []int32{e.ApprovalWorkStatusOk}
		makeUpApplyReq.BeginTime = dates[0]
		makeUpApplyReq.EndTime = dates[len(dates)-1]
		makeUpApplyReq.Page = 1
		makeUpApplyReq.PageSize = 1000
		makeUpApplyRes, err := GrpcOAImpl.QueryOaApply(context.Background(), makeUpApplyReq)
		if err != nil {
			return nil, err
		}
		collection.MakeUp = int32(makeUpApplyRes.Total)*/

	//  算旷工时  会把 休息日算上  所以 要 减去 休息日(基础)
	//collection.MissDay = collection.MissDay - collection.RestDays
	//collection.MissDay = collection.MissDay

	fmt.Println("step ==============================  03  ============================== ")
	fmt.Printf(" 审批  collection is  : %+v \n", collection)
	fmt.Println("step ==============================  03  ============================== ")

	return collection, nil
}

/*
出勤天数  平均工时  休息  缺卡  迟到 早退  旷工 外勤
*/
func StaffCollectionClickInV1(staffUID uint64, month string, dates []string) (collection *oa_model.StaffCollection, err error) {
	collection = new(oa_model.StaffCollection)

	/*
		// 查询 统计记录
			collection, err = findCollection(staffUID, month)
			if err != nil {
				return nil, err
			}
	*/
	// 查询 考勤记录
	recordReq := new(oa.OARecordReq)
	recordReq.StaffUID = staffUID
	recordReq.Dates = dates
	recordReq.Page = 1
	recordReq.PageSize = 9999
	recordRes, err := GrpcOAImpl.OARecord(context.Background(), recordReq)
	if err != nil {
		return nil, err
	}

	var (
		totalHour float64 // 时长
		hourDay   float64 // 用于 计算  平均工时
		workDays  int32   // 用于 统计  考勤天数
		missDay   int32   // 旷工次数
		outWork   int32   // 外勤次数
		late      int32   // 迟到次数
		before    int32   // 早退次数
		miss      int32   // 缺卡次数
		makeUp    int32   // 补卡次数
	)

	// 不计入 totalHour 的 不参与 平均工时计算
	for i := 0; i < len(recordRes.Data); i++ {
		// 缺卡  缺卡次数 和 考勤班次 次数一致  则  算旷工
		var (
			isMissDay int
			//isWork    bool = false
			//isOut     bool = false
			//isLate    bool = false
			//isBefore  bool = false
			hours float64
		)

		for j := 0; j < len(recordRes.Data[i].Records); j++ {
			// 用于 确认是否 旷工
			if recordRes.Data[i].Records[j].ActionType == oa_model.TypeMiss {
				miss += 1
				isMissDay += 1
				//isWork = false
			}
			if recordRes.Data[i].Records[j].ActionType == oa_model.TypeOutWork {
				outWork += 1
			}
			if recordRes.Data[i].Records[j].ActionType == oa_model.TypeLate {
				late += 1
			}
			if recordRes.Data[i].Records[j].ActionType == oa_model.TypeBefore {
				before += 1
			}
			if recordRes.Data[i].Records[j].ActionType == oa_model.TypeMakeUp {
				makeUp += 1
			}
		}

		if isMissDay == len(recordRes.Data[i].Records) {
			missDay += 1
		} else {
			workDays += 1
		}

		//fmt.Println("==========================================  recordRes.Data[i].Records  ========================================== ")
		//fmt.Printf("recordRes.Data[i].Records is : %+v \n", recordRes.Data[i].Records)
		//fmt.Println("==========================================  recordRes.Data[i].Records  =========================================== ")
		//fmt.Println()

		//  统计工时   没有 完整的  上下班 打卡 记录 则 不计算 平均工时
		for j := 0; j < len(recordRes.Data[i].Records); j = j + 2 {

			if j+1 >= len(recordRes.Data[i].Records) {
				break
			}

			// 判断是否是 缺卡   是  跳过
			if recordRes.Data[i].Records[j].ActionType == oa_model.TypeMiss || recordRes.Data[i].Records[j+1].ActionType == oa_model.TypeMiss {
				continue
			}

			// 成对 出现
			//now := time.Now().Format("2006-01-02")
			var startTime time.Time
			var endTime time.Time
			//if recordRes.Data[i].Records[j].ActionType != oa_model.TypeCommon {
			//	startTime, _ = time.ParseInLocation("2006-01-02 15:04:05", now+" "+recordRes.Data[i].Records[j].WorkTime+":00", time.Local)
			//} else {
			startTime, _ = time.ParseInLocation("2006-01-02 15:04:05", recordRes.Data[i].Records[j].ActionTime+":00", time.Local)
			//}

			//if recordRes.Data[i].Records[j+1].ActionType != oa_model.TypeCommon {
			//	endTime, _ = time.ParseInLocation("2006-01-02 15:04:05", now+" "+recordRes.Data[i].Records[j+1].WorkTime+":00", time.Local)
			//} else {
			endTime, _ = time.ParseInLocation("2006-01-02 15:04:05", recordRes.Data[i].Records[j+1].ActionTime+":00", time.Local)
			//}

			//fmt.Println("==========================================  startTime endTime  ========================================== ")
			//fmt.Println("startTime is : ", startTime)
			//fmt.Println("endTime is : ", endTime)
			//fmt.Println("==========================================  startTime endTime  ========================================== ")
			//fmt.Println()

			hours = hours + endTime.Sub(startTime).Hours()
			//fmt.Println("==========================================   hours  ========================================== ")
			//fmt.Println("hours is : ", hours)
			//fmt.Println("==========================================   hours  ========================================== ")
			//fmt.Println()
			//j = j + 2
		}

		if hours > 0 {
			hourDay += 1
			totalHour += hours
		}
	}

	collection.StaffUID = staffUID
	collection.WorkDays = workDays
	collection.Miss = miss
	collection.MakeUp = makeUp
	collection.Late = late
	collection.Before = before
	collection.MissDay = missDay
	collection.OutWork = outWork

	//fmt.Println("==========================================  totalHour hourDay  ========================================== ")
	//fmt.Println("totalHour is : ", totalHour)
	//fmt.Println("hourDay is : ", hourDay)
	//fmt.Println("==========================================  totalHour hourDay = ========================================== ")
	//fmt.Println()

	if totalHour != 0 && hourDay != 0 {
		averageHour, _ := strconv.ParseFloat(strconv.FormatFloat(math.Trunc(totalHour/hourDay*math.Pow10(1))/math.Pow10(1), 'f', -1, 64), 64)
		//fmt.Println("==========================================  averageHour  ========================================== ")
		//fmt.Println("averageHour is : ", averageHour)
		//fmt.Println("==========================================  averageHour  ========================================== ")
		//fmt.Println()
		collection.AverageHour = float32(averageHour)
	}

	//if collection.Miss != 0 || collection.Late != 0 || collection.Before != 0 || collection.MissDay != 0 {
	//	collection.Status = false
	//} else {
	//	collection.Status = true
	//}

	return
}

func StaffCollectionClickInV2(staffUID uint64, month string, dates []string) (collection *oa_model.StaffCollection, err error) {
	collection = new(oa_model.StaffCollection)

	/*
		// 查询 统计记录
			collection, err = findCollection(staffUID, month)
			if err != nil {
				return nil, err
			}
	*/
	// 查询 考勤记录
	recordReq := new(oa.OARecordReq)
	recordReq.StaffUID = staffUID
	recordReq.Dates = dates
	recordReq.Page = 1
	recordReq.PageSize = 9999
	recordRes, err := GrpcOAImpl.OARecord(context.Background(), recordReq)
	if err != nil {
		fmt.Println("查询 考勤记录  GrpcOAImpl.OARecord err :", err.Error())
		return nil, errors.New(e.ErrQueryOaRecord)
	}

	var (
		totalHour float64 // 时长
		hourDay   float64 // 用于 计算  平均工时
		workDays  int32   // 用于 统计  考勤天数
		missDay   int32   // 旷工次数
		outWork   int32   // 外勤次数
		late      int32   // 迟到次数
		before    int32   // 早退次数
		miss      int32   // 缺卡次数
		makeUp    int32   // 补卡次数
		lastCycle string
	)

	// 不计入 totalHour 的 不参与 平均工时计算
	for i := 0; i < len(recordRes.Data); i++ {
		// 缺卡  缺卡次数 和 考勤班次 次数一致  则  算旷工
		var (
			isMissDay int
			hours     float64
		)

		fmt.Printf("==========================================  recordRes.Data[%v].Records  ========================================== \n", i+1)
		fmt.Printf("recordRes.Data[i].Records is : %+v \n", recordRes.Data[i])
		fmt.Printf("==========================================  recordRes.Data[%v].Records  ========================================== \n", i+1)
		fmt.Println()

		// 如果 考勤 中的 打卡周期 不为空  则将 此次 打卡周期记录 为 之后 考勤记录为 空的 提供 判断依据
		if recordRes.Data[i].Cycle != "" {
			lastCycle = recordRes.Data[i].Cycle
		}

		// 全天考勤正常 与否
		oaRecord := oa_model.OARecord{}
		var isWork bool

		// 如果 考勤记录不为 空  则将 记录中的 IsWork 赋值给 统计中的 IsWork (不用)
		// 无论记录是否存在 都将 记录中的 IsWork 赋值给 统计中的 IsWork
		isWork = ConfirmIsWork(lastCycle, recordRes.Data[i].WorkDate)

		if isWork {
			recordRes.Data[i].IsWork = oa_model.IsWork
		}

		if len(recordRes.Data[i].Records) > 0 {
			oaRecord.IsWork = oa_model.IsWork
		}

		currentDate, _ := time.ParseInLocation("2006-01-02 15:04:05", time.Now().Format("2006-01-02")+" 00:00:00", time.Local)

		if len(recordRes.Data[i].Records) == 0 {
			workDate, _ := time.ParseInLocation("2006-01-02 15:04:05", recordRes.Data[i].WorkDate+" 00:00:00", time.Local)
			if recordRes.Data[i].IsWork == oa_model.IsWork && workDate.Before(currentDate) {
				//if len(recordRes.Data[i].Records) == 0 {
				//	if oaRecord.IsWork == oa_model.IsWork { // 特殊情况  工作日  没有考勤记录
				missDay += 1
			}
			continue
		}

		if oaRecord.IsWork == oa_model.IsWork && recordRes.Data[i].IsWork == oa_model.IsFree {
			collection.RestDays += -1
		}

		records := make([]oa_model.Record, 0)
		currentMap := make(map[string]*oa_model.Record, 0) // 剔除重复考勤记录数据
		for j := 0; j < len(recordRes.Data[i].Records); j++ {
			record := new(oa_model.Record)
			err = copier.CopyWithOption(&record, recordRes.Data[i].Records[j], copier.Option{DeepCopy: true})
			if err != nil {
				fmt.Println("复制 考勤记录  err :", err.Error())
				return
			}
			if _, ok := currentMap[recordRes.Data[i].Records[j].WorkTime]; ok {
				if recordRes.Data[i].Records[j].ActionType == oa_model.TypeCommon || recordRes.Data[i].Records[j].ActionType == oa_model.TypeBefore {
					records = records[:len(records)-1]
				} else {
					continue
				}
			}
			currentMap[recordRes.Data[i].Records[j].WorkTime] = record
			records = append(records, *record)
		}

		timeOptions := make([]*oa_model.TimeOption, 0)
		_ = copier.CopyWithOption(&timeOptions, recordRes.Data[i].TimeOptions, copier.Option{DeepCopy: true})
		fmt.Println("========================================== confirmLeaveType  ========================================== ")
		fmt.Println("confirmLeaveType req  staffUID is : ", staffUID)
		fmt.Printf("confirmLeaveType req timeOptions is %+v\n : ", timeOptions)
		fmt.Printf("confirmLeaveType req dates is %+v\n : ", []string{recordRes.Data[i].WorkDate, recordRes.Data[i].WorkDate})
		fmt.Println("==========================================  confirmLeaveType  =========================================== ")
		fmt.Println()

		leaveTypeMap := confirmLeaveType(staffUID, timeOptions, []string{recordRes.Data[i].WorkDate, recordRes.Data[i].WorkDate})

		fmt.Println("========================================== confirmLeaveType  ========================================== ")
		fmt.Printf("confirmLeaveType is : %+v \n", leaveTypeMap)
		fmt.Println("==========================================  confirmLeaveType  =========================================== ")
		fmt.Println()

		for j := 0; j < len(records); j++ {

			//  判断  当前 打卡记录是否 在 某一次 请假  调休 中, 如果存在  则将 ActionType 置为 相应请假类型 ActionTime 置为 打卡时间 （计算工时使用）
			//  会过滤掉  缺卡 迟到 早退情况
			if leaveTypeMap[records[j].WorkTime] != nil {
				records[j].ActionType = leaveTypeMap[records[j].WorkTime].ActionType
				records[j].ActionTime = recordRes.Data[i].WorkDate + " " + records[j].WorkTime
				continue
			}

			// 用于 确认是否 旷工
			if records[j].ActionType == oa_model.TypeMiss {
				miss += 1
				isMissDay += 1
			}
			if records[j].ActionType == oa_model.TypeLate {
				late += 1
			}
			if records[j].ActionType == oa_model.TypeBefore {
				before += 1
			}

		}

		if isMissDay == len(records) {
			missDay += 1
		} else {
			workDays += 1
		}

		fmt.Printf("==========================================  records %v ========================================== \n", i+1)
		fmt.Printf("records is : %+v \n", records)
		fmt.Printf("==========================================  records %v ========================================== \n", i+1)
		fmt.Println()

		//  统计工时   没有 完整的  上下班 打卡 记录 则 不计算 平均工时
		for j := 0; j < len(records); j = j + 2 {

			if j+1 >= len(records) {
				break
			}

			// 判断是否是 缺卡   是  跳过
			//if records[j].ActionType == oa_model.TypeMiss || records[j+1].ActionType == oa_model.TypeMiss {
			//	continue
			//}

			if (records[j].ActionType != oa_model.TypeCommon && records[j].ActionType != oa_model.TypeBefore && records[j].ActionType != oa_model.TypeLate && records[j].ActionType != oa_model.TypeMakeUp && records[j].ActionType != oa_model.TypeOutWork && records[j].ActionType != oa_model.TypeOverTime) ||
				(records[j+1].ActionType != oa_model.TypeCommon && records[j+1].ActionType != oa_model.TypeBefore && records[j+1].ActionType != oa_model.TypeLate && records[j+1].ActionType != oa_model.TypeMakeUp && records[j+1].ActionType != oa_model.TypeOutWork && records[j+1].ActionType != oa_model.TypeOverTime) {
				continue
			}

			// 成对 出现
			var startTime time.Time
			var endTime time.Time
			startTime, _ = time.ParseInLocation("2006-01-02 15:04:05", records[j].ActionTime+":00", time.Local)

			endTime, _ = time.ParseInLocation("2006-01-02 15:04:05", records[j+1].ActionTime+":00", time.Local)

			//fmt.Println("==========================================  startTime endTime  ========================================== ")
			//fmt.Println("startTime is : ", startTime)
			//fmt.Println("endTime is : ", endTime)
			//fmt.Println("==========================================  startTime endTime  ========================================== ")
			//fmt.Println()

			hours = hours + endTime.Sub(startTime).Hours()
			//fmt.Println("==========================================   hours  ========================================== ")
			//fmt.Println("hours is : ", hours)
			//fmt.Println("==========================================   hours  ========================================== ")
			//fmt.Println()
			//j = j + 2
		}

		//if hours > 0 {
		//	hourDay += 1
		//	totalHour += float64(HandleHourLeaveASeat(hours))
		//}
		if len(records) > 0 && (isMissDay != len(records)) {
			hourDay += 1
			totalHour += float64(HandleHourLeaveASeat(hours))
		}
	}

	collection.StaffUID = staffUID
	collection.WorkDays = workDays
	collection.Miss = miss
	collection.MakeUp = makeUp
	collection.Late = late
	collection.Before = before
	collection.MissDay = missDay
	collection.OutWork = outWork

	//fmt.Println("==========================================  totalHour hourDay  ========================================== ")
	//fmt.Println("totalHour is : ", totalHour)
	//fmt.Println("hourDay is : ", hourDay)
	//fmt.Println("==========================================  totalHour hourDay = ========================================== ")
	//fmt.Println()

	if totalHour != 0 && hourDay != 0 {
		//averageHour, _ := strconv.ParseFloat(strconv.FormatFloat(math.Trunc(totalHour/hourDay*math.Pow10(1))/math.Pow10(1), 'f', -1, 64), 64)
		collection.AverageHour = HandleHourLeaveASeat(totalHour / hourDay)
		/*fmt.Println("==========================================  averageHour  ========================================== ")
		fmt.Println("averageHour is : ", collection.AverageHour)
		fmt.Println("totalHour is : ", totalHour)
		fmt.Println("hourDay is : ", hourDay)
		fmt.Println("==========================================  averageHour  ========================================== ")
		fmt.Println()*/

	}

	//if collection.Miss != 0 || collection.Late != 0 || collection.Before != 0 || collection.MissDay != 0 {
	//	collection.Status = false
	//} else {
	//	collection.Status = true
	//}

	return
}

// 确认  当前 打卡 类型     根据  当天的 请假 调休  判断    并 在 考勤 记录 上 打上 相应的标签(leave, dayOff)
func confirmLeaveType(staffUUID uint64, times []*oa_model.TimeOption, dates []string) map[string]*oa_model.ConfirmActionType {
	// 查询 请假记录

	// oa 申请  请假  加班  调休
	applyReq := new(oa.ApplyRecordReq)
	applyReq.StaffUID = staffUUID
	applyReq.ApplyType = []string{
		oa_model.TypeLeave, // 请假
		oa_model.TypeSick,
		oa_model.TypeAnnualLeave,
		oa_model.TypeMaritalLeave,
		oa_model.TypeMatingCheckLeave,
		oa_model.TypeMaternityLeave,
		oa_model.TypePaternityLeave,
		oa_model.TypeParentalLeave,
		oa_model.TypeNursingLeave,
		oa_model.TypeFuneralLeave,
		oa_model.TypeBreastFeedingLeave,
		oa_model.TypeAbortLeave,
		oa_model.TypeDayOff, // 调休
		//oa_model.TypeOverTime, // 加班
		oa_model.TypeOutWork, // 外勤
		oa_model.TypeMakeUp,  // 补卡
	}
	applyReq.ApplyStatus = []int32{e.ApprovalWorkStatusOk}
	applyReq.BeginTime = dates[0]
	applyReq.EndTime = dates[len(dates)-1]
	applyReq.Page = 1
	applyReq.PageSize = 1000
	apply, err := GrpcOAImpl.QueryOaApply(context.Background(), applyReq)
	if err != nil {
		return nil
	}

	fmt.Printf("==========================================  apply ========================================== \n")
	fmt.Printf("apply is : %+v \n", apply.Data)
	fmt.Printf("==========================================  apply ========================================== \n")
	fmt.Println()

	fmt.Printf("==========================================  times ========================================== \n")
	fmt.Printf("times is : %+v \n", times)
	fmt.Printf("==========================================  times ========================================== \n")
	fmt.Println()

	confirmTypes := make(map[string]*oa_model.ConfirmActionType, 0)

	for i := 0; i < len(apply.Data); i++ {
		var (
			applyStartTimeStr string
			applyEndTimeStr   string
			applyStartTime    time.Time
			applyEndTime      time.Time
		)

		fmt.Printf("==========================================  apply.Data %v ========================================== \n", i+1)
		fmt.Printf("apply.Data is : %+v \n", apply.Data[i])
		fmt.Printf("==========================================  apply.Data %v ========================================== \n", i+1)
		fmt.Println()

		if len(apply.Data[i].ApplyTimes) == 1 {
			applyStartTimeStr = apply.Data[i].ApplyTimes[0].Date + " " + times[0].OnWorkTime + ":00"
		} else {
			applyTimes := make([]oa_model.ApplyTime, 0)
			_ = copier.CopyWithOption(&applyTimes, apply.Data[i].ApplyTimes, copier.Option{DeepCopy: true})
			applyStartTimeStr, applyEndTimeStr = oa_logic.ConvertApplyTimeV2(applyTimes, times)
		}

		applyStartTime, _ = time.ParseInLocation("2006-01-02 15:04:05", applyStartTimeStr, time.Local)
		applyEndTime, _ = time.ParseInLocation("2006-01-02 15:04:05", applyEndTimeStr, time.Local)
		for j := 0; j < len(times); j++ {

			onWorkTime, _ := time.ParseInLocation("2006-01-02 15:04:05", applyStartTime.Format("2006-01-02")+" "+times[j].OnWorkTime+":00", time.Local)
			offWorkTime, _ := time.ParseInLocation("2006-01-02 15:04:05", applyEndTime.Format("2006-01-02")+" "+times[j].OffWorkTime+":00", time.Local)

			/*
				开始时间 在 上班 之后   结束时间 在 下班之前    跳出 循环
			*/
			if applyStartTime.After(onWorkTime) && applyEndTime.Before(offWorkTime) {
				break
			}

			if !applyStartTime.After(onWorkTime) { // 开始时间  不在 上班之后
				if applyEndTime.After(onWorkTime) { // 结束时间  在 上班之后    记  上班时间 为 请假 或者 调休
					makeActionTypeRecord(confirmTypes, apply.Data[i].ApplyType, "", times[j].OnWorkTime, apply.Data[i].UUID)
					if applyEndTime.Before(offWorkTime) { // 结束时间  在 下班之前   跳出 循环
						break
					} else { // 结束时间  不在 下班之前   记  下班时间 为 请假 或者 调休
						makeActionTypeRecord(confirmTypes, apply.Data[i].ApplyType, "", times[j].OffWorkTime, apply.Data[i].UUID)
					}
				}
			} else {
				if !applyEndTime.Before(offWorkTime) && applyStartTime.Before(offWorkTime) {
					makeActionTypeRecord(confirmTypes, apply.Data[i].ApplyType, "", times[j].OffWorkTime, apply.Data[i].UUID)
				}
			}
		}
	}
	return confirmTypes
}

func confirmMakeUpType(staffUUID uint64, dates []string) map[string]*oa_model.ConfirmActionType {
	// 查询 请假记录

	// oa 申请  请假  加班  调休
	applyReq := new(oa.ApplyRecordReq)
	applyReq.StaffUID = staffUUID
	applyReq.ApplyType = []string{
		oa_model.TypeMakeUp, // 补卡
	}
	applyReq.ApplyStatus = []int32{e.ApprovalWorkStatusOk}
	applyReq.BeginTime = dates[0]
	applyReq.EndTime = dates[len(dates)-1]
	applyReq.Page = 1
	applyReq.PageSize = 1000
	apply, err := GrpcOAImpl.QueryOaApply(context.Background(), applyReq)
	if err != nil {
		return nil
	}

	confirmTypes := make(map[string]*oa_model.ConfirmActionType, 0)

	for i := 0; i < len(apply.Data); i++ {
		makeActionTypeRecord(confirmTypes, apply.Data[i].ApplyType, apply.Data[i].ApplyTimes[0].Date, apply.Data[i].ApplyTimes[0].Hour, apply.Data[i].UUID)
	}
	return confirmTypes
}

// 构建 打卡 类型 以及  添加 相关申请的UUID
func makeActionTypeRecord(confirmTypes map[string]*oa_model.ConfirmActionType, applyType, workDate, workTime, applyUUID string) {
	confirm := &oa_model.ConfirmActionType{
		WorkTime:  workTime,
		ApplyUUID: applyUUID,
	}
	// actionType   为 相应的 请假类型  不做统一 处理( 请假 归总 为 leave [不使用])
	if oa_model.LeaveType[applyType] != "" || applyType == oa_model.TypeLeave || applyType == oa_model.TypeSick || applyType == oa_model.TypeDayOff || applyType == oa_model.TypeMakeUp || applyType == oa_model.TypeOutWork {
		confirm.ActionType = applyType
	}

	confirmTypes[workDate+workTime] = confirm
}

func StaffCollectionDetail(dates []string, staffUID uint64, outWorkDates []string) (collectionDetail *oa_model.CollectionDetailRes, err error) {

	collectionDetail = new(oa_model.CollectionDetailRes)

	// 获取日历
	holidayRes, err := holiday.GetMultiData(holiday.ConvertTime(dates[0]), holiday.ConvertTime(dates[len(dates)-1]), true)
	if err != nil {
		return
	}
	holidayMap := make(map[string]*holiday.RespData, 0)
	for i := 0; i < len(holidayRes); i++ {
		var (
			yearStr  string
			dayStr   string
			monthStr string
		)
		yearStr = strconv.Itoa(int(holidayRes[i].Year))
		if holidayRes[i].Day < 10 {
			dayStr = "0" + strconv.Itoa(int(holidayRes[i].Day))
		} else {
			dayStr = strconv.Itoa(int(holidayRes[i].Day))
		}
		if holidayRes[i].Month < 10 {
			monthStr = "0" + strconv.Itoa(int(holidayRes[i].Month))
		} else {
			monthStr = strconv.Itoa(int(holidayRes[i].Month))
		}
		holidayMap[yearStr+"-"+monthStr+"-"+dayStr] = holidayRes[i]
	}
	fmt.Println(holidayMap)

	fmt.Println("step : =================================  2  =================================")
	fmt.Println()

	// 获取 oa 申请 信息
	applyReq := new(oa.ApplyRecordReq)
	applyReq.StaffUID = staffUID
	applyReq.ApplyType = []string{
		oa_model.TypeLeave, // 请假
		oa_model.TypeSick,
		oa_model.TypeAnnualLeave,
		oa_model.TypeMaritalLeave,
		oa_model.TypeMatingCheckLeave,
		oa_model.TypeMaternityLeave,
		oa_model.TypePaternityLeave,
		oa_model.TypeParentalLeave,
		oa_model.TypeNursingLeave,
		oa_model.TypeFuneralLeave,
		oa_model.TypeBreastFeedingLeave,
		oa_model.TypeAbortLeave,
		oa_model.TypeDayOff,   // 调休
		oa_model.TypeOverTime, // 加班
		oa_model.TypeOutWork,  // 外勤
		oa_model.TypeMakeUp,   // 补卡
	}
	applyReq.ApplyStatus = []int32{e.ApprovalWorkStatusOk}
	applyReq.BeginTime = outWorkDates[0]
	applyReq.EndTime = outWorkDates[len(outWorkDates)-1]
	applyReq.Page = 1
	applyReq.PageSize = 1000
	applyRes, err := GrpcOAImpl.QueryOaApply(context.Background(), applyReq)
	if err != nil {
		return
	}

	/*// 外勤
	outWorkApplyReq := new(oa.ApplyRecordReq)
	outWorkApplyReq.StaffUID = staffUID
	outWorkApplyReq.ApplyType = []string{
		oa_model.TypeOutWork, // 外勤
	}
	outWorkApplyReq.ApplyStatus = []int32{e.ApprovalWorkStatusOk}
	outWorkApplyReq.BeginTime = outWorkDates[0]
	outWorkApplyReq.EndTime = outWorkDates[len(outWorkDates)-1]
	outWorkApplyReq.Page = 1
	outWorkApplyReq.PageSize = 1000
	outWorkApplyRes, err := GrpcOAImpl.QueryOaApply(context.Background(), outWorkApplyReq)
	if err != nil {
		return
	}
	if outWorkApplyRes.Data != nil || len(outWorkApplyRes.Data) != 0 {
		if applyRes.Data != nil {
			applyRes.Data = append(applyRes.Data, outWorkApplyRes.Data...)
		} else if applyRes.Data == nil || len(applyRes.Data) == 0 {
			applyRes.Data = outWorkApplyRes.Data
		}
	}

	// 调休
	dayOffApplyReq := new(oa.ApplyRecordReq)
	dayOffApplyReq.StaffUID = staffUID
	dayOffApplyReq.ApplyType = []string{
		oa_model.TypeDayOff, // 外勤
		//oa_model.TypeMakeUp,   // 补卡
	}
	dayOffApplyReq.ApplyStatus = []int32{e.ApprovalWorkStatusOk}
	dayOffApplyReq.BeginTime = outWorkDates[0]
	dayOffApplyReq.EndTime = outWorkDates[len(outWorkDates)-1]
	dayOffApplyReq.Page = 1
	dayOffApplyReq.PageSize = 1000
	dayOffApplyRes, err := GrpcOAImpl.QueryOaApply(context.Background(), dayOffApplyReq)
	if err != nil {
		return nil, err
	}

	if dayOffApplyRes.Data != nil || len(dayOffApplyRes.Data) != 0 {
		if applyRes.Data != nil {
			applyRes.Data = append(applyRes.Data, dayOffApplyRes.Data...)
		} else if applyRes.Data == nil || len(applyRes.Data) == 0 {
			applyRes.Data = dayOffApplyRes.Data
		}
	}*/

	/*fmt.Println("applyRes ============================ start ======================================")
	fmt.Printf("applyRes : %+v\n", applyRes)
	fmt.Println("applyRes ============================ end   ======================================")
	fmt.Println()*/

	fmt.Println("step : =================================  3  =================================")
	fmt.Println()

	makeUp := make([]oa_model.CommonInfo, 0)
	leave := make([]oa_model.CommonInfo, 0)
	dayOff := make([]oa_model.CommonInfo, 0)
	overWork := make([]oa_model.CommonInfo, 0)
	outWork := make([]oa_model.CommonInfo, 0)

	for i := 0; i < len(applyRes.Data); i++ {
		commonInfo := oa_model.CommonInfo{}
		commonInfo.UUID = applyRes.Data[i].UUID
		commonInfo.Hours = applyRes.Data[i].Hours
		commonInfo.Days = applyRes.Data[i].Days
		commonInfo.LeaveType = applyRes.Data[i].ApplyType
		commonInfo.LeaveTypeZhCN = oa_model.TypeZhCN[applyRes.Data[i].ApplyType]
		commonInfo.ApplyTimes = make([]*oa_model.ApplyTime, 0)

		weeklys := make([]string, 0)
		for j := 0; j < len(applyRes.Data[i].ApplyTimes); j++ {
			at := &oa_model.ApplyTime{
				Date: applyRes.Data[i].ApplyTimes[j].Date,
				Hour: applyRes.Data[i].ApplyTimes[j].Hour,
				M:    applyRes.Data[i].ApplyTimes[j].M,
			}
			commonInfo.ApplyTimes = append(commonInfo.ApplyTimes, at)
			fmt.Println("at.Date :", at.Date)
			fmt.Println("holidayMap[at.Date] :", holidayMap[at.Date])
			//fmt.Println("holidayMap[at.Date].WeekDay :", holidayMap[at.Date].WeekDay)
			if holidayMap[at.Date] != nil {
				weeklys = append(weeklys, oa_model.WeekZhCN[holidayMap[at.Date].WeekDay])
			} else {
				v, _ := holiday.GetCacheHolidayInfo([]string{strings.ReplaceAll(at.Date, "-", "")})
				if len(v) == 1 {
					weeklys = append(weeklys, oa_model.WeekZhCN[v[0].WeekDay])
				}
			}
		}
		if len(weeklys) == 1 {
			commonInfo.Weekly = weeklys[0]
		} else if len(weeklys) > 1 {
			commonInfo.Weekly = strings.Join(weeklys, " - ")
		}

		//  组装 申请 的数据
		if applyRes.Data[i].ApplyType == oa_model.TypeMakeUp {
			makeUp = append(makeUp, commonInfo)
		} else if applyRes.Data[i].ApplyType == oa_model.TypeOverTime {
			if CheckDateInMonth(applyRes.Data[i].ApplyTimes[0].Date, dates) {
				overWork = append(overWork, commonInfo)
			}
		} else if applyRes.Data[i].ApplyType == oa_model.TypeDayOff {
			dayOff = append(dayOff, commonInfo)
		} else if applyRes.Data[i].ApplyType == oa_model.TypeOutWork {
			commonInfo.ApplyOutAddress = applyRes.Data[i].OutWorkAddress
			outWork = append(outWork, commonInfo)
		} else if applyRes.Data[i].ApplyType == oa_model.TypeLeave ||
			applyRes.Data[i].ApplyType == oa_model.TypeSick ||
			applyRes.Data[i].ApplyType == oa_model.TypeAnnualLeave ||
			applyRes.Data[i].ApplyType == oa_model.TypeMaritalLeave ||
			applyRes.Data[i].ApplyType == oa_model.TypeMatingCheckLeave ||
			applyRes.Data[i].ApplyType == oa_model.TypeMaternityLeave ||
			applyRes.Data[i].ApplyType == oa_model.TypePaternityLeave ||
			applyRes.Data[i].ApplyType == oa_model.TypeParentalLeave ||
			applyRes.Data[i].ApplyType == oa_model.TypeNursingLeave ||
			applyRes.Data[i].ApplyType == oa_model.TypeAbortLeave ||
			applyRes.Data[i].ApplyType == oa_model.TypeBreastFeedingLeave ||
			applyRes.Data[i].ApplyType == oa_model.TypeFuneralLeave {
			leave = append(leave, commonInfo)
		}
	}

	fmt.Println("step : =================================  4  =================================")
	fmt.Println()

	collectionDetail.MakeUp = makeUp
	collectionDetail.Leave = leave
	collectionDetail.DayOff = dayOff
	collectionDetail.OverWork = overWork
	collectionDetail.OutWork = outWork

	/*fmt.Println("collectionDetail ============================ start ======================================")
	fmt.Printf("collectionDetail : %+v\n", collectionDetail)
	fmt.Println("collectionDetail ============================ end   ======================================")
	fmt.Println()*/

	// 打卡 记录 查询 统计
	recordReq := new(oa.OARecordReq)
	recordReq.StaffUID = staffUID
	recordReq.Dates = dates
	recordReq.Page = 1
	recordReq.PageSize = 9999
	recordRes, err := GrpcOAImpl.OARecord(context.Background(), recordReq)
	if err != nil {
		//service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		//	Msg:    err.Error(),
		//	Status: e.Failed,
		//})
		return
	}

	/*	fmt.Println("recordRes ============================ start ======================================")
		fmt.Printf("recordRes : %+v\n", recordRes)
		fmt.Println("recordRes ============================ end   ======================================")
		fmt.Println()*/

	averageHour := make([]oa_model.CommonInfo, 0)
	workDays := make([]oa_model.CommonInfo, 0)
	miss := make([]oa_model.CommonInfo, 0)
	late := make([]oa_model.CommonInfo, 0)
	before := make([]oa_model.CommonInfo, 0)
	missDay := make([]oa_model.CommonInfo, 0)
	restDays := make([]oa_model.CommonInfo, 0)

	var lastCycle string

	for i := 0; i < len(recordRes.Data); i++ {
		var hours float64
		// 判断  是否是 旷工
		/*
		   旷工：   考勤记录为空  且  workDate 为工作日
		           考勤记录不为空 且  考勤记录均为  miss
		*/

		if recordRes.Data[i].Cycle != "" {
			lastCycle = recordRes.Data[i].Cycle
			fmt.Println("lastCycle is == ", lastCycle)
		}

		var isWork bool

		isWork = ConfirmIsWork(lastCycle, recordRes.Data[i].WorkDate)

		if isWork {
			recordRes.Data[i].IsWork = oa_model.IsWork
		}

		currentDate, _ := time.ParseInLocation("2006-01-02 15:04:05", time.Now().Format("2006-01-02")+" 00:00:00", time.Local)

		if len(recordRes.Data[i].Records) == 0 {
			workDate, _ := time.ParseInLocation("2006-01-02 15:04:05", recordRes.Data[i].WorkDate+" 00:00:00", time.Local)
			if workDate.Before(currentDate) {
				if recordRes.Data[i].IsWork == oa_model.IsWork {
					missDay = append(missDay, oa_model.CommonInfo{
						Date:   recordRes.Data[i].WorkDate,
						Weekly: oa_model.WeekZhCN[holidayMap[recordRes.Data[i].WorkDate].WeekDay],
						Days:   1,
					})
				}

				if recordRes.Data[i].IsWork == oa_model.IsFree && lastCycle == "" && (holidayMap[recordRes.Data[i].WorkDate].Type == oa_model.HolidayDay || holidayMap[recordRes.Data[i].WorkDate].Type == oa_model.FestivalHoliday) { //  测试 从未有打卡记录的 休息日 数量问题
					restDays = append(restDays, oa_model.CommonInfo{
						Date:   recordRes.Data[i].WorkDate,
						Weekly: oa_model.WeekZhCN[holidayMap[recordRes.Data[i].WorkDate].WeekDay],
						Days:   1,
					})
				}

				if recordRes.Data[i].IsWork == oa_model.IsFree && lastCycle != "" { //  测试 从未有打卡记录的 休息日 数量问题
					restDays = append(restDays, oa_model.CommonInfo{
						Date:   recordRes.Data[i].WorkDate,
						Weekly: oa_model.WeekZhCN[holidayMap[recordRes.Data[i].WorkDate].WeekDay],
						Days:   1,
					})
				}
			}

			continue
		}

		timeOption := make([]*oa_model.TimeOption, 0)
		_ = copier.CopyWithOption(&timeOption, recordRes.Data[i].TimeOptions, copier.Option{DeepCopy: true})

		leaveTypeMap := confirmLeaveType(staffUID, timeOption, []string{recordRes.Data[i].WorkDate, recordRes.Data[i].WorkDate})

		// =========================  考勤记录  ==========================================

		records := make([]oa_model.Record, 0)
		currentMap := make(map[string]*oa_model.Record, 0) // 剔除重复考勤记录数据
		for j := 0; j < len(recordRes.Data[i].Records); j++ {
			record := new(oa_model.Record)
			err = copier.CopyWithOption(&record, recordRes.Data[i].Records[j], copier.Option{DeepCopy: true})
			if err != nil {
				fmt.Println("复制 考勤记录  err :", err.Error())
				return
			}
			if _, ok := currentMap[recordRes.Data[i].Records[j].WorkTime]; ok {
				//records = records[:len(records)-1]
				if recordRes.Data[i].Records[j].ActionType == oa_model.TypeCommon || recordRes.Data[i].Records[j].ActionType == oa_model.TypeBefore {
					records = records[:len(records)-1]
				} else {
					continue
				}
			}
			currentMap[recordRes.Data[i].Records[j].WorkTime] = record
			records = append(records, *record)
		}

		isMissDay := 0
		singleMiss := make([]oa_model.CommonInfo, 0)
		for j := 0; j < len(records); j++ {
			if leaveTypeMap[records[j].WorkTime] != nil {
				records[j].ActionType = leaveTypeMap[records[j].WorkTime].ActionType
				records[j].ActionTime = recordRes.Data[i].WorkDate + " " + records[j].WorkTime
				continue
			}
			if records[j].ActionType == oa_model.TypeMiss {
				singleMiss = append(singleMiss, oa_model.CommonInfo{
					Date:   recordRes.Data[i].WorkDate,
					Weekly: oa_model.WeekZhCN[holidayMap[recordRes.Data[i].WorkDate].WeekDay],
					Time:   records[j].WorkTime,
				})
				isMissDay += 1
			}
		}

		if isMissDay == len(records) {
			missDay = append(missDay, oa_model.CommonInfo{
				Date:   recordRes.Data[i].WorkDate,
				Weekly: oa_model.WeekZhCN[holidayMap[recordRes.Data[i].WorkDate].WeekDay],
				Days:   1,
			})
			miss = append(miss, singleMiss...)
			continue
		}

		for j := 0; j < len(records); j++ {
			if records[j].ActionType == oa_model.TypeLate {
				commonInfo := oa_model.CommonInfo{
					Date:   recordRes.Data[i].WorkDate,
					Weekly: oa_model.WeekZhCN[holidayMap[recordRes.Data[i].WorkDate].WeekDay],
					Time:   strings.Split(records[j].ActionTime, " ")[1],
				}
				late = append(late, commonInfo)
			}

			if records[j].ActionType == oa_model.TypeMiss {
				commonInfo := oa_model.CommonInfo{
					Date:   recordRes.Data[i].WorkDate,
					Weekly: oa_model.WeekZhCN[holidayMap[recordRes.Data[i].WorkDate].WeekDay],
					Time:   records[j].WorkTime,
				}
				miss = append(miss, commonInfo)
			}

			if records[j].ActionType == oa_model.TypeBefore {
				commonInfo := oa_model.CommonInfo{
					Date:   recordRes.Data[i].WorkDate,
					Weekly: oa_model.WeekZhCN[holidayMap[recordRes.Data[i].WorkDate].WeekDay],
					Time:   strings.Split(records[j].ActionTime, " ")[1], //  使用的 是  第一次打 早退卡 的 记录
				}
				before = append(before, commonInfo)
			}

		}

		fmt.Println("son step : =================================  1  =================================")
		fmt.Printf("restore  records id %+v\n", records)
		fmt.Println()

		// =========================  旷工 ==========================================

		for j := 0; j < len(records); j = j + 2 {

			if j+1 >= len(records) {
				break
			}
			fmt.Println("son step : =================================  2  =================================")
			fmt.Println()

			// 判断是否是 缺卡   是  跳过
			//if records[j].ActionType == oa_model.TypeMiss || records[j+1].ActionType == oa_model.TypeMiss {
			//	continue
			//}

			if (records[j].ActionType != oa_model.TypeCommon && records[j].ActionType != oa_model.TypeBefore && records[j].ActionType != oa_model.TypeLate && records[j].ActionType != oa_model.TypeMakeUp && records[j].ActionType != oa_model.TypeOutWork && records[j].ActionType != oa_model.TypeOverTime) ||
				(records[j+1].ActionType != oa_model.TypeCommon && records[j+1].ActionType != oa_model.TypeBefore && records[j+1].ActionType != oa_model.TypeLate && records[j+1].ActionType != oa_model.TypeMakeUp && records[j+1].ActionType != oa_model.TypeOutWork && records[j+1].ActionType != oa_model.TypeOverTime) {
				continue
			}
			fmt.Println("son step : =================================  3  =================================")
			fmt.Println()

			// 成对 出现
			var startTime time.Time
			var endTime time.Time
			startTime, _ = time.ParseInLocation("2006-01-02 15:04:05", records[j].ActionTime+":00", time.Local)

			endTime, _ = time.ParseInLocation("2006-01-02 15:04:05", records[j+1].ActionTime+":00", time.Local)

			hours = hours + endTime.Sub(startTime).Hours()
			fmt.Println("son step : =================================  4  =================================")
			fmt.Println("计算时长 :  ", j, hours)
			fmt.Println()
		}

		if hours > 0 {
			averageHour = append(averageHour, oa_model.CommonInfo{
				Date:   recordRes.Data[i].WorkDate,
				Weekly: oa_model.WeekZhCN[holidayMap[recordRes.Data[i].WorkDate].WeekDay],
				Hours:  HandleHourLeaveASeat(hours),
			})
			workDays = append(workDays, oa_model.CommonInfo{Date: recordRes.Data[i].WorkDate, Weekly: oa_model.WeekZhCN[holidayMap[recordRes.Data[i].WorkDate].WeekDay]})
		}

		if hours == 0 && isMissDay != len(records) {
			averageHour = append(averageHour, oa_model.CommonInfo{
				Date:   recordRes.Data[i].WorkDate,
				Weekly: oa_model.WeekZhCN[holidayMap[recordRes.Data[i].WorkDate].WeekDay],
				Hours:  0,
			})
			workDays = append(workDays, oa_model.CommonInfo{Date: recordRes.Data[i].WorkDate, Weekly: oa_model.WeekZhCN[holidayMap[recordRes.Data[i].WorkDate].WeekDay]})
		}
	}

	fmt.Println("step : =================================  5  =================================")
	fmt.Println()

	collectionDetail.AverageHour = averageHour
	collectionDetail.WorkDays = workDays
	collectionDetail.Late = late
	collectionDetail.Miss = miss
	collectionDetail.Before = before
	collectionDetail.MissDay = missDay
	collectionDetail.RestDays = restDays

	//fmt.Println("collectionDetail ============================ start ======================================")
	//fmt.Printf("collectionDetail : %+v\n", collectionDetail)
	//fmt.Println("collectionDetail ============================ end   ======================================")
	//fmt.Println()
	return
}

func CheckMonthIsAfter(month string) bool {
	queryMonth, _ := time.ParseInLocation("2006-01-02 15:04:05", month+" 00:00:00", time.Local)
	nowMonth, _ := time.ParseInLocation("2006-01-02 15:04:05", time.Now().Format("2006-01")+"-25"+" 00:00:00", time.Local)
	return queryMonth.After(nowMonth)
}

// 是否在周期内
func CheckDateInMonth(date string, dates []string) bool {
	dateMap := make(map[string]string)
	for i := 0; i < len(dates); i++ {
		dateMap[dates[i]] = dates[i]
	}

	if _, ok := dateMap[date]; ok {
		return true
	} else {
		return false
	}
}
