package oa_new

import (
	"context"
	"dubbo.apache.org/dubbo-go/v3/common/logger"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/fonchain_enterprise/fonchain-main/api/account"
	"github.com/fonchain_enterprise/fonchain-main/api/oa"
	"github.com/fonchain_enterprise/fonchain-main/api/rule"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/login"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/oa_model"
	"github.com/fonchain_enterprise/fonchain-main/pkg/serializer"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/oa_new/common"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/oa_new/model"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/oa_new/oa_logic"
	"github.com/gin-gonic/gin"
	"strconv"
	"time"
)

func ClockIn(c *gin.Context) {
	req := new(model.ClockInReq)
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("ClockIn ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	logger.Infof("ClockIn req: %+v", req)

	userInfo := login.GetUserInfoFromC(c)

	// 查询 考勤信息
	workingTime, workingTimeErr := oa_logic.QueryWorkingTimeInUse(userInfo.PositionUsers, userInfo.ID)
	if workingTimeErr != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    workingTimeErr.Error(),
			Status: e.Failed,
		})
		return
	}

	logger.Infof("workingTime: %+v", workingTime)

	// 当前时间
	currentTime := common.CurrentTimeSubOneMinute()

	// 根据  workingTime 信息  判断是否可以外勤打卡
	in, checkClickInErr := oa_logic.CheckClickInAddress(req.Longitude, req.Latitude, workingTime.ChickIns)
	if checkClickInErr != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    checkClickInErr.Error(),
			Status: e.Failed,
		})
	}

	actionType, actionTypeErr := oa_logic.CheckActionType(req, workingTime, &currentTime, in)
	if actionTypeErr != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    actionTypeErr.Error(),
			Status: e.Failed,
		})
		return
	}

	if req.CheckAddress == "" && req.Longitude != "" && req.Latitude != "" {
		req.CheckAddress = oa_logic.ReverseGeo(req.Longitude, req.Latitude)
	}

	oaProfile := new(oa.OAProfile)
	oaProfile.StaffUID = userInfo.ID
	oaProfile.Domain = userInfo.Domain
	oaProfile.StaffName = userInfo.NickName
	oaProfile.DepartmentUID = workingTime.DepartmentUID
	oaProfile.PositionUID = workingTime.PositionUID
	oaProfile.Week = oa_model.Week[common.CheckIsBeforeFourTime().Weekday().String()]
	oaProfile.WorkDate = common.CheckIsBeforeFour()
	oaProfile.ActionTime = currentTime.Add(1 * time.Minute).Format(common.YYMMDDHHmm)
	oaProfile.FacePhoto = req.FacePhoto
	oaProfile.ClientID = req.ClientID
	oaProfile.CheckMethod = req.CheckMethod
	oaProfile.CheckAddress = req.CheckAddress
	oaProfile.Longitude = req.Longitude
	oaProfile.Latitude = req.Latitude
	oaProfile.Time = workingTime.Time
	oaProfile.WorkTime = actionType.WorkTime
	oaProfile.ActionType = actionType.ActionType
	oaProfile.IsWork = int32(actionType.IsWorkDay)
	oaProfile.OnOff = actionType.OnOff

	fmt.Println("===================================================")
	fmt.Println(oaProfile)
	logger.Infof("组装 打卡  请求参数 oaProfile: %+v", oaProfile)
	fmt.Println("===================================================")

	var err error

	if actionType.ActionType == model.ActionTypeOutNoClock || actionType.ActionType == model.ActionTypeBackNoClock || actionType.ActionType == model.ActionTypeNoClock {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    e.ErrorNotAllowedClockIn,
			Status: e.Failed,
		})
		return
	}

	if actionType.IsWorkDay == model.IsWork {
		logger.Infof("工作日 打卡 oaProfile: %+v", oaProfile)
		_, err = service.GrpcOAImpl.CreateOARecordWithMiss(context.Background(), oaProfile)
	} else {
		logger.Infof("非工作日 打卡 oaProfile: %+v", oaProfile)
		_, err = service.GrpcOAImpl.UseFaceCreateOARecord(context.Background(), oaProfile)
	}

	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    e.ErrorClockIn,
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Msg:    e.SuccessClockIn,
		Status: e.Ok,
	})

}

func ActionType(c *gin.Context) {
	req := new(model.ClockInReq)
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("ActionType ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	logger.Infof("ClockIn req: %+v", req)

	userInfo := login.GetUserInfoFromC(c)

	// 查询 考勤信息
	workingTime, workingTimeErr := oa_logic.QueryWorkingTimeInUse(userInfo.PositionUsers, userInfo.ID)
	if workingTimeErr != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    workingTimeErr.Error(),
			Status: e.Failed,
		})
		return
	}

	logger.Infof("workingTime: %+v\n", workingTime)
	fmt.Printf("workingTime: %+v\n", workingTime)

	// 当前时间
	currentTime := common.CurrentTimeSubOneMinute()

	fmt.Println("================== CheckClickInAddress  enter =================================")
	// 根据  workingTime 信息  判断是否可以外勤打卡
	in, checkClickInErr := oa_logic.CheckClickInAddress(req.Longitude, req.Latitude, workingTime.ChickIns)
	if checkClickInErr != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    checkClickInErr.Error(),
			Status: e.Failed,
		})
	}
	fmt.Printf("CheckClickInAddress in  %+v\n", in)

	fmt.Println("================== CheckClickInAddress  out =================================")

	fmt.Println("================== CheckActionType  enter =================================")

	fmt.Printf("CheckActionType req: %+v\n", req)
	fmt.Printf("CheckActionType workingTime: %+v\n", workingTime)
	fmt.Printf("CheckActionType currentTime: %+v\n", currentTime)

	actionType, actionTypeErr := oa_logic.CheckActionType(req, workingTime, &currentTime, in)
	if actionTypeErr != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    actionTypeErr.Error(),
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   actionType,
		Status: e.Ok,
	})

}

func ChickInMapInfo(c *gin.Context) {
	userInfo := login.GetUserInfoFromC(c)

	// 查询 考勤信息
	workingTime, workingTimeErr := oa_logic.QueryWorkingTimeInUse(userInfo.PositionUsers, userInfo.ID)
	if workingTimeErr != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    workingTimeErr.Error(),
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   workingTime.ChickIns,
		Status: e.Ok,
	})
}

func WorkingTimeInfo(c *gin.Context) {
	userInfo := login.GetUserInfoFromC(c)

	// 查询 考勤信息
	workingTime, workingTimeErr := oa_logic.QueryWorkingTimeInUse(userInfo.PositionUsers, userInfo.ID)
	if workingTimeErr != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    workingTimeErr.Error(),
			Status: e.Failed,
		})
		return
	}

	if workingTime == nil && workingTime.Time == nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    e.ErrNotHadWorkingTime,
			Status: e.Failed,
		})
		return

	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   workingTime,
		Status: e.Ok,
	})
}

func SystemMakeUp(c *gin.Context) {
	req := new(oa.SystemMakeUpReq)
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("SystemMakeUp ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if req.StaffUID == 0 || req.WorkTime == "" || req.WorkDate == "" {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    e.ErrMissSystemMakeUpParam,
			Status: e.Failed,
		})
		return
	}

	if (req.ActionTime != "" || req.ActionDate != "") && req.ActionDate != req.WorkDate {
		if common.ConvertWorkDateAndWorkTime(req.ActionDate, req.ActionTime).After(common.ConvertWorkDateAndWorkTime(req.ActionDate, common.NotWorkDayWorkTimeOn())) {
			service.ResponseMsg(c, e.SUCCESS, serializer.Response{
				Msg:    e.ErrActionTimeNotBeforeNextDateFour,
				Status: e.Failed,
			})
			return
		}
	}

	res, err := service.GrpcOAImpl.SystemMakeUp(context.Background(), req)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Msg:    res.Msg,
		Status: e.Ok,
	})
}

func SystemCommon(c *gin.Context) {
	req := new(oa.SystemCommonReq)
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("SystemCommon ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if req.StaffUID == 0 || req.Month == "" {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    e.ErrSystemCommonParam,
			Status: e.Failed,
		})
		return
	}

	res, err := service.GrpcOAImpl.SystemCommon(context.Background(), req)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Msg:    res.Msg,
		Status: e.Ok,
	})
}

// ClockInUseFace
// 打卡机打卡 ( 工作期间不做处理,  周末 加班 只记录 一个完整的周期 <上班卡只记录 第一次   下班卡 时间点做 更新>)
func ClockInUseFace(req oa_model.ClockInUseFace) error {
	if req.Logs == nil {
		return errors.New(e.GetMsg(e.ErrNoID))
	}
	jsonString, _ := json.Marshal(req)
	//if err != nil {
	//	fmt.Println("JSON serialization error:", err)
	//	return
	//}

	// 将JSON字符串打印到控制台
	fmt.Println(string(jsonString))
	/* 检查打卡 是否合规  前端 操作*/

	userIDStr := ""
	facePhoto := ""
	for i := 0; i < len(req.Logs); i++ {
		userIDStr = req.Logs[i].UserID
		facePhoto = req.Logs[i].Photo
	}

	if userIDStr == "" || userIDStr == "0" {
		return errors.New(e.GetMsg(e.ErrorGetUserInfo))
	}

	userID, _ := strconv.ParseUint(userIDStr, 0, 64)
	infoReq := account.InfoRequest{ID: userID}

	fmt.Println("======== infoReq  1  ==============")
	fmt.Printf("infoReq %+v\n", infoReq)

	infoRes, err := service.AccountProvider.Info(context.Background(), &infoReq)
	if err != nil {
		fmt.Println("======== AccountProvider.Info  2.1 ==============")
		fmt.Printf("err %+v\n", err)
		fmt.Printf("infoRes %+v\n", infoRes)
		return errors.New(e.GetMsg(e.ErrorGetUserInfo))
	}

	fmt.Println("======== userInfo  2 ==============")
	fmt.Printf("userInfo %+v\n", infoRes)

	if infoRes.Info.ID == 0 || infoRes.Info.NickName == "" || infoRes.Info.Domain == "" {
		return errors.New(e.GetMsg(e.ErrorGetUserInfo))
	}

	// 组装 考勤班次信息
	//userInfo := login.GetUserInfoFromC(c)
	departmentUUIDs := make([]string, 0)
	departmentAndPositionReq := new(rule.RulesRequest)
	departmentAndPositionReq.AccountID = infoRes.Info.ID
	departmentAndPositionRes, err := service.RuleProvider.UserInfo(context.Background(), departmentAndPositionReq)
	if err != nil {
		return err
	}

	userInfo := login.Info{
		ID:       infoRes.Info.ID,
		NickName: infoRes.Info.NickName,
		Domain:   infoRes.Info.Domain,
	}

	queryTimeOptions := make([]*rule.PositionUser, 0)
	for i := 0; i < len(departmentAndPositionRes.PositionUsers); i++ {
		departmentUUIDs = append(departmentUUIDs, strconv.FormatUint(departmentAndPositionRes.PositionUsers[i].DepartmentId, 10))
		queryTimeOption := &rule.PositionUser{
			PositionID:   departmentAndPositionRes.PositionUsers[i].PositionID,
			DepartmentId: departmentAndPositionRes.PositionUsers[i].DepartmentId,
		}
		queryTimeOptions = append(queryTimeOptions, queryTimeOption)
	}

	fmt.Println("======== queryTimeOptions  3 ==============")
	fmt.Printf("queryTimeOptions %+v\n", queryTimeOptions)

	workingTime, workingTimeErr := oa_logic.QueryWorkingTimeInUse(queryTimeOptions, userInfo.ID)
	if workingTimeErr != nil {
		return workingTimeErr
	}

	var currentTime time.Time
	// 给默认值
	currentTime = common.CurrentTimeSubOneMinute()
	if len(req.Logs) == 1 {
		currentTime, _ = time.ParseInLocation("2006-01-02 15:04:05", req.Logs[0].RecogTime, time.Local)
		currentTime.Add(-1 * time.Minute).Format(common.YYMMDDHHmm)
	}

	// 组装 chickInReq
	chickInReq := new(model.ClockInReq)
	chickInReq.StaffUID = userInfo.ID

	actionType, actionTypeErr := oa_logic.CheckActionType(chickInReq, workingTime, &currentTime, true)
	if actionTypeErr != nil {
		return actionTypeErr
	}

	oaProfile := new(oa.OAProfile)
	oaProfile.StaffUID = userInfo.ID
	oaProfile.Domain = userInfo.Domain
	oaProfile.StaffName = userInfo.NickName
	oaProfile.DepartmentUID = workingTime.DepartmentUID
	oaProfile.PositionUID = workingTime.PositionUID
	oaProfile.Week = oa_model.Week[common.CheckIsBeforeFourTime().Weekday().String()]
	oaProfile.WorkDate = common.CheckIsBeforeFour()
	oaProfile.ActionTime = currentTime.Add(1 * time.Minute).Format(common.YYMMDDHHmm)
	oaProfile.FacePhoto = facePhoto
	oaProfile.CheckAddress = req.Sn
	oaProfile.CheckMethod = oa_model.CheckMethodPunchClock
	oaProfile.Time = workingTime.Time
	oaProfile.WorkTime = actionType.WorkTime
	oaProfile.ActionType = actionType.ActionType
	oaProfile.IsWork = int32(actionType.IsWorkDay)
	oaProfile.OnOff = actionType.OnOff

	if actionType.IsWorkDay == model.IsWork {
		if oaProfile.ActionType == oa_model.TypeCommon {

			logger.Infof("工作日 打卡 oaProfile: %+v", oaProfile)
			_, err = service.GrpcOAImpl.CreateOARecordWithMiss(context.Background(), oaProfile)

			logger.Errorf("模版消息---1")
			templateMsgReq := account.SendClockInWechatRequest{
				OperatedAt: currentTime.Add(1 * time.Minute).Format(common.YYMMDDHHmm),
				ClockType:  oaProfile.OnOff,
				UserId:     uint32(userID),
				GhId:       "gh_d838598b1a23",
				Address:    req.Sn,
			}

			go service.AccountProvider.SendClockInWechat(context.Background(), &templateMsgReq)

			fmt.Println("================模版发送完毕=============================")
		}

	} else {
		logger.Infof("非工作日 打卡 oaProfile: %+v", oaProfile)
		_, err = service.GrpcOAImpl.UseFaceCreateOARecord(context.Background(), oaProfile)
	}

	return err
}

func QueryOtherRecord(c *gin.Context) {
	req := new(oa.QueryOtherRecordReq)
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("QueryOtherRecord ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if req.StaffUID == 0 {
		userInfo := login.GetUserInfoFromC(c)
		req.StaffUID = userInfo.ID
	}

	// 默认查询所有
	req.Page = 1
	req.PageSize = 9999999

	if req.WorkDate == "" {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    e.ErrParamWithoutWorkDate,
			Status: e.Failed,
		})
		return
	}

	res, err := service.GrpcOAImpl.QueryOtherRecord(context.Background(), req)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})

}
