package oa_new

import (
	"dubbo.apache.org/dubbo-go/v3/common/logger"
	"errors"
	"fmt"
	"github.com/fonchain_enterprise/fonchain-main/api/oa"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/login"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/oa_model"
	"github.com/fonchain_enterprise/fonchain-main/pkg/serializer"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/oa_new/common"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/oa_new/model"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/oa_new/oa_logic"
	"github.com/fonchain_enterprise/fonchain-main/pkg/utils/holiday"
	"github.com/gin-gonic/gin"
	"time"
)

// OverTimeDuration
// 计算 加班时长  <根据 设置的考勤班次>
func OverTimeDuration(c *gin.Context) {
	req := new(model.DurationReq)
	res := new(oa_model.DurationRes)
	if err := c.ShouldBind(&req); err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if len(req.ApplyTimes) < 1 {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    e.ErrOvertimeDisagreement,
			Status: e.Failed,
		})
		return
	}

	err := oa_logic.VerifyTimeOrder(req.ApplyTimes)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	logger.Infof("DurationReq req: %+v", req)
	fmt.Printf("DurationReq req: %+v", req)

	userInfo := login.GetUserInfoFromC(c)

	// 查询 考勤信息
	workingTime, workingTimeErr := oa_logic.QueryWorkingTimeInUse(userInfo.PositionUsers, userInfo.ID)
	if workingTimeErr != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    workingTimeErr.Error(),
			Status: e.Failed,
		})
		return
	}

	fmt.Println("====================================== workingTime  ====================================================")
	fmt.Printf("workingTime is %+v\n", workingTime)
	fmt.Println("====================================== workingTime  ====================================================")

	// 解析出 加班规则
	overtimeRule := new(oa.Overtime)
	for i := 0; i < len(workingTime.OaRules); i++ {
		if workingTime.OaRules[i].Keyword == model.OvertimeRule && workingTime.OaRules[i].OvertimeRule != nil {
			overtimeRule = workingTime.OaRules[i].OvertimeRule
			break
		}
	}

	fmt.Println("====================================== overtimeRule  ====================================================")
	fmt.Printf("overtimeRule is %+v\n", overtimeRule)
	fmt.Println("====================================== overtimeRule  ====================================================")

	startTime := common.ConvertApplyTimeToTime(req.ApplyTimes[0].Date, req.ApplyTimes[0].Hour)
	endTime := common.ConvertApplyTimeToTime(req.ApplyTimes[1].Date, req.ApplyTimes[1].Hour)

	fmt.Println("====================================== ApplyTimes  ====================================================")
	fmt.Printf("startTime is %+v\n", startTime)
	fmt.Printf("endTime is %+v\n", endTime)
	fmt.Println("====================================== ApplyTimes  ====================================================")

	if !startTime.Before(endTime) {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    e.ErrTimeOrder,
			Status: e.Failed,
		})
		return
	}

	var subHours float64
	if req.ApplyTimes[0].Date == req.ApplyTimes[1].Date {
		if req.ApplyTimes[0].Hour == req.ApplyTimes[1].Hour {
			service.ResponseMsg(c, e.SUCCESS, serializer.Response{
				Msg:    e.ErrOvertimeIsRep,
				Status: e.Failed,
			})
			return
		}
		var subTimeDurationSingleErr error
		subHours, subTimeDurationSingleErr = subTimeDurationSingle(req, workingTime, overtimeRule, startTime, endTime)
		if subTimeDurationSingleErr != nil {
			service.ResponseMsg(c, e.SUCCESS, serializer.Response{
				Msg:    subTimeDurationSingleErr.Error(),
				Status: e.Failed,
			})
			return
		}

	} else {
		// 处理 加班申请开始时间
		startTime = common.ApplyStartTime(startTime)
		endTime = common.ApplyEndTimeDefault(startTime, endTime)
		var subTimeDurationMultipleErr error
		subHours, subTimeDurationMultipleErr = subTimeDurationMultiple(req, workingTime, overtimeRule, startTime, endTime)
		if subTimeDurationMultipleErr != nil {
			service.ResponseMsg(c, e.SUCCESS, serializer.Response{
				Msg:    subTimeDurationMultipleErr.Error(),
				Status: e.Failed,
			})
			return
		}
	}

	fmt.Println("====================================== OverTimeDuration subHours ====================================================")
	fmt.Printf("OverTimeDuration subHours is %+v\n", subHours)
	fmt.Println("====================================== OverTimeDuration subHours ====================================================")

	fmt.Println("====================================== ApplyTimes after ====================================================")
	fmt.Printf("startTime is %+v\n", startTime)
	fmt.Printf("endTime is %+v\n", endTime)
	fmt.Println("====================================== ApplyTimes after ====================================================")

	originHour := endTime.Sub(startTime).Hours()

	fmt.Println("====================================== OverTimeDuration originHour ====================================================")
	fmt.Printf("OverTimeDuration originHour is %+v\n", originHour)
	fmt.Println("====================================== OverTimeDuration originHour ====================================================")

	res.NotTakeOut.Hours = float32(common.HandleHourLeaveFiveV2(originHour - subHours))
	res.TakeOut.Hours = float32(common.HandleHourLeaveFiveV2(originHour - subHours))

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Msg:    e.GetMsg(e.SUCCESS),
		Data:   res,
		Status: e.Ok,
	})
}

func subTimeDurationSingle(req *model.DurationReq, workingTime *oa.WorkingTime, overtimeRule *oa.Overtime, startTime, endTime time.Time) (subHours float64, err error) {
	//  判断 申请的时间是否在工作时间内
	workDay, isWorkDayErr := oa_logic.IsWorkDay(req.ApplyTimes[0].Date, workingTime.Week, workingTime.IsWorkDay)
	if isWorkDayErr != nil {
		return 0, isWorkDayErr
	}

	fmt.Println("======================= subTimeDurationSingle  IsWorkDay ==============================")
	fmt.Printf("workDay is %+v\n", workDay)
	fmt.Println("======================= subTimeDurationSingle  IsWorkDay ==============================")

	if workDay {
		subHours, err = workDayDurationSingle(req, workingTime, overtimeRule, startTime, endTime)
		fmt.Println("====================================== subHours  workDay ====================================================")
		fmt.Printf("subHours is %+v\n", subHours)
		fmt.Println("====================================== subHours  workDay ====================================================")
		return
	} else {
		subHours, err = notWorkDayDurationSingle(req, workingTime, overtimeRule, startTime, endTime)
		fmt.Println("====================================== subHours notWorkDay ====================================================")
		fmt.Printf("subHours is %+v\n", subHours)
		fmt.Println("====================================== subHours notWorkDay ====================================================")
		return
	}
}

func subTimeDurationMultiple(req *model.DurationReq, workingTime *oa.WorkingTime, overtimeRule *oa.Overtime, startTime, endTime time.Time) (subHours float64, err error) {

	startDay := common.ConvertApplyTimeToTime(req.ApplyTimes[0].Date, "00:00")
	endDay := common.ConvertApplyTimeToTime(req.ApplyTimes[1].Date, "00:00")
	offsetDay := endDay.Sub(startDay).Hours() / 24
	if offsetDay >= 2 {
		return 0, errors.New(e.ErrCanNotCalcOverTimeBehindTwoDays)
	}

	//  判断 申请的时间是否在工作时间内
	workDayStart, isWorkDayStartErr := oa_logic.IsWorkDay(startDay.Format(common.YYMMDD), workingTime.Week, workingTime.IsWorkDay)
	if isWorkDayStartErr != nil {
		return 0, isWorkDayStartErr
	}

	//  判断 申请的时间是否在工作时间内
	//workDayEnd, isWorkDayEndErr := oa_logic.IsWorkDay(endDay.Format(common.YYMMDD), workingTime.Week, workingTime.IsWorkDay)
	//if isWorkDayEndErr != nil {
	//	return startTime, endTime, 0, isWorkDayEndErr
	//}

	if workDayStart {
		// 申请的日期都为  工作日
		return workDayDurationSingle(req, workingTime, overtimeRule, startTime, endTime)
	} else if !workDayStart {
		// 申请的日期都不为  工作日
		return notWorkDayDurationSingle(req, workingTime, overtimeRule, startTime, endTime)
	}

	return
}

func workDayDurationSingle(req *model.DurationReq, workingTime *oa.WorkingTime, overtimeRule *oa.Overtime, startTime, endTime time.Time) (subHours float64, err error) {

	// 是工作日
	startWorkTime := common.ConvertWorkDateAndWorkTime(req.ApplyTimes[0].Date, workingTime.Time[0].OnWorkTime)
	endWorkTime := common.ConvertWorkDateAndWorkTime(req.ApplyTimes[0].Date, workingTime.Time[len(workingTime.Time)-1].OffWorkTime)

	fmt.Println("======================================  workTime  ====================================================")
	fmt.Printf("startWorkTime is %+v\n", startWorkTime)
	fmt.Printf("endWorkTime is %+v\n", endWorkTime)
	fmt.Println("====================================== workTime  ====================================================")

	if startTime.Before(startWorkTime) {
		if !endTime.After(startWorkTime) {
			if endTime.Sub(startTime).Hours() < float64(overtimeRule.WorkHead) {
				subHours += startWorkTime.Sub(startTime).Hours()
				fmt.Println("====================================== subHours WorkHead 1 ====================================================")
				fmt.Printf("subHours is %+v\n", subHours)
				fmt.Println("====================================== subHours WorkHead 1 ====================================================")
				return subHours, nil
			}
		} else {
			if startWorkTime.Sub(startTime).Hours() < float64(overtimeRule.WorkHead) {
				subHours += startWorkTime.Sub(startTime).Hours()
				fmt.Println("====================================== subHours WorkHead 2 ====================================================")
				fmt.Printf("subHours is %+v\n", subHours)
				fmt.Println("====================================== subHours WorkHead 2 ====================================================")
			}
		}
	}

	fmt.Println("====================================== subHours WorkHead 3 ====================================================")
	fmt.Printf("subHours is %+v\n", subHours)
	fmt.Println("====================================== subHours WorkHead 3  ====================================================")

	fmt.Println("======================================endTime.Sub(endWorkTime).Hours()  ====================================================")
	fmt.Printf("endTime.Sub(endWorkTime).Hours() is %+v\n", endTime.Sub(endWorkTime).Hours())
	fmt.Printf("float64(overtimeRule.WorkEnd) is %+v\n", float64(overtimeRule.WorkEnd))
	fmt.Printf("float64(overtimeRule.Least) is %+v\n", float64(overtimeRule.Least))
	fmt.Println("====================================== endTime.Sub(endWorkTime).Hours()  ====================================================")

	if endTime.After(endWorkTime) {
		// 参与计算的 开始时间
		var calcStartTime time.Time
		if !startTime.Before(endWorkTime) {
			calcStartTime = startTime
		} else {
			calcStartTime = endWorkTime
		}

		fmt.Println("====================================== subHours WorkEnd calcStartTime ====================================================")
		fmt.Printf("calcStartTime is %+v\n", calcStartTime)
		fmt.Println("====================================== subHours WorkEnd calcStartTime")

		if endTime.Sub(calcStartTime).Hours() < float64(overtimeRule.WorkEnd) {

			subHours += endTime.Sub(calcStartTime).Hours()
			fmt.Println("====================================== subHours WorkEnd 1 ====================================================")
			fmt.Printf("subHours is %+v\n", subHours)
			fmt.Println("====================================== subHours WorkEnd 1 ====================================================")

		} else if endTime.Sub(calcStartTime).Hours() >= float64(overtimeRule.WorkEnd) &&
			endTime.Sub(calcStartTime).Hours() < float64(overtimeRule.Least) {

			subHours += float64(overtimeRule.WorkEnd)
			fmt.Println("====================================== subHours WorkEnd 2 ====================================================")
			fmt.Printf("subHours is %+v\n", subHours)
			fmt.Println("====================================== subHours WorkEnd 2 ====================================================")

		}

	}

	fmt.Println("====================================== subHours WorkEnd 3 ====================================================")
	fmt.Printf("subHours is %+v\n", subHours)
	fmt.Println("====================================== subHours WorkEnd 3 ====================================================")

	if !startTime.After(startWorkTime) && !endTime.Before(endWorkTime) {
		// 固定 减去 工作日 工作时长  此时  不管 是否有休息时间
		subHours += endWorkTime.Sub(startWorkTime).Hours()
	} else if startTime.After(startWorkTime) && startTime.Before(endWorkTime) && !endTime.Before(endWorkTime) {
		// 申请开始时间 在工作时间内 且 申请结束时间 在工作时间之后
		subHours += endWorkTime.Sub(startTime).Hours()
	} else if !startTime.Before(startWorkTime) && !endTime.After(endWorkTime) && endTime.After(startWorkTime) {
		// 申请开始时间 在工作时间之前 且 申请结束时间 在工作时间内
		subHours += endTime.Sub(startTime).Hours()
	}

	return
}

func notWorkDayDurationSingle(req *model.DurationReq, workingTime *oa.WorkingTime, overtimeRule *oa.Overtime, startTime, endTime time.Time) (subHours float64, err error) {

	startDay := common.ConvertApplyTimeToTime(req.ApplyTimes[0].Date, "00:00")
	//endDay := common.ConvertApplyTimeToTime(req.ApplyTimes[1].Date, "00:00")

	// 获取 休息时间
	var breakOnWorkTime time.Time
	var breakOffWorkTime time.Time

	hasBreakTime := false
	if overtimeRule.RemovingBreakPeriods == model.RemovingBreakPeriodsYes {
		if workingTime.HasBreakTime == model.HasBreakTime {
			if workingTime.BreakOption.OnWorkTime != "" && workingTime.BreakOption.OffWorkTime != "" {
				hasBreakTime = true
				breakOnWorkTime = common.ConvertWorkDateAndWorkTime(startDay.Format(common.YYMMDD), workingTime.BreakOption.OnWorkTime)
				breakOffWorkTime = common.ConvertWorkDateAndWorkTime(startDay.Format(common.YYMMDD), workingTime.BreakOption.OffWorkTime)
				fmt.Println("====================================== breakOnWorkTime breakOffWorkTime ====================================================")
				fmt.Printf("breakOnWorkTime is %+v\n", breakOnWorkTime)
				fmt.Printf("breakOffWorkTime is %+v\n", breakOffWorkTime)
				fmt.Println("====================================== breakOnWorkTime breakOffWorkTime  ====================================================")
			}
		}
	}

	if hasBreakTime {
		if !startTime.After(breakOnWorkTime) {
			//  申请开始时间 在 休息开始时间之前 且 包含 休息时间点
			if !endTime.After(breakOnWorkTime) {
				//  申请结束时间 在 休息开始时间之前 且 包含 休息时间点
				return
			} else if !endTime.Before(breakOffWorkTime) {
				//  申请结束时间 在 休息结束时间之后 且 包含 休息时间点
				subHours += breakOffWorkTime.Sub(breakOnWorkTime).Hours()
				fmt.Println("====================================== subHours notWorkDay ====================================================")
				fmt.Printf("subHours is %+v\n", subHours)
				fmt.Println("====================================== subHours notWorkDay ====================================================")
				fmt.Println("====================================== subHours notWorkDay 1 ====================================================")
				return
			} else {
				// 申请结束时间  在 休息时间之间
				subHours += endTime.Sub(breakOnWorkTime).Hours()
				fmt.Println("====================================== subHours notWorkDay ====================================================")
				fmt.Printf("subHours is %+v\n", subHours)
				fmt.Println("====================================== subHours notWorkDay ====================================================")
				fmt.Println("====================================== subHours notWorkDay 2 ====================================================")
				return
			}

		} else {
			if endTime.After(breakOffWorkTime) {
				//  申请结束时间 在 休息结束时间之后 且 包含 休息时间点
				if !startTime.After(breakOffWorkTime) {
					//  申请开始时间 在 休息结束时间之前 且 包含 休息时间点
					subHours += breakOffWorkTime.Sub(startTime).Hours()
					fmt.Println("====================================== subHours notWorkDay ====================================================")
					fmt.Printf("subHours is %+v\n", subHours)
					fmt.Println("====================================== subHours notWorkDay ====================================================")
					fmt.Println("====================================== subHours notWorkDay 3 ====================================================")
					return
				}
				return
			}
		}
	}

	return
}

func workDayDurationMultiple(req *model.DurationReq, workingTime *oa.WorkingTime, overtimeRule *oa.Overtime, startTime, endTime time.Time) (startTimeAfter, endTimeAfter time.Time, subHours float32, err error) {
	// 处理 加班申请开始时间
	startTimeAfter = common.ApplyStartTime(startTime)
	endTimeAfter = common.ApplyEndTimeDefault(startTime, endTime)

	// 是工作日
	startWorkTime := common.ConvertWorkDateAndWorkTime(req.ApplyTimes[0].Date, workingTime.Time[0].OnWorkTime)
	endWorkTime := common.ConvertWorkDateAndWorkTime(req.ApplyTimes[1].Date, workingTime.Time[len(workingTime.Time)-1].OffWorkTime)

	fmt.Println("======================================  workTime  ====================================================")
	fmt.Printf("startWorkTime is %+v\n", startWorkTime)
	fmt.Printf("endWorkTime is %+v\n", endWorkTime)
	fmt.Println("====================================== workTime  ====================================================")

	if startTimeAfter.Before(startWorkTime) {
		if !endTime.After(startWorkTime) {
			if endTime.Sub(startTimeAfter).Hours() < float64(overtimeRule.WorkHead) {
				subHours += float32(startWorkTime.Sub(startTimeAfter).Hours())
				fmt.Println("====================================== subHours WorkHead 1 ====================================================")
				fmt.Printf("subHours is %+v\n", subHours)
				fmt.Println("====================================== subHours WorkHead 1 ====================================================")
				return startTimeAfter, endTimeAfter, subHours, nil
			}
		} else {
			if startWorkTime.Sub(startTimeAfter).Hours() < float64(overtimeRule.WorkHead) {
				subHours += float32(startWorkTime.Sub(startTimeAfter).Hours())
				fmt.Println("====================================== subHours WorkHead 2 ====================================================")
				fmt.Printf("subHours is %+v\n", subHours)
				fmt.Println("====================================== subHours WorkHead 2 ====================================================")
			}
		}
	}

	fmt.Println("====================================== subHours WorkHead 3 ====================================================")
	fmt.Printf("subHours is %+v\n", subHours)
	fmt.Println("====================================== subHours WorkHead 3  ====================================================")

	fmt.Println("======================================endTime.Sub(endWorkTime).Hours()  ====================================================")
	fmt.Printf("endTime.Sub(endWorkTime).Hours() is %+v\n", endTimeAfter.Sub(endWorkTime).Hours())
	fmt.Printf("float64(overtimeRule.WorkEnd) is %+v\n", float64(overtimeRule.WorkEnd))
	fmt.Printf("float64(overtimeRule.Least) is %+v\n", float64(overtimeRule.Least))
	fmt.Println("====================================== endTime.Sub(endWorkTime).Hours()  ====================================================")

	if endTimeAfter.After(endWorkTime) {
		// 参与计算的 开始时间
		var calcStartTime time.Time
		if !startTimeAfter.Before(endWorkTime) {
			calcStartTime = startTimeAfter
		} else {
			calcStartTime = endWorkTime
		}

		fmt.Println("====================================== subHours WorkEnd calcStartTime ====================================================")
		fmt.Printf("calcStartTime is %+v\n", calcStartTime)
		fmt.Println("====================================== subHours WorkEnd calcStartTime")

		if endTimeAfter.Sub(calcStartTime).Hours() < float64(overtimeRule.WorkEnd) {

			subHours += float32(endTimeAfter.Sub(calcStartTime).Hours())
			fmt.Println("====================================== subHours WorkEnd 1 ====================================================")
			fmt.Printf("subHours is %+v\n", subHours)
			fmt.Println("====================================== subHours WorkEnd 1 ====================================================")

		} else if endTimeAfter.Sub(calcStartTime).Hours() >= float64(overtimeRule.WorkEnd) &&
			endTimeAfter.Sub(calcStartTime).Hours() < float64(overtimeRule.Least) {

			subHours += overtimeRule.WorkEnd
			fmt.Println("====================================== subHours WorkEnd 2 ====================================================")
			fmt.Printf("subHours is %+v\n", subHours)
			fmt.Println("====================================== subHours WorkEnd 2 ====================================================")

		}

	}

	fmt.Println("====================================== subHours WorkEnd 3 ====================================================")
	fmt.Printf("subHours is %+v\n", subHours)
	fmt.Println("====================================== subHours WorkEnd 3 ====================================================")

	if !startTimeAfter.After(startWorkTime) && !endTimeAfter.Before(endWorkTime) {
		// 固定 减去 工作日 工作时长  此时  不管 是否有休息时间
		subHours += float32(endWorkTime.Sub(startWorkTime).Hours())
	}

	return
}

func notWorkDayDurationMultiple(req *model.DurationReq, workingTime *oa.WorkingTime, overtimeRule *oa.Overtime, startTime, endTime time.Time) (startTimeAfter, endTimeAfter time.Time, subHours float32, err error) {

	startDay := common.ConvertApplyTimeToTime(req.ApplyTimes[0].Date, "00:00")
	//endDay := common.ConvertApplyTimeToTime(req.ApplyTimes[1].Date, "00:00")

	// 处理 加班申请开始时间
	startTimeAfter = common.ApplyStartTime(startTime)
	endTimeAfter = common.ApplyEndTimeDefault(startTime, endTime)

	// 获取 休息时间
	var breakOnWorkTime time.Time
	var breakOffWorkTime time.Time

	hasBreakTime := false
	if overtimeRule.RemovingBreakPeriods == model.RemovingBreakPeriodsYes {
		if workingTime.HasBreakTime == model.HasBreakTime {
			if workingTime.BreakOption.OnWorkTime != "" && workingTime.BreakOption.OffWorkTime != "" {
				hasBreakTime = true
				breakOnWorkTime = common.ConvertWorkDateAndWorkTime(startDay.Format(common.YYMMDD), workingTime.BreakOption.OnWorkTime)
				breakOffWorkTime = common.ConvertWorkDateAndWorkTime(startDay.Format(common.YYMMDD), workingTime.BreakOption.OffWorkTime)
				fmt.Println("====================================== breakOnWorkTime breakOffWorkTime ====================================================")
				fmt.Printf("breakOnWorkTime is %+v\n", breakOnWorkTime)
				fmt.Printf("breakOffWorkTime is %+v\n", breakOffWorkTime)
				fmt.Println("====================================== breakOnWorkTime breakOffWorkTime  ====================================================")
			}
		}
	}

	if hasBreakTime {
		if !startTime.After(breakOnWorkTime) {
			//  申请开始时间 在 休息开始时间之前 且 包含 休息时间点
			if !endTime.After(breakOnWorkTime) {
				//  申请结束时间 在 休息开始时间之前 且 包含 休息时间点
				return
			} else if !endTime.Before(breakOffWorkTime) {
				//  申请结束时间 在 休息结束时间之后 且 包含 休息时间点
				subHours += float32(breakOffWorkTime.Sub(breakOnWorkTime).Hours())
				return
			} else {
				// 申请结束时间  在 休息时间之间
				subHours += float32(endTime.Sub(breakOnWorkTime).Hours())
				return
			}

		} else {
			if endTime.After(breakOffWorkTime) {
				//  申请结束时间 在 休息结束时间之后 且 包含 休息时间点
				if !startTime.After(breakOffWorkTime) {
					//  申请开始时间 在 休息结束时间之前 且 包含 休息时间点
					subHours += float32(breakOffWorkTime.Sub(startTime).Hours())
					return
				}
				return
			}
		}
	}

	return
}

// LeaveDuration 计算 请假时长  <根据 设置的考勤班次>
func LeaveDuration(c *gin.Context) {
	var applyTimes model.Data
	if err := c.ShouldBind(&applyTimes); err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	logger.Infof("DurationReq req: %+v", applyTimes)
	fmt.Printf("DurationReq req: %+v\n", applyTimes)

	userInfo := login.GetUserInfoFromC(c)

	// 查询 考勤信息
	workingTime, workingTimeErr := oa_logic.QueryWorkingTimeInUse(userInfo.PositionUsers, userInfo.ID)
	if workingTimeErr != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    workingTimeErr.Error(),
			Status: e.Failed,
		})
		return
	}

	res, err := oa_logic.LeaveDurationHandle(applyTimes, workingTime)

	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})
}

func LeaveDurationOld(c *gin.Context) {
	var applyTimes model.Data
	if err := c.ShouldBind(&applyTimes); err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	logger.Infof("DurationReq req: %+v", applyTimes)
	fmt.Printf("DurationReq req: %+v\n", applyTimes)

	userInfo := login.GetUserInfoFromC(c)

	// 查询 考勤信息
	workingTime, workingTimeErr := oa_logic.QueryWorkingTimeInUse(userInfo.PositionUsers, userInfo.ID)
	if workingTimeErr != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    workingTimeErr.Error(),
			Status: e.Failed,
		})
		return
	}

	req := new(model.DurationReq)

	req.ApplyTimes = applyTimes.ApplyTimes

	fmt.Println("====================================== workingTime  ====================================================")
	fmt.Printf("workingTime is %+v\n", workingTime)
	fmt.Println("====================================== workingTime  ====================================================")

	var times []*model.TimeOption
	for i := 0; i < len(workingTime.Time); i++ {
		times = append(times, &model.TimeOption{
			OnWorkTime:  workingTime.Time[i].OnWorkTime,
			OffWorkTime: workingTime.Time[i].OffWorkTime,
		})
	}
	req.TimeOptions.Times = times
	if workingTime.IsWorkDay == model.IsWorkDay {
		req.TimeOptions.Week = "1,5"
	}
	if workingTime.Week != "" {
		req.TimeOptions.Week = workingTime.Week
	}

	fmt.Println("====================================== workingTime  ====================================================")
	fmt.Printf("workingTime is %+v\n", workingTime)
	fmt.Println("====================================== workingTime  ====================================================")

	// 计算 请假时长
	if len(req.ApplyTimes) != 2 {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    e.ErrApplytimeDisagreement,
			Status: e.Failed,
		})
		return
	}

	verifyTimeOrderErr := oa_logic.VerifyTimeOrder(req.ApplyTimes)
	if verifyTimeOrderErr != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    verifyTimeOrderErr.Error(),
			Status: e.Failed,
		})
		return
	}

	res := model.DurationRes{}

	var freeTime float64
	if len(req.TimeOptions.Times) > 1 {
		for i := 0; i < len(req.TimeOptions.Times); i++ {
			var offWorkTime, onWorkTime = time.Time{}, time.Time{}
			offWorkTime, _ = time.ParseInLocation("2006-01-02 15:04:05", req.ApplyTimes[0].Date+" "+req.TimeOptions.Times[i].OffWorkTime+":00", time.Local)
			if i+1 < len(req.TimeOptions.Times) {
				onWorkTime, _ = time.ParseInLocation("2006-01-02 15:04:05", req.ApplyTimes[0].Date+" "+req.TimeOptions.Times[i+1].OnWorkTime+":00", time.Local)
			} else {
				break
			}

			freeTime = freeTime + onWorkTime.Sub(offWorkTime).Hours()
		}
	}

	//  请假 日期 是 同一天
	//  那么只需要 拿出 hours 或者  m  做 计算
	//  请假 日期 是 同一天
	//  那么只需要 拿出 hours 或者  m  做 计算
	if req.ApplyTimes[0].Date == req.ApplyTimes[1].Date {

		if req.ApplyTimes[0].M != "" && req.ApplyTimes[1].M != "" {
			if req.ApplyTimes[0].M == req.ApplyTimes[1].M {
				res.TakeOut.Days = 0.5
				res.NotTakeOut.Days = 0.5
			} else {
				res.TakeOut.Days = 1
				res.NotTakeOut.Days = 1
			}
		}

		if req.ApplyTimes[0].Hour != "" {

			res = oa_logic.SingleDateDurationHandle(req)

		}

	} else {

		basicDays := len(holiday.SplitDates(req.ApplyTimes[0].Date, req.ApplyTimes[1].Date, "2006-01-02")) - 1

		if req.ApplyTimes[0].M != "" && req.ApplyTimes[1].M != "" {

			if req.ApplyTimes[0].M == req.ApplyTimes[1].M { //  如果 同为 上午 或者 下午  则 记为 1.5
				res.TakeOut.Days = res.TakeOut.Days + 1.5
				res.NotTakeOut.Days = res.NotTakeOut.Days + 1.5
			} else {
				if req.ApplyTimes[0].M == "上午" { // 如果  不相同    则  记为  1
					res.TakeOut.Days = res.TakeOut.Days + 2
					res.NotTakeOut.Days = res.NotTakeOut.Days + 2
				} else if req.ApplyTimes[0].M == "下午" {
					res.TakeOut.Days = res.TakeOut.Days + 1
					res.NotTakeOut.Days = res.NotTakeOut.Days + 1
				}
			}

			isWeek, num, err := oa_logic.WeekNum(req)
			if err != nil {
				service.ResponseMsg(c, e.SUCCESS, serializer.Response{
					Msg:    err.Error(),
					Status: e.Failed,
				})
				return
			}
			if isWeek {
				basicDays = basicDays - num
			}

			fmt.Println("===================== 打印 isWeek  num  info  ===================================")
			fmt.Printf("isWeek  is : %+v\n", isWeek)
			fmt.Printf("num  is : %+v\n", num)
			fmt.Println("===================== 打印 isWeek  num  info ===================================")

			res.TakeOut.Days = res.TakeOut.Days + float32(basicDays-1)
			res.NotTakeOut.Days = res.NotTakeOut.Days + float32(basicDays-1)
		}

		if req.ApplyTimes[0].Hour != "" {

			res = oa_logic.MultiDateDurationHandle(req, freeTime, basicDays)

		}
	}

	if res.TakeOut.Days == 0 && res.TakeOut.Hours == 0 && res.NotTakeOut.Days == 0 && res.NotTakeOut.Hours == 0 {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg: e.ErrLeaveDurationIsZero,
			//Data:   res,
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})
}

func AnnualLeaveDuration(c *gin.Context) {
	var applyTimes model.Data
	res := new(model.DurationRes)
	if err := c.ShouldBind(&applyTimes); err != nil {
		logger.Errorf("AnnualLeaveDuration ShouldBind err %+v", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if len(applyTimes.ApplyTimes) < 1 {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    e.ErrOvertimeDisagreement,
			Status: e.Failed,
		})
		return
	}

	err := oa_logic.VerifyTimeOrder(applyTimes.ApplyTimes)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	now := common.CurrentDateStr()
	for i := 0; i < len(applyTimes.ApplyTimes); i++ {
		if applyTimes.ApplyTimes[i].Date == "" {
			applyTimes.ApplyTimes[i].Date = now
		}
	}

	userInfo := login.GetUserInfoFromC(c)
	// 查询 考勤信息
	workingTime, workingTimeErr := oa_logic.QueryWorkingTimeInUse(userInfo.PositionUsers, userInfo.ID)
	if workingTimeErr != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    workingTimeErr.Error(),
			Status: e.Failed,
		})
		return
	}

	res, err = oa_logic.AnnualLeaveDurationHandle(applyTimes, workingTime)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})
}

func AnnualLeaveDurationOld(c *gin.Context) {
	var applyTimes model.Data
	res := new(model.DurationRes)
	if err := c.ShouldBind(&applyTimes); err != nil {
		logger.Errorf("AnnualLeaveDuration ShouldBind err %+v", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if len(applyTimes.ApplyTimes) < 1 {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    e.ErrOvertimeDisagreement,
			Status: e.Failed,
		})
		return
	}

	err := oa_logic.VerifyTimeOrder(applyTimes.ApplyTimes)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	now := common.CurrentDateStr()
	for i := 0; i < len(applyTimes.ApplyTimes); i++ {
		if applyTimes.ApplyTimes[i].Date == "" {
			applyTimes.ApplyTimes[i].Date = now
		}
	}

	userInfo := login.GetUserInfoFromC(c)
	// 查询 考勤信息
	workingTime, workingTimeErr := oa_logic.QueryWorkingTimeInUse(userInfo.PositionUsers, userInfo.ID)
	if workingTimeErr != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    workingTimeErr.Error(),
			Status: e.Failed,
		})
		return
	}

	// 记录 打卡 周期
	isWork, err := oa_logic.IsWorkDay(common.CurrentDateStr(), workingTime.Week, workingTime.IsWorkDay)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    e.ErrConfirmweek,
			Status: e.Failed,
		})
		return
	}

	//startTime,_ := time.ParseInLocation("2006-01-02 15:04:05", req.ApplyTimes[0].Date+" "+"00:00:00", time.Local)
	//endTime,_ := time.ParseInLocation("2006-01-02 15:04:05", req.ApplyTimes[1].Date+" "+"00:00:00", time.Local)

	holidayRes, err := holiday.GetMultiData(holiday.ConvertTime(applyTimes.ApplyTimes[0].Date), holiday.ConvertTime(applyTimes.ApplyTimes[1].Date), true)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    e.ErrHolidayMiss,
			Status: e.Failed,
		})
		return
	}

	for i := 0; i < len(holidayRes); i++ {
		if holidayRes[i].Type == 0 || isWork {
			res.TakeOut.Days += 1
			res.NotTakeOut.Days += 1
		}
	}

	if res.TakeOut.Days == 0 && res.TakeOut.Hours == 0 && res.NotTakeOut.Days == 0 && res.NotTakeOut.Hours == 0 {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg: e.ErrLeaveDurationIsZero,
			//Data:   res,
			Status: e.Failed,
		})
		return
	}

	if applyTimes.ApplyTimes[0].M == "下午" {
		res.TakeOut.Days = res.TakeOut.Days - 0.5
		res.NotTakeOut.Days = res.NotTakeOut.Days - 0.5
	}

	if applyTimes.ApplyTimes[1].M == "上午" {
		res.TakeOut.Days = res.TakeOut.Days - 0.5
		res.NotTakeOut.Days = res.NotTakeOut.Days - 0.5
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})
}

func BusinessTripDuration(c *gin.Context) {
	var applyTimes model.Data
	res := new(model.DurationRes)
	if err := c.ShouldBind(&applyTimes); err != nil {
		logger.Errorf("BusinessTripDuration ShouldBind err %+v", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if len(applyTimes.ApplyTimes) < 1 {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    e.ErrOvertimeDisagreement,
			Status: e.Failed,
		})
		return
	}

	err := oa_logic.VerifyTimeOrder(applyTimes.ApplyTimes)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	now := common.CurrentDateStr()
	for i := 0; i < len(applyTimes.ApplyTimes); i++ {
		if applyTimes.ApplyTimes[i].Date == "" {
			applyTimes.ApplyTimes[i].Date = now
		}
	}

	//req := new(model.DurationReq)
	//
	//req.ApplyTimes = applyTimes.ApplyTimes

	res, err = oa_logic.CalcAllDayDurationHandle(applyTimes)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})
}

func BusinessTripDurationOld(c *gin.Context) {
	var applyTimes model.Data
	res := new(model.DurationRes)
	if err := c.ShouldBind(&applyTimes); err != nil {
		logger.Errorf("BusinessTripDuration ShouldBind err %+v", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if len(applyTimes.ApplyTimes) < 1 {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    e.ErrOvertimeDisagreement,
			Status: e.Failed,
		})
		return
	}

	err := oa_logic.VerifyTimeOrder(applyTimes.ApplyTimes)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	now := common.CurrentDateStr()
	for i := 0; i < len(applyTimes.ApplyTimes); i++ {
		if applyTimes.ApplyTimes[i].Date == "" {
			applyTimes.ApplyTimes[i].Date = now
		}
	}

	req := new(model.DurationReq)

	req.ApplyTimes = applyTimes.ApplyTimes

	holidayRes, err := holiday.GetMultiData(holiday.ConvertTime(applyTimes.ApplyTimes[0].Date), holiday.ConvertTime(applyTimes.ApplyTimes[1].Date), true)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    e.ErrHolidayMiss,
			Status: e.Failed,
		})
		return
	}

	for i := 0; i < len(holidayRes); i++ {
		res.TakeOut.Days += 1
		res.NotTakeOut.Days += 1
	}

	if res.TakeOut.Days == 0 && res.TakeOut.Hours == 0 && res.NotTakeOut.Days == 0 && res.NotTakeOut.Hours == 0 {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    e.ErrLeaveDurationIsZero,
			Status: e.Failed,
		})
		return
	}

	if applyTimes.ApplyTimes[0].M == "下午" {
		res.TakeOut.Days = res.TakeOut.Days - 0.5
		res.NotTakeOut.Days = res.NotTakeOut.Days - 0.5
	}

	if applyTimes.ApplyTimes[1].M == "上午" {
		res.TakeOut.Days = res.TakeOut.Days - 0.5
		res.NotTakeOut.Days = res.NotTakeOut.Days - 0.5
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})
}

func OutWorkDuration(c *gin.Context) {
	var applyTimes model.Data
	res := new(model.DurationRes)
	if err := c.ShouldBind(&applyTimes); err != nil {
		logger.Errorf("OutWorkDuration ShouldBind err %+v ", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	err := oa_logic.VerifyTimeOrder(applyTimes.ApplyTimes)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	res, err = oa_logic.OutWorkDurationHandle(applyTimes)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})
}

func OutWorkDurationOld(c *gin.Context) {
	var applyTimes model.Data
	res := new(model.DurationRes)
	if err := c.ShouldBind(&applyTimes); err != nil {
		logger.Errorf("OutWorkDuration ShouldBind err %+v ", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	err := oa_logic.VerifyTimeOrder(applyTimes.ApplyTimes)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	startTime, _ := time.ParseInLocation("2006-01-02 15:04:05", applyTimes.ApplyTimes[0].Date+" "+applyTimes.ApplyTimes[0].Hour+":00", time.Local)
	endTime, _ := time.ParseInLocation("2006-01-02 15:04:05", applyTimes.ApplyTimes[1].Date+" "+applyTimes.ApplyTimes[1].Hour+":00", time.Local)
	if applyTimes.ApplyTimes[1].Hour == "23:59" {
		endTime = endTime.Add(1 * time.Minute)
	}

	// 计算 请假时长
	res.NotTakeOut.Hours = service.HandleHourLeaveFive(endTime.Sub(startTime).Hours())
	res.TakeOut.Hours = res.NotTakeOut.Hours

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})
}
