package oa_new

import (
	"context"
	"fmt"
	"github.com/fonchain_enterprise/fonchain-main/api/oa"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/login"
	"github.com/fonchain_enterprise/fonchain-main/pkg/serializer"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/oa_new/common"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/oa_new/oa_logic"
	"github.com/fonchain_enterprise/fonchain-main/pkg/utils/holiday"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
	"go.uber.org/zap"
	"strings"
	"time"
)

func ExportActionRecord(c *gin.Context) {
	req := new(oa.OaMonthReq)
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	// 获取 当前用户的账号信息
	userInfo := login.GetUserInfoFromC(c)

	if req.Month == "" {
		req.Month = holiday.CurrentMonth()
	}

	req.Month = common.Add25ForMonth(req.Month)

	dates := holiday.SplitMonthV1(req.Month, common.YYMMDD)

	holidaysMap, holidaysMapErr := oa_logic.HolidayList(dates)

	if holidaysMapErr != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    holidaysMapErr.Error(),
			Status: e.Failed,
		})
		return
	}

	//fileName := fmt.Sprintf(common.ActionTimeFileName, dates[0], dates[len(dates)-1])
	fileName := fmt.Sprintf(common.ActionTimeFileName, strings.ReplaceAll(dates[0], "-", ""), strings.ReplaceAll(dates[len(dates)-1], "-", ""))
	exportRecordReq := &oa.ExportRecordReq{
		UserId:   userInfo.ID,
		FileName: fileName,
		TaskType: "action_time",
		Status:   2,
		FilePath: "",
	}
	exportRecord, err := service.GrpcOAImpl.CreateExportRecord(context.Background(), exportRecordReq)
	// 创建成功  则 将 导出记录的 id 记录
	if exportRecord.Id != 0 {
		exportRecordReq.Id = exportRecord.Id
	}

	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    e.ErrorCreateExportTask,
			Status: e.Failed,
		})
		return
	}

	go func() {

		isLeader, staffUIDs, authErr := IsCollectionDetailAuth(c)
		if authErr != nil {
			fmt.Println("IsCollectionDetailAuth ", fmt.Sprintf(" v: %+v", authErr.Error()))
			zap.L().Error("IsCollectionDetailAuth ", zap.Any("IsCollectionDetailAuth ", fmt.Sprintf(" v: %+v", authErr.Error())))
			return
		}

		fmt.Println("====================================  StaffUID 0 ====================================   ")
		fmt.Printf("isLeader %+v\n", isLeader)
		fmt.Printf("StaffUID %+v\n", staffUIDs)
		fmt.Println("====================================  StaffUID 0 ====================================   ")

		if isLeader == false {
			//  如果  请求参数中不携带 员工 uid  则 根据权限 获取 员工
			if len(req.StaffUID) == 0 {
				req.StaffUID = staffUIDs
			}
		}
		fmt.Println("====================================  StaffUID 1 ====================================   ")
		fmt.Printf("StaffUID %+v\n", req.StaffUID)
		fmt.Println("====================================  StaffUID 1 ====================================   ")

		/*	// 根据 部门的 ID 导出  需要 剔除 权限以外的 员工
			if req.DepartmentUID != 0 {
				employeeFile, employeeFileErr := service.GrpcEmployeeProvider.CheckEmployeeFilesByDepartmentID(c, &employee.CheckEmployeeFilesByDepartmentIDRequest{
					DepartmentIds: []uint64{uint64(req.DepartmentUID)},
				})
				if employeeFileErr != nil {
					fmt.Println("CheckEmployeeFilesByDepartmentID ", fmt.Sprintf(" v: %+v", employeeFileErr.Error()))
					zap.L().Error("CheckEmployeeFilesByDepartmentID ", zap.Any("CheckEmployeeFilesByDepartmentID ", fmt.Sprintf(" v: %+v", employeeFileErr.Error())))
					failedErr := oa_logic.UpdateExportRecordExportFailed(exportRecordReq)
					if failedErr != nil {
						fmt.Println("UpdateExportRecordExportFailed ", fmt.Sprintf(" v: %+v", authErr.Error()))
						zap.L().Error("UpdateExportRecordExportFailed ", zap.Any("UpdateExportRecordExportFailed ", fmt.Sprintf(" v: %+v", failedErr.Error())))
					}
					return
				}
				req.StaffUID = common.CheckStaffUIDsIn(req.StaffUID, employeeFile.UserIds)
			}*/

		fmt.Println("====================================  StaffUID 2 ====================================   ")
		fmt.Printf("StaffUID %+v\n", req.StaffUID)
		fmt.Println("====================================  StaffUID 2 ====================================   ")

		// 查询  员工档案 信息
		employees, employeesErr := oa_logic.GetAllEmployeesInfoForEmployees(req.StaffUID)
		if employeesErr != nil {
			fmt.Println("GetAllEmployeesInfoForEmployees ", fmt.Sprintf(" v: %+v", employeesErr.Error()))
			zap.L().Error("GetAllEmployeesInfoForEmployees ", zap.Any("GetAllEmployeesInfoForEmployees ", fmt.Sprintf(" v: %+v", employeesErr.Error())))
			failedErr := oa_logic.UpdateExportRecordExportFailed(exportRecordReq)
			if failedErr != nil {
				fmt.Println("UpdateExportRecordExportFailed ", fmt.Sprintf(" v: %+v", failedErr.Error()))
				zap.L().Error("UpdateExportRecordExportFailed ", zap.Any("UpdateExportRecordExportFailed ", fmt.Sprintf(" v: %+v", failedErr.Error())))
			}
			return
		}

		// 根据 req 的 DepartmentName 筛选员工
		employees = oa_logic.FilterEmployeesInfoEmployee(employees, req.StaffName, req.DepartmentUID, req.Month, req.StartDate, req.EndDate)

		// 导出 操作
		url, exportErr := oa_logic.ExportActionRecord(dates, holidaysMap, employees, fileName)
		if exportErr != nil {
			fmt.Println("ExportActionRecord ", fmt.Sprintf(" v: %+v", exportErr.Error()))
			zap.L().Error("ExportActionRecord ", zap.Any("ExportActionRecord ", fmt.Sprintf(" v: %+v", exportErr.Error())))
			failedErr := oa_logic.UpdateExportRecordExportFailed(exportRecordReq)
			if failedErr != nil {
				fmt.Println("UpdateExportRecordExportFailed ", fmt.Sprintf(" v: %+v", failedErr.Error()))
				zap.L().Error("UpdateExportRecordExportFailed ", zap.Any("UpdateExportRecordExportFailed ", fmt.Sprintf(" v: %+v", failedErr.Error())))
			}
			return
		}

		exportRecordReq.FilePath = url
		exportRecordReq.DerivationTime = time.Now().Format("2006-01-02 15:04:05")
		successErr := oa_logic.UpdateExportRecordExportSuccess(exportRecordReq)
		if successErr != nil {
			fmt.Println("UpdateExportRecordExportSuccess ", fmt.Sprintf(" v: %+v", successErr.Error()))
			zap.L().Error("UpdateExportRecordExportSuccess ", zap.Any("UpdateExportRecordExportSuccess ", fmt.Sprintf(" v: %+v", successErr.Error())))
		}
		return
	}()

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Msg:    e.GetMsg(e.SUCCESS),
		Data:   "导出任务已提交，稍后可查看导出结果。",
		Status: e.Ok,
	})

}
