package oa_new

import (
	"errors"
	"fmt"
	"github.com/fonchain_enterprise/fonchain-main/api/employee"
	"github.com/fonchain_enterprise/fonchain-main/api/oa"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/logic"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/login"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/oa_model"
	"github.com/fonchain_enterprise/fonchain-main/pkg/serializer"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/oa_new/model"
	"github.com/fonchain_enterprise/fonchain-main/pkg/utils"
	employee1 "github.com/fonchain_enterprise/fonchain-main/pkg/utils/employee"
	"github.com/fonchain_enterprise/fonchain-main/pkg/utils/excel"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
	"os"
	"strconv"
	"time"
)

func GetLeaveBalanceList(c *gin.Context) {
	var req oa.LeaveBalanceListReq
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	isLeader, employeeIds, err := employee1.IsHaveEmployeeAuth(c)
	if err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}
	if isLeader == false {
		req.EmployeeFileIds = employeeIds
	}
	res, err := service.GrpcOAImpl.GetLeaveBalanceList(c, &req)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	leaveBalanceRes := new(model.LeaveBalanceRes)

	leaveBalanceRes.LeaveBalanceList = res.Data
	leaveBalanceRes.Msg = res.Msg
	leaveBalanceRes.Total = res.Total
	leaveBalanceRes.LeaveBalanceList = res.Data
	leaveBalanceRes.Page = req.Page
	leaveBalanceRes.PageSize = req.PageSize

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   leaveBalanceRes,
		Status: e.Ok,
	})
}
func GetLeaveBalanceInfo(c *gin.Context) {
	var req oa.LeaveBalanceInfoReq
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	userInfo := login.GetUserInfoFromC(c)
	req.StaffUID = userInfo.ID
	res, err := service.GrpcOAImpl.GetLeaveBalanceInfo(c, &req)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})
	return
}
func CreateLeaveBalanceExcel(c *gin.Context) {
	file, err := c.FormFile("file")
	if err != nil {
		service.Error(c, e.Error, err)
	}
	safeFilename := fmt.Sprintf("%d", time.Now().Unix()) + "_" + file.Filename
	userInfo := login.GetUserInfoFromC(c)
	fileDir := fmt.Sprintf("/balance/excel/%s/%s_%s", time.Now().Format("2006-01-02"), userInfo.NickName, safeFilename)
	// 保存上传的文件到本地
	if err = c.SaveUploadedFile(file, safeFilename); err != nil {
		service.Error(c, e.Error, err)
		return
	}
	defer func() {
		if err := os.Remove(safeFilename); err != nil {
			// 处理删除文件失败的情况
			fmt.Println("Failed to delete file:", err)
		}
	}()
	inputUrl, err := service.PutBosWithName(safeFilename, false, fileDir)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}
	employeeList, err := logic.ImportEmployee(c, safeFilename)
	if len(employeeList) == 0 {
		service.Error(c, e.Error, errors.New("未获取到数据"))
		return
	}
	jobNumList := []string{}
	for _, i := range employeeList {
		jobNumList = append(jobNumList, i.JobNumber)
	}
	employeeIdList, err := service.GrpcEmployeeProvider.FindEmployeeFileIDForJobNum(c, &employee.FindEmployeeFileIDForJobNumRequest{JobNumber: jobNumList})
	if err != nil || len(employeeIdList.List) == 0 {
		service.Error(c, e.Error, errors.New("员工档案信息不存在"))
		return
	}
	employeeFileIds := []uint64{}
	for _, i := range employeeIdList.List {
		employeeFileIds = append(employeeFileIds, i.EmployeeFileId)
	}
	employeeInfo, err := service.GrpcEmployeeProvider.FindEmployeeFileList(c, &employee.FindEmployeeFileListRequest{Ids: employeeFileIds})
	if err != nil {
		service.Error(c, e.Error, errors.New("获取员工档案信息错误"))
		return
	}
	errLeaveBalanceRes := []*oa_model.ErrLeaveBalanceRes{}
	createLeaveBalanceReq := []*oa.LeaveBalance{}
	userMap := make(map[string]*employee.FindEmployeeFileListInfo)
	for _, user := range employeeInfo.EmployeeFileList {
		userMap[user.JobNumber] = user
	}
	for _, employeeReq := range employeeList {
		if user, exists := userMap[employeeReq.JobNumber]; exists {
			departmentPosition := []*oa.LeaveBalancePosition{}
			if user.DepartmentPostList != nil {
				for _, s := range user.DepartmentPostList {
					departmentPosition = append(departmentPosition, &oa.LeaveBalancePosition{
						DepartmentUID:  strconv.FormatUint(s.DepartmentId, 10),
						DepartmentName: s.DepartmentName,
						PositionUID:    strconv.FormatUint(s.PostId, 10),
						PositionName:   s.PostName,
					})
				}
			} else {
				errLeaveBalanceRes = append(errLeaveBalanceRes, &oa_model.ErrLeaveBalanceRes{
					Name:      user.Name,
					JobNumber: user.JobNumber,
					Remark:    "用户无部门岗位信息",
				})
			}
			createLeaveBalanceReq = append(createLeaveBalanceReq, &oa.LeaveBalance{
				StaffUID:       user.UserId,
				StaffNum:       user.JobNumber,
				StaffName:      user.Name,
				EmployeeFileId: user.Id,
				Year:           int32(time.Now().Year()),
				Positions:      departmentPosition,
				AnnualLeave:    logic.NewEmployeeWithDefaults(employeeReq.AnnualLeave),
				//Sick:           logic.NewEmployeeWithDefaults(employeeReq.SickLeave),
				ParentalLeave: logic.NewEmployeeWithDefaults(employeeReq.ParentalLeave),
				NursingLeave:  logic.NewEmployeeWithDefaults(employeeReq.NursingLeave),
			})
		} else {
			errLeaveBalanceRes = append(errLeaveBalanceRes, &oa_model.ErrLeaveBalanceRes{
				Name:      employeeReq.Name,
				JobNumber: employeeReq.JobNumber,
				Remark:    "员工档案信息不存在",
			})
		}

	}
	Ids := ""
	if len(createLeaveBalanceReq) > 0 {
		for _, i := range createLeaveBalanceReq {
			_, err = service.GrpcOAImpl.SaveLeaveBalance(c, i)
			if err != nil {
				errLeaveBalanceRes = append(errLeaveBalanceRes, &oa_model.ErrLeaveBalanceRes{
					Name:      i.StaffName,
					JobNumber: i.StaffNum,
					Remark:    fmt.Sprintf("创建假勤错误-%v", err.Error()),
				})
			}
			Ids += strconv.FormatUint(i.EmployeeFileId, 10) + ";"
		}
	}

	outUrl := ""
	Status := uint64(0)
	if len(errLeaveBalanceRes) > 0 {
		Status = 1
		var sheet1Columns = []string{"姓名", "工号", "备注"}
		var sheet1 = excel.NewSheet("Sheet1", errLeaveBalanceRes, sheet1Columns)
		ex, err := excel.NewExcelCreatorFromTemplate("demo.xlsx", "./", "", sheet1)
		ex.UseOption(excel.OptionFileNameSuffixWithUnixTime)
		path, _, err := ex.WriteToFile()
		outUrl, err = service.PutBos(path, "", false)
		if err != nil {
			service.Error(c, e.Error, err)
			return
		}
		defer func() {
			os.Remove(path)
		}()
	}
	//创建导入记录
	_, err = service.GrpcEmployeeProvider.CreateImportRecord(c, &employee.CreateImportRecordRequest{
		EmployeeFiles: Ids,
		Status:        Status,
		UserId:        userInfo.ID,
		ToLeadUrl:     inputUrl,
		DeriveUrl:     outUrl,
		Type:          "leaveBalance",
	})
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}
	service.Success(c, outUrl)
	return
}
func DownloadLeaveBalanceExcel(c *gin.Context) {
	var req oa.LeaveBalanceListReq
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	isLeader, employeeIds, err := employee1.IsHaveEmployeeAuth(c)
	if err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}
	if isLeader == false {
		req.EmployeeFileIds = employeeIds
	}
	res, err := service.GrpcOAImpl.GetLeaveBalanceList(c, &req)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	employeeFileIds := []uint64{}
	for _, i := range res.Data {
		employeeFileIds = append(employeeFileIds, i.EmployeeFileId)
	}
	employeeList, err := service.GrpcEmployeeProvider.QueryBasicInfoAboutEmployees(c, &employee.QueryBasicInfoAboutEmployeesRequest{
		EmployeeFileId: employeeFileIds,
	})
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	// 使用 map 来加速匹配
	employeeInfoMap := make(map[uint64]*employee.QueryBasicInfoAboutEmployeesInfo)
	for _, emp := range employeeList.List {
		employeeInfoMap[emp.EmployeeFileId] = emp
	}
	titleList := []string{
		"员工姓名", "工号", "状态", "部门", "岗位", "年假(天)", "育儿假(天)", "独生子女护理假(天)",
	}
	var dataList []interface{}
	for _, item := range res.Data {
		var status string
		if item.WorkingConditionId == 80 {
			status = "在职"
		} else if item.WorkingConditionId == 81 {
			status = "离职"
		} else {
			status = "未知"
		}
		if emp, found := employeeInfoMap[item.EmployeeFileId]; found {
			data := []any{
				item.StaffName,
				item.StaffNum,
				status,
				emp.DepartmentNames,
				emp.PositionNames,
				item.AnnualLeave,
				//item.Sick,
				item.ParentalLeave,
				item.NursingLeave,
			}
			dataList = append(dataList, &data)
		}
	}
	content, err := utils.ToExcelByType(titleList, dataList, "slice", "")
	if err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}
	utils.ResponseXls(c, content, "假勤余额")
	return
}

func BalanceUpdateAnnual(c *gin.Context) {
	var req oa.UpdateLeaveBalance
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	res, err := service.GrpcOAImpl.UpdateAnnualLeave(c, &req)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})

}
