package oa_new

import (
	"context"
	"dubbo.apache.org/dubbo-go/v3/common/logger"
	"fmt"
	"github.com/fonchain_enterprise/fonchain-main/api/oa"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/login"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/oa_model"
	"github.com/fonchain_enterprise/fonchain-main/pkg/serializer"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/oa_new/common"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/oa_new/model"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/oa_new/oa_logic"
	"github.com/fonchain_enterprise/fonchain-main/pkg/utils/holiday"
	"github.com/gin-gonic/gin"
	"sort"
	"strings"
	"time"
)

// MakeUpNum
// 查询 已补卡 及 剩余补卡次数
func MakeUpNum(c *gin.Context) {
	req := oa_model.MakeUpNumReq{}
	if err := c.ShouldBind(&req); err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	logger.Infof("MakeUpNumReq req: %+v", req)

	userInfo := login.GetUserInfoFromC(c)

	// 查询 考勤信息
	workingTime, workingTimeErr := oa_logic.QueryWorkingTimeInUse(userInfo.PositionUsers, userInfo.ID)
	if workingTimeErr != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    workingTimeErr.Error(),
			Status: e.Failed,
		})
		return
	}

	logger.Infof("workingTime: %+v\n", workingTime)
	fmt.Printf("workingTime: %+v\n", workingTime)

	makeUpRule := new(oa.MakeUp)
	// 获取 设置的 补卡次数
	for i := 0; i < len(workingTime.OaRules); i++ {
		if workingTime.OaRules[i].Keyword == model.MakeUpRule && workingTime.OaRules[i].MakeUpRule != nil {
			makeUpRule = workingTime.OaRules[i].MakeUpRule
			break
		}
	}

	if makeUpRule.Num == 0 {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    e.ErrMissMakeUpRule,
			Status: e.Failed,
		})
		return
	}

	fmt.Println("================================= makeUpRule =============================================")
	fmt.Printf("makeUpRule: %v\n", makeUpRule)
	fmt.Println("================================= makeUpRule =============================================")

	// 获取当前考勤周期
	month := ""
	if req.Month == "" {
		month = common.CurrentMonth()
	} else {
		month = req.Month
	}

	month = common.Add25ForMonth(month)

	dates := holiday.SplitMonthV1(month, common.YYMMDD)

	var queryMakeUpDates []string

	//  暂时 使用 当前周期的 日期
	queryMakeUpDates = dates

	//if makeUpRule.AttendanceCycle == model.AttendanceCycleYes {
	//	queryMakeUpDates = dates
	//} else if makeUpRule.AttendanceCycle == model.AttendanceCycleNo {
	//	datesMap := make(map[string]string)
	//	for i := 0; i < len(dates); i++ {
	//		datesMap[dates[i]] = dates[i]
	//	}
	//
	//	currentDate := common.CurrentDateTime()
	//	for i := int(makeUpRule.PriorDate) - 1; i > 0; i-- {
	//		addDate := currentDate.AddDate(0, 0, -i).Format(common.YYMMDD)
	//		if _, ok := datesMap[addDate]; ok {
	//			queryMakeUpDates = append(queryMakeUpDates, addDate)
	//		}
	//	}
	//
	//	if _, ok := datesMap[currentDate.Format(common.YYMMDD)]; ok {
	//		queryMakeUpDates = append(queryMakeUpDates, currentDate.Format(common.YYMMDD))
	//	}
	//}

	fmt.Println("======================================== queryMakeUpDates   ======================================================")
	fmt.Printf(" queryMakeUpDates %+v\n", queryMakeUpDates)
	fmt.Println("======================================== queryMakeUpDates   ======================================================")

	// 查询 oa审批
	oaApplys, oaApplyErr := oa_logic.QueryOaApplyDates(userInfo.ID, queryMakeUpDates, []int32{e.ApprovalWorkStatusDoing, e.ApprovalWorkStatusOk}, []string{oa_model.TypeMakeUp})
	if oaApplyErr != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    oaApplyErr.Error(),
			Status: e.Failed,
		})
		return
	}

	temp := "%s - %s 已申请 %v 次补卡，剩余 %v 次"
	res := oa_model.MakeUpNumRes{}
	res.Num = makeUpRule.Num - int64(len(oaApplys))
	limit := 0
	if res.Num > 0 {
		limit = int(res.Num)
	}
	res.Title = fmt.Sprintf(temp, queryMakeUpDates[0], queryMakeUpDates[len(queryMakeUpDates)-1], len(oaApplys), limit)

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})
}

func MissRecordForMakeUp(c *gin.Context) {
	req := new(oa.OtherRecordReq)

	userInfo := login.GetUserInfoFromC(c)

	// 查询 考勤信息
	workingTime, workingTimeErr := oa_logic.QueryWorkingTimeInUse(userInfo.PositionUsers, userInfo.ID)
	if workingTimeErr != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    workingTimeErr.Error(),
			Status: e.Failed,
		})
		return
	}

	logger.Infof("workingTime: %+v", workingTime)
	fmt.Printf("workingTime: %+v", workingTime)

	makeUpRule := new(oa.MakeUp)
	// 获取 设置的 补卡次数
	for i := 0; i < len(workingTime.OaRules); i++ {
		if workingTime.OaRules[i].Keyword == model.MakeUpRule && workingTime.OaRules[i].MakeUpRule != nil {
			makeUpRule = workingTime.OaRules[i].MakeUpRule
			break
		}
	}

	if makeUpRule.Num == 0 {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    e.ErrMissMakeUpRule,
			Status: e.Failed,
		})
		return
	}

	fmt.Println("================================= makeUpRule =============================================")
	fmt.Printf("makeUpRule: %v\n", makeUpRule)
	fmt.Println("================================= makeUpRule =============================================")

	// 获取当前考勤周期
	month := common.CurrentMonth()

	month = common.Add25ForMonth(month)

	dates := holiday.SplitMonthV1(month, common.YYMMDD)

	var queryMakeUpDates []string

	if makeUpRule.AttendanceCycle == model.AttendanceCycleYes {
		queryMakeUpDates = dates
	} else if makeUpRule.AttendanceCycle == model.AttendanceCycleNo {
		fmt.Println("================================= AttendanceCycleNo =============================================")
		datesMap := make(map[string]string)
		for i := 0; i < len(dates); i++ {
			datesMap[dates[i]] = dates[i]
		}

		currentDate := common.CurrentDateTime()
		for i := int(makeUpRule.PriorDate) - 1; i > 0; i-- {
			addDate := currentDate.AddDate(0, 0, -i).Format(common.YYMMDD)
			fmt.Println("================================= addDate =============================================")
			fmt.Printf("addDate: %v\n", addDate)
			fmt.Println("================================= addDate =============================================")
			if _, ok := datesMap[addDate]; ok {
				queryMakeUpDates = append(queryMakeUpDates, addDate)
			}
		}

		if _, ok := datesMap[currentDate.Format(common.YYMMDD)]; ok {
			queryMakeUpDates = append(queryMakeUpDates, currentDate.Format(common.YYMMDD))
		}
	}

	// 添加 当前考勤周期的 前一天
	//currentDate := common.CurrentDateTime()
	if queryMakeUpDates[0] == common.CurrentDateStr() {
		addDate := common.PreDate().Format(common.YYMMDD)
		if !common.IsInArray(addDate, dates) {
			queryMakeUpDates = append([]string{addDate}, queryMakeUpDates...)
		}
	}

	fmt.Println("================================= queryMakeUpDates =============================================")
	fmt.Printf("queryMakeUpDates: %v\n", queryMakeUpDates)
	fmt.Println("================================= queryMakeUpDates =============================================")

	req.StartDate = queryMakeUpDates[len(queryMakeUpDates)-len(queryMakeUpDates)]
	req.EndDate = queryMakeUpDates[len(queryMakeUpDates)-1]

	req.ActionType = []string{oa_model.TypeMiss}
	req.Page = 1
	req.PageSize = 9999
	req.StaffUID = userInfo.ID

	res, err := service.GrpcOAImpl.OtherRecord(context.Background(), req) // 查询 缺卡记录

	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	miss := make([]string, 0)
	missMap := make(map[string]string, 0)
	timeOptionsMap := make(map[string][]*oa.TimeOption, 0)
	//  整理 缺卡 记录
	records := make([]*oa.OAProfile, 0)
	for i := 0; i < len(res.Data); i++ {
		miss = append(miss, strings.Join([]string{res.Data[i].WorkDate, res.Data[i].WorkTime}, " "))
		records = append(records, res.Data[i])
		missMap[strings.Join([]string{res.Data[i].WorkDate, res.Data[i].WorkTime}, " ")] = strings.Join([]string{res.Data[i].WorkDate, res.Data[i].WorkTime}, " ")
		timeOptionsMap[res.Data[i].WorkDate] = res.Data[i].Time
	}

	sort.Strings(miss)

	fmt.Println("================================   miss  for  makeUp =======================================")
	fmt.Printf("missMap : %+v \n", missMap)
	fmt.Printf("timeOptionsMap : %+v \n", timeOptionsMap)
	fmt.Printf("records : %+v \n", records)
	fmt.Println("================================   miss  for  makeUp =======================================")

	// oa 申请  请假  加班  调休
	applyReq := new(oa.ApplyRecordReq)
	applyReq.StaffUID = userInfo.ID
	applyReq.ApplyType = []string{
		oa_model.TypeLeave, // 请假
		oa_model.TypeSick,
		oa_model.TypeAnnualLeave,
		oa_model.TypeMaritalLeave,
		oa_model.TypeMatingCheckLeave,
		oa_model.TypeMaternityLeave,
		oa_model.TypePaternityLeave,
		oa_model.TypeParentalLeave,
		oa_model.TypeNursingLeave,
		oa_model.TypeFuneralLeave,
		oa_model.TypeDayOff, // 调休
		//oa_model.TypeOverTime, // 加班
		//oa_model.TypeOutWork, // 外勤
		oa_model.TypeMakeUp, // 补卡
	}
	applyReq.ApplyStatus = []int32{e.ApprovalWorkStatusDoing, e.ApprovalWorkStatusOk}
	applyReq.BeginTime = queryMakeUpDates[len(queryMakeUpDates)-len(queryMakeUpDates)]
	applyReq.EndTime = queryMakeUpDates[len(queryMakeUpDates)-1]
	applyReq.Page = 1
	applyReq.PageSize = 1000
	apply, err := service.GrpcOAImpl.QueryOaApply(context.Background(), applyReq)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	fmt.Println("===============================   MissRecordForMakeUp  =======================================")
	fmt.Printf("apply %+v\n", apply)
	fmt.Println("===============================   MissRecordForMakeUp  =======================================")

	if len(apply.Data) == 0 {
		goto end
	}

	for i := 0; i < len(records); i++ {
		missTime, _ := time.ParseInLocation("2006-01-02 15:04:05", records[i].WorkDate+" "+records[i].WorkTime+":00", time.Local)
		for j := 0; j < len(apply.Data); j++ {
			//   判断  缺卡的 日期 在 申请的 日期内
			if apply.Data[j].ApplyType == oa_model.TypeMakeUp {
				if apply.Data[j].ApplyTimes[len(apply.Data[j].ApplyTimes)-len(apply.Data[j].ApplyTimes)].Date == records[i].WorkDate && apply.Data[j].ApplyTimes[len(apply.Data[j].ApplyTimes)-len(apply.Data[j].ApplyTimes)].Hour == records[i].WorkTime {
					missMap[strings.Join([]string{records[i].WorkDate, records[i].WorkTime}, " ")] = strings.Join([]string{res.Data[i].WorkDate, res.Data[i].WorkTime}, " ")
				}
			} else {
				if len(apply.Data[j].ApplyTimes) == 2 {
					var (
						startTime time.Time
						endTime   time.Time
					)
					if apply.Data[j].ApplyTimes[0].Hour != "" {
						startTime, _ = time.ParseInLocation("2006-01-02 15:04:05", apply.Data[j].ApplyTimes[0].Date+" "+apply.Data[j].ApplyTimes[0].Hour+":00", time.Local)
						endTime, _ = time.ParseInLocation("2006-01-02 15:04:05", apply.Data[j].ApplyTimes[1].Date+" "+apply.Data[j].ApplyTimes[1].Hour+":00", time.Local)
						if !startTime.After(missTime) && !endTime.Before(missTime) {
							missMap[strings.Join([]string{records[i].WorkDate, records[i].WorkTime}, " ")] = strings.Join([]string{records[i].WorkDate, records[i].WorkTime}, " ")
						}
					} else {
						if timeOptionsMap[apply.Data[j].ApplyTimes[0].Date] == nil && timeOptionsMap[apply.Data[j].ApplyTimes[1].Date] == nil {
							missMap[strings.Join([]string{records[i].WorkDate, records[i].WorkTime}, " ")] = strings.Join([]string{records[i].WorkDate, records[i].WorkTime}, " ")
							continue
						}

						var (
							startHour      string
							endHour        string
							applyStartDate string
							applyEndDate   string
						)
						if timeOptionsMap[apply.Data[j].ApplyTimes[0].Date] == nil {
							applyStartDate = req.StartDate
						} else {
							applyStartDate = apply.Data[j].ApplyTimes[0].Date
						}

						if timeOptionsMap[apply.Data[j].ApplyTimes[1].Date] == nil {
							applyEndDate = req.EndDate
						} else {
							applyEndDate = apply.Data[j].ApplyTimes[1].Date
						}
						if apply.Data[j].ApplyTimes[0].M == "上午" {
							startHour = timeOptionsMap[applyStartDate][0].OnWorkTime
						} else {
							if len(timeOptionsMap[applyStartDate]) == 1 {
								startHour = "12:00"
							} else if len(timeOptionsMap[applyStartDate]) == 2 {
								startHour = timeOptionsMap[applyStartDate][1].OnWorkTime
							}
						}

						if apply.Data[j].ApplyTimes[1].M == "上午" {
							if len(timeOptionsMap[applyStartDate]) == 1 {
								endHour = "12:00"
							} else if len(timeOptionsMap[applyStartDate]) == 2 {
								endHour = timeOptionsMap[applyStartDate][0].OffWorkTime
							}
						} else {
							if len(timeOptionsMap[applyEndDate]) == 1 {
								endHour = timeOptionsMap[applyEndDate][0].OffWorkTime
							} else if len(timeOptionsMap[applyEndDate]) == 2 {
								endHour = timeOptionsMap[applyEndDate][1].OffWorkTime
							}
						}
						startTime, _ = time.ParseInLocation("2006-01-02 15:04:05", apply.Data[j].ApplyTimes[0].Date+" "+startHour+":00", time.Local)
						endTime, _ = time.ParseInLocation("2006-01-02 15:04:05", apply.Data[j].ApplyTimes[1].Date+" "+endHour+":00", time.Local)
						if !startTime.After(missTime) && !endTime.Before(missTime) {
							missMap[strings.Join([]string{records[i].WorkDate, records[i].WorkTime}, " ")] = strings.Join([]string{records[i].WorkDate, records[i].WorkTime}, " ")
						}
					}
				}
			}
		}
	}

end:

	newRes := new(oa.MissRecordForMakeUpRes)
	newRes.MissRecord = make([]string, 0)

	// 排序
	for i := 0; i < len(miss); i++ {
		if missMap[miss[i]] != "" {
			newRes.MissRecord = append(newRes.MissRecord, missMap[miss[i]])
		}
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Msg:    newRes.Msg,
		Data:   newRes,
		Status: e.Ok,
	})

}
