package model

type CollectionDetailReq struct {
	StaffUID       uint64   `json:"staffUID"`       // 员工ID
	Month          string   `json:"month"`          // 月份
	Date           string   `json:"date"`           // 某天   2006-01-02
	Dates          []string `json:"dates"`          // 某些天  2006-01-02
	CollectionType uint64   `json:"collectionType"` // 统计的类型
}

// collectionType   统计的类型
// 1 出勤详情 2 平均工时  3 休息天数  4 缺卡详情  5 补卡详情  6 迟到详情  7 早退详情  8 旷工详情  9 外勤详情  10 加班详情  11 请假详情  12 调休详情  13 年假详情  14 病假详情  15 哺乳假详情
// 16 陪产假详情  17 婚假详情  18 丧假详情   19 产假详情  20 外出详情  21 出差详情  22 全部请假详情   23 全部 24 育儿假详情 25 独生子女护理假详情  26 流产假   27 孕检假
const (
	WorkDay            = iota + 1 // 出勤详情
	AverageHour                   // 平均工时
	RestDay                       // 休息天数
	Miss                          // 缺卡详情
	MakeUp                        // 补卡详情
	Late                          // 迟到详情
	Before                        // 早退详情
	AbsentDay                     // 旷工详情
	OutWork                       // 外勤详情
	OverTime                      // 加班详情
	Leave                         // 请假详情
	DayOff                        // 调休详情
	AnnualLeave                   // 年假详情
	Sick                          // 病假详情
	BreastFeedingLeave            // 哺乳假详情
	PaternityLeave                // 陪产假详情
	MaritalLeave                  // 婚假详情
	FuneralLeave                  // 丧假详情
	MaternityLeave                // 产假详情
	GoOut                         // 外出详情
	BusinessTrip                  // 出差详情
	AllLeave                      // 全部请假详情
	All
	ParentalLeave    // 育儿假详情
	NursingLeave     // 独生子女护理假详情
	AbortLeave       // 流产假
	MatingCheckLeave // 孕检假
)

type CollectionDetailRes struct {
	StaffUID             uint64               `json:"staffUID"`             // 员工ID
	StaffName            string               `json:"staffName"`            // 员工姓名
	StaffNum             string               `json:"staffNum"`             // 员工编号
	DepartmentName       string               `json:"departmentName"`       // 部门名称
	DepartmentUID        int64                `json:"departmentUID"`        // 部门ID
	PositionName         string               `json:"positionName"`         // 职位名称
	PositionUID          int64                `json:"positionUID"`          // 职位ID
	AttendanceGroupName  string               `json:"attendanceGroupName"`  // 考勤组名称
	Status               bool                 `json:"status"`               // 考勤状态
	Month                string               `json:"month"`                //  月份
	CollectionType       uint64               `json:"collectionType"`       // 统计的类型
	NeedAttendanceDayNum int                  `json:"needAttendanceDayNum"` // 应出勤天数
	AttendanceDayNum     int                  `json:"attendanceDayNum"`     // 出勤天数
	AttendanceRestDayNum int                  `json:"attendanceRestDayNum"` // 休息日出勤天数
	AttendanceHolidayNum int                  `json:"attendanceHolidayNum"` // 节假日出勤天数
	AttendanceWorkDayNum int                  `json:"attendanceWorkDayNum"` // 工作日出勤天数
	WorkDay              []*WorkDayDetail     `json:"workDay"`              // 出勤详情
	TotalHours           float64              `json:"totalHours"`           // 总工时
	AverageHourNum       float64              `json:"averageHourNum"`       // 平均工时
	AverageHour          []*AverageHourDetail `json:"averageHour"`          // 平均工时详情
	RestDayNum           int                  `json:"restDayNum"`           // 休息天数
	RestDay              []*RestDayDetail     `json:"restDay"`              // 休息天数详情
	MissNum              int                  `json:"missNum"`              // 缺卡次数
	Miss                 []*MissDetail        `json:"miss"`                 // 缺卡详情
	MakeUpNum            int                  `json:"makeUpNum"`            // 补卡次数
	//MakeUpDoingNum             int                  `json:"makeUpDoingNum"`             // 审批中补卡次数
	MakeUp                     []*MakeUpDetail   `json:"makeUp"`                     // 补卡详情
	LateNum                    int               `json:"lateNum"`                    // 迟到次数
	CommonLateNum              int               `json:"commonLateNum"`              // 普通迟到次数
	CommonLateDuration         float64           `json:"commonLateDuration"`         // 普通迟到时长
	ExtremeLateNum             int               `json:"extremeLateNum"`             // 严重迟到次数
	ExtremeLateDuration        float64           `json:"extremeLateDuration"`        // 严重迟到时长
	Late                       []*LateDetail     `json:"late"`                       // 迟到详情
	BeforeNum                  int               `json:"beforeNum"`                  // 早退次数
	CommonBeforeDuration       float64           `json:"commonBeforeDuration"`       // 普通早退时长
	Before                     []*BeforeDetail   `json:"before"`                     // 早退详情
	MissDayNum                 int               `json:"missDayNum"`                 // 旷工天数
	MissDay                    []*MissDayDetail  `json:"missDay"`                    // 旷工详情
	DayOffNum                  int               `json:"dayOffNum"`                  // 调休次数
	DayOffDoingNum             int               `json:"dayOffDoingNum"`             // 审批中调休次数
	DayOffHours                float64           `json:"dayOffHours"`                // 调休时长
	DayOff                     []*LeaveDetail    `json:"dayOff"`                     // 调休详情
	OverTimeNum                int               `json:"overTimeNum"`                // 加班次数
	OverTimeDoingNum           int               `json:"overTimeDoingNum"`           // 审批中加班次数
	OverTime                   *OverWorkDetail   `json:"overTime"`                   // 加班详情
	OutWorkNum                 int               `json:"outWorkNum"`                 // 外勤次数
	OutWorkDoingNum            int               `json:"outWorkDoingNum"`            // 审批中外勤次数
	OutWork                    []*OutWorkDetail  `json:"outWork"`                    // 外勤详情
	AnnualLeaveNum             float64           `json:"annualLeaveNum"`             // 年假天数
	AnnualLeaveDoingNum        float64           `json:"annualLeaveDoingNum"`        // 审批中年假天数
	AnnualLeave                []*LeaveDetail    `json:"annualLeave"`                // 年假详情
	LeaveNum                   float64           `json:"leaveNum"`                   // 请假天数
	LeaveDoingNum              float64           `json:"leaveDoingNum"`              // 审批中请假天数
	Leave                      []*LeaveDetail    `json:"leave"`                      // 请假详情
	SickDayNum                 float64           `json:"sickDayNum"`                 // 病假天数
	SickDayDoingNum            float64           `json:"sickDayDoingNum"`            // 审批中病假天数
	Sick                       []*LeaveDetail    `json:"sick"`                       // 病假详情
	BreastFeedingLeaveNum      float64           `json:"breastFeedingLeaveNum"`      // 哺乳假时长
	BreastFeedingLeaveDoingNum float64           `json:"breastFeedingLeaveDoingNum"` // 审批中哺乳假时长
	BreastFeedingLeave         []*LeaveDetail    `json:"breastFeedingLeave"`         // 哺乳假详情
	PaternityLeaveNum          int               `json:"paternityLeaveNum"`          // 陪产假天数
	PaternityLeaveDoingNum     int               `json:"paternityLeaveDoingNum"`     // 审批中陪产假天数
	PaternityLeave             []*LeaveDetail    `json:"paternityLeave"`             // 陪产假详情
	MaritalLeaveNum            int               `json:"maritalLeaveNum"`            // 婚假天数
	MaritalLeaveDoingNum       int               `json:"maritalLeaveDoingNum"`       // 审批中婚假天数
	MaritalLeave               []*LeaveDetail    `json:"maritalLeave"`               // 婚假详情
	FuneralLeaveNum            float64           `json:"funeralLeaveNum"`            // 丧假天数
	FuneralLeaveDoingNum       float64           `json:"funeralLeaveDoingNum"`       // 审批中丧假天数
	FuneralLeave               []*LeaveDetail    `json:"funeralLeave"`               // 丧假详情
	MaternityLeaveNum          float64           `json:"maternityLeaveNum"`          // 产假天数
	MaternityLeaveDoingNum     float64           `json:"maternityLeaveDoingNum"`     // 审批中产假天数
	MaternityLeave             []*LeaveDetail    `json:"maternityLeave"`             // 产假详情
	AllLeaveNum                int               `json:"allLeaveNum"`                // 全部请假次数
	AllLeaveDoingNum           int               `json:"allLeaveDoingNum"`           // 审批中全部请假次数
	AllLeaveDetail             []*AllLeaveDetail `json:"allLeaveDetail"`             // 全部请假详情
	AllDayLeaveNum             int               `json:"allDayLeaveNum"`             // 全天请假天数
	AllDayLeave                []*WorkDayDetail  `json:"allDayLeave"`                // 全天请假详情
	GoOutDuration              float64           `json:"goOutDuration"`              // 外出时长
	GoOutDurationDoing         float64           `json:"goOutDurationDoing"`         // 审批中外出时长
	GoOut                      []*LeaveDetail    `json:"goOut"`                      // 外出详情
	BusinessTripDuration       float64           `json:"businessTripDuration"`       // 出差时长
	BusinessTripDurationDoing  float64           `json:"businessTripDurationDoing"`  // 审批中出差时长
	BusinessTrip               []*LeaveDetail    `json:"businessTrip"`               // 出差详情
	ParentalLeaveNum           float64           `json:"parentalLeaveNum"`           // 育儿假天数
	ParentalLeaveDoingNum      float64           `json:"parentalLeaveDoingNum"`      // 审批中育儿假天数
	ParentalLeave              []*LeaveDetail    `json:"parentalLeave"`              // 育儿假详情
	NursingLeaveNum            float64           `json:"nursingLeaveNum"`            // 独生子女护理假天数
	NursingLeaveDoingNum       float64           `json:"nursingLeaveDoingNum"`       // 审批中独生子女护理假天数
	NursingLeave               []*LeaveDetail    `json:"nursingLeave"`               // 独生子女护理假详情
	AbortLeaveNum              float64           `json:"abortLeaveNum"`              // 流产假天数
	AbortLeaveDoingNum         float64           `json:"abortLeaveDoingNum"`         // 审批中流产假天数
	AbortLeave                 []*LeaveDetail    `json:"abortLeave"`                 // 流产假详情
	MatingCheckLeaveNum        float64           `json:"matingCheckLeaveNum"`        // 孕检假天数
	MatingCheckLeaveDoingNum   float64           `json:"matingCheckLeaveDoingNum"`   // 审批中孕检假天数
	MatingCheckLeave           []*LeaveDetail    `json:"matingCheckLeave"`           // 孕检假详情
}

type CollectionEmployeesDetailRes struct {
	StaffUID             uint64               `json:"StaffUID"`             // 员工ID
	StaffName            string               `json:"StaffName"`            // 员工姓名
	StaffNum             string               `json:"StaffNum"`             // 员工编号
	DepartmentName       string               `json:"DepartmentName"`       // 部门名称
	DepartmentUID        int64                `json:"DepartmentUID"`        // 部门ID
	PositionName         string               `json:"PositionName"`         // 职位名称
	PositionUID          int64                `json:"PositionUID"`          // 职位ID
	AttendanceGroupName  string               `json:"AttendanceGroupName"`  // 考勤组名称
	Status               bool                 `json:"Status"`               // 考勤状态
	Month                string               `json:"Month"`                //  月份
	CollectionType       uint64               `json:"CollectionType"`       // 统计的类型
	NeedAttendanceDayNum int                  `json:"NeedAttendanceDayNum"` // 应出勤天数
	WorkDays             int                  `json:"WorkDays"`             // 出勤天数
	AttendanceDayNum     int                  `json:"AttendanceDayNum"`     // 出勤天数
	AttendanceRestDayNum int                  `json:"AttendanceRestDayNum"` // 休息日出勤天数
	AttendanceHolidayNum int                  `json:"AttendanceHolidayNum"` // 节假日出勤天数
	AttendanceWorkDayNum int                  `json:"AttendanceWorkDayNum"` // 工作日出勤天数
	WorkDay              []*WorkDayDetail     `json:"WorkDay"`              // 出勤详情
	TotalHours           float64              `json:"TotalHours"`           // 总工时
	AverageHour          float64              `json:"AverageHour"`          // 平均工时
	AverageHourDetail    []*AverageHourDetail `json:"AverageHourDetail"`    // 平均工时详情
	RestDays             int                  `json:"RestDays"`             // 休息天数
	RestDaysDetail       []*RestDayDetail     `json:"RestDaysDetail"`       // 休息天数详情
	Miss                 int                  `json:"Miss"`                 // 缺卡次数
	MissDetail           []*MissDetail        `json:"MissDetail"`           // 缺卡详情
	MakeUp               int                  `json:"MakeUp"`               // 补卡次数
	//MakeUpDoing               int                  `json:"MakeUpDoing"`               // 审批中补卡次数
	MakeUpDetail              []*MakeUpDetail   `json:"MakeUpDetail"`              // 补卡详情
	Late                      int               `json:"Late"`                      // 迟到次数
	CommonLateNum             int               `json:"CommonLateNum"`             // 普通迟到次数
	CommonLateDuration        float64           `json:"CommonLateDuration"`        // 普通迟到时长
	ExtremeLateNum            int               `json:"ExtremeLateNum"`            // 严重迟到次数
	ExtremeLateDuration       float64           `json:"ExtremeLateDuration"`       // 严重迟到时长
	LateDetail                []*LateDetail     `json:"LateDetail"`                // 迟到详情
	Before                    int               `json:"Before"`                    // 早退次数
	CommonBeforeDuration      float64           `json:"CommonBeforeDuration"`      // 普通早退时长
	BeforeDetail              []*BeforeDetail   `json:"BeforeDetail"`              // 早退详情
	MissDay                   int               `json:"MissDay"`                   // 旷工天数
	MissDayDetail             []*MissDayDetail  `json:"MissDayDetail"`             // 旷工详情
	DayOff                    float64           `json:"DayOff"`                    // 调休时长
	DayOffNum                 int               `json:"DayOffNum"`                 // 调休次数
	DayOffNumDoing            int               `json:"DayOffNumDoing"`            // 审批中调休次数
	DayOffDetail              []*LeaveDetail    `json:"DayOffDetail"`              // 调休详情
	OverWork                  float32           `json:"OverWork"`                  // 加班时长
	OverWorkDoing             float32           `json:"OverWorkDoing"`             // 审批中加班时长
	OverWorkWorkDay           float32           `json:"OverWorkWorkDay"`           // 加班次数
	OverWorkRestDay           float32           `json:"OverWorkRestDay"`           // 加班次数
	OverWorkHoliday           float32           `json:"OverWorkHoliday"`           // 加班次数
	OverWorkDetail            *OverWorkDetail   `json:"OverWorkDetail"`            // 加班详情
	OutWork                   int               `json:"OutWork"`                   // 外勤次数
	OutWorkDoing              int               `json:"OutWorkDoing"`              // 审批中外勤次数
	OutWorkDetail             []*OutWorkDetail  `json:"OutWorkDetail"`             // 外勤详情
	AnnualLeave               float64           `json:"AnnualLeave"`               // 年假天数
	AnnualLeaveDoing          float64           `json:"AnnualLeaveDoing"`          // 审批中年假天数
	AnnualLeaveDetail         []*LeaveDetail    `json:"AnnualLeaveDetail"`         // 年假详情
	Leave                     float64           `json:"Leave"`                     // 请假天数
	LeaveDoing                float64           `json:"LeaveDoing"`                // 审批中请假天数
	LeaveDetail               []*LeaveDetail    `json:"LeaveDetail"`               // 请假详情
	Sick                      float64           `json:"Sick"`                      // 病假天数
	SickDoing                 float64           `json:"SickDoing"`                 // 审批中病假天数
	SickDetail                []*LeaveDetail    `json:"SickDetail"`                // 病假详情
	BreastFeedingLeave        float64           `json:"BreastFeedingLeave"`        // 哺乳假时长
	BreastFeedingLeaveDoing   float64           `json:"BreastFeedingLeaveDoing"`   // 审批中哺乳假时长
	BreastFeedingLeaveDetail  []*LeaveDetail    `json:"BreastFeedingLeaveDetail"`  // 哺乳假详情
	PaternityLeave            int               `json:"PaternityLeave"`            // 陪产假天数
	PaternityLeaveDoing       int               `json:"PaternityLeaveDoing"`       // 审批中陪产假天数
	PaternityLeaveDetail      []*LeaveDetail    `json:"PaternityLeaveDetail"`      // 陪产假详情
	MaritalLeave              int               `json:"MaritalLeave"`              // 婚假天数
	MaritalLeaveDoing         int               `json:"MaritalLeaveDoing"`         // 审批中婚假天数
	MaritalLeaveDetail        []*LeaveDetail    `json:"MaritalLeaveDetail"`        // 婚假详情
	FuneralLeave              float64           `json:"FuneralLeave"`              // 丧假天数
	FuneralLeaveDoing         float64           `json:"FuneralLeaveDoing"`         // 审批中丧假天数
	FuneralLeaveDetail        []*LeaveDetail    `json:"FuneralLeaveDetail"`        // 丧假详情
	MaternityLeave            float64           `json:"MaternityLeave"`            // 产假天数
	MaternityLeaveDoing       float64           `json:"MaternityLeaveDoing"`       // 审批中产假天数
	MaternityLeaveDetail      []*LeaveDetail    `json:"MaternityLeaveDetail"`      // 产假详情
	AllLeave                  int               `json:"AllLeave"`                  // 全部请假次数
	AllLeaveDetail            []*AllLeaveDetail `json:"AllLeaveDetail"`            // 全部请假详情
	AllDayLeaveNum            int               `json:"AllDayLeaveNum"`            // 全天请假天数
	AllDayLeave               []*WorkDayDetail  `json:"AllDayLeave"`               // 全天请假详情
	GoOutDuration             float64           `json:"GoOutDuration"`             // 外出时长
	GoOutDurationDoing        float64           `json:"GoOutDurationDoing"`        // 审批中外出时长
	GoOutDetail               []*LeaveDetail    `json:"GoOutDetail"`               // 外出详情
	BusinessTripDuration      float64           `json:"BusinessTripDuration"`      // 出差时长
	BusinessTripDurationDoing float64           `json:"BusinessTripDurationDoing"` // 审批中出差时长
	BusinessTripDetail        []*LeaveDetail    `json:"BusinessTripDetail"`        // 出差详情
	ParentalLeave             float64           `json:"ParentalLeave"`             // 育儿假天数
	ParentalLeaveDoing        float64           `json:"ParentalLeaveDoing"`        // 审批中育儿假天数
	ParentalLeaveDetail       []*LeaveDetail    `json:"ParentalLeaveDetail"`       // 育儿假详情
	NursingLeave              float64           `json:"NursingLeave"`              // 独生子女护理假天数
	NursingLeaveDoing         float64           `json:"NursingLeaveDoing"`         // 审批中独生子女护理假天数
	NursingLeaveDetail        []*LeaveDetail    `json:"NursingLeaveDetail"`        // 独生子女护理假详情
	AbortLeave                float64           `json:"AbortLeave"`                // 流产假天数
	AbortLeaveDoing           float64           `json:"AbortLeaveDoing"`           // 审批中流产假天数
	AbortLeaveDetail          []*LeaveDetail    `json:"AbortLeaveDetail"`          // 流产假详情
	MatingCheckLeave          float64           `json:"MatingCheckLeave"`          // 孕检假天数
	MatingCheckLeaveDoing     float64           `json:"MatingCheckLeaveDoing"`     // 审批中孕检假天数
	MatingCheckLeaveDetail    []*LeaveDetail    `json:"MatingCheckLeaveDetail"`    // 孕检假详情
}

type WorkDayDetail struct {
	WorkDate    string        `json:"workDate"`    // 日期
	Week        int32         `json:"week"`        // 星期 数字
	Weekly      string        `json:"weekly"`      // 星期
	DateType    int32         `json:"dateType"`    // 日期类型   0 工作日 1 休息日 2 节假日
	Abnormal    bool          `json:"abnormal"`    // 是否异常   false 表示正常   ture  表示异常
	Records     []*DayRecord  `json:"records"`     // 打卡记录
	ActionTypes []string      `json:"actionTypes"` // 打卡类型  迟到 早退 缺卡
	ApplyTypes  []string      `json:"applyTypes"`  // 申请类型  请假 加班 外勤 补卡申请
	IsWork      int8          `json:"isWork"`      // 是否出勤  1 出勤  2 休息
	WorkTimes   []*TimeOption `json:"workTimes"`   // 考勤信息
}

type DayRecord struct {
	ActionType        string         `json:"actionType"`        // 打卡类型
	ApplyInfo         []*ApplyDetail `json:"applyInfo"`         // 申请详情
	ActionTime        string         `json:"actionTime"`        // 实际的打卡时间
	WorkTime          string         `json:"workTime"`          // 设置的打卡时间
	OnOff             string         `json:"onOff"`             // 上下班   上班 on 下班 off
	CheckAddress      string         `json:"checkAddress"`      // 打卡地址
	CheckMethod       int32          `json:"checkMethod"`       // 打卡方式
	FacePhoto         string         `json:"facePhoto"`         // 人脸照片  存放 照片URL
	ClientID          string         `json:"clientID"`          // 使用的手机  存放 手机的唯一标识
	OutWorkPhoto      []string       `json:"outWorkPhoto"`      // 外勤照片  存放 照片URL
	IsWork            int8           `json:"isWork"`            // 是否出勤  1 出勤  2 休息
	IsExtremeLateness bool           `json:"isExtremeLateness"` // 是否严重迟到 true 严重迟到  false 正常
}

type ApplyDetail struct {
	ApplyID       string       `json:"applyID"`       // 申请ID
	ApplyType     string       `json:"applyType"`     // 申请类型
	ApplyTypeZhCN string       `json:"applyTypeZhCN"` // 申请类型 中文
	ApplyStatus   int32        `json:"applyStatus"`   // 申请状态
	ItineraryList []*Itinerary `json:"itineraryList"` // 行程
}

// DateType 日期类型
// 1 工作日  2 节假日  3 周末
const (
	WorkDayType    = iota // 工作日
	WeekendDayType        // 休息日
	HolidayDayType        // 节假日
)

// CheckMethod 打卡方式
// 1 人脸识别  2 手机打卡
const (
	FaceCheckMethod  = iota + 1 // 人脸识别
	PhoneCheckMethod            // 手机打卡
)

type AverageHourDetail struct {
	WorkDate string  `json:"workDate"` // 日期
	Weekly   string  `json:"weekly"`   // 星期
	Hours    float64 `json:"hours"`    // 工时
}

type RestDayDetail struct {
	WorkDate string  `json:"workDate"` // 日期
	Weekly   string  `json:"weekly"`   // 星期
	Day      float64 `json:"day"`      // 休息天数
}

type MissDetail struct {
	WorkDate    string        `json:"workDate"`    // 日期
	Weekly      string        `json:"weekly"`      // 星期
	ActionTime  string        `json:"actionTime"`  // 缺卡时间
	WorkTime    string        `json:"workTime"`    // 设置的打卡时间
	ApplyID     string        `json:"applyID"`     // 申请ID
	ApplyStatus int32         `json:"applyStatus"` // 申请状态
	WorkTimes   []*TimeOption `json:"workTimes"`   // 考勤信息
}

type MakeUpDetail struct {
	WorkDate    string `json:"workDate"`    // 日期
	Weekly      string `json:"weekly"`      // 星期
	WorkTime    string `json:"workTime"`    // 设置的打卡时间
	ActionTime  string `json:"actionTime"`  // 补卡申请时设置的时间
	ApplyID     string `json:"applyID"`     // 申请ID
	ApplyStatus int32  `json:"applyStatus"` // 申请状态
}

type LateDetail struct {
	WorkDate                string  `json:"workDate"`                // 日期
	Weekly                  string  `json:"weekly"`                  // 星期
	WorkTime                string  `json:"workTime"`                // 设置的打卡时间
	ActionTime              string  `json:"actionTime"`              // 迟到时间
	ApplyID                 string  `json:"applyID"`                 // 申请ID
	ApplyStatus             int32   `json:"applyStatus"`             // 申请状态
	Duration                float64 `json:"duration"`                // 迟到时长
	ExtremeLatenessDuration float64 `json:"extremeLatenessDuration"` // 严重迟到时长
	ActionType              string  `json:"actionType"`              // 打卡类型
}

type BeforeDetail struct {
	WorkDate    string  `json:"workDate"`    // 日期
	Weekly      string  `json:"weekly"`      // 星期
	WorkTime    string  `json:"workTime"`    // 设置的打卡时间
	ActionTime  string  `json:"actionTime"`  // 早退时间
	ApplyID     string  `json:"applyID"`     // 申请ID
	ApplyStatus int32   `json:"applyStatus"` // 申请状态
	Duration    float64 `json:"duration"`    // 早退时长
	ActionType  string  `json:"actionType"`  // 打卡类型
}

type MissDayDetail struct {
	WorkDate    string  `json:"workDate"`    // 日期
	Weekly      string  `json:"weekly"`      // 星期
	Day         float64 `json:"day"`         // 旷工天数
	ApplyID     string  `json:"applyID"`     // 申请ID
	ApplyStatus int32   `json:"applyStatus"` // 申请状态
}

type OutWorkDetail struct {
	WorkDate    string        `json:"workDate"`    // 日期
	Weekly      string        `json:"weekly"`      // 星期
	Records     []*DayRecord  `json:"records"`     // 打卡记录
	WorkTimes   []*TimeOption `json:"workTimes"`   // 考勤信息
	ApplyID     string        `json:"applyID"`     // 申请ID
	ApplyStatus int32         `json:"applyStatus"` // 申请状态
}

type LeaveDetail struct {
	ApplyID         string       `json:"applyID"`         // 申请ID
	ApplyType       string       `json:"applyType"`       // 申请类型
	ApplyStatus     int32        `json:"applyStatus"`     // 申请状态
	ApplyStartTime  string       `json:"applyStartTime"`  // 请假开始时间
	StartDayType    int32        `json:"startDayType"`    // 请假开始时间类型
	StartDayWeekly  string       `json:"startDayWeekly"`  // 请假开始时间星期
	StartAbnormal   int32        `json:"startAbnormal"`   // 请假开始时间异常
	ApplyEndTime    string       `json:"applyEndTime"`    // 请假结束时间
	EndDayType      int32        `json:"endDayType"`      // 请假结束时间类型
	EndDayWeekly    string       `json:"endDayWeekly"`    // 请假结束时间星期
	EndAbnormal     int32        `json:"endAbnormal"`     // 请假结束时间异常
	Duration        float64      `json:"duration"`        // 请假时长
	DurationType    int32        `json:"durationType"`    // 请假时长类型   day  hour
	ItineraryList   []*Itinerary `json:"itineraryList"`   // 行程
	Reason          string       `json:"reason"`          // 理由
	StartActionTime string       `json:"startActionTime"` // 异常时间
	EndActionTime   string       `json:"endActionTime"`   // 异常时间
}

type AllLeaveDetail struct {
	LeaveType    string  `json:"leaveType"`
	LeaveNum     float64 `json:"leaveNum"`
	DurationType int32   `json:"durationType"` // 请假时长类型   day  hour
	Leave        []*LeaveDetail
}

// 时长类型
const (
	DurationTypeDay = iota + 1
	DurationTypeHour
)

type OverWorkDetail struct {
	WorkDayHour    float64        `json:"workDayHour"`    // 工作日加班时长
	RestDayHour    float64        `json:"restDayHour"`    // 休息日加班时长
	HolidayHour    float64        `json:"holidayHour"`    // 节假日加班时长
	OverWorkHour   float64        `json:"overWorkHour"`   // 总加班时长
	OverWorkDetail []*LeaveDetail `json:"overWorkDetail"` // 加班详情
}

const (
	IsWork = iota + 1
	IsFree
)

// 是否有休息时间
const (
	NotHasBreakTime = iota + 1
	HasBreakTime
)

// 休息时间去除规则
const (
	// RemovingBreakPeriodsNo 不去除休息时间
	RemovingBreakPeriodsNo = 1
	// RemovingBreakPeriodsYes 去除休息时间
	RemovingBreakPeriodsYes = 2
)

type TimeOption struct {
	OnWorkTime  string `json:"onWorkTime"`
	OffWorkTime string `json:"offWorkTime"`
}

// Itinerary 行程
type Itinerary struct {
	Vehicle        string  `json:"vehicle"`
	StartAddress   string  `json:"startAddress"`
	EndAddress     string  `json:"endAddress"`
	ApplyStartTime string  `json:"applyStartTime"` // 请假开始时间
	StartDayType   int32   `json:"startDayType"`   // 请假开始时间类型
	StartDayWeekly string  `json:"startDayWeekly"` // 请假开始时间星期
	StartAbnormal  int32   `json:"startAbnormal"`  // 请假开始时间异常
	ApplyEndTime   string  `json:"applyEndTime"`   // 请假结束时间
	EndDayType     int32   `json:"endDayType"`     // 请假结束时间类型
	EndDayWeekly   string  `json:"endDayWeekly"`   // 请假结束时间星期
	EndAbnormal    int32   `json:"endAbnormal"`    // 请假结束时间异常
	Days           float32 `json:"days"`
}

// Abnormal 异常
const (
	IsAbnormal = iota + 1
	NotAbnormal
)
