package oa_new

import (
	"github.com/fonchain_enterprise/fonchain-main/api/oa"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/serializer"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/oa_new/model"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
)

// QueryVersionHis 查询发版信息
func QueryVersionHis(c *gin.Context) {
	var req model.Version
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	his := new(oa.VersionHisReq)
	res, err := service.GrpcOAImpl.QueryVersionHis(c, his)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})
}

// SaveVersionHis 查询发版信息
func SaveVersionHis(c *gin.Context) {
	var req model.Version
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	his := oa.VersionHis{
		Describe:       req.Describe,
		EditionUrl:     req.EditionUrl,
		EditionForce:   req.EditionForce,
		PackageType:    req.PackageType,
		EditionIssue:   req.EditionIssue,
		EditionNumber:  req.EditionNumber,
		EditionName:    req.EditionName,
		EditionSilence: req.EditionSilence,
		VersionType:    req.VersionType,
	}

	res, err := service.GrpcOAImpl.SaveVersionHis(c, &his)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})
}

func QueryVersionHisList(c *gin.Context) {
	var req model.Version
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	his := new(oa.VersionHisListReq)
	res, err := service.GrpcOAImpl.QueryVersionHisList(c, his)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	list := new(model.VersionList)
	list.Total = res.Total
	list.Msg = res.Msg
	list.List = make([]*model.Version, 0)
	for i := 0; i < len(res.Data); i++ {
		list.List = append(list.List, &model.Version{
			Describe:       res.Data[i].Describe,
			EditionUrl:     res.Data[i].EditionUrl,
			EditionForce:   res.Data[i].EditionForce,
			PackageType:    res.Data[i].PackageType,
			EditionIssue:   res.Data[i].EditionIssue,
			EditionNumber:  res.Data[i].EditionNumber,
			EditionName:    res.Data[i].EditionName,
			EditionSilence: res.Data[i].EditionSilence,
			VersionType:    res.Data[i].VersionType,
		})
	}

	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   res,
		Status: e.Ok,
	})
}
