package ocr

import (
	"encoding/base64"
	"errors"
	"fmt"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/response"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/utils"
	baiduUtils "github.com/fonchain_enterprise/utils/baidu"
	"github.com/gin-gonic/gin"
	"github.com/google/uuid"
	"github.com/jinzhu/copier"
	"go.uber.org/zap"
	"os"
	"path"
)

func CardInfo(ctx *gin.Context) {
	var err error
	file, err := ctx.FormFile("file")
	side := ctx.PostForm("side")
	source := ctx.PostForm("source")
	mask := ctx.PostForm("mask")
	typeS := ctx.PostForm("type")
	if file == nil {
		err = errors.New(e.GetMsg(e.InvalidParams))
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	if source == "" || mask == "" || typeS == "" || side == "" {
		err = errors.New(e.GetMsg(e.InvalidParams))
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	uuidNew := uuid.NewString()
	imgPath := fmt.Sprintf("%s/%s/%s/%s", "./runtime", source, typeS, mask)
	_, _ = utils.CheckDirPath(imgPath, true)
	dst := fmt.Sprintf("%s/%s%s", imgPath, uuidNew, path.Ext(file.Filename))
	err = ctx.SaveUploadedFile(file, dst)
	fileByte, err := os.ReadFile(dst)
	defer os.Remove(dst)
	btBase64 := base64.StdEncoding.EncodeToString(fileByte)
	result, err := baiduUtils.OcrGetIdCard(btBase64, side)
	zap.L().Info("result", zap.Any("result", result))
	if err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, err.Error(), nil)
		return
	}
	if side == "front" {
		if result.IdCard == "" {
			service.ResponseQuickMsg(ctx, e.Failed, e.ERROR_CARDID_OCR, nil)
			return
		}
	} else {
		if result.ExpireEndTime == "" {
			service.ResponseQuickMsg(ctx, e.Failed, e.ERROR_CARDID_OCR, nil)
			return
		}
	}
	url, _err := service.PutBos(dst, "image", true)
	if _err != nil {
		service.ResponseQuickMsg(ctx, e.Failed, _err.Error(), nil)
		return
	}
	var resp response.OrcInfoRes
	_ = copier.Copy(&resp, result)
	resp.Url = url
	service.ResponseQuickMsg(ctx, e.Ok, e.GetMsg(e.SUCCESS), resp)
	return
}
