package order

import (
	"errors"
	"fmt"
	"github.com/fonchain_enterprise/fonchain-main/api/account"
	"github.com/fonchain_enterprise/fonchain-main/api/department"
	"github.com/fonchain_enterprise/fonchain-main/api/order"
	"github.com/fonchain_enterprise/fonchain-main/api/position"
	"github.com/fonchain_enterprise/fonchain-main/api/rule"
	"github.com/fonchain_enterprise/fonchain-main/pkg/common/untils/timetool"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/login"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/query"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/vo"
	"github.com/fonchain_enterprise/fonchain-main/pkg/serializer"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
)

func ControlSure(c *gin.Context) {
	req := &order.SetStatusRequest{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	req.Status = e.Order_Need_Finance_Sure
	res, err := service.OrderProvider.SetStatus(c, req)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func Statist(c *gin.Context) {
	req := &order.OrderStatisticsInfoRequest{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	res, err := service.OrderProvider.OrderStatisticsInfo(c, req)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func FinanceSures(c *gin.Context) {

	rr := &query.BatchOrderIds{}

	if err := c.ShouldBindBodyWith(&rr, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	for _, id := range rr.IDs {
		req := &order.SetStatusRequest{ID: id}
		req.Status = e.Order_Done
		_, err := service.OrderProvider.SetStatus(c, req)

		if err != nil {
			service.Error(c, e.Error, err)
			return
		}

	}

	service.Success(c, nil)

	return
}

func FinanceSure(c *gin.Context) {
	req := &order.SetStatusRequest{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	req.Status = e.Order_Done
	res, err := service.OrderProvider.SetStatus(c, req)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func FinancePass(c *gin.Context) {
	req := &order.OrderStageSetStatusReq{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	req.IsPass = true
	res, err := service.OrderProvider.OrderStageSetStatus(c, req)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func FinanceRefuse(c *gin.Context) {
	req := &order.OrderStageSetStatusReq{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	req.IsPass = false
	res, err := service.OrderProvider.OrderStageSetStatus(c, req)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

// Transactions 提交的销售报表列表
func Transactions(c *gin.Context) {
	req := &order.TransactionsRequest{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	var SiteUidS []uint32
	userInfo := login.GetUserInfoFromC(c)

	if len(userInfo.PositionUsers) <= 0 {
		service.Error(c, e.InvalidParams, errors.New("无销售部门"))
		return
	}

	for _, tt := range userInfo.PositionUsers {
		SiteUidS = append(SiteUidS, uint32(tt.DepartmentId))
	}
	userList, err := service.AccountProvider.ListV2(c, &account.ListV2Request{
		Domain:        "fontree",
		Page:          1,
		DepartmentIds: SiteUidS,
	})
	var userIds []uint64
	for _, i := range userList.Data {
		userIds = append(userIds, i.ID)
	}
	req.SellerIds = userIds
	//if len(SiteUidS) <= 0 {
	//	service.Error(c, e.InvalidParams, errors.New("less data"))
	//	return
	//}
	//
	//req.SaleSiteUidS = SiteUidS

	res, err := service.OrderProvider.GetTransactions(c, req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}
	puRequest := &rule.PositionUserListRequest{
		//SiteUidS: SiteUidS,
	}
	puRes, err := service.RuleProvider.PositionUserList(c, puRequest)
	res1 := &order.TransactionsResponse{}

	for _, i := range res.Data {
		if len(i.TransactionStages) == 0 {
			continue
		}
		found := false // 标志是否找到匹配
		for _, s := range puRes.Data {
			//if i.SaleSiteUid == s.SyncId && i.SellerName == s.UserName {
			if i.SellerName == s.UserName {
				i.Colour = "black"
				found = true // 找到匹配，设置标志为 true
				break        // 不再继续搜索
			}
		}
		if !found { // 如果未找到匹配，设置颜色为 red
			i.Colour = "red"
		}
		res1.Data = append(res1.Data, i)
	}
	res1.Count = res.Count
	service.Success(c, serializer.BuildTransactionPage(res1))
	return
}

func Users(c *gin.Context) {

	response := vo.PageRes{
		Count: 0,
	}

	req := &order.TransactionsRequest{}

	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	if req.SaleSite == "" {
		service.Error(c, e.InvalidParams, errors.New("less Data"))
		return

	}

	departmentID := getSaleSiteIDFromContext(c, req.SaleSite)
	if departmentID == 0 {
		service.Error(c, e.InvalidParams, errors.New("less Data 0"))
		return
	}

	UserInfoMap := make(map[uint64]*department.UserInfo, 0)

	positionIds, ok := getSellerPositionID(c, []uint64{departmentID})

	if ok == false { //不存在销售的岗位
		service.Success(c, response)
		return
	}

	dUsersreq := department.UsersRequest{
		InPositionIds: positionIds,
	}

	res, err := service.DepartmentProvider.Users(c, &dUsersreq)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	if res.Count == 0 {
		service.Success(c, response)
		return
	}

	var userIds []uint64

	for i, tt := range res.Users {
		userIds = append(userIds, tt.UserID)
		UserInfoMap[tt.UserID] = res.Users[i]
	}

	IdsReq := account.ListByIDsRequest{
		IDs: userIds,
	}

	userRes, userErr := service.AccountProvider.ListByIDs(c, &IdsReq)

	if userErr != nil {
		service.Error(c, e.Error, err)
		return
	}

	response = vo.PageRes{
		Count: userRes.Count,
	}

	response.Data = serializer.BuildUsers(userRes.Data, UserInfoMap)

	service.Success(c, response)
}

func Publish(c *gin.Context) {
	req := &order.SetStatusRequest{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	req.Status = e.ORDER_STATUS_NEED_CONFIRM
	res, err := service.OrderProvider.SetStatus(c, req)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

// OrderList 订单列表
func OrderList(c *gin.Context) {
	req := &order.OrderList{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}
	req.Page = 1
	req.PageSize = 999

	res, err := service.OrderProvider.GetOrders(c, req)

	fmt.Println("1----", err)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	resPage := vo.PageRes{
		Count: uint64(len(res.Data)),
		Data:  serializer.BuildOrders(res.Data),
	}
	service.Success(c, resPage)

	return
}

// OrderDetail 订单列表
func OrderDetail(c *gin.Context) {
	req := &order.CommonRequest{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	res, err := service.OrderProvider.GetOrder(c, req)

	fmt.Println("1----", err)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	infoReq := &account.InfoRequest{
		ID: res.SellerId,
	}

	infoRes, err := service.AccountProvider.Info(c, infoReq)
	if err == nil && infoRes != nil && res != nil {
		res.SellerName = infoRes.Info.NickName
		res.SellerNum = infoRes.Info.JobNum
		res.SellerImg = infoRes.Info.Avatar
	}

	service.Success(c, serializer.BuildOrder(res))

	return
}

// SetOrder 更新订单
func SetOrder(c *gin.Context) {
	req := &order.OrderRequest{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	req.Gtv = req.Price
	res, err := service.OrderProvider.OrderUpdate(c, req)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func UpdateOrder(c *gin.Context) {
	req := &order.OrderBatchUpdateRequest{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	res, err := service.OrderProvider.OrderBatchUpdate(c, req)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func CashierSure(c *gin.Context) {
	req := &order.SetStatusRequest{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	req.Status = e.Order_Need_Finance_Sure
	res, err := service.OrderProvider.SetStatus(c, req)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

// FinanceOrders 订单列表
func FinanceOrders(c *gin.Context) {
	req := &order.OrderList{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}
	//Todo 徐嘉红修改需求:内控管理 取消查看订单状态限制,查询所有订单数据
	//if len(req.Status) == 0 {
	//	req.Status = []uint64{e.Order_Need_Finance_Sure, e.Order_Done}
	//}

	res, err := service.OrderProvider.GetOrders(c, req)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	for index, tt := range res.Data {
		if tt.Status == e.Order_Need_Finance_Sure {
			res.Data[index].IsSure = 2
		} else {
			res.Data[index].IsSure = 1
		}
	}

	for i, t := range res.Data {
		res.Data[i].SellTime = timetool.GetYmdFromString(t.SellTime)
	}

	service.Success(c, res)

	return
}

// ControlOrders 订单列表
func ControlOrders(c *gin.Context) {
	req := &order.OrderList{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	if len(req.Status) == 0 {
		req.Status = []uint64{e.Order_Need_Control_Sure, e.Order_Need_Finance_Sure, e.Order_Done}
	}

	res, err := service.OrderProvider.GetOrders(c, req)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	for index, tt := range res.Data {
		if tt.Status == e.Order_Need_Control_Sure {
			res.Data[index].IsSure = 2
		} else {
			res.Data[index].IsSure = 1
		}
	}

	service.Success(c, res)

	return
}

// CashierOrders 订单列表
func CashierOrders(c *gin.Context) {
	req := &order.OrderList{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	userInfo := login.GetUserInfoFromC(c)

	if len(userInfo.PositionUsers) <= 0 {
		service.Success(c, nil)
		return
	}

	var saleSites []string
	for _, tt := range userInfo.PositionUsers {
		saleSites = append(saleSites, tt.DepartmentName)
	}

	req.SaleSites = saleSites
	//req.SaleSite = userInfo.PositionUsers[0].DepartmentName

	if len(req.Status) == 0 {
		req.Status = []uint64{e.Order_Done, e.Order_Need_Control_Sure, e.Order_Need_Finance_Sure} //查看内控和财务，以及完成的数据
	}

	res, err := service.OrderProvider.GetOrders(c, req)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	for i, t := range res.Data {
		res.Data[i].SellTime = timetool.GetYmdFromString(t.SellTime)
	}

	service.Success(c, res)

	return
}

// SiteOrders 订单列表
func SiteOrders(c *gin.Context) {
	req := &order.OrderList{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	userInfo := login.GetUserInfoFromC(c)
	req.SaleSite = userInfo.PositionUsers[0].DepartmentName

	res, err := service.OrderProvider.GetOrders(c, req)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)

	return
}

func Transaction(c *gin.Context) {
	req := &order.CommonRequest{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	res, err := service.OrderProvider.GetTransaction(c, req)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	for i, t := range res.Orders {
		res.Orders[i].SellTime = timetool.GetYmdFromString(t.SellTime)
	}

	service.Success(c, res)

	return
}

func SiteTransactions(c *gin.Context) {
	req := &order.TransactionsRequest{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	userInfo := login.GetUserInfoFromC(c)
	if len(userInfo.PositionUsers) <= 0 {
		service.Success(c, nil)
		return
	}

	//var saleSiteUids []string
	//for _, tt := range userInfo.PositionUsers {
	//	saleSiteUids = append(saleSiteUids, tt.SyncId)
	//}

	//req.SaleSiteUidS = saleSiteUids

	//req.SaleSite = userInfo.PositionUsers[0].DepartmentName
	req.Status = []uint64{e.Transaction_Need_Cashier_Sure}

	res, err := service.OrderProvider.GetTransactions(c, req)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}
	orderNotSureMap := make(map[uint64]struct{})
	orderNotSureMap[e.Order_Done] = struct{}{}
	orderNotSureMap[e.Order_Need_Finance_Sure] = struct{}{}
	orderNotSureMap[e.Order_Need_Control_Sure] = struct{}{}

	//修改
	for index, tt := range res.Data {
		for i, t := range tt.Orders {
			if _, ok := orderNotSureMap[t.Status]; ok { //是完成的
				res.Data[index].Orders[i].IsSure = 1
			} else {
				res.Data[index].Orders[i].IsSure = 2
			}

			res.Data[index].Orders[i].SellTime = timetool.GetYmdFromString(t.SellTime)
		}
	}

	service.Success(c, res)

	return
}

func DepartmentBases(c *gin.Context) {
	var req = department.DepartmentsByAuthRequest{
		Url: e.SellerBossKey,
	}

	infoByUserIdResponse, err1 := service.DepartmentProvider.DepartmentsByAuth(c, &req)

	if err1 != nil {
		service.Error(c, e.Error, err1)
		return
	}

	service.Success(c, infoByUserIdResponse.Data)

}
func TransactionUpdate(c *gin.Context) {
	req := &order.UpdateTransactionDateReq{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}
	infoByUserIdResponse, err1 := service.OrderProvider.UpdateTransactionDate(c, req)

	if err1 != nil {
		service.Error(c, e.Error, err1)
		return
	}

	service.Success(c, infoByUserIdResponse.ID)
}
func getDepartmentIds(c *gin.Context) []uint64 {

	var depatmentIds []uint64
	userInfo := login.GetUserInfoFromC(c)

	if len(userInfo.PositionUsers) <= 0 {
		return depatmentIds
	}

	for _, tt := range userInfo.PositionUsers {
		depatmentIds = append(depatmentIds, tt.DepartmentId)
	}

	return depatmentIds
}

func getSaleSiteIDFromContext(c *gin.Context, departmentName string) uint64 {

	userInfo := login.GetUserInfoFromC(c)

	if len(userInfo.PositionUsers) <= 0 {
		return 0
	}

	for _, tt := range userInfo.PositionUsers {
		if departmentName == tt.DepartmentName {
			return tt.DepartmentId
		}
	}

	return 0

}

func getSaleSitesFromContext(c *gin.Context) []string {

	var saleSites []string
	userInfo := login.GetUserInfoFromC(c)

	if len(userInfo.PositionUsers) <= 0 {
		return saleSites
	}

	for _, tt := range userInfo.PositionUsers {
		saleSites = append(saleSites, tt.SyncId)
	}

	return saleSites

}

func getSellerPositionID(c *gin.Context, departmentIds []uint64) (id []uint64, ok bool) {
	var ids []uint64
	ok = false
	fReq := position.PositionByUrlsRequest{
		Urls:          []string{e.SellerStaffKey, e.SellerBossKey},
		DepartmentIds: departmentIds,
	}

	fres, err := service.PositionProvider.PositionByUrls(c, &fReq)

	if err == nil && fres != nil && fres.List != nil {
		for _, tt := range fres.List {
			ids = append(ids, tt.ID)
			ok = true
		}
	}

	return ids, ok
}
