package pay_bill

import (
	"errors"
	api "github.com/fonchain_enterprise/fonchain-main/api/paybill"
	"github.com/fonchain_enterprise/fonchain-main/api/position"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/login"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
)

func Des(c *gin.Context) {
	var req api.GetSecretDataRequest
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.Error, err)
		return
	}

	if err := isHaveDriverAuth(c); err != nil {
		service.Error(c, e.Error, err)
		return
	}

	res, err := service.GrpcPayBillImpl.GetSecretData(c, &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
}

func Update(c *gin.Context) {
	var req api.UpdateSecretDataRequest
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.Error, err)
		return
	}

	if err := isHaveDriverAuth(c); err != nil {
		service.Error(c, e.Error, err)
		return
	}

	res, err := service.GrpcPayBillImpl.UpdateSecretData(c, &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
}

func Send(c *gin.Context) {
	var req api.GetPhoneSecretRequest
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.Error, err)
		return
	}

	if err := isHaveDriverAuth(c); err != nil {
		service.Error(c, e.Error, err)
		return
	}

	res, err := service.GrpcPayBillImpl.GetPhoneSecret(c, &req)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
}

func isHaveDriverAuth(c *gin.Context) error {

	userInfoAny, _ := c.Get("jwtInfo")
	userInfo := userInfoAny.(login.Info)
	if len(userInfo.PositionUsers) <= 0 {
		return errors.New("缺少权限,没有绑定岗位")
	}

	req := position.DoIHavaAuthRequest{
		UserId:       userInfo.ID,
		Url:          e.SecretDataBoss,
		DepartmentID: userInfo.PositionUsers[0].DepartmentId,
	}

	res, err := service.PositionProvider.DoIHavaAuth(c, &req)

	if err == nil && res.Hava == true {
		return nil
	}

	return errors.New("缺少权限")
}
