package push

import (
	"context"
	"errors"
	"fmt"
	"net/http"
	"strconv"

	"dubbo.apache.org/dubbo-go/v3/common/logger"
	"github.com/fonchain_enterprise/fonchain-main/api/account"
	"github.com/fonchain_enterprise/fonchain-main/api/collections"
	"github.com/fonchain_enterprise/fonchain-main/api/push"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/serializer"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/gin-gonic/gin"
)

func PutIn(c *gin.Context) {
	fmt.Println("第一处")
	var putInRequest push.PutInRequest
	if err := c.ShouldBind(&putInRequest); err != nil {
		logger.Errorf("putInRequest ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	//获取藏品相关信息
	collectionsInfoRes, err := service.GrpcCollectionImpl.CollectionsDetail(context.Background(), &collections.CollectionsInfoReq{
		CollectionsUID: putInRequest.CollectionUid,
		Language:       "zh-CN",
	})
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	fmt.Println("第一处", collectionsInfoRes.Data.SaleType)

	if collectionsInfoRes.Data.SaleType != "normal" {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   nil,
			Msg:    "赠品不能进行空投",
			Status: e.Failed,
		})
		return
	}

	putInRequest.Collection = collectionsInfoRes.Data.Name
	putInRequest.Desc = collectionsInfoRes.Data.Intro
	putInRequest.PicUrl = collectionsInfoRes.Data.CollectionImg
	//获取用户id切片
	if putInRequest.DropTarget != "全服" {

		if putInRequest.DropTargetNum != 1 {
			service.ResponseMsg(c, e.SUCCESS, serializer.Response{
				Data:   nil,
				Msg:    errors.New("空投数量需要为1").Error(),
				Status: e.Failed,
			})
			return
		}

		userid, err := strconv.Atoi(putInRequest.DropTarget)
		if err != nil {
			service.ResponseMsg(c, e.SUCCESS, serializer.Response{
				Data:   nil,
				Msg:    err.Error(),
				Status: e.Failed,
			})
			return
		}

		putInRequest.UserId = []int32{int32(userid)}
	} else {

		//账号服务
		var updateReq = account.ListRequest{
			Domain:   "mall",
			PageSize: uint64(putInRequest.DropTargetNum),
		}

		ures, err := service.AccountProvider.RandList(context.Background(), &updateReq)
		if err != nil {
			service.ResponseMsg(c, e.SUCCESS, serializer.Response{
				Msg:    err.Error(),
				Status: e.Failed,
			})
			return
		}

		//做一下判断，让空投的人数不超过全服的人数
		if putInRequest.DropTargetNum > int32(ures.AllCount) {
			service.ResponseMsg(c, e.SUCCESS, serializer.Response{
				Data:   nil,
				Msg:    errors.New("空投数量不可以大于全服总人数").Error(),
				Status: e.Failed,
			})
			return
		}

		var tempIds []int32
		for _, v := range ures.Data {
			tempIds = append(tempIds, int32(v.ID))
		}
		putInRequest.CollectionNum = int32(ures.Count)
		putInRequest.UserId = tempIds
	}

	rep, err := service.GrpcPushImpl.PutIn(context.Background(), &putInRequest)

	fmt.Println("rep:", rep)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Msg:    rep.Msg,
		Status: e.Ok,
		Data:   rep,
	})
}

func DropHistortList(c *gin.Context) {
	var dropHistortListRequest push.DropHistortListRequest
	rep, err := service.GrpcPushImpl.DropHistortList(context.Background(), &dropHistortListRequest)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	service.ResponseMsg(c, e.SUCCESS, serializer.Response{
		Msg:    rep.Msg,
		Status: e.Ok,
		Data:   rep,
	})
}

func DropHistortDel(c *gin.Context) {
	var dropHistortDelRequest push.DropHistortDelRequest
	if err := c.ShouldBind(&dropHistortDelRequest); err != nil {
		logger.Errorf("dropHistortDelRequest ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	rep, err := service.GrpcPushImpl.DropHistortDel(context.Background(), &dropHistortDelRequest)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	c.JSON(http.StatusOK, serializer.Response{
		Status: e.Ok,
		Code:   e.SUCCESS,
		Data:   rep,
	})
}

func PushAnnouncement(c *gin.Context) {
	fmt.Println("第一处")
	var pushAnnouncementRequest push.PushAnnouncementRequest
	if err := c.ShouldBind(&pushAnnouncementRequest); err != nil {
		logger.Errorf("pushAnnouncementRequest ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if pushAnnouncementRequest.Target != "全服" {
		userid, err := strconv.Atoi(pushAnnouncementRequest.Target)
		if err != nil {
			return
		}
		pushAnnouncementRequest.AnnounceNum = 1
		pushAnnouncementRequest.UserId = []int32{int32(userid)}
	} else {
		//账号服务
		var updateReq1 = account.ListRequest{
			Domain:   "mall",
			PageSize: uint64(1),
		}

		ures1, err := service.AccountProvider.RandList(context.Background(), &updateReq1)
		if err != nil {
			service.ResponseMsg(c, e.SUCCESS, serializer.Response{
				Msg:    err.Error(),
				Status: e.Failed,
			})
			return
		}

		var updateReq2 = account.ListRequest{
			Domain:   "mall",
			PageSize: ures1.Count,
		}

		ures2, err := service.AccountProvider.RandList(context.Background(), &updateReq2)
		if err != nil {
			service.ResponseMsg(c, e.SUCCESS, serializer.Response{
				Msg:    err.Error(),
				Status: e.Failed,
			})
			return
		}

		var tempIds []int32
		for _, v := range ures2.Data {
			tempIds = append(tempIds, int32(v.ID))
		}
		pushAnnouncementRequest.AnnounceNum = int32(ures2.Count)
		pushAnnouncementRequest.UserId = tempIds
	}

	rep, err := service.GrpcPushImpl.PushAnnouncement(context.Background(), &pushAnnouncementRequest)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	c.JSON(http.StatusOK, serializer.Response{
		Status: e.Ok,
		Code:   e.SUCCESS,
		Data:   rep,
	})
}

func AnnouncementHisList(c *gin.Context) {
	var announcementHisListRequest push.AnnouncementHisListRequest
	rep, err := service.GrpcPushImpl.AnnouncementHisList(context.Background(), &announcementHisListRequest)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	c.JSON(http.StatusOK, serializer.Response{
		Status: e.Ok,
		Code:   e.SUCCESS,
		Data:   rep,
	})
}

func AnnouncementDel(c *gin.Context) {
	var announcementDelRequest push.AnnouncementDelRequest
	if err := c.ShouldBind(&announcementDelRequest); err != nil {
		logger.Errorf("pushAnnouncementRequest ShouldBind err", err)
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   nil,
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	rep, err := service.GrpcPushImpl.AnnouncementDel(context.Background(), &announcementDelRequest)
	if err != nil {
		service.ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	c.JSON(http.StatusOK, serializer.Response{
		Status: e.Ok,
		Code:   e.SUCCESS,
		Data:   rep,
	})
}
