package shop

import (
	"context"
	"dubbo.apache.org/dubbo-go/v3/common/logger"
	"github.com/fonchain_enterprise/fonchain-main/api/shopbrand"
	"github.com/fonchain_enterprise/fonchain-main/pkg/common"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/gin-gonic/gin"
)

func CreateBrand(c *gin.Context) {
	var req shopbrand.CreateBrandreq
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("CreateBrand ShouldBind err", err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	if err := req.Validate(); err != nil {
		err = common.SubstrError(err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}

	if req.LanguageNum == 1 {

		//var accreq chain.CreateAccountRequest
		//accres, err := service.ChainProvider.CreateAccount(context.Background(), &accreq)
		//if err != nil {
		//	logger.Errorf("CreateAccount err", err)
		//	err = errors.New(e.GetMsg(e.ErrorCreateAccount))
		//	return
		//}
		//req.Address = accres.Address
		//req.Mnemonic = accres.Mnemonic
	}
	//
	//req.Address = "Address"
	//req.Mnemonic = "Mnemonic"

	resp, err := service.GrpcShopBrandImpl.CreateBrand(context.Background(), &req)
	if err != nil {
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(c, e.Ok, resp.Msg, map[string]interface{}{
		"BrandUid": resp.BrandUid,
	})
	return
}

func BrandInfo(c *gin.Context) {
	var req shopbrand.BrandInforeq
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("BrandInfo ShouldBind err", err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	if err := req.Validate(); err != nil {
		err = common.SubstrError(err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	resp, err := service.GrpcShopBrandImpl.BrandInfo(context.Background(), &req)
	if err != nil {
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(c, e.Ok, resp.Msg, resp)
	return
}

func BrandUpdate(c *gin.Context) {
	var req shopbrand.UpdateBrandreq
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("BrandUpdate ShouldBind err", err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	if err := req.Validate(); err != nil {
		err = common.SubstrError(err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	resp, err := service.GrpcShopBrandImpl.UpdateBrand(context.Background(), &req)
	if err != nil {
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(c, e.Ok, resp.Msg, nil)
	return
}

func BrandDel(c *gin.Context) {
	var req shopbrand.DeleteBrandreq
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("BrandUpdate ShouldBind err", err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	if err := req.Validate(); err != nil {
		err = common.SubstrError(err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	resp, err := service.GrpcShopBrandImpl.DeleteBrand(context.Background(), &req)
	if err != nil {
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(c, e.Ok, resp.Msg, nil)
	return
}

func BrandList(c *gin.Context) {
	var req shopbrand.BrandListreq
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("BrandList ShouldBind err", err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	if err := req.Validate(); err != nil {
		err = common.SubstrError(err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	resp, err := service.GrpcShopBrandImpl.BrandList(context.Background(), &req)
	if err != nil {
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(c, e.Ok, resp.Msg, resp)
	return
}

func AllBrand(c *gin.Context) {
	var req shopbrand.AllBrandreq
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("AllBrand ShouldBind err", err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	resp, err := service.GrpcShopBrandImpl.AllBrand(context.Background(), &req)
	if err != nil {
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(c, e.Ok, resp.Msg, resp.Data)
	return
}

func UpdateLanguageInfo(c *gin.Context) {
	var req shopbrand.UpdateLanguageInfoReq
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("UpdateLanguageInfoReq ShouldBind err", err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	resp, err := service.GrpcShopBrandImpl.UpdateLanguageInfo(context.Background(), &req)
	if err != nil {
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	service.Success(c, resp)
	return
}

func GetAllLanguageInfo(c *gin.Context) {
	var req shopbrand.GetAllLanguageInfoReq
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("GetAllLanguageInfoReq ShouldBind err", err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	resp, err := service.GrpcShopBrandImpl.GetAllLanguageInfo(context.Background(), &req)
	if err != nil {
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	service.Success(c, resp)
	return
}
