package shop

import (
	"context"
	"dubbo.apache.org/dubbo-go/v3/common/logger"
	"errors"
	"fmt"
	"github.com/fonchain_enterprise/fonchain-main/api/collection_record"
	"github.com/fonchain_enterprise/fonchain-main/api/collections"
	"github.com/fonchain_enterprise/fonchain-main/api/shopbrand"
	"github.com/fonchain_enterprise/fonchain-main/pkg/common"
	"github.com/fonchain_enterprise/fonchain-main/pkg/config"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/order"
	"github.com/fonchain_enterprise/fonchain-main/pkg/utils"
	"github.com/fonchain_enterprise/fonchain-main/pkg/utils/qr"
	"github.com/gin-gonic/gin"
	"github.com/skip2/go-qrcode"
	"io"
	"io/ioutil"
	"net/http"
	"os"
)

func AddCollections(c *gin.Context) {
	var req collections.CollectionsInfo
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("AddCollections ShouldBind err", err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}

	fmt.Println("测试1")

	if err := req.Validate(); err != nil {
		err = common.SubstrError(err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	fmt.Println("测试2")

	if len(req.CarouselFigureList) <= 0 || len(req.DetailImgList) <= 0 {
		service.Error(c, e.Error, errors.New("新增藏品，主图和详情图不可为空，请您修改后保存"))
		return
	}
	fmt.Println("测试3")

	//获取到语言信息
	getLanguageInfoByNumRep, err := service.GrpcShopBrandImpl.GetLanguageInfoByNum(context.Background(), &shopbrand.GetLanguageInfoByNumReq{LanguageNum: req.LanguageNum})
	if err != nil {
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	fmt.Println("测试4")

	req.Language = getLanguageInfoByNumRep.Language
	fmt.Println("测试5")
	resp, err := service.GrpcCollectionImpl.AddCollections(context.Background(), &req)
	fmt.Println("测试6")
	if err != nil {
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	fmt.Println("测试7")
	service.ResponseQuickMsg(c, e.Ok, resp.Msg, resp)
	return
}

func UpdateCollectionsProfile(c *gin.Context) {
	var req collections.CollectionsInfo
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("UpdateCollectionsProfile ShouldBind err", err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	if err := req.Validate(); err != nil {
		err = common.SubstrError(err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}

	if req.IsDelete != 1 { // 删除藏品字段
		//获取到语言信息
		getLanguageInfoByNumRep, err := service.GrpcShopBrandImpl.GetLanguageInfoByNum(context.Background(), &shopbrand.GetLanguageInfoByNumReq{LanguageNum: req.LanguageNum})
		if err != nil {
			service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
			return
		}
		req.Language = getLanguageInfoByNumRep.Language
	}

	resp, err := service.GrpcCollectionImpl.UpdateCollections(context.Background(), &req)
	if err != nil {
		//service.ResponseQuickMsg(c, e.Failed, resp.Msg, nil)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	go order.SynPayCollectionData(c, req.CollectionUID)
	service.ResponseQuickMsg(c, e.Ok, resp.Msg, nil)
	return
}

func CollectionsList(c *gin.Context) {
	var req collections.CollectionsListReq
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("DelCollections ShouldBind err", err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	if err := req.Validate(); err != nil {
		err = common.SubstrError(err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	resp, err := service.GrpcCollectionImpl.CollectionsList(context.Background(), &req)
	if err != nil {
		service.ResponseQuickMsg(c, e.Failed, resp.Msg, nil)
		return
	}
	service.ResponseQuickMsg(c, e.Ok, resp.Msg, resp)
	return
}

func RecordList(c *gin.Context) {
	var req collection_record.OrderRecordList
	if err := c.ShouldBind(&req); err != nil {
		service.Error(c, e.Error, err)
		return
	}

	resp, err := service.GrpcCollectionRecordImpl.List(context.Background(), &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}
	service.Success(c, resp)
	return
}

func CollectionsDetail(c *gin.Context) {
	var req collections.CollectionsInfoReq
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("DelCollections ShouldBind err", err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	if err := req.Validate(); err != nil {
		err = common.SubstrError(err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	resp, err := service.GrpcCollectionImpl.CollectionsDetail(context.Background(), &req)
	if err != nil {
		service.ResponseQuickMsg(c, e.Failed, resp.Msg, nil)
		return
	}
	//var data any
	//if len(resp.Data) > 0 {
	//	data = resp.Data[0]
	//}
	service.ResponseQuickMsg(c, e.Ok, resp.Msg, resp.Data)
	return
}

func Qr(c *gin.Context) {
	var req collections.CollectionsInfoReq

	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("DelCollections ShouldBind err", err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}

	url := fmt.Sprintf("%s/#/pages/index/detail/detail?CollectionUID=%s", config.ShopHost, req.CollectionsUID)
	s, _ := qr.CreateQrBase64(url, "")

	resMap := make(map[string]interface{})
	resMap["base"] = s
	resMap["url"] = url
	service.Success(c, resMap)
	return
}

func Extend(c *gin.Context) {
	var req collections.CommodityPromotionReq
	type ll struct {
		Name     string
		Index    string
		Hash     string
		RealHash string
		ShopHash string
		Url      string
	}
	var data []interface{}
	var OrderColumnName = []string{"名称", "编号", "藏品hash", "编号hash", "真正的hash", "url"}
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("DelCollections ShouldBind err", err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	if req.StartNum > req.EndNum {
		c.String(http.StatusRequestedRangeNotSatisfiable, fmt.Sprintf("Incorrect request parameters"))
		return
	}
	tempDir := "./temp/images"
	err := os.MkdirAll(tempDir, os.ModePerm)
	if err != nil {
		c.String(http.StatusInternalServerError, fmt.Sprintf("Failed to create temporary directory: %v", err))
		return
	}
	defer os.RemoveAll(tempDir) // 在处理完成后删除临时目录及其中的文件
	resp, err := service.GrpcCollectionImpl.CommodityPromotion(context.Background(), &req)
	str := "https://shop.szjixun.cn/#/pages/login/login?collectionsUID=%s&num=%d"
	for _, r := range resp.Data {
		for i := req.StartNum; i <= req.EndNum; i++ {
			temp := ll{
				Name:     r.Name,
				Index:    fmt.Sprintf("%d", i),
				Hash:     r.TransactionHash,
				RealHash: fmt.Sprintf("%s_%d", r.TransactionHash, i),
				Url:      fmt.Sprintf(str, r.CollectionUid, i)}
			temp.ShopHash = utils.SHA256V([]byte(temp.RealHash))

			qrCode, err := qrcode.Encode(temp.Url, qrcode.Medium, 256)
			if err != nil {
				fmt.Printf("Failed to generate QR code: %v\n", err)
				return
			}
			// 将二维码图片保存到文件
			err = ioutil.WriteFile("./temp/images/"+temp.Index+".png", qrCode, 0644)
			if err != nil {
				fmt.Printf("Failed to save QR code image: %v\n", err)
				return
			}
			data = append(data, &temp)
		}
	}
	content := utils.ToExcel(OrderColumnName, data, "struct")
	file, err := os.Create("./temp/藏品哈希.xlsx")
	if err != nil {
		fmt.Println("Failed to create file:", err)
		return
	}
	defer file.Close()
	_, err = io.Copy(file, content)
	if err != nil {
		fmt.Println("Failed to save file:", err)
		return
	}
	qr.ImagesOnZip(c, "./temp/")

}

func GetCollectionLanguageInfo(c *gin.Context) {
	var req collections.GetCollectionLanguageInfoReq
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("GetCollectionLanguageInfoReq ShouldBind err", err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	if err := req.Validate(); err != nil {
		err = common.SubstrError(err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}

	getAllLanguageInforesp, err := service.GrpcShopBrandImpl.GetAllLanguageInfo(context.Background(), &shopbrand.GetAllLanguageInfoReq{})
	if err != nil {
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}

	for _, s := range getAllLanguageInforesp.LanguageInfo {
		languageInfo := &collections.LanguageInfo{
			LanguageNum: s.LanguageNum,
			Language:    s.Language,
		}

		req.LanguageInfo = append(req.LanguageInfo, languageInfo)
	}

	resp, err := service.GrpcCollectionImpl.GetCollectionLanguageInfo(context.Background(), &req)
	if err != nil {
		service.ResponseQuickMsg(c, e.Failed, resp.Msg, nil)
		return
	}
	//var data any
	//if len(resp.Data) > 0 {
	//	data = resp.Data[0]
	//}
	service.ResponseQuickMsg(c, e.Ok, resp.Msg, resp)
	return
}
