package shop

import (
	"context"
	"dubbo.apache.org/dubbo-go/v3/common/logger"
	backendSeries "github.com/fonchain_enterprise/fonchain-main/api/series"
	"github.com/fonchain_enterprise/fonchain-main/api/shopbrand"
	"github.com/fonchain_enterprise/fonchain-main/pkg/common"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/order"
	"github.com/gin-gonic/gin"
)

func SaveSeries(c *gin.Context) {
	var req backendSeries.SaveSeriesReq
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("Create ShouldBind err", err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	if err := req.Validate(); err != nil {
		err = common.SubstrError(err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}

	//获取到语言信息
	getLanguageInfoByNumRep, err := service.GrpcShopBrandImpl.GetLanguageInfoByNum(context.Background(), &shopbrand.GetLanguageInfoByNumReq{LanguageNum: req.LanguageNum})
	if err != nil {
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	req.Language = getLanguageInfoByNumRep.Language

	if req.LanguageNum == 1 {
		// 查看品牌uid是否存在
		brandInfoResp, err := service.GrpcShopBrandImpl.BrandInfo(context.Background(), &shopbrand.BrandInforeq{BrandUid: req.BrandId})
		if err != nil {
			service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
			return
		}
		if brandInfoResp == nil {
			service.ResponseQuickMsg(c, e.Failed, "品牌方不存在", nil)
			return
		}
	}

	//atta := make(map[string]interface{})
	//atta["string-value"] = "string-demo"
	//atta["int-value"] = 1231242
	//reqContext := context.WithValue(context.Background(), constant.DubboCtxKey("attachment"), atta)
	resp, err := service.GrpcBackendSeriesImpl.SaveSeries(context.Background(), &req)
	if err != nil {
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(c, e.Ok, resp.Msg, map[string]interface{}{
		"Uuid": resp.Uuid,
	})
	return
}

func SeriesDetail(c *gin.Context) {
	var req backendSeries.SeriesDetailReq
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("SeriesDetailReq ShouldBind err", err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	if err := req.Validate(); err != nil {
		err = common.SubstrError(err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	resp, err := service.GrpcBackendSeriesImpl.SeriesDetail(context.Background(), &req)
	if err != nil {
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(c, e.Ok, resp.Msg, resp)
	return
}

func UpdateSeriesLang(c *gin.Context) {
	var req backendSeries.UpdateSeriesLangReq
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("UpdateSeriesLang ShouldBind err", err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	if err := req.Validate(); err != nil {
		err = common.SubstrError(err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	resp, err := service.GrpcBackendSeriesImpl.UpdateSeriesLang(context.Background(), &req)
	if err != nil {
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(c, e.Ok, resp.Msg, nil)
	return
}

func AutoShelf(c *gin.Context) {
	var req backendSeries.AutoShelfReq
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("AutoShelf ShouldBind err", err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	if err := req.Validate(); err != nil {
		err = common.SubstrError(err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	resp, err := service.GrpcBackendSeriesImpl.AutoShelf(context.Background(), &req)
	if err != nil {
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	order.SynPaySeriesData(c, req.SeriesUuid)
	service.ResponseQuickMsg(c, e.Ok, resp.Msg, nil)
	return
}

func HandShelf(c *gin.Context) {
	var req backendSeries.HandShelfReq
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("AutoShelf ShouldBind err", err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	if err := req.Validate(); err != nil {
		err = common.SubstrError(err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	resp, err := service.GrpcBackendSeriesImpl.HandShelf(context.Background(), &req)
	if err != nil {
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	order.SynPaySeriesData(c, req.SeriesUuid)
	/*if resp.BrandUuid != "" && resp.SeriesUuid != "" {
		// 请求brand 获取hash
		var brandReq shopbrand.BrandListreq
		brandReq.BrandUid = append(brandReq.BrandUid, resp.BrandUuid)
		brandResp, err := service.GrpcShopBrandImpl.BrandList(context.Background(), &brandReq)
		if err != nil {
			logger.Errorf("HandShelf BrandList err", err)
			err = errors.New(e.GetMsg(e.ErrorBrandList))
			return
		}
		var ntfReq collections.StoreNFTList
		var ntfInfo collections.StoreNFTInfo
		ntfInfo.SeriesUID = resp.SeriesUuid
		ntfInfo.BrandHash = brandResp.Data[0].Address
		ntfReq.NFT = append(ntfReq.NFT, &ntfInfo)
		ntfResp, err := service.GrpcCollectionImpl.StoreNFT(context.Background(), &ntfReq)
		logger.Info("ntfResp", ntfResp, "ntfReq", ntfReq.NFT)
		if err != nil {
			logger.Errorf("HandShelf StoreNFT err", err)
			err = errors.New(e.GetMsg(e.ErrorStoreNTF))
			return
		}
		if len(ntfResp.SeriesUID) == 0 {
			// 通知成功
			var seriesReq backendSeries.UpdateSeriesChainReq
			seriesReq.SeriesUuid = resp.SeriesUuid
			seriesReq.ChainStatus = 1
			_, _ = service.GrpcBackendSeriesImpl.UpdateSeriesChain(context.Background(), &seriesReq)
		}
	}*/
	service.ResponseQuickMsg(c, e.Ok, resp.Msg, nil)
	return
}

func SeriesList(c *gin.Context) {
	var req backendSeries.SeriesListReq
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("AutoShelf ShouldBind err", err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	if err := req.Validate(); err != nil {
		err = common.SubstrError(err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	if req.Language == "" {
		req.Language = model.LangChinese
	}

	getAllLanguageInfoResp, err := service.GrpcShopBrandImpl.GetAllLanguageInfo(context.Background(), &shopbrand.GetAllLanguageInfoReq{})
	if err != nil {
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	for _, l := range getAllLanguageInfoResp.LanguageInfo {
		req.LanguageNum = append(req.LanguageNum, l.LanguageNum)
	}

	resp, err := service.GrpcBackendSeriesImpl.SeriesList(context.Background(), &req)
	if err != nil {
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(c, e.Ok, resp.Msg, resp)
	return
}

func SeriesDel(c *gin.Context) {
	var req backendSeries.SeriesDelReq
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("SeriesDel ShouldBind err", err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	if err := req.Validate(); err != nil {
		err = common.SubstrError(err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	resp, err := service.GrpcBackendSeriesImpl.SeriesDel(context.Background(), &req)
	if err != nil {
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(c, e.Ok, resp.Msg, nil)
	return
}

func GetSeriesLanguageInfo(c *gin.Context) {
	var req backendSeries.GetSeriesLanguageInfoReq
	if err := c.ShouldBind(&req); err != nil {
		logger.Errorf("GetAllLanguageInfoReq ShouldBind err", err)
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	getAllLanguageInforesp, err := service.GrpcShopBrandImpl.GetAllLanguageInfo(context.Background(), &shopbrand.GetAllLanguageInfoReq{})
	if err != nil {
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}

	for _, s := range getAllLanguageInforesp.LanguageInfo {
		languageInfo := &backendSeries.LanguageInfo{
			LanguageNum: s.LanguageNum,
			Language:    s.Language,
		}

		req.LanguageInfo = append(req.LanguageInfo, languageInfo)
	}

	resp, err := service.GrpcBackendSeriesImpl.GetSeriesLanguageInfo(context.Background(), &req)
	if err != nil {
		service.ResponseQuickMsg(c, e.Failed, err.Error(), nil)
		return
	}
	service.ResponseQuickMsg(c, e.Ok, resp.Msg, resp)
	return
}
