package service

import (
	"context"

	"dubbo.apache.org/dubbo-go/v3/common/logger"
	"github.com/fonchain_enterprise/fonchain-main/api/shopbrand"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/serializer"
	"github.com/gin-gonic/gin"
)

func CreateBrand(c *gin.Context) {
	//声明一个变量
	var CreateBrandService shopbrand.CreateBrandreq
	if err := c.ShouldBind(&CreateBrandService); err != nil {
		logger.Errorf("CreateBrand ShouldBind err", err)
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Msg:  err.Error(),
			Code: e.Failed,
		})
		return
	}
	resp, err := GrpcShopBrandImpl.CreateBrand(context.Background(), &CreateBrandService)
	if err != nil {
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Msg:  err.Error(),
			Code: e.Failed,
		})
		return
	}
	ResponseMsg(c, e.UpdatePasswordSuccess, serializer.Response{
		Msg:  resp.Msg,
		Code: e.Ok,
		Data: resp,
	})
}

func BrandList(c *gin.Context) {
	//声明一个对象
	var BrandListService shopbrand.BrandListreq

	if err := c.ShouldBind(&BrandListService); err != nil {
		logger.Errorf("BrandList ShouldBind err", err)
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Msg:  err.Error(),
			Code: e.Failed,
		})
		return
	}
	resp, err := GrpcShopBrandImpl.BrandList(context.Background(), &BrandListService)
	if err != nil {
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Msg:  err.Error(),
			Code: e.Failed,
		})
		return
	}
	ResponseMsg(c, e.SUCCESS, serializer.Response{
		Msg:  resp.Msg,
		Code: e.Ok,
		Data: resp,
	})
}

func BrandInfo(c *gin.Context) {
	//声明一个对象
	var BrandInfoService shopbrand.BrandInforeq

	if err := c.ShouldBind(&BrandInfoService); err != nil {
		logger.Errorf("BrandInfo ShouldBing err", err)
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Msg:  err.Error(),
			Code: e.Failed,
		})
		return
	}
	resp, err := GrpcShopBrandImpl.BrandInfo(context.Background(), &BrandInfoService)
	if err != nil {
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Msg:  err.Error(),
			Code: e.Failed,
		})
		return
	}
	ResponseMsg(c, e.UpdatePasswordSuccess, serializer.Response{
		Msg:  resp.Msg,
		Code: e.Ok,
		Data: resp,
	})
}

func DeleteBrand(c *gin.Context) {
	//声明一个变量
	var DeleteBrandService shopbrand.DeleteBrandreq
	if err := c.ShouldBind(&DeleteBrandService); err != nil {
		logger.Errorf("DeleteBrand ShouldBind err", err)
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	resp, err := GrpcShopBrandImpl.DeleteBrand(context.Background(), &DeleteBrandService)
	if err != nil {
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	ResponseMsg(c, e.SUCCESS, serializer.Response{
		Msg:    resp.Msg,
		Status: e.Ok,
	})
	return
}

func UpdateBrand(c *gin.Context) {
	//声明一个变量
	var UpdateBrandService shopbrand.UpdateBrandreq
	if err := c.ShouldBind(&UpdateBrandService); err != nil {
		logger.Errorf("UpdateBrand ShouldBind err", err)
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	resp, err := GrpcShopBrandImpl.UpdateBrand(context.Background(), &UpdateBrandService)
	if err != nil {
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	ResponseMsg(c, e.SUCCESS, serializer.Response{
		Msg:    resp.Msg,
		Status: e.Ok,
	})
	return
}
