package service

import (
	"context"
	"dubbo.apache.org/dubbo-go/v3/common/logger"
	artShow "github.com/fonchain_enterprise/fonchain-main/api/artShow"
	"github.com/fonchain_enterprise/fonchain-main/api/artist"
	"github.com/fonchain_enterprise/fonchain-main/api/artwork_third_party"
	"github.com/fonchain_enterprise/fonchain-main/api/paybill"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/login"
	"github.com/fonchain_enterprise/fonchain-main/pkg/serializer"
	"github.com/fonchain_enterprise/fonchain-main/pkg/utils/stime"
	"github.com/gin-gonic/gin"
)

// CreateInvoice 上传单据
func CreateInvoice(c *gin.Context) {
	req := new(artShow.Invoice)
	if err := c.ShouldBindJSON(req); err != nil {
		logger.Errorf("CreateInvoice ShouldBind err", err)
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	if req.ArtworkPrice == "" {
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Msg:    e.ErrorShowInvoiceMiss,
			Status: e.Failed,
		})
		return
	}

	userInfo := login.GetUserInfoFromC(c)

	req.UpOperator = userInfo.NickName
	req.OperationTime = stime.Currentime(stime.Format_Normal_YMDhms)

	createInvoiceRes, createInvoiceResErr := GrpcArtShowImpl.CreateInvoice(c, req)
	if createInvoiceResErr != nil {
		ResponseMsg(c, e.Error, serializer.Response{
			Msg:    createInvoiceRes.Msg,
			Status: e.Failed,
		})
		return
	}

	ResponseMsg(c, e.SUCCESS, serializer.Response{
		Status: e.Ok,
	})
}

// ShowInvoice  详情
func ShowInvoice(c *gin.Context) {
	req := new(artShow.InvoiceDetailReq)
	if err := c.ShouldBindJSON(req); err != nil {
		logger.Errorf("InvoiceDetailReq ShouldBind err", err)
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	showInvoiceRes, showInvoiceResErr := GrpcArtShowImpl.QueryInvoice(c, req)
	if showInvoiceResErr != nil {
		ResponseMsg(c, e.Error, serializer.Response{
			Msg:    showInvoiceResErr.Error(),
			Status: e.Failed,
		})
		return
	}

	ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   showInvoiceRes,
		Status: e.Ok,
	})
}

// ShowInvoiceList  列表
func ShowInvoiceList(c *gin.Context) {
	req := new(artShow.InvoiceListReq)
	if err := c.ShouldBindJSON(req); err != nil {
		logger.Errorf("InvoiceListReq ShouldBind err", err)
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	invoiceListRes, invoiceListResErr := GrpcArtShowImpl.QueryInvoiceList(c, req)
	if invoiceListResErr != nil {
		ResponseMsg(c, e.Error, serializer.Response{
			Msg:    invoiceListResErr.Error(),
			Status: e.Failed,
		})
		return
	}

	ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   invoiceListRes,
		Status: e.Ok,
	})
}

// UpdateInvoicePrintNum  更新单据打印次数
func UpdateInvoicePrintNum(c *gin.Context) {
	req := new(artShow.UpdatePrintNum)
	if err := c.ShouldBindJSON(req); err != nil {
		logger.Errorf("InvoiceListReq ShouldBind err", err)
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	updateInvoicePrintNumRes, updateInvoicePrintNumErr := GrpcArtShowImpl.UpdateInvoicePrintNum(c, req)
	if updateInvoicePrintNumErr != nil {
		ResponseMsg(c, e.Error, serializer.Response{
			Msg:    updateInvoicePrintNumRes.Msg,
			Status: e.Failed,
		})
		return
	}

	ResponseMsg(c, e.SUCCESS, serializer.Response{
		Status: e.Ok,
	})
}

// UpdateShowPrintNum  更新画展包打印次数
func UpdateShowPrintNum(c *gin.Context) {
	req := new(artShow.UpdatePrintNum)
	if err := c.ShouldBindJSON(req); err != nil {
		logger.Errorf("InvoiceListReq ShouldBind err", err)
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	updateInvoicePrintNumRes, updateInvoicePrintNumErr := GrpcArtShowImpl.UpdateShowPrintNum(c, req)
	if updateInvoicePrintNumErr != nil {
		ResponseMsg(c, e.Error, serializer.Response{
			Msg:    updateInvoicePrintNumRes.Msg,
			Status: e.Failed,
		})
		return
	}

	ResponseMsg(c, e.SUCCESS, serializer.Response{
		Status: e.Ok,
	})
}

// QueryPrice  查询价格
func QueryPrice(c *gin.Context) {
	req := new(model.QueryInvoiceInfo)
	if err := c.ShouldBindJSON(req); err != nil {
		logger.Errorf("QueryPrice ShouldBind err", err)
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	//  组装 获取 画作 基础信息参数
	artworkDataReq := artwork_third_party.AwDataByShowIdRequest{}
	artworkDataReq.ArtShowIds = append(artworkDataReq.ArtShowIds, req.ShowUID)
	//  获取 画作 基础信息
	resp, err := GrpcArtworkThirdPartyImpl.AwDataByShowId(context.Background(), &artworkDataReq)
	if err != nil {
		logger.Errorf("AwDataByShowId query err", err)
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	if len(resp.Data) < 1 {
		logger.Errorf("ExportShowPrice  query params  err :", "画作信息为空")
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   nil,
			Msg:    "画作信息为空",
			Status: e.Failed,
		})
		return
	}

	// 组装 画作 基础信息
	artworkData := make(map[string]*artwork_third_party.AwDataByShowIdResponse_Info, 0)
	for i := 0; i < len(resp.Data); i++ {
		artworkData[resp.Data[i].ArtworkUuid] = resp.Data[i]
	}

	// 获取 画作价格信息
	artworkPriceList := new(artShow.ArtworkPriceListRes)
	// 获取 画作价格信息
	artworkPriceReq := artShow.ShowDetailReq{}
	artworkPriceReq.ShowUID = append(artworkPriceReq.ShowUID, req.ShowUID)
	artworkPriceList, err = GrpcArtShowImpl.ArtworkPriceList(context.Background(), &artworkPriceReq)
	if err != nil {
		logger.Errorf("ArtworkPriceList query err", err)
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	// 获取 画展包信息
	showDetailReq := new(artShow.ShowDetailReq)
	showDetailReq.ShowUID = append(showDetailReq.ShowUID, req.ShowUID)
	artShowInfo, err := GrpcArtShowImpl.ShowDetail(context.Background(), showDetailReq)
	if err != nil {
		logger.Errorf("ShowDetail query err", err)
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	// 组装 画作 基础信息
	artShowMap := make(map[string]*artShow.ShowDetail, 0)
	for i := 0; i < len(artShowInfo.Data); i++ {
		if artShowInfo.Data[i].Price == 0 {
			ResponseMsg(c, e.SUCCESS, serializer.Response{
				Msg:    e.ErrorNoPrice,
				Status: e.Failed,
			})
			return
		}
		artShowMap[artShowInfo.Data[i].ShowUID] = artShowInfo.Data[i]
	}

	// 获取画家 信息
	artistInfoReq := new(artist.ArtistDetailDataReq)
	artistInfoReq.Uids = append(artistInfoReq.Uids, artShowInfo.Data[0].ArtistUID)
	artistInfo, err := GrpcArtistImpl.ArtistDetailData(context.Background(), artistInfoReq)
	if err != nil {
		logger.Errorf("ArtistDetailData query err", err)
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	artistMap := make(map[string]*artist.ArtistDetailDataResp_Info, 0)
	for i := 0; i < len(artistInfo.Data); i++ {
		if artistMap[artistInfo.Data[i].ProfileInfo.Name] == nil {
			artistMap[artistInfo.Data[i].ProfileInfo.Name] = artistInfo.Data[i]
		}
	}

	priceInfo := serializer.BuildInvoiceInfo(artworkPriceList.Data, artworkData, artistMap)

	priceInfo.Address = artShowMap[req.ShowUID].Address
	priceInfo.ShowTime = artShowMap[req.ShowUID].ShowTime

	if artistMap[artShowInfo.Data[0].ArtistName] != nil {
		priceInfo.BankNo = artistMap[artShowInfo.Data[0].ArtistName].ProfileInfo.BankNum
		priceInfo.Bank = artistMap[artShowInfo.Data[0].ArtistName].ProfileInfo.BankName
		priceInfo.BankCode = artistMap[artShowInfo.Data[0].ArtistName].ProfileInfo.BankCode
	}

	ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   priceInfo,
		Status: e.Ok,
	})
}

func QueryPrice_withoutCopyright(c *gin.Context) {
	req := new(model.QueryInvoiceInfo)
	if err := c.ShouldBindJSON(req); err != nil {
		logger.Errorf("QueryPrice ShouldBind err", err)
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	//  组装 获取 画作 基础信息参数
	artworkDataReq := artwork_third_party.AwDataByShowIdRequest{}
	artworkDataReq.ArtShowIds = append(artworkDataReq.ArtShowIds, req.ShowUID)
	//  获取 画作 基础信息
	resp, err := GrpcArtworkThirdPartyImpl.AwDataByShowId(context.Background(), &artworkDataReq)
	if err != nil {
		logger.Errorf("AwDataByShowId query err", err)
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	if len(resp.Data) < 1 {
		logger.Errorf("ExportShowPrice  query params  err :", "画作信息为空")
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Data:   nil,
			Msg:    "画作信息为空",
			Status: e.Failed,
		})
		return
	}

	// 组装 画作 基础信息
	artworkData := make(map[string]*artwork_third_party.AwDataByShowIdResponse_Info, 0)
	for i := 0; i < len(resp.Data); i++ {
		artworkData[resp.Data[i].ArtworkUuid] = resp.Data[i]
	}

	// 获取 画作价格信息
	artworkPriceList := new(artShow.ArtworkPriceListRes)
	// 获取 画作价格信息
	artworkPriceReq := artShow.ShowDetailReq{}
	artworkPriceReq.ShowUID = append(artworkPriceReq.ShowUID, req.ShowUID)
	artworkPriceList, err = GrpcArtShowImpl.ArtworkPriceList(context.Background(), &artworkPriceReq)
	if err != nil {
		logger.Errorf("ArtworkPriceList query err", err)
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	// 获取 画展包信息
	showDetailReq := new(artShow.ShowDetailReq)
	showDetailReq.ShowUID = append(showDetailReq.ShowUID, req.ShowUID)
	artShowInfo, err := GrpcArtShowImpl.ShowDetail(context.Background(), showDetailReq)
	if err != nil {
		logger.Errorf("ShowDetail query err", err)
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}

	// 组装 画作 基础信息
	artShowMap := make(map[string]*artShow.ShowDetail, 0)
	for i := 0; i < len(artShowInfo.Data); i++ {
		if artShowInfo.Data[i].Price == 0 {
			ResponseMsg(c, e.SUCCESS, serializer.Response{
				Msg:    e.ErrorNoPrice,
				Status: e.Failed,
			})
			return
		}
		artShowMap[artShowInfo.Data[i].ShowUID] = artShowInfo.Data[i]
	}

	// 获取画家 信息
	artistInfoReq := new(artist.ArtistDetailDataReq)
	artistInfoReq.Uids = append(artistInfoReq.Uids, artShowInfo.Data[0].ArtistUID)
	artistInfo, err := GrpcArtistImpl.ArtistDetailData(context.Background(), artistInfoReq)
	if err != nil {
		logger.Errorf("ArtistDetailData query err", err)
		ResponseMsg(c, e.SUCCESS, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	artistMap := make(map[string]*artist.ArtistDetailDataResp_Info, 0)
	for i := 0; i < len(artistInfo.Data); i++ {
		if artistMap[artistInfo.Data[i].ProfileInfo.Name] == nil {
			artistMap[artistInfo.Data[i].ProfileInfo.Name] = artistInfo.Data[i]
		}
	}

	priceInfo := serializer.BuildInvoiceInfo_withoutCopyright(artworkPriceList.Data, artworkData, artistMap)

	priceInfo.Address = artShowMap[req.ShowUID].Address
	priceInfo.ShowTime = artShowMap[req.ShowUID].ShowTime
	priceInfo.TalentAgency = artShowMap[req.ShowUID].TalentAgency

	if artistMap[artShowInfo.Data[0].ArtistName] != nil {
		priceInfo.BankNo = artistMap[artShowInfo.Data[0].ArtistName].ProfileInfo.BankNum
		priceInfo.Bank = artistMap[artShowInfo.Data[0].ArtistName].ProfileInfo.BankName
		priceInfo.BankCode = artistMap[artShowInfo.Data[0].ArtistName].ProfileInfo.BankCode
	}

	ResponseMsg(c, e.SUCCESS, serializer.Response{
		Data:   priceInfo,
		Status: e.Ok,
	})
}

// UpdateShowInvoiceStatus  更新单据状态
func UpdateShowInvoiceStatus(c *gin.Context) {
	req := new(artShow.UpdateShowInvoiceStatusReq)
	if err := c.ShouldBindJSON(req); err != nil {
		logger.Errorf("InvoiceListReq ShouldBind err", err)
		ResponseMsg(c, e.InvalidParams, serializer.Response{
			Msg:    err.Error(),
			Status: e.Failed,
		})
		return
	}
	userInfo := login.GetUserInfoFromC(c)

	updateShowInvoiceStatusRes, updateShowInvoiceStatusErr := GrpcArtShowImpl.UpdateShowInvoiceStatus(c, req)
	if updateShowInvoiceStatusErr != nil {
		ResponseMsg(c, e.Error, serializer.Response{
			Msg:    updateShowInvoiceStatusRes.Msg,
			Status: e.Failed,
		})
		return
	}

	// 更新单据状态成功后，更新画展包付款单
	setDelivered, setDeliveredErr := GrpcPayBillImpl.SetDelivered(context.Background(), &paybill.ShowRequest{
		ShowUid:      req.ShowUID,
		OperatorId:   uint32(userInfo.ID),
		OperatorName: userInfo.NickName,
	})
	if setDeliveredErr != nil {
		ResponseMsg(c, e.Error, serializer.Response{
			Msg:    setDelivered.Msg,
			Status: e.Failed,
		})
		return
	}

	ResponseMsg(c, e.SUCCESS, serializer.Response{
		Status: e.Ok,
	})
}
