package task

import (
	"dubbo.apache.org/dubbo-go/v3/common/logger"
	"encoding/json"
	"errors"
	"fmt"
	art_show "github.com/fonchain_enterprise/fonchain-main/api/artShow"
	"github.com/fonchain_enterprise/fonchain-main/api/artwork_query"
	"github.com/fonchain_enterprise/fonchain-main/api/department"
	"github.com/fonchain_enterprise/fonchain-main/api/task"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/login"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/account"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service/auth"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
)

var (
	AuthAssuranceRule = "task_auth_assurance_1" //鉴证科权限
	AuthMathRule      = "task_auth_math"        //数字化权限
)

func Detail(c *gin.Context) {
	req := &task.TaskDetail{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	res, err := service.TaskProvider.Detail(c, req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func Create(c *gin.Context) {
	req := &task.TaskRequest{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	userInfo := login.GetUserInfoFromC(c)
	fmt.Println(userInfo)

	if len(req.JoinUsers) != 1 {
		service.Error(c, e.Error, errors.New("申请人数仅能为一人"))
		return
	}

	b := auth.HasAuth(c, req.JoinUsers[0].UserID, AuthAssuranceRule)
	b2 := auth.HasAuth(c, req.JoinUsers[0].UserID, AuthMathRule)

	if b == false && b2 == false {
		service.Error(c, e.Error, errors.New(req.JoinUsers[0].Name+"当前没有 数字化或者鉴证科 的数据接收权限"))
		return
	}

	if b == true && b2 == true {
		service.Error(c, e.Error, errors.New(req.JoinUsers[0].Name+"同时拥有数字化和鉴证科数据接收权限权限，无法进行默认二选一，请修改其权限"))
		return
	}

	res, err := service.TaskProvider.Create(c, req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}
	positionName := "鉴证科"
	if b2 == true {
		positionName = "数字化"
	}

	err = SendArtwork(c, req, positionName)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}
	service.Success(c, res)

	return
}

func Update(c *gin.Context) {
	req := &task.TaskRequest{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	res, err := service.TaskProvider.Update(c, req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func Remove(c *gin.Context) {
	req := &task.TaskDetail{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	oldRes, err := service.TaskProvider.Detail(c, req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	joinUserInfo, err := account.GetUserInfoById(c, oldRes.JoinUsers[0].UserID, "")
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	res, err := service.TaskProvider.Remove(c, req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	var userIDs []int32
	var artworkIDs []int32
	for _, t := range oldRes.JoinUsers {
		userIDs = append(userIDs, int32(t.UserID))
	}

	for _, t := range oldRes.Artworks {
		artworkIDs = append(artworkIDs, int32(t.ArtworkID))
	}

	//删除按照需求要
	userInfo := login.GetUserInfoFromC(c)
	awReq := artwork_query.UpdateAwStockStatusRequest{
		AdminId:    int32(userInfo.ID),
		ActionType: 4,
		ArtworkIds: artworkIDs,
		AllotUids:  userIDs,
		PostName:   joinUserInfo.PositionUsers[0].PositionName,
	}

	s, e1 := json.Marshal(awReq)
	fmt.Println("请求之前", awReq)
	logger.Errorf("主管工作台task--删除", string(s), e1)
	newRes, err := service.GrpcArtworkQueryImpl.UpdateAwStockStatus(c, &awReq)
	fmt.Println("请求之后", err)
	s2, e2 := json.Marshal(newRes)
	logger.Errorf("主管工作台task-res--删除", string(s2), e2, err)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func Finish(c *gin.Context) {
	req := &task.SetStatusRequest{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}
	req.Status = 2
	res, err := service.TaskProvider.SetStatus(c, req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func DataInfo(c *gin.Context) {
	req := &task.DataInfoRequest{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}
	req.Type = e.Task_Type_Art

	userInfo := login.GetUserInfoFromC(c)

	//获取自己管理的小弟id列表
	bossReq := department.UserIdByBossIdRequest{
		UserId: userInfo.ID,
	}

	bossRes, err := service.DepartmentProvider.UserIdByBossId(c, &bossReq)
	fmt.Println("获取管理之下的userId", bossRes, err)

	req.SubmitterIDs = bossRes.UserIds

	res, err := service.TaskProvider.DataInfo(c, req)

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func List(c *gin.Context) {
	req := &task.TaskList{}
	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	req.Type = e.Task_Type_Art
	userInfo := login.GetUserInfoFromC(c)

	//获取自己管理的小弟id列表
	bossReq := department.UserIdByBossIdRequest{
		UserId: userInfo.ID,
	}
	bossRes, err := service.DepartmentProvider.UserIdByBossId(c, &bossReq)
	fmt.Println("获取管理之下的userId", bossRes, err)

	req.SubmitterIDs = bossRes.UserIds

	res, err := service.TaskProvider.List(c, req)
	var artistIds []string

	for _, v := range res.Data {
		for _, v1 := range v.Artworks {
			if v1.ArtistID != "" {
				artistIds = append(artistIds, v1.ArtistID)
			}
		}
	}

	if len(artistIds) > 0 {
		artshowResp, err := service.GrpcArtShowImpl.ArtistExhibitionCount(c, &art_show.ArtistExhibitionCountReq{ArtistUid: artistIds})
		if err != nil {
			service.Error(c, e.InvalidParams, err)
			return
		}
		artistMap := make(map[string]*art_show.ArtistCount, len(artshowResp.ArtistShowCount))

		for k, v := range artshowResp.ArtistShowCount {
			artistMap[v.ArtistUid] = artshowResp.ArtistShowCount[k]
		}

		for k, v := range res.Data {
			for k1, v1 := range v.Artworks {
				if o, ok := artistMap[v1.ArtistID]; ok == true {
					tempArtistName := v1.ArtistName
					if o.BackStatus == 2 {
						tempArtistName = "#" + v1.ArtistName
					}

					res.Data[k].Artworks[k1].ArtistShowCount = fmt.Sprintf("%s(%d)", tempArtistName, o.ShowCount)
				}
			}
		}
	}

	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func SendArtwork(c *gin.Context, taskReq *task.TaskRequest, positionName string) error {
	var err error
	var artworkIds, allotUids []int32

	for _, tt := range taskReq.Artworks {
		artworkIds = append(artworkIds, int32(tt.ArtworkID))
	}

	for _, tt := range taskReq.JoinUsers {
		allotUids = append(allotUids, int32(tt.UserID))
	}

	//非同一岗位请分批处理
	/*
		joinUserInfo, err := account.GetUserInfoById(c, taskReq.JoinUsers[0].UserID, "")
		if err != nil {
			return err
		}
	*/

	req := &artwork_query.UpdateAwStockStatusRequest{
		ActionType:  3,
		ArtworkIds:  artworkIds,
		AllotUids:   allotUids,
		ReceiveDate: taskReq.StartAt,
		PostName:    positionName,
		//PostName:    joinUserInfo.PositionUsers[0].PositionName,
	}

	userInfo := login.GetUserInfoFromC(c)

	s, e1 := json.Marshal(req)
	logger.Errorf("主管工作台创建---", string(s), e1)
	fmt.Println("3-----------请求之前", req)
	req.AdminId = int32(userInfo.ID)
	newRes, errs := service.GrpcArtworkQueryImpl.UpdateAwStockStatus(c, req)
	fmt.Println("3-----------请求之后", newRes, errs)
	a, e2 := json.Marshal(newRes)
	logger.Errorf("主管工作台创建-res--", string(a), e2, errs)
	if errs != nil {
		return err
	}

	return nil
}
