package turnstile

import (
	"context"
	"encoding/json"
	"fmt"
	"github.com/fonchain_enterprise/fonchain-main/pkg/cache"
	"github.com/fonchain_enterprise/fonchain-main/pkg/common/dingding"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/vo/turnstile"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/gin-gonic/gin"
	"github.com/gorilla/websocket"
	"log"
	"net/http"
	"sync"
	"time"
)

func DeviceWs(c *gin.Context) {

	// 获取WebSocket连接
	var cstUpgrader = websocket.Upgrader{
		Subprotocols:      []string{"p0", "p1"},
		ReadBufferSize:    1024,
		WriteBufferSize:   1024,
		EnableCompression: true,
		Error: func(w http.ResponseWriter, r *http.Request, status int, reason error) {
			http.Error(w, reason.Error(), status)
		},
		CheckOrigin: func(r *http.Request) bool {
			return true
		},
	}

	ws, err := cstUpgrader.Upgrade(c.Writer, c.Request, http.Header{"Set-Cookie": {"sessionID=1234"}})
	if err != nil {
		service.Error(c, e.Failed, nil, "websocket升级失败")
		return
	}

	defer ws.Close()

	activeMsg(ws)

	fmt.Println("结束")

	return
}

func activeMsg(ws *websocket.Conn) {
	var wg sync.WaitGroup
	var deviceInfo turnstile.DevicePing

	_, p, err := ws.ReadMessage()
	if err != nil {
		println("连接错误", err)
		return
	}

	fmt.Println("首次数据为:", string(p))
	err = json.Unmarshal(p, &deviceInfo)
	if err != nil || deviceInfo.Sn == "" {
		fmt.Println("连接信息解析失败", deviceInfo)
		return
	}
	ctx, cancel := context.WithCancel(context.Background())
	stdoutDone := make(chan struct{}, 3)
	deviceId := deviceInfo.Sn
	fmt.Println("设备标号是", deviceId)
	wg.Add(3)
	go ping(ctx, ws, &wg, stdoutDone, deviceId) //存活检测，检测客户端是否存活
	go sendMsg(ctx, ws, &wg, deviceId)          //推送给机器数据
	go getMsg(ctx, ws, &wg, deviceId)           //获取机器推送数据

	go func(stdoutDone chan struct{}) {
		<-stdoutDone
		cancel()
	}(stdoutDone)

	wg.Wait()

}

// sendMsg 向推送数据
func sendMsg(ctx context.Context, ws *websocket.Conn, wg *sync.WaitGroup, deviceId string) {
	defer wg.Done()

	ticker := time.NewTicker(10 * time.Second)
	defer ticker.Stop()

	println("goroutinue is running")
	// 处理WebSocket连接
	for {
		time.Sleep(1 * time.Second)
		select {
		case <-ctx.Done():
			println("goroutinue-send is stop")
			return
		default:
			//println("goroutinue is running")
		}

		info := cache.RedisClient.LPop(cache.GetTurnstileSendListKey(deviceId)).Val()

		if info == "" {
			continue
		}

		println("主动推送数据", info)
		err := ws.WriteMessage(websocket.TextMessage, []byte(info))
		if err != nil {
			println("推送数据失败", err)
		}

		//监控

	}

}

// getMsg 主动ping客户端
func getMsg(ctx context.Context, ws *websocket.Conn, wg *sync.WaitGroup, deviceId string) {
	defer wg.Done()

	// 处理WebSocket连接
	for {
		var req turnstile.DevicePing
		select {
		case <-ctx.Done():
			println("goroutinue-getmsg is stop")
			return
		default:
			println("goroutinue-getmsg is running")
		}

		messageType, p, err := ws.ReadMessage()
		if err != nil {
			break
		}

		fmt.Println("解析设备数据:", messageType, string(p))

		err = json.Unmarshal(p, &req)
		if err != nil {
			log.Println("解析失败提示,提示:", err)
			continue
		}
		if req.Cmd == "ping" {
			fmt.Println("推送数据")
			cache.RedisClient.RPush(cache.GetTurnstileSendListKey(deviceId), "{cmd: \"pong\"}")
		}

		if req.Extra != "" {
			cache.RedisClient.Set(cache.GetPushResInfo(req.Extra), string(p), 60*time.Second)
		}

	}
}

// ping 主动ping客户端
func ping(ctx context.Context, ws *websocket.Conn, wg *sync.WaitGroup, done chan struct{}, deviceSn string) {
	defer wg.Done()
	ticker := time.NewTicker(10 * time.Second)
	defer ticker.Stop()
	for {
		cache.RedisClient.Set(cache.GetPushDeviceSnInfo(deviceSn), 1, 20*time.Second)
		select {
		case <-ctx.Done():
			log.Println("健康监测监测被父协程停止")
			cache.RedisClient.Del(cache.GetPushDeviceSnInfo(deviceSn))
			return
		case <-ticker.C:
			if err := ws.WriteControl(websocket.PingMessage, []byte{}, time.Now().Add(10*time.Second)); err != nil {
				done <- struct{}{}
				dingding.Send("打卡机下线请关注(服务端ping打卡机器异常),提示:"+err.Error(), err)
				log.Println("打卡机下线请关注(服务端ping打卡机器异常),提示:", err)
				cache.RedisClient.Del(cache.GetPushDeviceSnInfo(deviceSn))
				return
			}
		}
	}
}
