package weekmeal

import (
	"github.com/fonchain_enterprise/fonchain-main/api/appointment"
	"github.com/fonchain_enterprise/fonchain-main/api/buy"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/fonchain_enterprise/fonchain-main/pkg/utils/qr"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
)

func Weeks(c *gin.Context) {

	var req appointment.GetNextWeekWorkDaysRequest

	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	res, err := service.AppointmentProvider.GetWeeks(c, &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func Update(c *gin.Context) {

	var req appointment.WeekInfoResponse

	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	res, err := service.AppointmentProvider.SetWeekInfo(c, &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func UpdateWeekInfo(c *gin.Context) {

	var req appointment.UpdateWeekInfoRequest

	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	res, err := service.AppointmentProvider.UpdateWeekInfo(c, &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func Week(c *gin.Context) {

	var req appointment.GetWeekInfoRequest

	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	res, err := service.AppointmentProvider.GetWeekInfo(c, &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

func CreateQr(c *gin.Context) {

	req := &buy.CreateQrCodeRequest{}

	res, err := service.BuyProvider.CreateQrCode(c, req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	logo := "https://dci-file-new.bj.bcebos.com/fonchain-main/prod/image/artist/recent_photo/0/9ed3bc16-a0f2-41c5-940e-7c40ad4c841d.png"
	s, _ := qr.CreateQrBase64(res.QrUrl, logo)

	resMap := make(map[string]interface{})
	resMap["base"] = s
	resMap["url"] = res
	service.Success(c, resMap)
	return
}
