package work_day_meal

import (
	"github.com/fonchain_enterprise/fonchain-main/api/appointment"
	"github.com/fonchain_enterprise/fonchain-main/pkg/e"
	"github.com/fonchain_enterprise/fonchain-main/pkg/model/login"
	"github.com/fonchain_enterprise/fonchain-main/pkg/service"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
)

//NowNextWorkDay 当天的下一个工作日
func NowNextWorkDay(c *gin.Context) {
	res, err := service.AppointmentProvider.NowNextWorkDay(c, &appointment.EmptyRequest{})
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

//MealList 工作日餐的列表
func MealList(c *gin.Context) {

	var req appointment.MealListRequest

	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	res, err := service.AppointmentProvider.MealList(c, &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

//UserWorkDayMeal 登陆者某天的预约记录
func UserWorkDayMeal(c *gin.Context) {

	var req appointment.WorkDayMealRequest

	if err := c.ShouldBindBodyWith(&req, binding.JSON); err != nil {
		service.Error(c, e.InvalidParams, err)
		return
	}

	userInfo := login.GetUserInfoFromC(c)
	req.UserID = uint32(userInfo.ID)
	req.UserTel = userInfo.TelNum
	req.UserName = userInfo.NickName
	req.UserAvatar = userInfo.Avatar

	res, err := service.AppointmentProvider.UserWorkDayMeal(c, &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

//SureWorkDayMeal 申请预约
func SureWorkDayMeal(c *gin.Context) {

	req := appointment.WorkDayMealRequest{}

	workRes, err := service.AppointmentProvider.NowNextWorkDay(c, &appointment.EmptyRequest{})
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	req.Day = workRes.WorkDay

	userInfo := login.GetUserInfoFromC(c)
	req.UserID = uint32(userInfo.ID)
	req.UserTel = userInfo.TelNum
	req.UserName = userInfo.NickName
	req.UserAvatar = userInfo.Avatar

	res, err := service.AppointmentProvider.SureWorkDayMeal(c, &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}

//CancelWorkDayMeal 取消预约
func CancelWorkDayMeal(c *gin.Context) {

	req := appointment.WorkDayMealRequest{}

	workRes, err := service.AppointmentProvider.NowNextWorkDay(c, &appointment.EmptyRequest{})
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	req.Day = workRes.WorkDay

	userInfo := login.GetUserInfoFromC(c)
	req.UserID = uint32(userInfo.ID)
	req.UserTel = userInfo.TelNum
	req.UserName = userInfo.NickName
	req.UserAvatar = userInfo.Avatar

	res, err := service.AppointmentProvider.CancelWorkDayMeal(c, &req)
	if err != nil {
		service.Error(c, e.Error, err)
		return
	}

	service.Success(c, res)
	return
}
