// Package excel -----------------------------
// @file      : templateInter_test.go
// @author    : JJXu
// @contact   : wavingbear@163.com
// @time      : 2022/7/23 15:59
// -------------------------------------------
package example

import (
	"fmt"
	"path/filepath"
	"strings"
	"testing"
	"time"

	"github.com/fonchain_enterprise/fonchain-main/pkg/utils/excel"
)

// 定义sheet表结构
type sheet1Define struct {
	Xid  int    `json:"xid" form:"xid" db:"column:xid;comment: "`
	Name string `json:"name" form:"name" db:"column:name;comment: "`
	Age  int    `json:"age" form:"age" db:"column:age;comment: "`
}

// 通过模板文件创建数据（表头已在模板中定义好了）
func TestExcelTpl_WriteToExcelUseTemplateFile(t *testing.T) {
	var sheet1Data = []sheet1Define{
		{1, "张三", 16},
		{2, "黑猫警长", 18},
	}

	var sheet1 = excel.NewTemplateSheet("Sheet1", sheet1Data)
	//var suffixFunc = func() string { return fmt.Sprintf("%v", time.Now().Unix()) }
	exCreator, err := excel.NewExcelCreatorFromTemplate("demo.xlsx", "./", "./demo.xlsx", sheet1)
	if err != nil {
		t.Log(err)
	}
	exCreator.UseOption(func(excel *excel.Excel) {
		ext := filepath.Ext(excel.SaveName)
		name := strings.Split(excel.SaveName, ext)[0]
		excel.SaveName = fmt.Sprintf("%s_%v%s", name, time.Now().Unix(), ext)
	})
	path, name, err := exCreator.WriteToFile()
	if err != nil {
		t.Log(err)
	} else {
		t.Log(path, name)
	}
}

// 不通过模板文件，自定义表头、数据内容
func TestExcelTpl_WriteToEmptyExcel(t *testing.T) {
	var sheet1Columns = []string{"序号", "姓名", "年龄"}
	var sheet1Data = []sheet1Define{
		{1, "张三", 16},
		{2, "黑猫警长", 18},
	}

	var sheet1 = excel.NewSheet("Sheet1", sheet1Data, sheet1Columns)
	//var suffixFunc = func() string { return fmt.Sprintf("%v", time.Now().Unix()) }
	exCreator, err := excel.NewExcelCreatorFromTemplate("demo.xlsx", "./", "", sheet1)
	if err != nil {
		t.Log(err)
	}
	exCreator.UseOption(func(excel *excel.Excel) {
		ext := filepath.Ext(excel.SaveName)
		name := strings.Split(excel.SaveName, ext)[0]
		excel.SaveName = fmt.Sprintf("%s_%v%s", name, time.Now().Unix(), ext)
	})
	path, name, err := exCreator.WriteToFile()
	if err != nil {
		t.Log(err)
	} else {
		t.Log(path, name)
	}
}

// 不通过模板文件，自定义表头、数据内容，手动指定要插入的字段和顺序
func TestExcelTpl_WriteToEmptyExcel_SpecifiedField(t *testing.T) {
	var sheet1Columns = []string{"年龄", "姓名"}
	var jsonFiledList = []string{"age", "name"}
	var sheet1Data = []sheet1Define{
		{1, "张三", 16},
		{2, "黑猫警长", 18},
	}

	var sheet1 = excel.NewSheet("Sheet1", sheet1Data, sheet1Columns, jsonFiledList)
	//var suffixFunc = func() string { return fmt.Sprintf("%v", time.Now().Unix()) }
	exCreator, err := excel.NewExcelCreatorFromTemplate("demo.xlsx", "./", "", sheet1)
	if err != nil {
		t.Log(err)
	}
	exCreator.UseOption(func(excel *excel.Excel) {
		ext := filepath.Ext(excel.SaveName)
		name := strings.Split(excel.SaveName, ext)[0]
		excel.SaveName = fmt.Sprintf("%s_%v%s", name, time.Now().Unix(), ext)
	})
	path, name, err := exCreator.WriteToFile()
	if err != nil {
		t.Log(err)
	} else {
		t.Log(path, name)
	}
}

// 传入结构体数据
func TestExcelTpl_WriteToEmptyExcel_UseStruct(t *testing.T) {
	var sheet1Columns = []string{"年龄", "姓名"}
	var jsonFiledList = []string{"age", "name"}
	var sheet1Data = []sheet1Define{
		{1, "张三", 16},
	}

	var sheet1 = excel.NewSheet("Sheet1", sheet1Data, sheet1Columns, jsonFiledList)
	//var suffixFunc = func() string { return fmt.Sprintf("%v", time.Now().Unix()) }
	exCreator, err := excel.NewExcelCreatorFromTemplate("demo.xlsx", "./", "", sheet1)
	if err != nil {
		t.Log(err)
	}
	exCreator.UseOption(func(excel *excel.Excel) {
		ext := filepath.Ext(excel.SaveName)
		name := strings.Split(excel.SaveName, ext)[0]
		excel.SaveName = fmt.Sprintf("%s_%v%s", name, time.Now().Unix(), ext)
	})
	path, name, err := exCreator.WriteToFile()
	if err != nil {
		t.Log(err)
	} else {
		t.Log(path, name)
	}
}

func TestReadExcel(t *testing.T) {
	var file = "demo_1671435052.xlsx"
	ex := excel.Excel{OriginFilePath: file}
	var datas []sheet1Define
	err := ex.ReadSheetData("Sheet1", func(rowIndex int, rows []string) {
		if rowIndex == 0 {
			//跳过首行
			return
		}
		datas = append(datas, sheet1Define{
			Xid:  excel.Int[int](rows[0]),
			Name: rows[1],
			Age:  excel.Int[int](rows[2]),
		})
	})
	if err != nil {
		t.Error(err.Error())
	}
	fmt.Println(datas)

}
